/**
*
* $Id: ARIES.cpp
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "Aries.h"
/*
double LiftCoeff (double aoa)
{
	const int nlift = 9;
	static const double AOA[nlift] = {-180*RAD,-165*RAD,-164*RAD,-45*RAD,0*RAD,45*RAD,164*RAD,165*RAD,180*RAD};
	static const double CL[nlift]  = {0.06375,  0.1275,       0,      0,    0,     0,      0,      0,0.06375};
	//	static const double CL[nlift]  = {       0,  0.1275,       0,      0,    0,     0,      0,-0.1275,      0};
	static const double SCL[nlift] = {(CL[1]-CL[0])/(AOA[1]-AOA[0]), (CL[2]-CL[1])/(AOA[2]-AOA[1]),
		                              (CL[3]-CL[2])/(AOA[3]-AOA[2]), (CL[4]-CL[3])/(AOA[4]-AOA[3]),
									  (CL[5]-CL[4])/(AOA[5]-AOA[4]), (CL[6]-CL[5])/(AOA[6]-AOA[5]),
									  (CL[7]-CL[6])/(AOA[7]-AOA[6]), (CL[8]-CL[7])/(AOA[8]-AOA[7])};
	for (int i = 0; i < nlift-1 && AOA[i+1] < aoa; i++);
	return CL[i] + (aoa-AOA[i])*SCL[i];
}
*/



// 1. vertical lift component (wings and body)


void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = {0.06375,  0.1275,  0,   0,  0,  0,    0,     0,      0,   0,     0,     0,     0,       0,       0,     0,    0,    0,   0,     0,     0,     0,     0,  0.06375};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,     0,     0,   0,     0,     0,     0,       0,       0,     0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.

	// change for drag increase at high Mach:
	// Subsonic Equivalents

//	double Cl0, Cl1, Cd0, Cd1, Cd2;
/*	if(M>=5.0) {
		Cl0 =  0.0;
		Cl1 =  1.675600;
		Cd0 =  0.068540;
		Cd1 = -0.412900;
		Cd2 =  1.539960;
		*cl = Cl0 + Cl1*aoa;
		*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
*/
	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
/*	if(M<1.25) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.04 + oapiGetInducedDrag (*cl, 2.1, 0.82);
	}
	else if(M<5.0) { 
 */
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cd = 0.2 + oapiGetInducedDrag (*cl, 2.1, 0.65);

// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
//	}
}

// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}


ARIES::ARIES (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hFront				= oapiLoadMeshGlobal ("2001/ArFront");
	hMid				= oapiLoadMeshGlobal ("2001/ArMid");
	hAft				= oapiLoadMeshGlobal ("2001/ArAft");
	hGear				= oapiLoadMeshGlobal ("2001/ArGear");
	hAero				= oapiLoadMeshGlobal ("2001/ArAero");
	hHab				= oapiLoadMeshGlobal ("2001/ArHab");
	hGCR				= oapiLoadMeshGlobal ("2001/ArGCR");
	hWindRing			= oapiLoadMeshGlobal ("2001/ArWindRing");
	hNoWindRing			= oapiLoadMeshGlobal ("2001/ArNoWindRing");
	hFwdLock			= oapiLoadMeshGlobal ("2001/ArFwdLock");
	hWeapRing			= oapiLoadMeshGlobal ("2001/ArWeapRing");
	hHercHab			= oapiLoadMeshGlobal ("2001/ArHercHab");
	hPaxInt				= oapiLoadMeshGlobal ("2001/ArPaxInt");
	hCgoInt				= oapiLoadMeshGlobal ("2001/ArCgoInt");
	hDummy1				= oapiLoadMeshGlobal ("2001/dummy");
	hDummy2				= oapiLoadMeshGlobal ("2001/dummy2");
	hDummy3				= oapiLoadMeshGlobal ("2001/dummy3");

	tex = oapiRegisterExhaustTexture("RealExhaust");

	/* set the propellant handles to NULL so I can check if I need to 
	/* create them in the SetMainStage() function. */
	ph_Aries=NULL;
	ph_LOX=NULL;
	Dock0=NULL;
	Dock1=NULL;
	Dock2=NULL;
	bShieldJettisoned=false;
	bDeorbitFired=false;
	bThrustTerminated=false;
	bEjected=false;
	bEVA=false;
	Drymass = ARIESIB_DRYMASS;
	Fuelmass = ARIES_PROP_MASS;
	Thrust = THRUST_FIRST_VAC;
	Ispvac = ISP_FIRST_VAC;
	bDoorsOpen = false;
	bWindows = true;
	bLegs = true;
	bLock = false;
	bAero = false;
	bHab = false;
	bGCR = false;
	bLOXInjection = false;
	bWeapRing = false;
	bHercHab = false;
	numatch = 0;
	gear_status=DOOR_OPEN;
	gear_proc=1.0;
	pldoors_status=DOOR_CLOSED;
	pldoors_proc=0.0;
	engdoors_status=DOOR_OPEN;
	engdoors_proc=1.0;
	hab_status=DOOR_CLOSED;
	hab_proc=0.0;
	oldsimt = -0.01;
	Vold = _V(0,0,0);
	bAbort = false;
	AbortChecked = false;
	bAnimationsMade = false;
	FwdZ = 0.0;
	AftZ = 0.0;
	FwdHelper = 0.0;
	AftHelper = 0.0;
	bEngReset = false;
	Firstsweep = true;

	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "ARIESIB" ) ) {
		Drymass = ARIESIB_DRYMASS;
		Fuelmass = ARIES_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = false;
		bWindows = true;
		bLegs = true;
		bLock = false;
		bAero = false;
		bHab = false;
	}
	else if( strstr( classname, "ARIESIC" ) ) {
		Drymass = ARIESIB_DRYMASS-PAXPROVISION_MASS;
		Fuelmass = ARIES_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = true;
		bLock = false;
		bAero = false;
		bHab = false;
		numatch = 7;
	}
	else if( strstr( classname, "ARIESIIB" ) ) {
		Drymass = ARIESIB_DRYMASS+AERO_DRYMASS+BIGGER_ENGINES;
		Fuelmass = ARIESII_PROP_MASS;
		Thrust = THRUST_MARS_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = false;
		bWindows = true;
		bLegs = true;
		bLock = true;
		bAero = true;
		bHab = false;
		bLOXInjection = true;
	}
	else if( strstr( classname, "ARIESIIC" ) ) {
		Drymass = ARIESIB_DRYMASS+AERO_DRYMASS+BIGGER_ENGINES-PAXPROVISION_MASS;
		Fuelmass = ARIESII_PROP_MASS;
		Thrust = THRUST_MARS_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = true;
		bLock = true;
		bAero = true;
		bHab = false;
		bLOXInjection = true;
		numatch = 7;
	}
	else if( strstr( classname, "TAURUSIB" ) ) {
		Drymass = ARIESIB_DRYMASS-LEGS_DRYMASS+LOCK_DRYMASS-PAXPROVISION_MASS;
		Fuelmass = ARIES_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHab = false;
		numatch = 14;
	}
	else if( strstr( classname, "AQUARIUSIC" ) ) {
		Drymass = ARIESIB_DRYMASS-PAXPROVISION_MASS-LEGS_DRYMASS+LOCK_DRYMASS+HAB_DRYMASS;
		Fuelmass = AQUARIUS_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHab = true;
		FwdZ = 10.0;
		AftZ = -15.5;
		FwdHelper = 10.0;
		numatch = 7;
	}
	else if( strstr( classname, "AQUARIUSIIC" ) ) {
		Drymass = ARIESIB_DRYMASS-PAXPROVISION_MASS+LOCK_DRYMASS+HAB_DRYMASS;
		Fuelmass = AQUARIUS_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = true;
		bLock = true;
		bAero = false;
		bHab = true;
		FwdZ = 10.0;
		AftZ = -15.5;
		FwdHelper = 10.0;
		numatch = 7;
	}
	else if( strstr( classname, "AQUARIUSID" ) ) {
		Drymass = ARIESIB_DRYMASS-PAXPROVISION_MASS-LEGS_DRYMASS+LOCK_DRYMASS+HAB_DRYMASS+GCR_DRYMASS;
		Fuelmass = AQUARIUS_PROP_MASS;
		Thrust = THRUST_GCR_VAC;
		Ispvac = ISP_GCR_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHab = true;
		bGCR = true;
		FwdZ = 10.0;
		AftZ = -15.5;
		FwdHelper = 10.0;
		AftHelper = -44.0+AftZ;
		numatch = 7;
	}
	else if( strstr( classname, "HERCULISIC" ) ) {
		Drymass = ARIESIB_DRYMASS-PAXPROVISION_MASS-LEGS_DRYMASS+LOCK_DRYMASS+LRTANKS_DRYMASS;
		Fuelmass = AQUARIUS_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHercHab = true;
		FwdZ = 10.0;
		AftZ = -15.5;
		FwdHelper = 10.0;
		numatch = 11;
	}
	else if( strstr( classname, "DRACOIB" ) ) {
		Drymass = ARIESIB_DRYMASS-LEGS_DRYMASS+LOCK_DRYMASS-PAXPROVISION_MASS+DRACO_DRYMASS;
		Fuelmass = ARIES_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHab = false;
		bWeapRing = true;
		FwdZ = 2.0;
		AftZ = -2.0;
		numatch = 14;
	}
	else if( strstr( classname, "1XE" ) ) {
		Drymass = ARIESIB_DRYMASS-PAXPROVISION_MASS-LEGS_DRYMASS+LOCK_DRYMASS+GCR_DRYMASS;
		Fuelmass = ARIES_PROP_MASS;
		Thrust = THRUST_GCR_VAC;
		Ispvac = ISP_GCR_VAC;
		bDoorsOpen = false;
		bWindows = false;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHab = false;
		bGCR = true;
		AftHelper = -44.0;
		numatch = 14;
	}



//	sat_attach = CreateAttachment (false, OFS_SAT+_V(0,0,FwdZ), _V(0,0,1), _V(0,1,0), "X");
	
	//		     			   child        pos               dir       rot      handle
	Dockpos = CreateAttachment (false, _V(0,0,9.82+FwdZ),   _V(0,0,1), _V(0,1,0), "DOCK");

	if (bHercHab) {
		attach[0] = _V(-3.0,4.4,5.5+FwdZ);
		attachdir[0] = _V(0,-1,0);
		attachrot[0] = _V(0,0,-1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[0] = CreateAttachment (false, attach[0],   attachdir[0], attachrot[0], "X");

		attach[1] = _V( 3.0,4.4,5.5+FwdZ);
		attachdir[1] = _V(0,-1,0);
		attachrot[1] = _V(0,0,-1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[1] = CreateAttachment (false, attach[1],   attachdir[1], attachrot[1], "X");

		attach[2] = _V( 0.0,0.0,5.5+FwdZ);
		attachdir[2] = _V(0,-1,0);
		attachrot[2] = _V(0,0,-1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[2] = CreateAttachment (false, attach[2],   attachdir[2], attachrot[2], "X");

		attach[3] = _V( 2.1,3.3,-6.0+FwdZ);
		attachdir[3] = _V(0,1,0);
		attachrot[3] = _V(0,0,1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[3] = CreateAttachment (false, attach[3],   attachdir[3], attachrot[3], "X");

		attach[4] = _V(-2.1,3.3,-6.0+FwdZ);
		attachdir[4] = _V(0,1,0);
		attachrot[4] = _V(0,0,1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[4] = CreateAttachment (false, attach[4],   attachdir[4], attachrot[4], "X");

		attach[5] = _V( 2.1,-3.3,-6.0+FwdZ);
		attachdir[5] = _V(0,-1,0);
		attachrot[5] = _V(0,0,1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[5] = CreateAttachment (false, attach[5],   attachdir[5], attachrot[5], "X");

		attach[6] = _V(-2.1,-3.3,-6.0+FwdZ);
		attachdir[6] = _V(0,-1,0);
		attachrot[6] = _V(0,0,1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[6] = CreateAttachment (false, attach[6],   attachdir[6], attachrot[6], "X");

		attach[7] = _V( 2.1,3.3,-16.0+FwdZ);
		attachdir[7] = _V(0,1,0);
		attachrot[7] = _V(0,0,1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[7] = CreateAttachment (false, attach[7],   attachdir[7], attachrot[7], "X");

		attach[8] = _V(-2.1,3.3,-16.0+FwdZ);
		attachdir[8] = _V(0,1,0);
		attachrot[8] = _V(0,0,1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[8] = CreateAttachment (false, attach[8],   attachdir[8], attachrot[8], "X");

		attach[9] = _V( 2.1,-3.3,-16.0+FwdZ);
		attachdir[9] = _V(0,-1,0);
		attachrot[9] = _V(0,0,1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[9] = CreateAttachment (false, attach[9],   attachdir[9], attachrot[9], "X");

		attach[10] = _V(-2.1,-3.3,-16.0+FwdZ);
		attachdir[10] = _V(0,-1,0);
		attachrot[10] = _V(0,0,1);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[10] = CreateAttachment (false, attach[10],   attachdir[10], attachrot[10], "X");

	}
	else {
		if (numatch > 0) {
			attach[0] = _V(-3.0,4.4,5.5+FwdZ);
			attachdir[0] = _V(0,-1,0);
			attachrot[0] = _V(0,0,-1);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[0] = CreateAttachment (false, attach[0],   attachdir[0], attachrot[0], "X");

			attach[1] = _V( 3.0,4.4,5.5+FwdZ);
			attachdir[1] = _V(0,-1,0);
			attachrot[1] = _V(0,0,-1);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[1] = CreateAttachment (false, attach[1],   attachdir[1], attachrot[1], "X");

			attach[2] = _V(-3.0,0.0,5.5+FwdZ);
			attachdir[2] = _V(0,-1,0);
			attachrot[2] = _V(0,0,-1);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[2] = CreateAttachment (false, attach[2],   attachdir[2], attachrot[2], "X");

			attach[3] = _V( 3.0,0.0,5.5+FwdZ);
			attachdir[3] = _V(0,-1,0);
			attachrot[3] = _V(0,0,-1);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[3] = CreateAttachment (false, attach[3],   attachdir[3], attachrot[3], "X");

			attach[4] = _V( 0.0,0.0,5.5+FwdZ);
			attachdir[4] = _V(0,-1,0);
			attachrot[4] = _V(0,0,-1);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[4] = CreateAttachment (false, attach[4],   attachdir[4], attachrot[4], "X");

			attach[5] = _V(-1.5,0.5,7.5+FwdZ);
			attachdir[5] = _V(0,-1,0);
			attachrot[5] = _V(0,0,-1);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[5] = CreateAttachment (false, attach[5],   attachdir[5], attachrot[5], "X");

			attach[6] = _V( 1.5,0.5,7.5+FwdZ);
			attachdir[6] = _V(0,-1,0);
			attachrot[6] = _V(0,0,-1);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[6] = CreateAttachment (false, attach[6],   attachdir[6], attachrot[6], "X");
		}
		if (numatch > 7) {

			attach[7] = _V( 0, 4.0,9.0+FwdZ);
			attachdir[7] = _V(0,0,1);
			attachrot[7] = _V(0,1,0);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[7] = CreateAttachment (false, attach[7],   attachdir[7], attachrot[7], "X");

			attach[8] = _V( 0,-4.0,9.0+FwdZ);
			attachdir[8] = _V(0,0,1);
			attachrot[8] = _V(0,-1,0);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[8] = CreateAttachment (false, attach[8],   attachdir[8], attachrot[8], "X");

			attach[9] = _V( 3.464,2.0,9.0+FwdZ);
			attachdir[9] = _V(0,0,1);
			attachrot[9] = _V(1,0,0);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[9] = CreateAttachment (false, attach[9],   attachdir[9], attachrot[9], "X");

			attach[10] = _V(-3.464,2.0,9.0+FwdZ);
			attachdir[10] = _V(0,0,1);
			attachrot[10] = _V(-1,0,0);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[10] = CreateAttachment (false, attach[10],   attachdir[10], attachrot[10], "X");

			attach[11] = _V( 3.464,-2.0,9.0+FwdZ);
			attachdir[11] = _V(0,0,1);
			attachrot[11] = _V(1,0,0);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[11] = CreateAttachment (false, attach[11],   attachdir[11], attachrot[11], "X");

			attach[12] = _V(-3.464,-2.0,9.0+FwdZ);
			attachdir[12] = _V(0,0,1);
			attachrot[12] = _V(-1,0,0);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[12] = CreateAttachment (false, attach[12],   attachdir[12], attachrot[12], "X");


			attach[13] = _V(0.0,0,9.9+FwdZ);
			attachdir[13] = _V(0,0,1);
			attachrot[13] = _V(0,1,0);
			//	  		       			     child   pos           dir           rot      handle
			attachpos[13] = CreateAttachment (false, attach[13],   attachdir[13], attachrot[13], "X");

		}
	}

	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
//		havesound=false;
//	else
//	{
		havesound=true;
		RequestLoadVesselWave3(MyID,PLAYGEAR,"Sound\\2001\\gear.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYBANG,"Sound\\2001\\stg2sep.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYJETT,"Sound\\2001\\fasep.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYDOORS,"Sound\\2001\\Payload_doors.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYHAB,"Sound\\2001\\Hab_fx.wav",DEFAULT);
//	}

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();  

	CreateVariableDragElement (&gear_proc, 0.6, _V(0, -1, 0));

//                  alignment      position      func    chord S   AR   
	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 18, 125, 0.5);
	// wing and body lift+drag components

	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 18, 60, 0.5);
	// vertical stabiliser and body lift and drag components



	Dock0 = CreateDock(_V(0,0,9.82+FwdZ),_V(0,0,1),_V(0,1,0));
	Dock1 = CreateDock(_V(-2.97,-6.97,0.667+AftZ),_V(-0.3907154387,-0.9204854517,0),_V(0,0,1));
//	Dock1 = CreateDock(_V(-2.97,-6.97,0.667+AftZ),_V(-0.389249,-0.917031,-0.086825),_V(-0.086825,-0.389249,0.917031));
	if (!bAero)
		Dock2 = CreateDock(_V(0,0,-5.275+AftZ),_V(0,0,-1),_V(0,1,0));

}


ARIES::~ARIES()
{

}

void ARIES::MakeAnimations()
{

	if (bAnimationsMade) return;
	bAnimationsMade = true;


	anim_gear = CreateAnimation (1);
	anim_pldoors = CreateAnimation(0);
	anim_engdoors = CreateAnimation(0);
	anim_hab = CreateAnimation(1);

/*	anim_gear = 0.0;
	anim_pldoors = 1.0;
	anim_engdoors = 0.0;
	anim_hab = 1.0;
*/
 

	if (bLegs) {
		static UINT GearDoor1Grp[1] = {16};
		MGROUP_TRANSFORM *GearDoor1 = new MGROUP_ROTATE (4, GearDoor1Grp, 1,
			_V( 7.5,0,1.0), _V(0,1,0), (float)(-9*RAD));
		static UINT GearDoor2Grp[1] = {19};
		MGROUP_TRANSFORM *GearDoor2 = new MGROUP_ROTATE (4, GearDoor2Grp, 1,
			_V(-7.5,0,1.0), _V(0,1,0), (float)( 9*RAD));
		static UINT GearDoor3Grp[1] = {18};
		MGROUP_TRANSFORM *GearDoor3 = new MGROUP_ROTATE (4, GearDoor3Grp, 1,
			_V(0,-7.5,1.0), _V(1,0,0), (float)(-9*RAD));
		static UINT GearDoor4Grp[1] = {17};
		MGROUP_TRANSFORM *GearDoor4 = new MGROUP_ROTATE (4, GearDoor4Grp, 1,
			_V(0, 7.5,1.0), _V(1,0,0), (float)( 9*RAD));

		Dr1 = AddAnimationComponent (anim_gear, 0.05, 0.2, GearDoor1);
		Dr2 = AddAnimationComponent (anim_gear, 0.05, 0.2, GearDoor2);
		Dr3 = AddAnimationComponent (anim_gear, 0.05, 0.2, GearDoor3);
		Dr4 = AddAnimationComponent (anim_gear, 0.05, 0.2, GearDoor4);

		static UINT Gear11Grp[1] = {3};
		MGROUP_TRANSFORM *Gear11 = new MGROUP_ROTATE (4, Gear11Grp, 1,
			_V( 7.0,0,-2.3), _V(0,1,0), (float)(-23*RAD));
		static UINT Gear12Grp[1] = {2};
		MGROUP_TRANSFORM *Gear12 = new MGROUP_ROTATE (4, Gear12Grp, 1,
			_V( 7.4,0,-4.4), _V(0,1,0), (float)(-80*RAD));
		static UINT Gear13Grp[2] = {1,0};
		MGROUP_TRANSFORM *Gear13 = new MGROUP_ROTATE (4, Gear13Grp, 2,
			_V( 8.54,0,-6.8), _V(0,1,0), (float)(-120*RAD));

		Gear1_1 = AddAnimationComponent (anim_gear, 0.2, 0.5, Gear11);
		Gear1_2 = AddAnimationComponent (anim_gear, 0.5, 0.8, Gear12, Gear1_1);
		Gear1_3 = AddAnimationComponent (anim_gear, 0.8, 0.95, Gear13, Gear1_2);

		static UINT Gear21Grp[1] = {7};
		MGROUP_TRANSFORM *Gear21 = new MGROUP_ROTATE (4, Gear21Grp, 1,
			_V(-7.0,0,-2.3), _V(0,1,0), (float)( 23*RAD));
		static UINT Gear22Grp[1] = {6};
		MGROUP_TRANSFORM *Gear22 = new MGROUP_ROTATE (4, Gear22Grp, 1,
			_V(-7.4,0,-4.4), _V(0,1,0), (float)( 80*RAD));
		static UINT Gear23Grp[2] = {4,5};
		MGROUP_TRANSFORM *Gear23 = new MGROUP_ROTATE (4, Gear23Grp, 2,
			_V(-8.54,0,-6.8), _V(0,1,0), (float)( 120*RAD));

		Gear2_1 = AddAnimationComponent (anim_gear, 0.2, 0.5, Gear21);
		Gear2_2 = AddAnimationComponent (anim_gear, 0.5, 0.8, Gear22, Gear2_1);
		Gear2_3 = AddAnimationComponent (anim_gear, 0.8, 0.95, Gear23, Gear2_2);

		static UINT Gear31Grp[1] = {11};
		MGROUP_TRANSFORM *Gear31 = new MGROUP_ROTATE (4, Gear31Grp, 1,
			_V(0,-7.0,-2.3), _V(1,0,0), (float)(-23*RAD));
		static UINT Gear32Grp[1] = {10};
		MGROUP_TRANSFORM *Gear32 = new MGROUP_ROTATE (4, Gear32Grp, 1,
			_V(0,-7.4,-4.4), _V(1,0,0), (float)(-80*RAD));
		static UINT Gear33Grp[2] = {8,9};
		MGROUP_TRANSFORM *Gear33 = new MGROUP_ROTATE (4, Gear33Grp, 2,
			_V(0,-8.54,-6.8), _V(1,0,0), (float)(-120*RAD));

		Gear3_1 = AddAnimationComponent (anim_gear, 0.2, 0.5, Gear31);
		Gear3_2 = AddAnimationComponent (anim_gear, 0.5, 0.8, Gear32, Gear3_1);
		Gear3_3 = AddAnimationComponent (anim_gear, 0.8, 0.95, Gear33, Gear3_2);

		static UINT Gear41Grp[1] = {15};
		MGROUP_TRANSFORM *Gear41 = new MGROUP_ROTATE (4, Gear41Grp, 1,
			_V(0, 7.0,-2.3), _V(1,0,0), (float)( 23*RAD));
		static UINT Gear42Grp[1] = {14};
		MGROUP_TRANSFORM *Gear42 = new MGROUP_ROTATE (4, Gear42Grp, 1,
			_V(0, 7.4,-4.4), _V(1,0,0), (float)( 80*RAD));
		static UINT Gear43Grp[2] = {12,13};
		MGROUP_TRANSFORM *Gear43 = new MGROUP_ROTATE (4, Gear43Grp, 2,
			_V(0, 8.54,-6.8), _V(1,0,0), (float)( 120*RAD));

		Gear4_1 = AddAnimationComponent (anim_gear, 0.2, 0.5, Gear41);
		Gear4_2 = AddAnimationComponent (anim_gear, 0.5, 0.8, Gear42, Gear4_1);
		Gear4_3 = AddAnimationComponent (anim_gear, 0.8, 0.95, Gear43, Gear4_2);

		if (bAero) {
			static UINT GearCvr1Grp[1] = {5};
			MGROUP_TRANSFORM *GearCvr1 = new MGROUP_ROTATE (8, GearCvr1Grp, 1,
				_V( 8.17,0,-3.3), _V(0,1,0), (float)(-70*RAD));
			static UINT GearCvr2Grp[1] = {6};
			MGROUP_TRANSFORM *GearCvr2 = new MGROUP_ROTATE (8, GearCvr2Grp, 1,
				_V(-8.17,0,-3.3), _V(0,1,0), (float)( 70*RAD));
			static UINT GearCvr3Grp[1] = {7};
			MGROUP_TRANSFORM *GearCvr3 = new MGROUP_ROTATE (8, GearCvr3Grp, 1,
				_V(0, 8.17,-3.3), _V(1,0,0), (float)( 70*RAD));
			static UINT GearCvr4Grp[1] = {8};
			MGROUP_TRANSFORM *GearCvr4 = new MGROUP_ROTATE (8, GearCvr4Grp, 1,
				_V(0,-8.17,-3.3), _V(1,0,0), (float)(-70*RAD));

			AddAnimationComponent (anim_gear, 0.05, 0.35, GearCvr1, Dr1);
			AddAnimationComponent (anim_gear, 0.05, 0.35, GearCvr2, Dr2);
			AddAnimationComponent (anim_gear, 0.05, 0.35, GearCvr3, Dr4);
			AddAnimationComponent (anim_gear, 0.05, 0.35, GearCvr4, Dr3);


		}

	}

// Known Groups:

// front,mid,aft,hab,gear,windows,aero,lock


// Front (0)
//
// 0 main body
// 1 Inner
// 2 L PL Door
// 3 R PL Door
// 4 CptInner
// 5 Nose
// 6 Strips
// 7 Window

// Mid (1)

// Aft (2)

// Hab (3)
//
// 0 Common Area
// 1 Spine
// 2 Tank
// 3 Tank
// 4 Hab 1 inner
// 5 Hab 2 inner
// 6 Hab 1 outer
// 7 Hab 2 outer

// Gear (4)
//
// 0-3 Gear 1 (Foot, shin, thigh, brace)
// 4-7 Gear 2
// 8-11 Gear 3
// 12-15 Gear 4
// 16 Gear 1 Cover
// 17 Gear 4 Cover
// 18 Gear 3 Cover
// 19 Gear 2 Cover

// Windows (5)

// Aero (6)
//
// 0 Main plate
// 1-4 Doors
// 5 Gear Cover 1
// 6 Gear Cover 2
// 7 Gear Cover 4
// 8 Gear Cover 3

	static UINT LPLDoorGrp[1] = {2};
	static MGROUP_ROTATE LPLDoor (7, LPLDoorGrp, 1,
		_V(-2.8,-5.3,6.71), _V( 0.2,0.4,0.7), (float)(-110*RAD));
	static UINT RPLDoorGrp[1] = {3};
	static MGROUP_ROTATE RPLDoor (7, RPLDoorGrp, 1,
		_V( 2.8,-5.3,6.71), _V(-0.2,0.4,0.7), (float)( 110*RAD));

	if (bHab) {
		static UINT LHabGrp[2] = {4,6};
		static MGROUP_ROTATE LHab (3, LHabGrp, 2,
			_V( 0.0, 5.2,-0.59), _V(1,0,0), (float)( 90*RAD));
		static UINT RHabGrp[2] = {5,7};
		static MGROUP_ROTATE RHab (3, RHabGrp, 2,
			_V( 0.0,-5.2,-0.59), _V(1,0,0), (float)(-90*RAD));

		AddAnimationComponent (anim_hab, 0.1, 0.9, &LHab);
		AddAnimationComponent (anim_hab, 0.1, 0.9, &RHab);

	}


	if (bAero) {
		static UINT EngDrGrp1[1] = {4};
		static MGROUP_ROTATE EngDr1 (8, EngDrGrp1, 1,
			_V(-3.6,-2.9,-6.49), _V(-0.42, 0.908,0), (float)( 110*RAD));
		static UINT EngDrGrp2[1] = {3};
		static MGROUP_ROTATE EngDr2 (8, EngDrGrp2, 1,
			_V( 3.6,-2.9,-6.49), _V( 0.42, 0.908,0), (float)(-110*RAD));
		static UINT EngDrGrp3[1] = {2};
		static MGROUP_ROTATE EngDr3 (8, EngDrGrp3, 1,
			_V(-3.6, 2.9,-6.49), _V(-0.42,-0.908,0), (float)(-110*RAD));
		static UINT EngDrGrp4[1] = {1};
		static MGROUP_ROTATE EngDr4 (8, EngDrGrp4, 1,
			_V( 3.6, 2.9,-6.49), _V( 0.42,-0.908,0), (float)( 110*RAD));
		AddAnimationComponent (anim_engdoors, 0.1, 0.9, &EngDr1);
		AddAnimationComponent (anim_engdoors, 0.1, 0.9, &EngDr2);
		AddAnimationComponent (anim_engdoors, 0.1, 0.9, &EngDr3);
		AddAnimationComponent (anim_engdoors, 0.1, 0.9, &EngDr4);
	}

/*
	AddAnimationComponent (anim_gear, 0.4, 0.9, &LGear);
	AddAnimationComponent (anim_gear, 0.4, 0.95, &LSupt);
	AddAnimationComponent (anim_gear, 0.05, 0.4, &RGearDoor);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &RGear);
	AddAnimationComponent (anim_gear, 0.4, 0.95, &RSupt);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &NoseGear);
	AddAnimationComponent (anim_gear, 0.1, 0.4, &NoseGearDoor);
	AddAnimationComponent (anim_gear, 0.5, 0.9, &NoseSupt);
*/
	AddAnimationComponent (anim_pldoors, 0.1, 0.9, &LPLDoor);
	AddAnimationComponent (anim_pldoors, 0.1, 0.9, &RPLDoor);




}

void ARIES::UpdateAnimations()
{	
	// animate landing gear
	if (gear_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * GEAR_OPERATING_SPEED;
		if (gear_status == DOOR_CLOSING) {
			if (gear_proc > 0.0)
				gear_proc = max (0.0, gear_proc-da);
			else {
				gear_status = DOOR_CLOSED;
				StopVesselWave3(MyID,PLAYGEAR);
			}
		} else  { // door opening
			if (gear_proc < 1.0)
				gear_proc = min (1.0, gear_proc+da);
			else {
				gear_status = DOOR_OPEN;
				StopVesselWave3(MyID,PLAYGEAR);
			}
		}
		SetAnimation (anim_gear, gear_proc);
	}

	// animate payload bay doors
	if (pldoors_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * PLDOORS_OPERATING_SPEED;
		if (pldoors_status == DOOR_CLOSING) {
			if (pldoors_proc > 0.0)
				pldoors_proc = max (0.0, pldoors_proc-da);
			else {
				pldoors_status = DOOR_CLOSED;
				StopVesselWave3(MyID,PLAYDOORS);
			}
		} else  { // door opening
			if (pldoors_proc < 1.0)
				pldoors_proc = min (1.0, pldoors_proc+da);
			else {
				pldoors_status = DOOR_OPEN;
				StopVesselWave3(MyID,PLAYDOORS);
			}
		}
		SetAnimation (anim_pldoors, pldoors_proc);
	}

	// animate engine doors
		if (engdoors_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * ENGDOORS_OPERATING_SPEED;
			if (engdoors_status == DOOR_CLOSING) {
				if (engdoors_proc > 0.0)
					engdoors_proc = max (0.0, engdoors_proc-da);
				else {
					engdoors_status = DOOR_CLOSED;
					StopVesselWave3(MyID,PLAYDOORS);
				}
			} else  { // door opening
				if (engdoors_proc < 1.0)
					engdoors_proc = min (1.0, engdoors_proc+da);
				else {
					engdoors_status = DOOR_OPEN;
					StopVesselWave3(MyID,PLAYDOORS);
				}
			}
			SetAnimation (anim_engdoors, engdoors_proc);
	}

	// animate hab
		if (hab_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * HAB_OPERATING_SPEED;
			if (hab_status == DOOR_CLOSING) {
				if (hab_proc > 0.0)
					hab_proc = max (0.0, hab_proc-da);
				else {
					hab_status = DOOR_CLOSED;
					StopVesselWave3(MyID,PLAYHAB);
					SetPMI (_V(43.16,43.65,42.37));
				}
			} else  { // door opening
				if (hab_proc < 1.0)
					hab_proc = min (1.0, hab_proc+da);
				else {
					hab_status = DOOR_OPEN;					
					StopVesselWave3(MyID,PLAYHAB);
					SetPMI (_V(203,173,42.37));
				}
			}
			SetAnimation (anim_hab, hab_proc);
	}

}

void ARIES::ActivateLandingGear (DoorStatus action)
{
	gear_status = action;
}

void ARIES::RevertLandingGear ()
{
	ActivateLandingGear (gear_status == DOOR_CLOSED || gear_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void ARIES::ActivatePayloadBayDoors (DoorStatus action)
{
	pldoors_status = action;
}

void ARIES::RevertPayloadBayDoors ()
{
	ActivatePayloadBayDoors (pldoors_status == DOOR_CLOSED || pldoors_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void ARIES::ActivateEngineDoors (DoorStatus action)
{
	engdoors_status = action;
}

void ARIES::RevertEngineDoors ()
{
	ActivateEngineDoors (engdoors_status == DOOR_CLOSED || engdoors_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void ARIES::ActivateHab (DoorStatus action)
{
	hab_status = action;
}

void ARIES::RevertHab ()
{
	ActivateHab (hab_status == DOOR_CLOSED || hab_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}


void ARIES::SetGearParameters (double state)
{

	double geary = (-1*(gear_proc-1.0)*0.6);
	if ((gear_status == DOOR_OPENING) || (gear_status == DOOR_CLOSING)) {
		SetTouchdownPoints (_V(0,-1.8+geary,14.0), _V(-9.15,-2.0+geary,-3.0), _V(9.15,-2.0+geary,-3.0));
	}
}

/*
void ARIES::SeparateEVA ()
{
	VESSELSTATUS2 vs;
	vs.version = 2;
	GetStatusEx (&vs);
//	vs.thruster[0].level = 0.0;
//	vs.fuel[0].level=1.0;
//	if (GetGravityRef() != vs.rbody) {
//		vs.status = 0;
//		vs.rbody = NULL;
//		vs.base = NULL;
//		vs.port = NULL;
//		vs.surf_lng = NULL;
//		vs.surf_lat = NULL;
//		vs.surf_hdg = NULL;
//	}

//	vs.status=0;
	VECTOR3 ofs = EVA_OFS;
	ofs.z += AftZ;
	Local2Rel (ofs, vs.rpos);
	char name[256], name2[256];
//	strcpy (name, GetName()); strcat (name, "-PLT");


	UINT i;
	for (i=1;i<8;i++) {
			strcpy (name2, GetName()); 
			sprintf(name, "-EV%d", i);
			strcat (name2, name);
			hvessel=oapiGetVesselByName(name2);
			if (hvessel == 0) break;
	}		
	if ( i < 7) 
	{
		oapiCreateVesselEx (name2, "2001/Astro2001", &vs);
		hvessel=oapiGetVesselByName(name2);
		oapiSetFocusObject(hvessel);
	}

}
*/

void ARIES::SeparateEVA ()
{
	VESSELSTATUS vs;
	GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	if (GetGravityRef() != GetSurfaceRef())				// I.E., Phobos
		vs.status = 0;
///	vs.status=0;
	VECTOR3 ofs = EVA_OFS;
	ofs.z += AftZ;
	Local2Rel (ofs, vs.rpos);
	char name[256], name2[256];
//	strcpy (name, GetName()); strcat (name, "-PLT");


	UINT i;
	for (i=1;i<8;i++) {
			strcpy (name2, GetName()); 
			sprintf(name, "-EV%d", i);
			strcat (name2, name);
			hvessel=oapiGetVesselByName(name2);
			if (hvessel == 0) break;
	}		
	if ( i < 7) 
	{
		oapiCreateVessel (name2, "2001/Astro2001", vs);
		hvessel=oapiGetVesselByName(name2);
		oapiSetFocusObject(hvessel);
	}

}

void ARIES::TimeStep(  double simt )
{
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */


	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
			SetMainStage();
	}

	UpdateAnimations();
	if (bAero)
		if (engdoors_proc < 1.0)
			SetThrusterGroupLevel(THGROUP_MAIN,0.0);
	if (bHab)
		if (hab_proc > 0.0)
			SetThrusterGroupLevel(THGROUP_MAIN,0.0);

	if (bLOXInjection) {

/*		if (GetThrusterGroupLevel(THGROUP_MAIN) < 0.95) {		// LOX injection only at "afterburner" throttle setting
			SetThrusterLevel(th_first[4],0.0);
			SetThrusterLevel(th_first[5],0.0);
			SetThrusterLevel(th_first[6],0.0);
			SetThrusterLevel(th_first[7],0.0);
		}
*/
		if ((GetPropellantMass(ph_LOX) == 0.0) && (!bEngReset)) {			// We're out of LOX -- redefine engines 
			bEngReset = true;												// so the thrust meter doesn't lie
			double ThrustLvl = GetThrusterGroupLevel(THGROUP_MAIN);
			SetMainStage();
			SetThrusterGroupLevel(THGROUP_MAIN,ThrustLvl);
		}
		else if ((GetPropellantMass(ph_LOX) > 0.0) && (bEngReset)) {		// Must've landed on a pad!
			bEngReset = false;												// Or someone's transmuting LH2 to LOX with 
			double ThrustLvl = GetThrusterGroupLevel(THGROUP_MAIN);			// FuelMFD (cheater!)
			SetMainStage();
			SetThrusterGroupLevel(THGROUP_MAIN,ThrustLvl);
		}
	}

	if (Firstsweep) {
//		SetMainStage();
		Firstsweep = false;
	}
//	SetGearParameters(gear_proc);

}


void ARIES::AttachVessel()
{
	int n;
	double bestdist = 100.0;
	OBJHANDLE hVBest = NULL;
	int nbest;
	ATTACHMENTHANDLE hAtt, hAttbest;
	bool AlreadyAttached = false;

			/* Now scan for nearby attach points; these need to stay in the bay!  */
		for (n = 0; n < numatch; n++) {
				
			Local2Global (attach[n], grms);  // global position of RMS tip
			GlobalRot (attachdir[n], gatchdir);
			GlobalRot (attachrot[n], gatchrot);

			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (int i = 0; i < oapiGetVesselCount(); i++) {
				hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < 25.0 + 3.0*oapiGetSize (hV)) { // in range
					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
//						if (strncmp (id, "BOLT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						v->GlobalRot (dir, gdir);
						v->GlobalRot (rot, grot);
						if ((!GetAttachmentStatus (hAtt)) && (dist (gpos, grms) > 0.0 ) && (dist (gpos, grms) < MAX_GRAPPLING_DIST)) { // found one!
							double angleoff = 0.5*PI*sqrt(pow(gdir.x-gatchdir.x,2.0) + pow(gdir.y-gatchdir.y,2.0) + pow(gdir.z-gatchdir.z,2.0));
							double angleoff2 = 0.5*PI*sqrt(pow(grot.x-gatchrot.x,2.0) + pow(grot.y-gatchrot.y,2.0) + pow(grot.z-gatchrot.z,2.0));
							if ((angleoff > 160*RAD) && (angleoff2 < 20*RAD)) {
								AlreadyAttached = false;
								for (int k = 0; k < nAttach; k++) { // now scan all attachment points of the candidate
									if (GetAttachmentStatus(v->GetAttachmentHandle(true,k)) != NULL)
										AlreadyAttached = true;
								}
								if (!AlreadyAttached) 
									if (dist (gpos, grms) < bestdist) {
										bestdist = dist (gpos, grms);
										hVBest = hV;
										nbest = n;
										hAttbest = hAtt;
									}
							}
						}
					}
				}
			}
		}
		if ((hVBest != NULL) && (bestdist < 100.0)) {
			AttachChild (hVBest, attachpos[nbest], hAttbest);
			AtchV[n] = oapiGetVesselInterface (hVBest);
//			return;
		}

}


/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */

void ARIES::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void ARIES::SetClassCaps(  FILEHANDLE cfg )
{

	CheckCentralMessage();

	SetMainStage();
	ClearMeshesEx();
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(6,MESHVIS_EXTERNAL|MESHVIS_VC);
}

void ARIES::SaveState( FILEHANDLE scn )
{
	char cbuf[256];

	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */


//	Undock(ALLDOCKS);

	if (bLegs) {
		sprintf (cbuf, "%d %0.4f", gear_status, gear_proc);
		oapiWriteScenario_string (scn, "GEAR", cbuf);
	}
	if (bDoorsOpen) {
		sprintf (cbuf, "%d %0.4f", pldoors_status, pldoors_proc);
		oapiWriteScenario_string (scn, "DOORS", cbuf);
	}
	if (bAero) {
		sprintf (cbuf, "%d %0.4f", engdoors_status, engdoors_proc);
		oapiWriteScenario_string (scn, "ENGINEDOORS", cbuf);
	}
	if (bHab) {
		sprintf (cbuf, "%d %0.4f", hab_status, hab_proc);
		oapiWriteScenario_string (scn, "HAB", cbuf);
	}


	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void ARIES::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
	if (!strnicmp (line, "GEAR", 4)) {
		sscanf (line+4, "%d%lf", &gear_status, &gear_proc);
		} else if (!strnicmp (line, "DOORS", 5)) {
		sscanf (line+5, "%d%lf", &pldoors_status, &pldoors_proc);
		} else if (!strnicmp (line, "ENGINEDOORS", 11)) {
		sscanf (line+11, "%d%lf", &engdoors_status, &engdoors_proc);
		} else if (!strnicmp (line, "HAB", 3)) {
		sscanf (line+3, "%d%lf", &hab_status, &hab_proc);
		} else ParsePayloadString(line);

		ParseScenarioLineEx (line, vs);
	}

	/* always set the stage whenever something may have changed */
	SetMainStage();

	if (!bDoorsOpen) {
		pldoors_status = DOOR_CLOSED;
		pldoors_proc = 0.0;
	}

	RevertLandingGear();
	RevertLandingGear();

	RevertPayloadBayDoors();
	RevertPayloadBayDoors();

	RevertEngineDoors();
	RevertEngineDoors();

	RevertHab();
	RevertHab();
}


void ARIES::PlaceComponents()
{
	COMMON_MESH tmp;

	if (bLock) {							// #0
		tmp.MeshHandle(hFwdLock);
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	} else {
		tmp.MeshHandle(hDummy3);
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}
	tmp.MeshHandle(hMid);					// #1
	tmp.MeshPosition(_V(0,0,AftZ));
	AddMeshEx (tmp);
	if (bGCR) {
		tmp.MeshHandle(hGCR);				// #2
		tmp.MeshPosition(_V(0,0,AftZ));
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hAft);			
		tmp.MeshPosition(_V(0,0,AftZ));
		AddMeshEx (tmp);
	}
	if (bHab) {
		tmp.MeshHandle(hHab);				// #3
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}
	else if (bHercHab) {
		tmp.MeshHandle(hHercHab);			// #3
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hDummy1);
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}
	if (bLegs) {							// #4
		tmp.MeshHandle(hGear);
		tmp.MeshPosition(_V(0,0,AftZ));
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hDummy2);
		tmp.MeshPosition(_V(0,0,AftZ));
		AddMeshEx (tmp);
	}
	if (bWindows) {							// #5 + 6
		tmp.MeshHandle(hWindRing);
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
		tmp.MeshHandle(hPaxInt);
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hNoWindRing);
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
		tmp.MeshHandle(hCgoInt);
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}

	tmp.MeshHandle(hFront);					// #7
	tmp.MeshPosition(_V(0,0,FwdZ));
	AddMeshEx (tmp);

	if (bAero) {							// #8
		tmp.MeshHandle(hAero);
		tmp.MeshPosition(_V(0,0,AftZ));
		AddMeshEx (tmp);
	}

	if (bWeapRing) {						// #9
		tmp.MeshHandle(hWeapRing);
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}

 }

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void ARIES::SetMainStage( )
{
	AttachedMass = 0.0;
	for (int i = 0; i < numatch; i++) {
		hV = GetAttachmentStatus(attachpos[i]);
		if (hV != NULL) {
			AtchV[i] = oapiGetVesselInterface (hV);
			atchmass[i] = AtchV[i]->GetMass();
			AttachedMass += atchmass[i];
		}
		else atchmass[i] = 0.0;
	}
	SetARIESStage();
//	ClearMeshesEx();
//	PlaceComponents();
//	Draw();

}


void ARIES::SetARIESStage( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 3.0, 7, 350, 0.3, 0.05, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_LIN, 0, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.2, 0.2
	};
/*
	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
*/

	PARTICLESTREAMSPEC contrail2 = 
	{
		0, 5.0, 7, 350, 0.3, 0.25, 7, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_LIN, 0, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.3, 0.3
	};

	contrail.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (30+FwdZ);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( Drymass + AttachedMass + CalculateMass() );

	if (bLegs)
		SetTouchdownPoints (_V(0,-10.0,-9.8+AftZ), _V(-7,7,-9.8+AftZ), _V(7,7,-9.8+AftZ));
	else
		SetTouchdownPoints (_V(0,-37.0,20.0), _V(-17,-36.0,-10), _V(17,-36.0,-10));

	SetPMI (_V(43.16,43.65,42.37));
//	SetPMI (_V(203,173,42.37));
	SetCrossSections (_V(212,214,207));
	SetCW (1.0, 1.0, 1.0, 1.0);
	SetRotDrag (_V(0.7,0.7,1.2));
//	SetPitchMomentScale (-1e-6);
//	SetBankMomentScale (-1e-6);
	SetTrimScale (0.05);
	SetCameraOffset(CAMERA_OFS+_V(0,0,FwdZ));
//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);
//	SetLiftCoeffFunc (LiftCoeff);

	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	MakeAnimations();

	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hARIES, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_Aries==NULL)  
		ph_Aries  = CreatePropellantResource(Fuelmass, Fuelmass,1.0); 
	if (bLOXInjection)
		if (ph_LOX==NULL)  
			ph_LOX  = CreatePropellantResource(LOX_PROP_MASS, LOX_PROP_MASS, 1.0); 

	SetDefaultPropellantResource (ph_Aries); 

	if (bGCR) {
		th_first[0] = CreateThruster ( _V( 0.0, 1.00,-65.0+AftZ), _V(0,0,1), Thrust, ph_Aries, Ispvac, Ispvac*0.9);
//		AddExhaustStream (th_first[0], _V( 0.0, 1.00,-63.2+AftZ), &contrail);
		AddExhaust (th_first[0], 0.5, 2.5);  

		th_first[1] = CreateThruster ( _V( 0.0,-1.00,-65.0+AftZ), _V(0,0,1), Thrust, ph_Aries, Ispvac, Ispvac*0.9);
//		AddExhaustStream (th_first[1], _V( 0.0,-1.40,-63.2+AftZ), &contrail);
		AddExhaust (th_first[1], 0.5, 2.5);  

		thg_first = CreateThrusterGroup (th_first, 2, THGROUP_MAIN);

	}
	else {
		th_first[0] = CreateThruster ( _V( 2.35, 2.35,-5.525+AftZ), _V(0,0,1), Thrust, ph_Aries, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[0], _V( 2.35, 2.35,-5.525+AftZ), &contrail);
		AddExhaust (th_first[0], 0.5, 2.5);  

		th_first[1] = CreateThruster ( _V( 2.35,-2.35,-5.525+AftZ), _V(0,0,1), Thrust, ph_Aries, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[1], _V( 2.35,-2.35,-5.525+AftZ), &contrail);
		AddExhaust (th_first[1], 0.5, 2.5);  

		th_first[2] = CreateThruster ( _V(-2.35, 2.35,-5.525+AftZ), _V(0,0,1), Thrust, ph_Aries, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[2], _V(-2.35, 2.35,-5.525+AftZ), &contrail);
		AddExhaust (th_first[2], 0.5, 2.5);  

		th_first[3] = CreateThruster ( _V(-2.35,-2.35,-5.525+AftZ), _V(0,0,1), Thrust, ph_Aries, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[3], _V(-2.35,-2.35,-5.525+AftZ), &contrail);
		AddExhaust (th_first[3], 0.5, 2.5);  

		if ((bLOXInjection) && (!bEngReset)) {

			th_first[4] = CreateThruster ( _V( 2.35, 2.35,-5.525+AftZ), _V(0,0,1), THRUST_LOX_VAC, ph_LOX, ISP_LOX_VAC, ISP_LOX_VAC*0.9);
			AddExhaustStream (th_first[4], _V( 2.35, 2.35,-5.525+AftZ), &contrail2);

			th_first[5] = CreateThruster ( _V( 2.35,-2.35,-5.525+AftZ), _V(0,0,1), THRUST_LOX_VAC, ph_LOX, ISP_LOX_VAC, ISP_LOX_VAC*0.9);
			AddExhaustStream (th_first[5], _V( 2.35,-2.35,-5.525+AftZ), &contrail2);

			th_first[6] = CreateThruster ( _V(-2.35, 2.35,-5.525+AftZ), _V(0,0,1), THRUST_LOX_VAC, ph_LOX, ISP_LOX_VAC, ISP_LOX_VAC*0.9);
			AddExhaustStream (th_first[6], _V(-2.35, 2.35,-5.525+AftZ), &contrail2);

			th_first[7] = CreateThruster ( _V(-2.35,-2.35,-5.525+AftZ), _V(0,0,1), THRUST_LOX_VAC, ph_LOX, ISP_LOX_VAC, ISP_LOX_VAC*0.9);
			AddExhaustStream (th_first[7], _V(-2.35,-2.35,-5.525+AftZ), &contrail2);

			thg_first = CreateThrusterGroup (th_first, 8, THGROUP_MAIN);
		}
		else 
			thg_first = CreateThrusterGroup (th_first, 4, THGROUP_MAIN);
		
	}


	SetAttControlsARIES();
	EnableTransponder(true);

}



int ARIES::ConsumeDirectKey(char *keystate)
{
	if (KEYDOWN (keystate, OAPI_KEY_ADD))
	{
		if (bAero) {
			if (engdoors_proc < 1.0)
				RESETKEY (keystate, OAPI_KEY_ADD);
		}
		if (bHab) {
			if (hab_proc > 0.0)
				RESETKEY (keystate, OAPI_KEY_ADD);
		}

		return 0;
	}
}

/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */


int ARIES::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	if (!fPressed) return 0; // only process keydown events
	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		if (KEYDOWN (keystate, OAPI_KEY_SPACE)) 
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_SPACE, 1.0))
			{

			oapiSetShowGrapplePoints (!oapiGetShowGrapplePoints ());

			return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_J)) 
		{ // "Join Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{
			/* CVE-lite function ! */
			/* IMPORTANT */
			/* call DeployPayload() to jettison the GPDS payloads in sequence */

			if (numatch > 0)
				AttachVessel();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();

			return 1;
			}
		}
	}
	else { // unmodified keys

		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) 
			{

			{
				OverridePayloadVelocity(_V(0,+0.25,0));
				VESSEL *depV;
				depV = DeployPayload();
				if (depV == NULL) {
					for (int i = 0; i < numatch; i++) {
						if (GetAttachmentStatus(attachpos[numatch-i-1]) != NULL) {
							DetachChild(attachpos[numatch-i-1],0.02);
							atchmass[numatch-i-1] = 0.0;
							i = numatch;
						}
					}
				}
			}

		

				/* CVE-lite function ! */
				/* IMPORTANT */
				/* call DeployPayload() to jettison the GPDS payloads in sequence */

				/* IMPORTANT */
				/* Always re-set the stage after any kind of change */
				SetMainStage();
				
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_G)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_G, 1.0)) {
				// Toggle the Gear
				if (bLegs) {
					RevertLandingGear ();
					StopVesselWave3(MyID,PLAYGEAR);
					PlayVesselWave3(MyID,PLAYGEAR);
				}
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_K)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_K, 1.0)) {
				// Toggle the bay doors
				if (bDoorsOpen) {
					RevertPayloadBayDoors ();
					StopVesselWave3(MyID,PLAYDOORS);
					PlayVesselWave3(MyID,PLAYDOORS);
				}
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_P)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_P, 1.0)) {
				// Toggle the hab
				if (bHab) {
					RevertHab ();
					StopVesselWave3(MyID,PLAYHAB);
					PlayVesselWave3(MyID,PLAYHAB);
				}
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_O)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_O, 1.0)) {
				// Toggle the engine doors
				if (bAero) {
					RevertEngineDoors();
					StopVesselWave3(MyID,PLAYDOORS);
					PlayVesselWave3(MyID,PLAYDOORS);
				}
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0)) {
				// EVA
				SeparateEVA();
				SetMainStage();
			}
			return 1;
		}

	}
	return 0;
}



int ARIES::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}




void ARIES::SetAttControlsARIES()
{
	THRUSTER_HANDLE th_rot[4];
	texRCS  = oapiRegisterExhaustTexture ("exhaust_atrcs");
	PARTICLESTREAMSPEC contrailR = 
	{
//		0, 0.25, 10.0, 9.0, 0.5, 1.0, 10, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
		0, 0.5, 29, 50.0, 0.3, 0.05, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_LIN, 0, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.2, 0.2
	};

	th_rot[0] = CreateThruster (_V( 0, 15, 0), _V(0, 0,-1), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V( 0,-15, 0), _V(0, 0, 1), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 0, 0, 15), _V(0, 1, 0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V( 0, 0,-15), _V(0,-1, 0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 5.95, 6.25,0.934+AftZ), _V(0,0, 1),texRCS); 
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-5.95, 6.25,0.934+AftZ), _V(0,0, 1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V( 6.69,-5.20,0.325+AftZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.25,-5.90,0.325+AftZ), _V(0,0,-1),texRCS);  
	AddExhaustStream (th_rot[0], _V( 5.95, 6.25,0.934+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-5.95, 6.25,0.934+AftZ), &contrailR);
	AddExhaustStream (th_rot[1], _V( 6.69,-5.20,0.325+AftZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.25,-5.90,0.325+AftZ), &contrailR);
	if (FwdHelper != 0) {
		AddExhaust (th_rot[2], 2.5, 0.5, _V(0,-7.75,3.38+FwdHelper), _V(0,-1,0),texRCS);  
		AddExhaustStream (th_rot[2], _V(0,-7.75,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[3], 2.5, 0.5, _V(0, 5.5,3.38+AftHelper), _V(0, 1,0),texRCS);  
		AddExhaustStream (th_rot[3], _V(0, 5.5,3.38+AftHelper), &contrailR);
	}
	thg_pitchup=CreateThrusterGroup (th_rot, 4, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,0,-15), _V(0,1,0),MAX_ATT_FIRST, ph_Aries, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(0,0,15), _V(0,1,0),MAX_ATT_FIRST, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.60,-5.53,0.61+AftZ), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.10,-6.10,0.61+AftZ), _V(0,-1.0,0.0),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.60,-5.53,0.61+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.10,-6.10,0.61+AftZ), &contrailR);
	if (FwdHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V(0,-7.75,3.38+FwdHelper), _V(0,-1,0),texRCS);  
		AddExhaustStream (th_rot[0], _V(0,-7.75,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[1], 2.5, 0.5, _V(0,-5.5,3.38+AftHelper), _V(0,-1,0),texRCS);  
		AddExhaustStream (th_rot[1], _V(0,-5.5,3.38+AftHelper), &contrailR);
	}
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);

	th_rot[0] = CreateThruster (_V(0,0,-15), _V(0,-1,0), MAX_ATT_FIRST, ph_Aries, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(0,0,15), _V(0,-1,0), MAX_ATT_FIRST, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-5.70, 6.43,0.61+AftZ), _V(0,1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 5.70, 6.43,0.61+AftZ), _V(0,1.0,0.0),texRCS);  
	AddExhaustStream (th_rot[0], _V(-5.70, 6.43,0.61+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V( 5.70, 6.43,0.61+AftZ), &contrailR);
	if (FwdHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V(0, 7.75,3.38+FwdHelper), _V(0, 1,0),texRCS);  
		AddExhaustStream (th_rot[0], _V(0, 7.75,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[1], 2.5, 0.5, _V(0, 5.5,3.38+AftHelper), _V(0, 1,0),texRCS);  
		AddExhaustStream (th_rot[1], _V(0, 5.5,3.38+AftHelper), &contrailR);
	}
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,-15,0), _V(0,0,-1), MAX_ATT_FIRST, ph_Aries, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(0, 15,0), _V(0,0, 1), MAX_ATT_FIRST, ph_Aries, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 0, 0, 15), _V(0,-1, 0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V( 0, 0,-15), _V(0, 1, 0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.60,-5.17,0.934+AftZ), _V(0,0, 1),texRCS); 
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.20,-5.82,0.934+AftZ), _V(0,0, 1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V( 5.85, 6.17,0.325+AftZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-5.85, 6.17,0.325+AftZ), _V(0,0,-1),texRCS);   
	AddExhaustStream (th_rot[0], _V( 6.60,-5.17,0.934+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.20,-5.82,0.934+AftZ), &contrailR);		
	AddExhaustStream (th_rot[1], _V( 5.85, 6.22,0.325+AftZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-5.85, 6.22,0.325+AftZ), &contrailR); 
	if (FwdHelper != 0) {
		AddExhaust (th_rot[2], 2.5, 0.5, _V(0, 7.75,3.38+FwdHelper), _V(0, 1,0),texRCS);  
		AddExhaustStream (th_rot[2], _V(0, 7.75,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[3], 2.5, 0.5, _V(0,-5.5,3.38+AftHelper), _V(0,-1,0),texRCS);  
		AddExhaustStream (th_rot[3], _V(0,-5.5,3.38+AftHelper), &contrailR);
	}
	thg_pitchdown=CreateThrusterGroup (th_rot, 4, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V( 15,0,0), _V(0,0, 1), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(-15,0,0), _V(0,0,-1), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 0, 0, 15), _V(-1,0, 0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V( 0, 0,-15), _V( 1,0, 0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.60,-5.17,0.325+AftZ), _V(0,0,-1),texRCS); 
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.20,-5.82,0.934+AftZ), _V(0,0, 1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 5.85, 6.17,0.325+AftZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-5.85, 6.17,0.924+AftZ), _V(0,0, 1),texRCS);   
	AddExhaustStream (th_rot[0], _V( 6.60,-5.17,0.325+AftZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.20,-5.82,0.934+AftZ), &contrailR);		
	AddExhaustStream (th_rot[0], _V( 5.85, 6.22,0.325+AftZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-5.85, 6.22,0.924+AftZ), &contrailR); 
	if (FwdHelper != 0) {
		AddExhaust (th_rot[2], 2.5, 0.5, _V( 7.75,0,3.38+FwdHelper), _V( 1,0,0),texRCS);  
		AddExhaustStream (th_rot[2], _V( 7.75,0,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[3], 2.5, 0.5, _V(-5.5,0,3.38+AftHelper), _V(-1,0,0),texRCS);  
		AddExhaustStream (th_rot[3], _V(-5.5,0,3.38+AftHelper), &contrailR);
	}
	CreateThrusterGroup (th_rot, 4, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V( 15,0,0), _V(0,0,-1), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(-15,0,0), _V(0,0, 1), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 0, 0, 15), _V( 1,0, 0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V( 0, 0,-15), _V(-1,0, 0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.60,-5.17,0.934+AftZ), _V(0,0, 1),texRCS); 
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.20,-5.82,0.934+AftZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 5.85, 6.17,0.325+AftZ), _V(0,0, 1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-5.85, 6.17,0.325+AftZ), _V(0,0,-1),texRCS);   
	AddExhaustStream (th_rot[0], _V( 6.60,-5.17,0.934+AftZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.20,-5.82,0.325+AftZ), &contrailR);		
	AddExhaustStream (th_rot[0], _V( 5.85, 6.22,0.934+AftZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-5.85, 6.22,0.325+AftZ), &contrailR); 
	if (FwdHelper != 0) {
		AddExhaust (th_rot[2], 2.5, 0.5, _V(-7.75,0,3.38+FwdHelper), _V(-1,0,0),texRCS);  
		AddExhaustStream (th_rot[2], _V(-7.75,0,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[3], 2.5, 0.5, _V( 5.5,0,3.38+AftHelper), _V( 1,0,0),texRCS);  
		AddExhaustStream (th_rot[3], _V( 5.5,0,3.38+AftHelper), &contrailR);
	}	CreateThrusterGroup (th_rot, 4, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.15, 6.01,0.61+AftZ), _V(1,0,0),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.96,-5.02,0.61+AftZ), _V(1,0,0),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.15, 6.01,0.61+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V( 6.96,-5.02,0.61+AftZ), &contrailR); 
	if (FwdHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V( 7.75,0,3.38+FwdHelper), _V( 1,0,0),texRCS);  
		AddExhaustStream (th_rot[0], _V( 7.75,0,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V( 5.5,0,3.38+AftHelper), _V( 1,0,0),texRCS);  
		AddExhaustStream (th_rot[0], _V( 5.5,0,3.38+AftHelper), &contrailR);
	}
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_FIRST, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.20, 6.10,0.61+AftZ), _V(-1,0,0),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.52,-5.68,0.61+AftZ), _V(-1,0,0),texRCS);  
	AddExhaustStream (th_rot[0], _V(-6.20, 6.10,0.61+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.52,-5.68,0.61+AftZ), &contrailR);  
	if (FwdHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V(-7.75,0,3.38+FwdHelper), _V(-1,0,0),texRCS);  
		AddExhaustStream (th_rot[0], _V(-7.75,0,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V(-5.5,0,3.38+AftHelper), _V(-1,0,0),texRCS);  
		AddExhaustStream (th_rot[0], _V(-5.5,0,3.38+AftHelper), &contrailR);
	}
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V( 3, 3,0), _V( 0.7,-0.7,0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(-3, 3,0), _V( 0.7, 0.7,0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 3,-3,0), _V(-0.7,-0.7,0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V(-3,-3,0), _V(-0.7, 0.7,0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 5.69, 6.40,0.61+AftZ), _V(-0.7, 0.7,0),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.19, 6.13,0.61+AftZ), _V(-0.7,-0.7,0),texRCS);  
	AddExhaust (th_rot[2], 2.5, 0.5, _V( 6.96,-5.05,0.61+AftZ), _V( 0.7, 0.7,0),texRCS);  
	AddExhaust (th_rot[3], 2.5, 0.5, _V(-6.52,-5.69,0.61+AftZ), _V( 0.7,-0.7,0),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.11, 5.98,0.61+AftZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-5.74, 6.55,0.61+AftZ), &contrailR);
	AddExhaustStream (th_rot[2], _V( 6.54,-5.54,0.61+AftZ), &contrailR);
	AddExhaustStream (th_rot[3], _V(-6.04,-6.11,0.61+AftZ), &contrailR);
	CreateThrusterGroup (th_rot, 4, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V( 3, 3,0), _V(-0.7, 0.7,0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(-3, 3,0), _V(-0.7,-0.7,0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 3,-3,0), _V( 0.7, 0.7,0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V(-3,-3,0), _V( 0.7,-0.7,0), MAX_ATT_FIRST/2, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.11, 5.98,0.61+AftZ), _V( 0.7,-0.7,0),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-5.74, 6.55,0.61+AftZ), _V( 0.7, 0.7,0),texRCS);  
	AddExhaust (th_rot[2], 2.5, 0.5, _V( 6.54,-5.54,0.61+AftZ), _V(-0.7,-0.7,0),texRCS);  
	AddExhaust (th_rot[3], 2.5, 0.5, _V(-6.52,-5.69,0.61+AftZ), _V(-0.7, 0.7,0),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.11, 5.98,0.61+AftZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-5.74, 6.55,0.61+AftZ), &contrailR);
	AddExhaustStream (th_rot[2], _V( 6.54,-5.54,0.61+AftZ), &contrailR);
	AddExhaustStream (th_rot[3], _V(-6.52,-5.69,0.61+AftZ), &contrailR);
	CreateThrusterGroup (th_rot, 4, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_FIRST*2, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 5.85, 6.17,0.325+AftZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-5.85, 6.17,0.325+AftZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.69,-5.20,0.325+AftZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.25,-5.90,0.325+AftZ), _V(0,0,-1),texRCS);  
	AddExhaustStream (th_rot[0], _V( 5.85, 6.17,0.325+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-5.85, 6.17,0.325+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V( 6.69,-5.20,0.325+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.25,-5.90,0.325+AftZ), &contrailR);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_FIRST*2, ph_Aries, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 5.95, 6.25,0.934+AftZ), _V(0,0,1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-5.95, 6.25,0.934+AftZ), _V(0,0,1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.60,-5.17,0.934+AftZ), _V(0,0,1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.20,-5.82,0.934+AftZ), _V(0,0,1),texRCS);  
	AddExhaustStream (th_rot[0], _V( 5.95, 6.25,0.934+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-5.95, 6.25,0.934+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V( 6.60,-5.17,0.934+AftZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.20,-5.82,0.934+AftZ), &contrailR);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}