/**
*
* $Id: CVEL-CentaurO.cpp,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "CentaurO.h"

CENTAURO::CENTAURO (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hCentaurO = oapiLoadMeshGlobal ("2001/CentaurO");
	tex = oapiRegisterExhaustTexture("Exhaust_atsme");

	/* set the propellant handle to NULL so I can check if I need to 
	/* create it in the SetMainStage() function. */
	ph_first=NULL; 
	atchmass = 0.0;
	attach = _V(0,0,3.1);
	attachdir = _V(0,0,1);
	//	  		     			   child   pos           dir       rot      handle
	attachpos = CreateAttachment (false, attach,   attachdir, _V(0,-1,0), "X");

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();   
}


CENTAURO::~CENTAURO()
{

}


void CENTAURO::TimeStep(  double simt )
{
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */
	if( CheckCentralMessage() )
		SetMainStage();
}

/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */

void CENTAURO::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void CENTAURO::SetClassCaps(  FILEHANDLE cfg )
{
	/* ensure the propellent handle is set to NULL before creation */
	ph_first=NULL;
	SetMainStage();
}

void CENTAURO::SaveState( FILEHANDLE scn )
{
	char cbuf[256];
	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */

	if ((attach.x != 0.0) || (attach.y != 0.0) || (attach.z != 3.1)) {
		sprintf (cbuf, "%0.4f %0.4f %0.4f", attach.x,attach.y,attach.z);
		oapiWriteScenario_string (scn, "ATTACHPOINT1", cbuf);
	}

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void CENTAURO::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		if (!strnicmp (line, "ATTACHPOINT1", 12)) {
		    sscanf (line+12, "%lf%lf%lf", &attach.x,&attach.y,&attach.z);
			//	   						   pos           dir       rot       
			SetAttachmentParams (attachpos, attach,   attachdir, _V(0,-1,0));
//			DetachChild(attachpos,0.1);
//			AttachChild (hV, attachpos, hAtch);

		}
		else ParsePayloadString(line);


		ParseScenarioLineEx (line, vs);
	}

	/* always set the stage whenever something may have changed */
	SetMainStage();
}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void CENTAURO::SetMainStage( )
{

	const VECTOR3 VECTOR_ZERO = _V(0,0,0);

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (4);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */

	hV = GetAttachmentStatus(attachpos);
	if (hV != NULL) {
		AtchVessel = hV;
		AtchV = oapiGetVesselInterface (AtchVessel);
		atchmass = AtchV->GetMass();
	}
	else atchmass = 0.0;

	SetEmptyMass ( CENTAURO_DRYMASS + CalculateMass() + atchmass);


	SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
	SetPMI (_V(17,17,1.5));
	SetCrossSections (_V(4,4,2.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 

	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	Draw();


	/* we're not using COMMON_MESH today, so draw our own */
	AddMesh(hCentaurO, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */
	if (ph_first==NULL)  
		ph_first  = CreatePropellantResource(FIRST_PROP_MASS,FIRST_PROP_MASS*InitFuelLevel,1.0); 


	SetDefaultPropellantResource (ph_first); 

	th_first[0] = CreateThruster ( _V(0,0,-1.9), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);


	thg_first = CreateThrusterGroup (th_first, 1, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 18.0, 1.9,_V(0,0,-2.6), _V(0,0,-1), tex);
	int numeng = 24;
	for (int i = 0; i < numeng; i++)
		AddExhaust(th_first[0],1.9,0.25, _V((-1.1*cos(i*(6.28/numeng))),(-1.1*sin(i*(6.28/numeng))),-1.9), _V((0.08*cos(i*(6.28/numeng))),(0.08*sin(i*(6.28/numeng))), -0.95),tex);


	SetAttControlsCentaurO();
	EnableTransponder(true);
}





/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */

int CENTAURO::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	if (!fPressed) return 0; // only process keydown events
	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		if (KEYDOWN (keystate, OAPI_KEY_SPACE)) 
		{ // "Join Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_SPACE, 1.0))
			{

			oapiSetShowGrapplePoints (!oapiGetShowGrapplePoints ());

			return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_J)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{
			/* CVE-lite function ! */
			/* IMPORTANT */
			/* call DeployPayload() to jettison the GPDS payloads in sequence */
	
			if (GetAttachmentStatus(attachpos) == NULL)
				AttachVessel();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();

			return 1;
			}
		}
	}
	else {

		if (KEYDOWN (keystate, OAPI_KEY_J)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{

			/* CVE-lite function ! */
			/* IMPORTANT */
			/* call DeployPayload() to jettison the GPDS payloads in sequence */
	
			if (GetAttachmentStatus(attachpos) == NULL)
				DeployPayload();
			else {
				DetachChild(attachpos,0.1);
				atchmass = 0.0;
			} 

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();

			return 1;
			}
		}
	}


	return 0;
}



void CENTAURO::AttachVessel()
{
	double bestdist = 100.0;
	OBJHANDLE hVBest = NULL;
	ATTACHMENTHANDLE hAtt, hAttbest;
	bool AlreadyAttached = false;
	
			/* Now scan for nearby attach points; these need to stay in the bay!  */
				
			Local2Global (attach, grms);  // global position of RMS tip
			GlobalRot (attachdir, gatchdir);

			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (int i = 0; i < oapiGetVesselCount(); i++) {
				hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < 3.0*oapiGetSize (hV)) { // in range
					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
//						if (strncmp (id, "BOLT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						v->GlobalRot (dir, gdir);
						if (dist (gpos, grms) < 1.0) { // found one!
							double angleoff = 0.5*PI*sqrt(pow(gdir.x-gatchdir.x,2.0) + pow(gdir.y-gatchdir.y,2.0) + pow(gdir.z-gatchdir.z,2.0));
							if (angleoff > 160*RAD) {
								AlreadyAttached = false;
								for (int k = 0; k < nAttach; k++) { // now scan all attachment points of the candidate
									if (GetAttachmentStatus(v->GetAttachmentHandle(true,k)) != NULL)
										AlreadyAttached = true;
								}
								if (!AlreadyAttached) 
									if (dist (gpos, grms) < bestdist) {
										bestdist = dist (gpos, grms);
										hVBest = hV;
										hAttbest = hAtt;
									}
							}
						}
					}
				}
			}
		if ((hVBest != NULL) && (bestdist < 100.0)) {
			AttachChild (hVBest, attachpos, hAttbest);
			AtchV = oapiGetVesselInterface (hVBest);
//			return;
		}
}


int CENTAURO::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}




void CENTAURO::SetAttControlsCentaurO()
{
	THRUSTER_HANDLE th_rot[2];
	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);
	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}
