/**
*
* $Id: CVEL-Centaur-O.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2006 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/orbitersdk.h"
#include "../../include/orbitersoundsdk3.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

// All data from astronautix.com
const double ISP_FIRST_SL		=     454.0*G; 
const double ISP_FIRST_VAC		=     454.0*G; //***
const double THRUST_FIRST_VAC	=    8485.0*G; //***
const double MAX_ATT_FIRST		=     250.0*G;
const double FIRST_PROP_MASS	=    9900.0; //*** the total mass of fueled CentaurO is 10900kg
const double CENTAURO_MAX_BURN	=     625.0; //***
const double CENTAURO_DRYMASS	=    1000.0; //***

#define PLAYGEAR					 20		// the gear sound
#define PLAYBANG					 24		// the jettison sound
#define PLAYJETT					 25     // entry shield jettison sound
#define PLAYJET						 26     // the jet noise sound
#define PLAYDOORS					 27     // the payload door sound
#define MYRADARSOUND				  2     // Custom radar sound
#define MYRADARCLOSE				  3     // Custom radar sound


/* make your own vessel class (named CENTAURO here, for example)
   subclassed from COMMON_VESSEL_EX */

class CENTAURO : public COMMON_VESSEL_EX {
public:
	CENTAURO (OBJHANDLE hObj, int fmodel);
	~CENTAURO();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void AttachVessel();
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 

	void SetAttControlsCentaurO();    // this is to set up the thrusters. I like to keep them separate.
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);

protected:
	PROPELLANT_HANDLE ph_first;
	THRUSTER_HANDLE   th_first[2];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hCentaurO;
	SURFHANDLE		  tex;
	ATTACHMENTHANDLE  attachpos;
	VECTOR3           attach;
	VECTOR3 gpos, grms, pos, dir, rot, attachdir, gatchdir, gdir;
	OBJHANDLE hV, AtchVessel, hvessel;
	VESSEL *AtchV;
	double atchmass;

};







/* basic vessel class setup */


// Initialisation


void CENTAURO::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void CENTAURO::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void CENTAURO::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void CENTAURO::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void CENTAURO::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void CENTAURO::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int CENTAURO::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new CENTAURO (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (CENTAURO*)vessel;
}
