/**
*
* $Id: GAGARIN.cpp
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "Gagarin.h"
/*
double LiftCoeff (double aoa)
{
	const int nlift = 9;
	static const double AOA[nlift] = {-180*RAD,-165*RAD,-164*RAD,-45*RAD,0*RAD,45*RAD,164*RAD,165*RAD,180*RAD};
	static const double CL[nlift]  = {0.06375,  0.1275,       0,      0,    0,     0,      0,      0,0.06375};
	//	static const double CL[nlift]  = {       0,  0.1275,       0,      0,    0,     0,      0,-0.1275,      0};
	static const double SCL[nlift] = {(CL[1]-CL[0])/(AOA[1]-AOA[0]), (CL[2]-CL[1])/(AOA[2]-AOA[1]),
		                              (CL[3]-CL[2])/(AOA[3]-AOA[2]), (CL[4]-CL[3])/(AOA[4]-AOA[3]),
									  (CL[5]-CL[4])/(AOA[5]-AOA[4]), (CL[6]-CL[5])/(AOA[6]-AOA[5]),
									  (CL[7]-CL[6])/(AOA[7]-AOA[6]), (CL[8]-CL[7])/(AOA[8]-AOA[7])};
	for (int i = 0; i < nlift-1 && AOA[i+1] < aoa; i++);
	return CL[i] + (aoa-AOA[i])*SCL[i];
}
*/



// 1. vertical lift component (wings and body)


void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = {0.06375,  0.1275,  0,   0,  0,  0,    0,     0,      0,   0,     0,     0,     0,       0,       0,     0,    0,    0,   0,     0,     0,     0,     0,  0.06375};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,     0,     0,   0,     0,     0,     0,       0,       0,     0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.

	// change for drag increase at high Mach:
	// Subsonic Equivalents

//	double Cl0, Cl1, Cd0, Cd1, Cd2;
/*	if(M>=5.0) {
		Cl0 =  0.0;
		Cl1 =  1.675600;
		Cd0 =  0.068540;
		Cd1 = -0.412900;
		Cd2 =  1.539960;
		*cl = Cl0 + Cl1*aoa;
		*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
*/
	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
/*	if(M<1.25) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.04 + oapiGetInducedDrag (*cl, 2.1, 0.82);
	}
	else if(M<5.0) { 
 */
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cd = 0.2 + oapiGetInducedDrag (*cl, 2.1, 0.65);

// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
//	}
}

// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}


GAGARIN::GAGARIN (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hFront				= oapiLoadMeshGlobal ("2001/GFwd");
	hMid				= oapiLoadMeshGlobal ("2001/GStretch");
	hAft				= oapiLoadMeshGlobal ("2001/GAft");
	hGear				= oapiLoadMeshGlobal ("2001/GGear");
	hAero				= oapiLoadMeshGlobal ("2001/GAero");
	hHab				= oapiLoadMeshGlobal ("2001/GHab");
	hGCR				= oapiLoadMeshGlobal ("2001/GSakharov");
	hFwdLock			= oapiLoadMeshGlobal ("2001/GFwdLock");
	hSeats				= oapiLoadMeshGlobal ("2001/GSeats");
	hWeapRing			= oapiLoadMeshGlobal ("2001/GWeapons");
	hRack				= oapiLoadMeshGlobal ("2001/GLuggRack");
	hDummy1				= oapiLoadMeshGlobal ("2001/dummy");
	hDummy2				= oapiLoadMeshGlobal ("2001/dummy2");
	hDummy3				= oapiLoadMeshGlobal ("2001/dummy3");

	tex = oapiRegisterExhaustTexture("RealExhaust");

	/* set the propellant handles to NULL so I can check if I need to 
	/* create them in the SetMainStage() function. */
	ph_Gagarin=NULL;
	ph_LOX=NULL;
	Dock0=NULL;
	Dock1=NULL;
	Dock2=NULL;
	malf = 0;
	bShieldJettisoned=false;
	bDeorbitFired=false;
	bThrustTerminated=false;
	bEjected=false;
	bEVA=false;
	Drymass = GAGARIN_DRYMASS;
	Fuelmass = GAGARIN_PROP_MASS;
	Thrust = THRUST_FIRST_VAC;
	Ispvac = ISP_FIRST_VAC;
	bDoorsOpen = false;
	bWindows = true;
	bLegs = true;
	bLock = false;
	bAero = false;
	bHab = false;
	bGCR = false;
	bStretch = false;
	bWeapRing = false;
	bRack = false;
	DockZ = 0.0;
	gear_status=DOOR_CLOSED;
	gear_proc=0.0;
	engdoors_status=DOOR_CLOSED;
	engdoors_proc=0.0;
	hab_status=DOOR_CLOSED;
	hab_proc=0.0;
	seats_status=DOOR_CLOSED;
	seats_proc=0.0;
	oldsimt = -0.01;
	Vold = _V(0,0,0);
	bAbort = false;
	AbortChecked = false;
	bAnimationsMade = false;
	FwdZ = 0.0;
	AftZ = 0.0;
	FwdHelper = 0.0;
	AftHelper = 0.0;
	Firstsweep = true;
	insignia_tex = NULL;
	PlatformZ = 0.0;
	numatch = 7;
	RadarFlagForSound=FALSE;
	RadarFlagForClose=FALSE;


	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "GAGARIN-M" ) ) {
		Drymass = GAGARIN_DRYMASS+LOCK_DRYMASS+AERO_DRYMASS+BIGGER_ENGINES;
		Fuelmass = GAGARIN_PROP_MASS;
		Thrust = THRUST_MARS_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = false;
		bWindows = true;
		bLegs = true;
		bLock = true;
		bAero = true;
		bHab = false;
	}
	else if( strstr( classname, "GAGARIN" ) ) {
		Drymass = GAGARIN_DRYMASS+LOCK_DRYMASS;
		Fuelmass = GAGARIN_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = false;
		bWindows = true;
		bLegs = true;
		bLock = true;
		bAero = false;
		bHab = false;
	}
	else if( strstr( classname, "BONDARENKO" ) ) {
		Drymass = GAGARIN_DRYMASS+HAB_DRYMASS+LOCK_DRYMASS;
		Fuelmass = GAGARIN_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = false;
		bWindows = true;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHab = true;
		PlatformZ = 43.0;
	}
	else if( strstr( classname, "PATSAYEV-M" ) ) {
		Drymass = GAGARIN_DRYMASS+STRETCH_DRYMASS+AERO_DRYMASS+BIGGER_ENGINES+RACK_DRYMASS+LOCK_DRYMASS;
		Fuelmass = KOMAROV_PROP_MASS;
		Thrust = THRUST_MARS_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = true;
		bLock = true;
		bAero = true;
		bHab = false;
		bStretch = true;
		bRack = true;
		FwdZ = 4.35;
		AftZ = -6.0;
		PlatformZ = 2.75;
		numatch = 13;
		DockZ = 2.35;
	}
	else if( strstr( classname, "PATSAYEV" ) ) {
		Drymass = GAGARIN_DRYMASS+STRETCH_DRYMASS+LOCK_DRYMASS;
		Fuelmass = KOMAROV_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHab = false;
		bStretch = true;
		FwdZ = 4.35;
		AftZ = -6.0;
	}
	else if( strstr( classname, "MERKUR" ) ) {
		Drymass = GAGARIN_DRYMASS+LOCK_DRYMASS+GCR_DRYMASS;
		Fuelmass = GAGARIN_PROP_MASS;
		Thrust = THRUST_GCR_VAC;
		Ispvac = ISP_GCR_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHab = false;
		bGCR = true;
		bStretch = false;
//		FwdHelper = 10.0;
		AftHelper = -42.5+AftZ;
	}
	else if( strstr( classname, "SKORPION" ) ) {
		Drymass = GAGARIN_DRYMASS+LOCK_DRYMASS+SKORPION_DRYMASS;
		Fuelmass = GAGARIN_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = false;
		bWindows = true;
		bLegs = true;
		bLock = true;
		bAero = false;
		bHab = false;
		bWeapRing = true;
		PlatformZ = 4.0;
		DockZ = 4.0;
	}
	else if( strstr( classname, "KOMAROV-BIS" ) ) {
		Drymass = GAGARIN_DRYMASS+LOCK_DRYMASS+STRETCH_DRYMASS+HAB_DRYMASS+GCR_DRYMASS;
		Fuelmass = KOMAROV_PROP_MASS;
		Thrust = THRUST_GCR_VAC;
		Ispvac = ISP_GCR_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHab = true;
		bGCR = true;
		bStretch = true;
		FwdZ = 4.35;
		AftZ = -6.0;
//		FwdHelper = 10.0;
		PlatformZ = 43.0;
		AftHelper = -42.5+AftZ;
	}
	else if( strstr( classname, "KOMAROV" ) ) {
		Drymass = GAGARIN_DRYMASS+LOCK_DRYMASS+STRETCH_DRYMASS+HAB_DRYMASS;
		Fuelmass = KOMAROV_PROP_MASS;
		Thrust = THRUST_FIRST_VAC;
		Ispvac = ISP_FIRST_VAC;
		bDoorsOpen = true;
		bWindows = false;
		bLegs = false;
		bLock = true;
		bAero = false;
		bHab = true;
		bStretch = true;
		FwdZ = 4.35;
		AftZ = -6.0;
		PlatformZ = 43.0;
//		FwdHelper = 10.0;
	}

//	sat_attach = CreateAttachment (false, OFS_SAT+_V(0,0,FwdZ), _V(0,0,1), _V(0,1,0), "X");

	
	//		     			   child        pos						    dir       rot      handle

	Dockpos = CreateAttachment (false, _V(0,0,7.22+PlatformZ+DockZ+FwdZ),   _V(0,0,1), _V(0,1,0), "DOCK");

	attach[0] = _V(-2.5, 4.33,6.5+PlatformZ+FwdZ);
	attachdir[0] = _V(0,0,1);
	attachrot[0] = _V(0.5,-0.866,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[0] = CreateAttachment (false, attach[0],   attachdir[0], attachrot[0], "X");
	
	attach[1] = _V( 2.5,-4.33,6.5+PlatformZ+FwdZ);
	attachdir[1] = _V(0,0,1);
	attachrot[1] = _V(-0.5,0.866,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[1] = CreateAttachment (false, attach[1],   attachdir[1], attachrot[1], "X");

	attach[2] = _V(-2.5,-4.33,6.5+PlatformZ+FwdZ);
	attachdir[2] = _V(0,0,1);
	attachrot[2] = _V(-0.5,-0.866,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[2] = CreateAttachment (false, attach[2],   attachdir[2], attachrot[2], "X");

	attach[3] = _V( 2.5, 4.33,6.5+PlatformZ+FwdZ);
	attachdir[3] = _V(0,0,1);
	attachrot[3] = _V(0.5,0.866,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[3] = CreateAttachment (false, attach[3],   attachdir[3], attachrot[3], "X");

	attach[4] = _V(-5.0, 0.0,6.5+PlatformZ+FwdZ);
	attachdir[4] = _V(0,0,1);
	attachrot[4] = _V(1,0,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[4] = CreateAttachment (false, attach[4],   attachdir[4], attachrot[4], "X");

	attach[5] = _V( 5.0, 0.0,6.5+PlatformZ+FwdZ);
	attachdir[5] = _V(0,0,1);
	attachrot[5] = _V(-1,0,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[5] = CreateAttachment (false, attach[5],   attachdir[5], attachrot[5], "X");

	if (bRack) {

		attach[6] = _V(-2.5, 4.33,6.5+FwdZ);
		attachdir[6] = _V(0,0,1);
		attachrot[6] = _V(0.5,-0.866,0);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[6] = CreateAttachment (false, attach[6],   attachdir[6], attachrot[6], "X");
		
		attach[7] = _V( 2.5,-4.33,6.5+FwdZ);
		attachdir[7] = _V(0,0,1);
		attachrot[7] = _V(-0.5,0.866,0);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[7] = CreateAttachment (false, attach[7],   attachdir[7], attachrot[7], "X");

		attach[8] = _V(-2.5,-4.33,6.5+FwdZ);
		attachdir[8] = _V(0,0,1);
		attachrot[8] = _V(-0.5,-0.866,0);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[8] = CreateAttachment (false, attach[8],   attachdir[8], attachrot[8], "X");

		attach[9] = _V( 2.5, 4.33,6.5+FwdZ);
		attachdir[9] = _V(0,0,1);
		attachrot[9] = _V(0.5,0.866,0);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[9] = CreateAttachment (false, attach[9],   attachdir[9], attachrot[9], "X");

		attach[10] = _V(-5.0, 0.0,6.5+FwdZ);
		attachdir[10] = _V(0,0,1);
		attachrot[10] = _V(1,0,0);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[10] = CreateAttachment (false, attach[10],   attachdir[10], attachrot[10], "X");

		attach[11] = _V( 5.0, 0.0,6.5+FwdZ);
		attachdir[11] = _V(0,0,1);
		attachrot[11] = _V(-1,0,0);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[11] = CreateAttachment (false, attach[11],   attachdir[11], attachrot[11], "X");

		attach[12] = _V( 0.0, 0.0,7.22+FwdZ);
		attachdir[12] = _V(0,0,1);
		attachrot[12] = _V(0,1,0);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[12] = CreateAttachment (false, attach[12],   attachdir[12], attachrot[12], "X");

	} else {	
		if (PlatformZ > 0.0)
			attach[6] = _V( 0.0, 0.0,6.5+PlatformZ+FwdZ);
		else attach[6] = _V( 0.0, 0.0,7.22+PlatformZ+FwdZ);
		attachdir[6] = _V(0,0,1);
		attachrot[6] = _V(0,1,0);
		//	  		       			     child   pos           dir           rot      handle
		attachpos[6] = CreateAttachment (false, attach[6],   attachdir[6], attachrot[6], "X");
	}

	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
//		havesound=false;
//	else
//	{
		havesound=true;
		RequestLoadVesselWave3(MyID,PLAYGEAR,"Sound\\2001\\gear.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYBANG,"Sound\\2001\\stg2sep.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYJETT,"Sound\\2001\\fasep.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYDOORS,"Sound\\2001\\Payload_doors.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYHAB,"Sound\\2001\\Hab_fx.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,MYRADARSOUND,"Sound\\2001\\rusbeep.wav",INTERNAL_ONLY);
		RequestLoadVesselWave3(MyID,MYRADARCLOSE,"Sound\\2001\\rusclose.wav",INTERNAL_ONLY);
		SoundOptionOnOff3(MyID,PLAYRADARBIP,FALSE);

//	}
//		ReplaceStockSound3(MyID,"Sound\\2001\\rusbeep.wav",PLAYRADARBIP);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();  

	CreateVariableDragElement (&gear_proc, 0.6, _V(0, -1, 0));

//                  alignment      position      func    chord S   AR   
	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 18, 125, 0.5);
	// wing and body lift+drag components

	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 18, 60, 0.5);
	// vertical stabiliser and body lift and drag components



	if (bHab)
		Dock0 = CreateDock(_V(0,0,49.6+FwdZ),_V(0,0,1),_V(0,1,0));
	else Dock0 = CreateDock(_V(0,0,7.22+DockZ+FwdZ),_V(0,0,1),_V(0,1,0));
	Dock1 = CreateDock(_V(0, 7.65,-0.1+AftZ),_V(0, 0.9204854517,-0.3907154387),_V(0,0,1));
	Dock2 = CreateDock(_V(0,-7.65,-0.1+AftZ),_V(0,-0.9204854517,-0.3907154387),_V(0,0,1));

}


GAGARIN::~GAGARIN()
{

	if (insignia_tex) oapiDestroySurface(insignia_tex);
}

void GAGARIN::MakeAnimations()
{

	if (bAnimationsMade) return;
	bAnimationsMade = true;

	anim_gear = CreateAnimation (0);
	anim_engdoors = CreateAnimation(0);
	anim_hab = CreateAnimation(0);
	anim_seats = CreateAnimation(0);
/*
	anim_gear = 0.0;
	anim_engdoors = 0.0;
	anim_hab = 0.0;
*/
 
/*
	if ((bLegs) && (bAero)) {
		static UINT GearDoor1Grp[1] = {16};
		static MGROUP_ROTATE GearDoor1 (4, GearDoor1Grp, 1,
			_V( 7.5,0,1.0), _V(0,1,0), (float)(-9*RAD));
		static UINT GearDoor2Grp[1] = {19};
		static MGROUP_ROTATE GearDoor2 (4, GearDoor2Grp, 1,
			_V(-7.5,0,1.0), _V(0,1,0), (float)( 9*RAD));
		static UINT GearDoor3Grp[1] = {18};
		static MGROUP_ROTATE GearDoor3 (4, GearDoor3Grp, 1,
			_V(0,-7.5,1.0), _V(1,0,0), (float)(-9*RAD));
		static UINT GearDoor4Grp[1] = {17};
		static MGROUP_ROTATE GearDoor4 (4, GearDoor4Grp, 1,
			_V(0, 7.5,1.0), _V(1,0,0), (float)( 9*RAD));

		Dr1 = AddAnimationComponent (anim_gear, 0.05, 0.2, &GearDoor1);
		Dr2 = AddAnimationComponent (anim_gear, 0.05, 0.2, &GearDoor2);
		Dr3 = AddAnimationComponent (anim_gear, 0.05, 0.2, &GearDoor3);
		Dr4 = AddAnimationComponent (anim_gear, 0.05, 0.2, &GearDoor4);

		static UINT Gear11Grp[1] = {3};
		static MGROUP_ROTATE Gear11 (4, Gear11Grp, 1,
			_V( 7.0,0,-2.3), _V(0,1,0), (float)(-23*RAD));
		static UINT Gear12Grp[1] = {2};
		static MGROUP_ROTATE Gear12 (4, Gear12Grp, 1,
			_V( 7.4,0,-4.4), _V(0,1,0), (float)(-80*RAD));
		static UINT Gear13Grp[2] = {1,0};
		static MGROUP_ROTATE Gear13 (4, Gear13Grp, 2,
			_V( 8.54,0,-6.8), _V(0,1,0), (float)(-120*RAD));

		Gear1_1 = AddAnimationComponent (anim_gear, 0.2, 0.5, &Gear11);
		Gear1_2 = AddAnimationComponent (anim_gear, 0.5, 0.8, &Gear12, Gear1_1);
		Gear1_3 = AddAnimationComponent (anim_gear, 0.8, 0.95, &Gear13, Gear1_2);

		static UINT Gear21Grp[1] = {7};
		static MGROUP_ROTATE Gear21 (4, Gear21Grp, 1,
			_V(-7.0,0,-2.3), _V(0,1,0), (float)( 23*RAD));
		static UINT Gear22Grp[1] = {6};
		static MGROUP_ROTATE Gear22 (4, Gear22Grp, 1,
			_V(-7.4,0,-4.4), _V(0,1,0), (float)( 80*RAD));
		static UINT Gear23Grp[2] = {4,5};
		static MGROUP_ROTATE Gear23 (4, Gear23Grp, 2,
			_V(-8.54,0,-6.8), _V(0,1,0), (float)( 120*RAD));

		Gear2_1 = AddAnimationComponent (anim_gear, 0.2, 0.5, &Gear21);
		Gear2_2 = AddAnimationComponent (anim_gear, 0.5, 0.8, &Gear22, Gear2_1);
		Gear2_3 = AddAnimationComponent (anim_gear, 0.8, 0.95, &Gear23, Gear2_2);

		static UINT Gear31Grp[1] = {11};
		static MGROUP_ROTATE Gear31 (4, Gear31Grp, 1,
			_V(0,-7.0,-2.3), _V(1,0,0), (float)(-23*RAD));
		static UINT Gear32Grp[1] = {10};
		static MGROUP_ROTATE Gear32 (4, Gear32Grp, 1,
			_V(0,-7.4,-4.4), _V(1,0,0), (float)(-80*RAD));
		static UINT Gear33Grp[2] = {8,9};
		static MGROUP_ROTATE Gear33 (4, Gear33Grp, 2,
			_V(0,-8.54,-6.8), _V(1,0,0), (float)(-120*RAD));

		Gear3_1 = AddAnimationComponent (anim_gear, 0.2, 0.5, &Gear31);
		Gear3_2 = AddAnimationComponent (anim_gear, 0.5, 0.8, &Gear32, Gear3_1);
		Gear3_3 = AddAnimationComponent (anim_gear, 0.8, 0.95, &Gear33, Gear3_2);

		static UINT Gear41Grp[1] = {15};
		static MGROUP_ROTATE Gear41 (4, Gear41Grp, 1,
			_V(0, 7.0,-2.3), _V(1,0,0), (float)( 23*RAD));
		static UINT Gear42Grp[1] = {14};
		static MGROUP_ROTATE Gear42 (4, Gear42Grp, 1,
			_V(0, 7.4,-4.4), _V(1,0,0), (float)( 80*RAD));
		static UINT Gear43Grp[2] = {12,13};
		static MGROUP_ROTATE Gear43 (4, Gear43Grp, 2,
			_V(0, 8.54,-6.8), _V(1,0,0), (float)( 120*RAD));

		Gear4_1 = AddAnimationComponent (anim_gear, 0.2, 0.5, &Gear41);
		Gear4_2 = AddAnimationComponent (anim_gear, 0.5, 0.8, &Gear42, Gear4_1);
		Gear4_3 = AddAnimationComponent (anim_gear, 0.8, 0.95, &Gear43, Gear4_2);

		if (bAero) {
			static UINT GearCvr1Grp[1] = {5};
			static MGROUP_ROTATE GearCvr1 (6, GearCvr1Grp, 1,
				_V( 8.17,0,-3.3), _V(0,1,0), (float)(-70*RAD));
			static UINT GearCvr2Grp[1] = {6};
			static MGROUP_ROTATE GearCvr2 (6, GearCvr2Grp, 1,
				_V(-8.17,0,-3.3), _V(0,1,0), (float)( 70*RAD));
			static UINT GearCvr3Grp[1] = {7};
			static MGROUP_ROTATE GearCvr3 (6, GearCvr3Grp, 1,
				_V(0, 8.17,-3.3), _V(1,0,0), (float)( 70*RAD));
			static UINT GearCvr4Grp[1] = {8};
			static MGROUP_ROTATE GearCvr4 (6, GearCvr4Grp, 1,
				_V(0,-8.17,-3.3), _V(1,0,0), (float)(-70*RAD));

			AddAnimationComponent (anim_gear, 0.05, 0.35, &GearCvr1, Dr1);
			AddAnimationComponent (anim_gear, 0.05, 0.35, &GearCvr2, Dr2);
			AddAnimationComponent (anim_gear, 0.05, 0.35, &GearCvr3, Dr4);
			AddAnimationComponent (anim_gear, 0.05, 0.35, &GearCvr4, Dr3);


		}

	}
*/
// Known Groups:

// front,mid,aft,hab,gear,windows,aero,lock


// Seats (0)

// Front (1)
//
// 0 main body
// 1 Inner
// 2 L PL Door
// 3 R PL Door
// 4 CptInner
// 5 Nose
// 6 Strips
// 7 Window

// Aft (2)
//
// 0 Aeroshield
// 1 EngDr1
// 2 EngDr2
// 3 EngDr3
// 4 GrDr1
// 5 GrDr2
// 6 GrDr3
// 7 GrDr4

// Hab (3)
//
// 0 Platform
// 1 Tunnel
// 2 T
// 3 Tunnel 1
// 4 OffB1
// 5 Hab 1 outer
// 6 Tunnel 2
// 7 OffB2
// 8 Hab 2 outer
// 9 Hab 1 inner
// 10 Hab 2 inner
// 11 FrontGray
// 12 Hab1Greeb
// 13 Hab2Greeb
// 14 TunlGreeb
// 15 Tunl1Pipe
// 16 Tunl2Pipe
// 17 FwdFixedGreeb
// 18 Hab1SmMotor
// 19 Hab2SmMotor
// 20 HabMotors
// 21 Hab 1 windows
// 22 Hab 2 windows

// Gear (4)

// Windows (5)

/*
	static UINT LPLDoorGrp[1] = {2};
	static MGROUP_ROTATE LPLDoor (0, LPLDoorGrp, 1,
		_V(-2.8,-5.3,6.71), _V( 0.2,0.4,0.7), (float)(-110*RAD));
	static UINT RPLDoorGrp[1] = {3};
	static MGROUP_ROTATE RPLDoor (0, RPLDoorGrp, 1,
		_V( 2.8,-5.3,6.71), _V(-0.2,0.4,0.7), (float)( 110*RAD));
*/

	static UINT SeatsGrp[4] = {0,1,2,3};
	static MGROUP_ROTATE Seats (0, SeatsGrp, 4,
		(PIVOTPOINT), _V(1,0,0), (float)( SeatRotate*RAD));
	AddAnimationComponent (anim_seats, 0.02, 0.98, &Seats);
	ofscam[0] = OFS_CAMERA1;
//	static MGROUP_ROTATE viewpoint (LOCALVERTEXLIST, MAKEGROUPARRAY(ofscam), 1,
//		(PIVOTPOINT), _V(1,0,0), (float)( SeatRotate*RAD));
//	AddAnimationComponent (anim_seats, 0.02, 0.98, &viewpoint);

	viewpoint = new MGROUP_ROTATE (LOCALVERTEXLIST, MAKEGROUPARRAY(ofscam), 1,
		PIVOTPOINT, _V(1,0,0), (float)( SeatRotate*RAD)); 
	AddAnimationComponent (anim_seats, 0, 1, viewpoint);


	if (bHab) {
		static UINT LHabGrp1[3] = {3, 15, 18};
		MGROUP_TRANSFORM *LHab1 = new MGROUP_ROTATE (3, LHabGrp1, 3,
			_V( 0.0, 0.0, 45.3), _V(0.707,0.707,0), (float)(-90*RAD));
//		static MGROUP_ROTATE LHab1 (3, LHabGrp1, 3,
//			_V( 0.0, 0.0, 45.3), _V(0,1,0), (float)(-90*RAD));
		static UINT LHabGrp2[5] = {4,5,9,12,21};
		MGROUP_TRANSFORM *LHab2 = new MGROUP_ROTATE (3, LHabGrp2, 5,
			_V( 0.0, 0.0, 23.7), _V(0.707,0.707,0), (float)( 90*RAD));
		static UINT LHabGrp3[8] = {3,4,5,9,12,15,18,21};
		MGROUP_TRANSFORM *LHab3 = new MGROUP_ROTATE (3, LHabGrp3, 8,
			_V( 1.414, 1.414, 45.3), _V(0,0,1), (float)( 90*RAD));
				
		static UINT RHabGrp1[3] = {6, 16, 19};
		MGROUP_TRANSFORM *RHab1 = new MGROUP_ROTATE (3, RHabGrp1, 3,
			_V( 0.0, 0.0, 45.3), _V(0.707,0.707,0), (float)( 90*RAD));
		static UINT RHabGrp2[5] = {7,8,10,13,22};
		MGROUP_TRANSFORM *RHab2 = new MGROUP_ROTATE (3, RHabGrp2, 5,
			_V( 0.0, 0.0, 23.7), _V(0.707,0.707,0), (float)(-90*RAD));
		static UINT RHabGrp3[8] = {6,7,8,10,13,16,19,22};
		MGROUP_TRANSFORM *RHab3 = new MGROUP_ROTATE (3, RHabGrp3, 8,
			_V(-1.414,-1.414, 45.3), _V(0,0,1), (float)( 90*RAD));
						
		hab1 = AddAnimationComponent (anim_hab, 0.1, 0.7, LHab1);
		AddAnimationComponent (anim_hab, 0.1, 0.7, LHab2, hab1);
		hab2 = AddAnimationComponent (anim_hab, 0.1, 0.7, RHab1);
		AddAnimationComponent (anim_hab, 0.1, 0.7, RHab2, hab2);
		AddAnimationComponent (anim_hab, 0.7, 0.9, LHab3);
		AddAnimationComponent (anim_hab, 0.7, 0.9, RHab3);

	}


	if (bAero) {

		static UINT GearGrp[1] = {0};
		static MGROUP_TRANSLATE Gear (4, GearGrp, 1,
			_V( 0,0,-2.25));

		static UINT EngDrGrp1[1] = {1};
		static MGROUP_ROTATE EngDr1 (2, EngDrGrp1, 1,
			_V( 2.296,-1.222,-8.80), _V( 0.42, 0.908,0), (float)(-165*RAD));
		static UINT EngDrGrp2[1] = {2};
		static MGROUP_ROTATE EngDr2 (2, EngDrGrp2, 1,
			_V(-2.296,-1.222,-8.80), _V(-0.42, 0.908,0), (float)( 165*RAD));
		static UINT EngDrGrp3[1] = {3};
		static MGROUP_ROTATE EngDr3 (2, EngDrGrp3, 1,
			_V( 0.0, 2.667,-8.80), _V(1,0,0), (float)( 165*RAD));
		static UINT GearDrGrp1[1] = {4};
		static MGROUP_ROTATE GearDr1 (2, GearDrGrp1, 1,
			_V(-4.63, 3.82,-7.40), _V(-0.707,-0.707,0), (float)(-220*RAD));
		static UINT GearDrGrp2[1] = {5};
		static MGROUP_ROTATE GearDr2 (2, GearDrGrp2, 1,
			_V( 4.63, 3.82,-7.40), _V( 0.707,-0.707,0), (float)( 220*RAD));
		static UINT GearDrGrp3[1] = {6};
		static MGROUP_ROTATE GearDr3 (2, GearDrGrp3, 1,
			_V( 2.55,-5.33,-7.40), _V( 0.866, 0.5,0), (float)(-220*RAD));
		static UINT GearDrGrp4[1] = {7};
		static MGROUP_ROTATE GearDr4 (2, GearDrGrp4, 1,
			_V(-2.55,-5.33,-7.40), _V(-0.866, 0.5,0), (float)( 220*RAD));
		AddAnimationComponent (anim_engdoors, 0.1, 0.9, &EngDr1);
		AddAnimationComponent (anim_engdoors, 0.1, 0.9, &EngDr2);
		AddAnimationComponent (anim_engdoors, 0.1, 0.9, &EngDr3);

		if (bLegs) {		
			AddAnimationComponent (anim_gear, 0.2, 0.9, &Gear);
			AddAnimationComponent (anim_gear, 0.05, 0.2, &GearDr1);
			AddAnimationComponent (anim_gear, 0.05, 0.2, &GearDr2);
			AddAnimationComponent (anim_gear, 0.05, 0.2, &GearDr3);
			AddAnimationComponent (anim_gear, 0.05, 0.2, &GearDr4);
		}
	}

/*
	AddAnimationComponent (anim_gear, 0.4, 0.9, &LGear);
	AddAnimationComponent (anim_gear, 0.4, 0.95, &LSupt);
	AddAnimationComponent (anim_gear, 0.05, 0.4, &RGearDoor);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &RGear);
	AddAnimationComponent (anim_gear, 0.4, 0.95, &RSupt);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &NoseGear);
	AddAnimationComponent (anim_gear, 0.1, 0.4, &NoseGearDoor);
	AddAnimationComponent (anim_gear, 0.5, 0.9, &NoseSupt);
*/
//	AddAnimationComponent (anim_pldoors, 0.1, 0.9, &LPLDoor);
//	AddAnimationComponent (anim_pldoors, 0.1, 0.9, &RPLDoor);




}

void GAGARIN::UpdateAnimations()
{	
	// animate landing gear
	if (gear_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * GEAR_OPERATING_SPEED;
		if (gear_status == DOOR_CLOSING) {
			if (gear_proc > 0.0)
				gear_proc = max (0.0, gear_proc-da);
			else {
				gear_status = DOOR_CLOSED;
				StopVesselWave3(MyID,PLAYGEAR);
			}
		} else  { // door opening
			if (gear_proc < 1.0)
				gear_proc = min (1.0, gear_proc+da);
			else {
				gear_status = DOOR_OPEN;
				StopVesselWave3(MyID,PLAYGEAR);
			}
		}
		SetAnimation (anim_gear, gear_proc);
	}
/*
	// animate payload bay doors
	if (pldoors_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * PLDOORS_OPERATING_SPEED;
		if (pldoors_status == DOOR_CLOSING) {
			if (pldoors_proc > 0.0)
				pldoors_proc = max (0.0, pldoors_proc-da);
			else {
				pldoors_status = DOOR_CLOSED;
			}
		} else  { // door opening
			if (pldoors_proc < 1.0)
				pldoors_proc = min (1.0, pldoors_proc+da);
			else {
				pldoors_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_pldoors, pldoors_proc);
	}
*/
	// animate engine doors
		if (engdoors_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * ENGDOORS_OPERATING_SPEED;
			if (engdoors_status == DOOR_CLOSING) {
				if (engdoors_proc > 0.0)
					engdoors_proc = max (0.0, engdoors_proc-da);
				else {
					engdoors_status = DOOR_CLOSED;
					StopVesselWave3(MyID,PLAYDOORS);
				}
			} else  { // door opening
				if (engdoors_proc < 1.0)
					engdoors_proc = min (1.0, engdoors_proc+da);
				else {
					engdoors_status = DOOR_OPEN;
					StopVesselWave3(MyID,PLAYDOORS);
				}
			}
			SetAnimation (anim_engdoors, engdoors_proc);
	}

	// animate hab
		if (hab_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * HAB_OPERATING_SPEED;
			if (hab_status == DOOR_CLOSING) {
				if (hab_proc > 0.0)
					hab_proc = max (0.0, hab_proc-da);
				else {
					hab_status = DOOR_CLOSED;
					StopVesselWave3(MyID,PLAYHAB);
					SetPMI (_V(43.16,43.65,42.37));
				}
			} else  { // door opening
				if (hab_proc < 1.0)
					hab_proc = min (1.0, hab_proc+da);
				else {
					hab_status = DOOR_OPEN;
					StopVesselWave3(MyID,PLAYHAB);
					SetPMI (_V(203,173,42.37));
				}
			}
			SetAnimation (anim_hab, hab_proc);
	}

	// animate seats
		if (seats_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * SEATS_OPERATING_SPEED;
			double tilt = 0.0;
			if (((seats_proc*SeatRotate*RAD) > PI/2))
				tilt = PI;
			if (seats_status == DOOR_CLOSING) {
				if (seats_proc > 0.0) {
					seats_proc = max (0.0, seats_proc-da);
					SetCameraDefaultDirection(_V(0,(-1*sin(seats_proc*SeatRotate*RAD)),cos(seats_proc*SeatRotate*RAD)),tilt);
/*					SetCameraOffset(PIVOTPOINT + _V(0,0,FwdZ) + _V(OFS_CAMERA1.x,
						((OFS_CAMERA1.y-PIVOTPOINT.y)*cos(seats_proc*SeatRotate*RAD)),
						((OFS_CAMERA1.z-PIVOTPOINT.z)*-1.0*sin(seats_proc*SeatRotate*RAD))));

*/
//					SetCameraOffset(OFS_CAMERA1*(1-seats_proc)+OFS_CAMERA2*(seats_proc)+_V(0,0,FwdZ));
					SetCameraOffset(ofscam[0]+_V(0,0,FwdZ));
				}
				else {
					seats_status = DOOR_CLOSED;
					StopVesselWave3(MyID,PLAYGEAR);
					SetCameraDefaultDirection(_V(0,(-1*sin(seats_proc*SeatRotate*RAD)),cos(seats_proc*SeatRotate*RAD)),tilt);
//					SetCameraDefaultDirection(_V(0,0,1));

				}
			} else  { // door opening
				if (seats_proc < 1.0) {
					seats_proc = min (1.0, seats_proc+da);
					SetCameraDefaultDirection(_V(0,(-1*sin(seats_proc*SeatRotate*RAD)),cos(seats_proc*SeatRotate*RAD)),tilt);
//					SetCameraOffset(OFS_CAMERA1*(1-seats_proc)+OFS_CAMERA2*(seats_proc)+_V(0,0,FwdZ));
					SetCameraOffset(ofscam[0]+_V(0,0,FwdZ));
				}
				else {
					seats_status = DOOR_OPEN;
					StopVesselWave3(MyID,PLAYGEAR);
					SetCameraDefaultDirection(_V(0,(-1*sin(seats_proc*SeatRotate*RAD)),cos(seats_proc*SeatRotate*RAD)),tilt);
//					SetCameraDefaultDirection(_V(0,-1,0));
					SetCameraOffset(ofscam[0]+_V(0,0,FwdZ));
				}
			}
			SetAnimation (anim_seats, seats_proc);
	}
}

void GAGARIN::ActivateLandingGear (DoorStatus action)
{
	gear_status = action;
}

void GAGARIN::RevertLandingGear ()
{
	ActivateLandingGear (gear_status == DOOR_CLOSED || gear_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void GAGARIN::ActivateEngineDoors (DoorStatus action)
{
	engdoors_status = action;
}

void GAGARIN::RevertEngineDoors ()
{
	ActivateEngineDoors (engdoors_status == DOOR_CLOSED || engdoors_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void GAGARIN::ActivateHab (DoorStatus action)
{
	hab_status = action;
}

void GAGARIN::RevertHab ()
{
	ActivateHab (hab_status == DOOR_CLOSED || hab_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void GAGARIN::ActivateSeats (DoorStatus action)
{
	seats_status = action;
}

void GAGARIN::RevertSeats ()
{
	ActivateSeats (seats_status == DOOR_CLOSED || seats_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}


void GAGARIN::SetGearParameters (double state)
{

	double geary = (-1*(gear_proc-1.0)*0.6);
	if ((gear_status == DOOR_OPENING) || (gear_status == DOOR_CLOSING)) {
		SetTouchdownPoints (_V(0,-1.8+geary,14.0), _V(-9.15,-2.0+geary,-3.0), _V(9.15,-2.0+geary,-3.0));
	}
}


void GAGARIN::SeparateEVA ()
{
	VESSELSTATUS vs;
	GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	if (GetGravityRef() != GetSurfaceRef())				// I.E., Phobos
		vs.status = 0;
///	vs.status=0;
	VECTOR3 ofs = EVA_OFS;
	ofs.z += AftZ;
	Local2Rel (ofs, vs.rpos);
	char name[256], name2[256];
//	strcpy (name, GetName()); strcat (name, "-PLT");


	UINT i;
	for (i=1;i<8;i++) {
			strcpy (name2, GetName()); 
			sprintf(name, "-EV%d", i);
			strcat (name2, name);
			hvessel=oapiGetVesselByName(name2);
			if (hvessel == 0) break;
	}		
	if ( i < 7) 
	{
		oapiCreateVessel (name2, "2001/Astro2001", vs);
		hvessel=oapiGetVesselByName(name2);
		oapiSetFocusObject(hvessel);
	}

}

void GAGARIN::RussianBeeps()
{
		double Altitude=0;
		double Distance=2000;
		oapiGetFocusAltitude(&Altitude);
		Local2Global (_V(0,0,0), grms);  // global position of RMS tip
		for (int i = 0; i < oapiGetVesselCount(); i++) {
			hV = oapiGetVesselByIndex (i);
			if (hV == GetHandle()) continue; // we don't want to check ourselves ...
			oapiGetGlobalPos (hV, &gpos);
			if ((oapiGetSize(hV) > 6.0) || (dist(gpos,grms) > 60.0))         // Skip attached boxes!
//				if (((dist (gpos, grms) - oapiGetSize(hV) - oapiGetSize(GetHandle())) < Distance) && ((dist (gpos, grms) - oapiGetSize(hV) - oapiGetSize(GetHandle())) > 10.0))
				if (((dist (gpos, grms) - oapiGetSize(GetHandle())) < Distance) && ((dist (gpos, grms) - oapiGetSize(GetHandle())) > 10.0))
					Distance = dist(gpos,grms) - oapiGetSize(GetHandle());
//					Distance = dist(gpos,grms) - oapiGetSize(hV) - oapiGetSize(GetHandle());
		}
		if ((Distance < Altitude) || (Altitude == 0))
			Altitude = Distance;			// Play beeps based on object distance instead!



		// ------------------------------------------------------
		// here we play our radar sound that slide with altitude
		// ------------------------------------------------------
		if(Altitude<600&&Altitude>25)
		{
			PlayVesselWave3(MyID,MYRADARSOUND,LOOP,255,(int)(16000+((600-Altitude)*10)));		// play looped and a frequency that increase with altitude
			RadarFlagForSound=TRUE;
			if(RadarFlagForClose=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARCLOSE);
				RadarFlagForClose = FALSE;
			}
		}
		else if (Altitude<25&&Altitude>10) 
		{
			PlayVesselWave3(MyID,MYRADARCLOSE,LOOP,255);		// play looped 
			RadarFlagForClose=TRUE;
			if(RadarFlagForSound=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARSOUND);
				RadarFlagForSound = FALSE;
			}
		}
		else {
			if(RadarFlagForSound=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARSOUND);
				RadarFlagForSound = FALSE;
			}
			if(RadarFlagForClose=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARCLOSE);
				RadarFlagForClose = FALSE;
			}
		}
}

void GAGARIN::TimeStep(  double simt )
{

	RussianBeeps();

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */


	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
			SetMainStage();
	}

	UpdateAnimations();
	if (bAero)
		if (engdoors_proc < 1.0)
			SetThrusterGroupLevel(THGROUP_MAIN,0.0);
	if (bHab)
		if (hab_proc > 0.0)
			SetThrusterGroupLevel(THGROUP_MAIN,0.0);


	if (Firstsweep) {
		double tilt = 0.0;
		if (((seats_proc*SeatRotate*RAD) > PI/2))
			tilt = PI;
		SetCameraDefaultDirection(_V(0,(-1*sin(seats_proc*SeatRotate*RAD)),cos(seats_proc*SeatRotate*RAD)),tilt);
		SetCameraOffset(ofscam[0]+_V(0,0,FwdZ));
		Firstsweep = false;
	}

//	SetGearParameters(gear_proc);

}



void GAGARIN::AttachVessel()
{
	int n;
	double bestdist = 100.0;
	OBJHANDLE hVBest = NULL;
	int nbest;
	ATTACHMENTHANDLE hAtt, hAttbest;
	bool AlreadyAttached = false;

			/* Now scan for nearby attach points; these need to stay in the bay!  */
		for (n = 0; n < numatch; n++) {
				
			Local2Global (attach[n], grms);  // global position of RMS tip
			GlobalRot (attachdir[n], gatchdir);
			GlobalRot (attachrot[n], gatchrot);

			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (int i = 0; i < oapiGetVesselCount(); i++) {
				hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < 25.0 + 3.0*oapiGetSize (hV)) { // in range
					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
//						if (strncmp (id, "BOLT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						v->GlobalRot (dir, gdir);
						v->GlobalRot (rot, grot);
						if ((!GetAttachmentStatus (hAtt)) && (dist (gpos, grms) > 0.0 ) && (dist (gpos, grms) < MAX_GRAPPLING_DIST)) { // found one!
							double angleoff = 0.5*PI*sqrt(pow(gdir.x-gatchdir.x,2.0) + pow(gdir.y-gatchdir.y,2.0) + pow(gdir.z-gatchdir.z,2.0));
							double angleoff2 = 0.5*PI*sqrt(pow(grot.x-gatchrot.x,2.0) + pow(grot.y-gatchrot.y,2.0) + pow(grot.z-gatchrot.z,2.0));
							if ((angleoff > 160*RAD) && (angleoff2 < 20*RAD)) {
								AlreadyAttached = false;
								for (int k = 0; k < nAttach; k++) { // now scan all attachment points of the candidate
									if (GetAttachmentStatus(v->GetAttachmentHandle(true,k)) != NULL)
										AlreadyAttached = true;
								}
								if (!AlreadyAttached) 
									if (dist (gpos, grms) < bestdist) {
										bestdist = dist (gpos, grms);
										hVBest = hV;
										nbest = n;
										hAttbest = hAtt;
									}
							}
						}
					}
				}
			}
		}
		if ((hVBest != NULL) && (bestdist < 100.0)) {
			AttachChild (hVBest, attachpos[nbest], hAttbest);
			AtchV[n] = oapiGetVesselInterface (hVBest);
//			return;
		}

}


/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */

void GAGARIN::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void GAGARIN::SetClassCaps(  FILEHANDLE cfg )
{

	CheckCentralMessage();

	SetMainStage();
	ClearMeshesEx();
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
	SetMeshVisibilityMode(1,MESHVIS_ALWAYS);
	SetMeshVisibilityMode(2,MESHVIS_ALWAYS);
	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);
	SetMeshVisibilityMode(4,MESHVIS_ALWAYS);
	SetMeshVisibilityMode(5,MESHVIS_ALWAYS);
	if (bWeapRing)
		SetMeshVisibilityMode(6,MESHVIS_ALWAYS);

}

void GAGARIN::SaveState( FILEHANDLE scn )
{
	char cbuf[256];

	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */


//	Undock(ALLDOCKS);

	sprintf (cbuf, "%d %0.4f", gear_status, gear_proc);
	oapiWriteScenario_string (scn, "GEAR", cbuf);
	sprintf (cbuf, "%d %0.4f", engdoors_status, engdoors_proc);
	oapiWriteScenario_string (scn, "ENGINEDOORS", cbuf);
	sprintf (cbuf, "%d %0.4f", hab_status, hab_proc);
	oapiWriteScenario_string (scn, "HAB", cbuf);
	if (malf > 0) {
		sprintf (cbuf, "%d", malf);
		oapiWriteScenario_string (scn, "MALF", cbuf);
	}

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void GAGARIN::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
	if (!strnicmp (line, "GEAR", 4)) {
		sscanf (line+4, "%d%lf", &gear_status, &gear_proc);
		} else if (!strnicmp (line, "ENGINEDOORS", 11)) {
		sscanf (line+11, "%d%lf", &engdoors_status, &engdoors_proc);
		} else if (!strnicmp (line, "HAB", 3)) {
		sscanf (line+3, "%d%lf", &hab_status, &hab_proc);
		} else if (!strnicmp (line, "MALF", 4)) {
		sscanf (line+4, "%d", &malf);
		} else ParsePayloadString(line);

		ParseScenarioLineEx (line, vs);
	}


	RevertLandingGear();
	RevertLandingGear();

	RevertEngineDoors();
	RevertEngineDoors();

	RevertHab();
	RevertHab();

	RevertSeats();
	RevertSeats();

	/* always set the stage whenever something may have changed */
//	SetMainStage();

}


void GAGARIN::PlaceComponents()
{
	COMMON_MESH tmp;

	tmp.MeshHandle(hSeats);					// #0
	tmp.MeshPosition(_V(0,0,FwdZ));
	AddMeshEx (tmp);

	tmp.MeshHandle(hFront);					// #1
	tmp.MeshPosition(_V(0,0,FwdZ));
	AddMeshEx (tmp);

	if (bGCR) {								// #2
		tmp.MeshHandle(hGCR);
		tmp.MeshPosition(_V(0,0,AftZ));
		AddMeshEx (tmp);
	}
	else if (bAero) {						
		tmp.MeshHandle(hAero);
		tmp.MeshPosition(_V(0,0,AftZ));
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hAft);						
		tmp.MeshPosition(_V(0,0,AftZ));
		AddMeshEx (tmp);
	}
	if (bHab) {
		tmp.MeshHandle(hHab);				// #3
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hDummy1);
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}
	if (bLegs) {					
		tmp.MeshHandle(hGear);				// #4
		if (bAero)
			tmp.MeshPosition(_V(0,0,AftZ+0.5));
		else
			tmp.MeshPosition(_V(0,0,AftZ));
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hDummy2);
		tmp.MeshPosition(_V(0,0,AftZ));
		AddMeshEx (tmp);
	}
				

	if (bRack) {
		tmp.MeshHandle(hRack);				// #5
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	} else {
		tmp.MeshHandle(hDummy3);
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}
	if (bWeapRing) {
		tmp.MeshHandle(hWeapRing);			// #6
		tmp.MeshPosition(_V(0,0,FwdZ));
		AddMeshEx (tmp);
	}
	if (bStretch) {					
		tmp.MeshHandle(hMid);				// #7
		tmp.MeshPosition(_V(0,0,0));
		AddMeshEx (tmp);
	}
	if (bLock) {							// #8
		tmp.MeshHandle(hFwdLock);
		tmp.MeshPosition(_V(0,0,FwdZ+DockZ));
		AddMeshEx (tmp);
	}




 }

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void GAGARIN::SetMainStage( )
{
	AttachedMass = 0.0;
	for (int i = 0; i < numatch; i++) {
		hV = GetAttachmentStatus(attachpos[i]);
		if (hV != NULL) {
			AtchV[i] = oapiGetVesselInterface (hV);
			atchmass[i] = AtchV[i]->GetMass();
			AttachedMass += atchmass[i];
		}
		else atchmass[i] = 0.0;
	}
	SetGAGARINStage();

}


void GAGARIN::SetGAGARINStage( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */

	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.5, 9, 350, 0.3, 0.05, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_LIN, 0, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.2, 0.2
	};

	PARTICLESTREAMSPEC contrail2 = 
	{
		0, 5.0, 7, 350, 0.3, 0.25, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_LIN, 0, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.2, 0.2
	};


	PARTICLESTREAMSPEC LeakingGas = 
	{
		0, 25.5, 5, 5.0, 5, 4, 8, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_LIN, 0, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.2, 0.2
	};

	contrail.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (30+FwdZ);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( Drymass + AttachedMass + CalculateMass() );

	if ((bAero) && (bLegs))
		SetTouchdownPoints (_V(0,-10.0,-8.2-1.75+AftZ), _V(-7,7,-8.2-1.75+AftZ), _V(7,7,-8.2-1.75+AftZ));
	else if (bLegs)
		SetTouchdownPoints (_V(0,-10.0,-8.2+AftZ), _V(-7,7,-8.2+AftZ), _V(7,7,-8.2+AftZ));
	else
		SetTouchdownPoints (_V(-37.0,0.0,20.0), _V(-37.0,-17.0,-10), _V(-37.0, 17.0,-10));

	SetPMI (_V(43.16,43.65,42.37));
//	SetPMI (_V(203,173,42.37));
	SetCrossSections (_V(212,214,207));
	SetCW (1.0, 1.0, 1.0, 1.0);
	SetRotDrag (_V(0.7,0.7,1.2));
//	SetPitchMomentScale (-1e-6);
//	SetBankMomentScale (-1e-6);
	SetTrimScale (0.05);
	double tilt = 0.0;
	if (((seats_proc*SeatRotate*RAD) > PI/2))
		tilt = PI;
	SetCameraDefaultDirection(_V(0,(-1*sin(seats_proc*SeatRotate*RAD)),cos(seats_proc*SeatRotate*RAD)),tilt);
	SetCameraOffset(ofscam[0]+_V(0,0,FwdZ));

//	SetCameraDefaultDirection(_V(0,(-1*sin(seats_proc*SeatRotate*RAD)),cos(seats_proc*SeatRotate*RAD)),tilt);
//	SetCameraOffset(OFS_CAMERA1*(1-seats_proc)+OFS_CAMERA2*(seats_proc)+_V(0,0,FwdZ));
//	SetLiftCoeffFunc (LiftCoeff);

	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */

	MakeAnimations();



	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hGAGARIN, &VECTOR_ZERO );

/*
	insignia_tex = oapiCreateTextureSurface (512, 512);
	oapiBlt (insignia_tex, oapiGetTextureHandle (hFront, 2), 0, 0, 0, 0, 512, 512);
//	  oapiColourFill (insignia_tex, oapiGetColour (198,52,33), 128, 0, 128, 64);
	HDC hDC = oapiGetDC (insignia_tex);
	HFONT hFont = CreateFont(20, 0, 0, 0, 700, 0, 0, 0, RUSSIAN_CHARSET, 0, 0, 0, 0, "Arial");
	HFONT pFont = (HFONT)SelectObject (hDC, hFont);
	SetTextColor (hDC, 0xD0D0D0);
	SetBkMode (hDC, TRANSPARENT);
	SetTextAlign (hDC, TA_CENTER);
	char cbuf[16];
	strncpy (cbuf, GetName(), 16);
//	strncpy (cbuf, "Komarov", 16);
	int len = min(strlen(GetName()), 16);
//	TextOut (hDC, 196, 10, cbuf, len);
//	TextOut (hDC, 196, 74, cbuf, len);
	SetTextColor (hDC, 0x808080);
	SetTextAlign (hDC, TA_RIGHT);
	TextOut (hDC, 100, 158, cbuf, len);
	SetTextAlign (hDC, TA_LEFT);
//	TextOut (hDC, 133, 158, cbuf, len);
	SelectObject (hDC, pFont);
	DeleteObject (hFont);
	oapiReleaseDC (insignia_tex, hDC);
	
	oapiSetTexture (hFront, 2, insignia_tex);
*/	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_Gagarin==NULL)  
		ph_Gagarin  = CreatePropellantResource(Fuelmass, Fuelmass,1.0); 

	SetDefaultPropellantResource (ph_Gagarin); 

	if (bGCR) {
		th_first[0] = CreateThruster ( _V(  1.90, 0.0, -44.8+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
//		AddExhaustStream (th_first[0], _V( 0.0, 1.00,-63.2+AftZ), &contrail);
		AddExhaust (th_first[0], 0.5, 2.5);  

		th_first[1] = CreateThruster ( _V( -1.90, 0.0, -44.8+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
//		AddExhaustStream (th_first[1], _V( 0.0,-1.40,-63.2+AftZ), &contrail);
		AddExhaust (th_first[1], 0.5, 2.5);  

		thg_first = CreateThrusterGroup (th_first, 2, THGROUP_MAIN);

	}
	else {
		th_first[0] = CreateThruster ( _V( 0.00, 0.00,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[0], _V( 0.00, 0.00,-6.9+AftZ), &contrail);
		AddExhaust (th_first[0], 0.5, 2.5);  

		th_first[1] = CreateThruster ( _V( 0.00,-1.00,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[1], _V( 0.00,-1.00,-6.9+AftZ), &contrail);
		AddExhaust (th_first[1], 0.5, 2.5);  

		th_first[2] = CreateThruster ( _V( 0.00, 1.00,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[2], _V( 0.00, 1.00,-6.9+AftZ), &contrail);
		AddExhaust (th_first[2], 0.5, 2.5);  

		th_first[3] = CreateThruster ( _V(-0.86,-0.50,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[3], _V(-0.86,-0.50,-6.9+AftZ), &contrail);
		AddExhaust (th_first[3], 0.5, 2.5);  

		th_first[4] = CreateThruster ( _V( 0.86,-0.50,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[1], _V( 0.86,-0.50,-6.9+AftZ), &contrail);
		AddExhaust (th_first[4], 0.5, 2.5);  

		th_first[5] = CreateThruster ( _V(-0.86, 0.50,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[2], _V(-0.86, 0.50,-6.9+AftZ), &contrail);
		AddExhaust (th_first[5], 0.5, 2.5);  

		th_first[6] = CreateThruster ( _V( 0.86, 0.50,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
		AddExhaustStream (th_first[3], _V( 0.86, 0.50,-6.9+AftZ), &contrail);
		AddExhaust (th_first[6], 0.5, 2.5);  

		if (bAero) {

			th_first[7] = CreateThruster ( _V( 1.07, 1.85,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
			AddExhaustStream (th_first[7], _V( 1.07, 1.85,-6.9+AftZ), &contrail);
			AddExhaust (th_first[7], 0.5, 2.5);  

			th_first[8] = CreateThruster ( _V(-1.07, 1.85,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
			AddExhaustStream (th_first[8], _V(-1.07, 1.85,-6.9+AftZ), &contrail);
			AddExhaust (th_first[8], 0.5, 2.5);  

			th_first[9] = CreateThruster ( _V( 2.11, 0.00,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
			AddExhaustStream (th_first[9], _V( 2.11, 0.00,-6.9+AftZ), &contrail);
			AddExhaust (th_first[9], 0.5, 2.5);  

			th_first[10] = CreateThruster ( _V(-2.11, 0.00,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
			AddExhaustStream (th_first[10], _V(-2.11, 0.00,-6.9+AftZ), &contrail);
			AddExhaust (th_first[10], 0.5, 2.5);  

			th_first[11] = CreateThruster ( _V( 1.07,-1.85,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
			AddExhaustStream (th_first[11], _V( 1.07,-1.85,-6.9+AftZ), &contrail);
			AddExhaust (th_first[11], 0.5, 2.5);  
			
			th_first[12] = CreateThruster ( _V(-1.07,-1.85,-6.9+AftZ), _V(0,0,1), Thrust, ph_Gagarin, Ispvac, Ispvac*0.9);
			AddExhaustStream (th_first[12], _V(-1.07,-1.85,-6.9+AftZ), &contrail);
			AddExhaust (th_first[12], 0.5, 2.5);  

			thg_first = CreateThrusterGroup (th_first, 13, THGROUP_MAIN);
		}
		else 
			thg_first = CreateThrusterGroup (th_first, 7, THGROUP_MAIN);
		
	}


	if (malf > 0) {
		ClearThrusterDefinitions();
		if (malf == 1) {
			th_first[0] = CreateThruster ( attach[6], _V(0,0,1), 0.00000001, ph_Gagarin, Ispvac, Ispvac*0.9);
			AddExhaustStream (th_first[0], attach[6], &LeakingGas);
		} else {
			if (bGCR) {
				th_first[0] = CreateThruster ( _V(  0.00, 0.0, -44.8+AftZ), _V(0,0,1), 0.00000001, ph_Gagarin, Ispvac, Ispvac*0.9);
				AddExhaustStream (th_first[0], _V( 0.0, 0.00,-44.8+AftZ), &LeakingGas);
			}
			else {
				th_first[0] = CreateThruster ( _V( 0.00, 0.00,-6.9+AftZ), _V(0,0,1), 0.00000001, ph_Gagarin, Ispvac, Ispvac*0.9);
				AddExhaustStream (th_first[0], _V( 0.00, 0.00,-6.9+AftZ), &LeakingGas);
			}
		}
		thg_first = CreateThrusterGroup (th_first, 1, THGROUP_USER);
		SetThrusterLevel(th_first[0],1.0);
	}

	if (malf != 1)
		SetAttControlsGAGARIN();
	EnableTransponder(true);


}



/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */


int GAGARIN::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	if (!fPressed) return 0; // only process keydown events
	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		if (KEYDOWN (keystate, OAPI_KEY_SPACE)) 
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_SPACE, 1.0))
			{

			oapiSetShowGrapplePoints (!oapiGetShowGrapplePoints ());

			return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_J)) 
		{ // "Join Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{
			/* CVE-lite function ! */
			/* IMPORTANT */
			/* call DeployPayload() to jettison the GPDS payloads in sequence */

			AttachVessel();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();

			return 1;
			}
		}
	}
	else { // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) 
			{

			{
				OverridePayloadVelocity(_V(0,+0.25,0));
//				VESSEL *depV;
//				depV = DeployPayload();
//				if (depV == NULL) {
					for (int i = 0; i < numatch; i++) {
						if (GetAttachmentStatus(attachpos[i]) != NULL) {
							DetachChild(attachpos[i],0.02);
							atchmass[i] = 0.0;
							i = numatch;
						}
					}
//				}
			}

		

				/* CVE-lite function ! */
				/* IMPORTANT */
				/* call DeployPayload() to jettison the GPDS payloads in sequence */

				/* IMPORTANT */
				/* Always re-set the stage after any kind of change */

				SetMainStage();
				
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_V)) 
		{ // Switch the Viewpoint
			if (oapiAcceptDelayedKey (OAPI_KEY_V, 1.0))
			{
				RevertSeats();
				StopVesselWave3(MyID,PLAYGEAR);
				PlayVesselWave3(MyID,PLAYGEAR);

/*				VECTOR3 defdir;
				GetCameraDefaultDirection(defdir);
				if (defdir.z == 1) {
					SetCameraDefaultDirection(_V(0,-1,0));
					SetCameraOffset(OFS_CAMERA2+_V(0,0,FwdZ));
				}
				else {
					SetCameraDefaultDirection(_V(0,0,1));			
					SetCameraOffset(OFS_CAMERA1+_V(0,0,FwdZ));
				}
*/
			}

				return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_G)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_G, 1.0)) {
				// Toggle the Gear
				if ((bLegs) && (bAero)) {
					RevertLandingGear ();
					StopVesselWave3(MyID,PLAYGEAR);
					PlayVesselWave3(MyID,PLAYGEAR);
				}
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_P)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_P, 1.0)) {
				// Toggle the hab
				if (bHab) {
					RevertHab ();
					StopVesselWave3(MyID,PLAYHAB);
					PlayVesselWave3(MyID,PLAYHAB);
				}
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_O)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_O, 1.0)) {
				// Toggle the engine doors
				if (bAero) {
					RevertEngineDoors();
					StopVesselWave3(MyID,PLAYDOORS);
					PlayVesselWave3(MyID,PLAYDOORS);
				}
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0)) {
				// EVA
				SeparateEVA();
				SetMainStage();
			}
			return 1;
		}

	}
	return 0;
}



int GAGARIN::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}




void GAGARIN::SetAttControlsGAGARIN()
{
	THRUSTER_HANDLE th_rot[4];
	texRCS  = oapiRegisterExhaustTexture ("exhaust_atrcs");

	PARTICLESTREAMSPEC contrailR = 
	{
//		0, 0.25, 10.0, 9.0, 0.5, 1.0, 10, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
		0, 0.5, 29, 50.0, 0.3, 0.05, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_LIN, 0, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.2, 0.2
	};

	th_rot[0] = CreateThruster (_V( 0, 15, 0), _V(0, 0,-1), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V( 0,-15, 0), _V(0, 0, 1), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 0, 0, 15), _V(0, 1, 0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V( 0, 0,-15), _V(0,-1, 0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30, 6.30,1.932+FwdZ), _V(0,0, 1),texRCS); 
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.30, 6.30,1.932+FwdZ), _V(0,0, 1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V( 6.30,-6.30,1.043+FwdZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.30,-6.30,1.043+FwdZ), _V(0,0,-1),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.30, 6.30,1.932+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.30, 6.30,1.932+FwdZ), &contrailR);
	AddExhaustStream (th_rot[1], _V( 6.30,-6.30,1.043+FwdZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.30,-6.30,1.043+FwdZ), &contrailR);
	if (FwdHelper != 0) {
		AddExhaust (th_rot[2], 2.5, 0.5, _V(0,-7.75,3.38+FwdHelper), _V(0,-1,0),texRCS);  
		AddExhaustStream (th_rot[2], _V(0,-7.75,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[3], 2.5, 0.5, _V(0, 5.5,3.38+AftHelper), _V(0, 1,0),texRCS);  
		AddExhaustStream (th_rot[3], _V(0, 5.5,3.38+AftHelper), &contrailR);
	}
	thg_pitchup=CreateThrusterGroup (th_rot, 4, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,0,-15), _V(0,1,0),MAX_ATT_FIRST, ph_Gagarin, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(0,0,15), _V(0,1,0),MAX_ATT_FIRST, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30,-6.67,1.49+FwdZ), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.30,-6.67,1.49+FwdZ), _V(0,-1.0,0.0),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.30,-6.67,1.49+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.30,-6.67,1.49+FwdZ), &contrailR);
	if (FwdHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V(0,-7.75,3.38+FwdHelper), _V(0,-1,0),texRCS);  
		AddExhaustStream (th_rot[0], _V(0,-7.75,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[1], 2.5, 0.5, _V(0,-5.5,3.38+AftHelper), _V(0,-1,0),texRCS);  
		AddExhaustStream (th_rot[1], _V(0,-5.5,3.38+AftHelper), &contrailR);
	}
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);

	th_rot[0] = CreateThruster (_V(0,0,-15), _V(0,-1,0), MAX_ATT_FIRST, ph_Gagarin, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(0,0,15), _V(0,-1,0), MAX_ATT_FIRST, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.30, 6.67,1.49+FwdZ), _V(0,1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30, 6.67,1.49+FwdZ), _V(0,1.0,0.0),texRCS);  
	AddExhaustStream (th_rot[0], _V(-6.30, 6.67,1.49+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V( 6.30, 6.67,1.49+FwdZ), &contrailR);
	if (FwdHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V(0, 7.75,3.38+FwdHelper), _V(0, 1,0),texRCS);  
		AddExhaustStream (th_rot[0], _V(0, 7.75,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[1], 2.5, 0.5, _V(0, 5.5,3.38+AftHelper), _V(0, 1,0),texRCS);  
		AddExhaustStream (th_rot[1], _V(0, 5.5,3.38+AftHelper), &contrailR);
	}
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,-15,0), _V(0,0,-1), MAX_ATT_FIRST, ph_Gagarin, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(0, 15,0), _V(0,0, 1), MAX_ATT_FIRST, ph_Gagarin, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 0, 0, 15), _V(0,-1, 0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V( 0, 0,-15), _V(0, 1, 0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30,-6.30,1.932+FwdZ), _V(0,0, 1),texRCS); 
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.30,-6.30,1.932+FwdZ), _V(0,0, 1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V( 6.30, 6.30,1.043+FwdZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.30, 6.30,1.043+FwdZ), _V(0,0,-1),texRCS);   
	AddExhaustStream (th_rot[0], _V( 6.30,-6.30,1.932+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.30,-6.30,1.932+FwdZ), &contrailR);		
	AddExhaustStream (th_rot[1], _V( 6.30, 6.30,1.043+FwdZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.30, 6.30,1.043+FwdZ), &contrailR); 
	if (FwdHelper != 0) {
		AddExhaust (th_rot[2], 2.5, 0.5, _V(0, 7.75,3.38+FwdHelper), _V(0, 1,0),texRCS);  
		AddExhaustStream (th_rot[2], _V(0, 7.75,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[3], 2.5, 0.5, _V(0,-5.5,3.38+AftHelper), _V(0,-1,0),texRCS);  
		AddExhaustStream (th_rot[3], _V(0,-5.5,3.38+AftHelper), &contrailR);
	}
	thg_pitchdown=CreateThrusterGroup (th_rot, 4, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V( 15,0,0), _V(0,0, 1), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(-15,0,0), _V(0,0,-1), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 0, 0, 15), _V(-1,0, 0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V( 0, 0,-15), _V( 1,0, 0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30,-6.30,1.043+FwdZ), _V(0,0,-1),texRCS); 
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.30,-6.30,1.932+FwdZ), _V(0,0, 1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30, 6.30,1.043+FwdZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.30, 6.30,1.932+FwdZ), _V(0,0, 1),texRCS);   
	AddExhaustStream (th_rot[0], _V( 6.30,-6.30,1.043+FwdZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.30,-6.30,1.932+FwdZ), &contrailR);		
	AddExhaustStream (th_rot[0], _V( 6.30, 6.30,1.043+FwdZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.30, 6.30,1.932+FwdZ), &contrailR); 
	if (FwdHelper != 0) {
		AddExhaust (th_rot[2], 2.5, 0.5, _V( 7.75,0,3.38+FwdHelper), _V( 1,0,0),texRCS);  
		AddExhaustStream (th_rot[2], _V( 7.75,0,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[3], 2.5, 0.5, _V(-5.5,0,3.38+AftHelper), _V(-1,0,0),texRCS);  
		AddExhaustStream (th_rot[3], _V(-5.5,0,3.38+AftHelper), &contrailR);
	}
	CreateThrusterGroup (th_rot, 4, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V( 15,0,0), _V(0,0,-1), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(-15,0,0), _V(0,0, 1), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 0, 0, 15), _V( 1,0, 0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V( 0, 0,-15), _V(-1,0, 0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30,-6.30,1.932+FwdZ), _V(0,0, 1),texRCS); 
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.30,-6.30,1.932+FwdZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30, 6.30,1.043+FwdZ), _V(0,0, 1),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.30, 6.30,1.043+FwdZ), _V(0,0,-1),texRCS);   
	AddExhaustStream (th_rot[0], _V( 6.30,-6.30,1.932+FwdZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.30,-6.30,1.932+FwdZ), &contrailR);		
	AddExhaustStream (th_rot[0], _V( 6.30, 6.30,1.043+FwdZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.30, 6.30,1.043+FwdZ), &contrailR); 
	if (FwdHelper != 0) {
		AddExhaust (th_rot[2], 2.5, 0.5, _V(-7.75,0,3.38+FwdHelper), _V(-1,0,0),texRCS);  
		AddExhaustStream (th_rot[2], _V(-7.75,0,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[3], 2.5, 0.5, _V( 5.5,0,3.38+AftHelper), _V( 1,0,0),texRCS);  
		AddExhaustStream (th_rot[3], _V( 5.5,0,3.38+AftHelper), &contrailR);
	}	CreateThrusterGroup (th_rot, 4, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.67, 6.30,1.49+FwdZ), _V(1,0,0),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.67,-6.30,1.49+FwdZ), _V(1,0,0),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.67, 6.30,1.49+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V( 6.67,-6.30,1.49+FwdZ), &contrailR); 
	if (FwdHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V( 7.75,0,3.38+FwdHelper), _V( 1,0,0),texRCS);  
		AddExhaustStream (th_rot[0], _V( 7.75,0,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V( 5.5,0,3.38+AftHelper), _V( 1,0,0),texRCS);  
		AddExhaustStream (th_rot[0], _V( 5.5,0,3.38+AftHelper), &contrailR);
	}
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_FIRST, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.67, 6.30,1.49+FwdZ), _V(-1,0,0),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.67,-6.30,1.49+FwdZ), _V(-1,0,0),texRCS);  
	AddExhaustStream (th_rot[0], _V(-6.67, 6.30,1.49+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.67,-6.30,1.49+FwdZ), &contrailR);  
	if (FwdHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V(-7.75,0,3.38+FwdHelper), _V(-1,0,0),texRCS);  
		AddExhaustStream (th_rot[0], _V(-7.75,0,3.38+FwdHelper), &contrailR);
	}
	if (AftHelper != 0) {
		AddExhaust (th_rot[0], 2.5, 0.5, _V(-5.5,0,3.38+AftHelper), _V(-1,0,0),texRCS);  
		AddExhaustStream (th_rot[0], _V(-5.5,0,3.38+AftHelper), &contrailR);
	}
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V( 3, 3,0), _V( 0.0,-1.0,0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(-3, 3,0), _V( 1.0, 0.0,0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 3,-3,0), _V(-1.0, 0.0,0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V(-3,-3,0), _V( 0.0, 1.0,0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30, 6.30,1.49+FwdZ), _V( 0.0, 1.0,0),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.30, 6.30,1.49+FwdZ), _V(-1.0, 0.0,0),texRCS);  
	AddExhaust (th_rot[2], 2.5, 0.5, _V( 6.30,-6.30,1.49+FwdZ), _V( 1.0, 0.0,0),texRCS);  
	AddExhaust (th_rot[3], 2.5, 0.5, _V(-6.30,-6.30,1.49+FwdZ), _V( 0.0,-1.0,0),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.30, 6.30,1.49+FwdZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.30, 6.30,1.49+FwdZ), &contrailR);
	AddExhaustStream (th_rot[2], _V( 6.30,-6.30,1.49+FwdZ), &contrailR);
	AddExhaustStream (th_rot[3], _V(-6.30,-6.30,1.49+FwdZ), &contrailR);
	CreateThrusterGroup (th_rot, 4, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V( 3, 3,0), _V(-1.0, 0.0,0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[1] = CreateThruster (_V(-3, 3,0), _V( 0.0,-1.0,0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[2] = CreateThruster (_V( 3,-3,0), _V( 0.0, 1.0,0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	th_rot[3] = CreateThruster (_V(-3,-3,0), _V( 1.0, 0.0,0), MAX_ATT_FIRST/2, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30, 6.30,1.49+FwdZ), _V( 1.0, 0.0,0),texRCS);  
	AddExhaust (th_rot[1], 2.5, 0.5, _V(-6.30, 6.30,1.49+FwdZ), _V( 0.0, 1.0,0),texRCS);  
	AddExhaust (th_rot[2], 2.5, 0.5, _V( 6.30,-6.30,1.49+FwdZ), _V( 0.0,-1.0,0),texRCS);  
	AddExhaust (th_rot[3], 2.5, 0.5, _V(-6.30,-6.30,1.49+FwdZ), _V(-1.0, 0.0,0),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.30, 6.30,1.49+FwdZ), &contrailR);
	AddExhaustStream (th_rot[1], _V(-6.30, 6.30,1.49+FwdZ), &contrailR);
	AddExhaustStream (th_rot[2], _V( 6.30,-6.30,1.49+FwdZ), &contrailR);
	AddExhaustStream (th_rot[3], _V(-6.30,-6.30,1.49+FwdZ), &contrailR);
	CreateThrusterGroup (th_rot, 4, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_FIRST*2, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30, 6.30,1.043+FwdZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.30, 6.30,1.043+FwdZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30,-6.30,1.043+FwdZ), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.30,-6.30,1.043+FwdZ), _V(0,0,-1),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.30, 6.30,1.043+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.30, 6.30,1.043+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V( 6.30,-6.30,1.043+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.30,-6.30,1.043+FwdZ), &contrailR);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_FIRST*2, ph_Gagarin, ISP_RCS_VAC);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30, 6.30,1.932+FwdZ), _V(0,0,1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.30, 6.30,1.932+FwdZ), _V(0,0,1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 6.30,-6.30,1.932+FwdZ), _V(0,0,1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-6.30,-6.30,1.932+FwdZ), _V(0,0,1),texRCS);  
	AddExhaustStream (th_rot[0], _V( 6.30, 6.30,1.932+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.30, 6.30,1.932+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V( 6.30,-6.30,1.932+FwdZ), &contrailR);
	AddExhaustStream (th_rot[0], _V(-6.30,-6.30,1.932+FwdZ), &contrailR);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}