/**
*
* $Id: Gagarin.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

const int nsurf = 10; // number of bitmap handles
const double SeatRotate = 120.0;

// All figures are completely made up
const double ISP_RCS_VAC		=	  786.0*G; //***
const double ISP_FIRST_SL		=     956.0*G; 
const double ISP_FIRST_VAC		=    1086.0*G; //***
const double ISP_LOX_VAC		=	  420.0*G;
const double ISP_GCR_VAC		=	 6500.0*G;
const double THRUST_FIRST_VAC	=    6300.0*G; //***
const double THRUST_MARS_VAC	=    9300.0*G; //***
const double THRUST_LOX_VAC		=   22800.0*G; //***
const double THRUST_GCR_VAC		=   90000.0*G; //***
const double MAX_ATT_FIRST		=    3000.0*G;
const double GAGARIN_PROP_MASS	=   84350.0; 
const double KOMAROV_PROP_MASS	=  210350.0; 
const double GAGARIN_DRYMASS	=   19250.0; //***
const double SKORPION_DRYMASS	=    9000.0; //***
const double AERO_DRYMASS		=    4550.0; //***
const double HAB_DRYMASS		=   10250.0; //***
const double BIGGER_ENGINES		=    5460.0; //***
const double LOCK_DRYMASS		=     200.0; //***
const double RACK_DRYMASS		=     250.0; //***
const double STRETCH_DRYMASS	=	 2350.0;
const double GCR_DRYMASS		=   14500.0; //***

const double R_EARTH = 6371000;			// radius of the earth (m)
const double U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
const double APO_ALT = 240.0;


const VECTOR3 FRONT_OFS			= _V(0,0,0);
const VECTOR3 MID_OFS			= _V(0,0,0);
const VECTOR3 AFT_OFS			= _V(0,0,0);
const VECTOR3 GAGARIN_OFS			= _V(0,0,0);
const VECTOR3 HAB_OFS			= _V(0,0,0);
const VECTOR3 LOCK_OFS			= _V(0,0,0);
const VECTOR3 AERO_OFS			= _V(0,0,0);
const VECTOR3 GEAR_OFS			= _V(0,0,0);
const VECTOR3 WINDRING_OFS		= _V(0,0,0);
const VECTOR3 ASTRO_OFS			= _V(0.0,-0.25,6.6);
const VECTOR3 EVA_OFS			= _V(0, 7.9,0.1);
const VECTOR3 OFS_SAT			= _V(0,-1.5,6.5);
const VECTOR3 OFS_CAMERA1		= _V(-0.44,-7.00,3.76);
const VECTOR3 OFS_CAMERA2		= _V(-0.44,-7.38,4.00);
const VECTOR3 PIVOTPOINT		= _V( 0.00,-7.40,3.87);

const double GEAR_OPERATING_SPEED = 0.25;
const double PLDOORS_OPERATING_SPEED = 0.07125;
const double ENGDOORS_OPERATING_SPEED = 0.07125;
const double HAB_OPERATING_SPEED = 0.025;
const double SEATS_OPERATING_SPEED = 0.25;

const double MAX_GRAPPLING_DIST		= 20.0;

#define PLAYGEAR					 20		// the gear sound
#define PLAYBANG					 24		// the jettison sound
#define PLAYJETT					 25     // entry shield jettison sound
#define PLAYDOORS					 26     // Doors open/close sound
#define PLAYHAB						 27     // Hab open/close sound
#define MYRADARSOUND				  2     // Custom radar sound
#define MYRADARCLOSE				  3     // Custom radar sound

/* make your own vessel class (named GAGARIN here, for example)
   subclassed from COMMON_VESSEL_EX */

class GAGARIN : public COMMON_VESSEL_EX {
public:
	GAGARIN (OBJHANDLE hObj, int fmodel);
	~GAGARIN();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } gear_status, engdoors_status, hab_status, seats_status;

	void MakeAnimations();
	void UpdateAnimations();
	void ActivateLandingGear (DoorStatus action);
	void RevertLandingGear();
	void ActivateEngineDoors (DoorStatus action);
	void RevertEngineDoors();
	void ActivateHab (DoorStatus action);
	void RevertHab();
	void ActivateSeats (DoorStatus action);
	void RevertSeats();
	void SeparateEVA();
	void SetGearParameters (double state);
	void AttachSweep();
	void AttachVessel();
	void RussianBeeps();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void PlaceComponents();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 
	void SetGAGARINStage();

	void SetAttControlsGAGARIN();    // this is to set up the thrusters. I like to keep them separate.
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC);

protected:
	PROPELLANT_HANDLE ph_Gagarin, ph_LOX;
	THRUSTER_HANDLE   th_first[13];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hFront,hMid,hAft,hGear,hAero,hHab,hGCR,hWindRing,hNoWindRing,hFwdLock,hSeats,hWeapRing,hRack,hDummy1,hDummy2,hDummy3;
	ANIMATIONCOMPONENT_HANDLE	Dr1,Dr2,Dr3,Dr4,Gear1_1,Gear1_2,Gear1_3,Gear2_1,Gear2_2,Gear2_3,
		Gear3_1,Gear3_2,Gear3_3,Gear4_1,Gear4_2,Gear4_3, hab1, hab2;
	SURFHANDLE		  tex,texRCS, retex;
//	UINT anim_lrudder,anim_rrudder,anim_laileron,anim_raileron,anim_lelevator,anim_relevator,anim_lelevatortrim,anim_relevatortrim;
	UINT anim_gear;     // handle for landing gear animation
	UINT anim_pldoors;     // handle for payload door animation
	UINT anim_engdoors;     // handle for payload door animation
	UINT anim_hab;	// handle for hab animation
	UINT anim_seats;	// handle for seats animation
	double gear_proc;
	double engdoors_proc;
	double hab_proc;
	double seats_proc;
	bool BrakesOn, havesound, bShieldJettisoned, bDeorbitFired, bThrustTerminated, bEjected, bEVA, bAbort, AbortChecked;
	UINT Exh1,Exh2,Exh3,Exh4;
	PSTREAM_HANDLE stream_a_1,stream_b_1,stream_a_2,stream_b_2,stream_a_3,stream_b_3,stream_a_4,stream_b_4;
	VECTOR3 Vold, Vnew;
	double dt, oldsimt;
	int MyID;
	VECTOR3 gpos, grms, pos, dir, rot;
	OBJHANDLE hV, BayVessel, hvessel;
	VESSEL *BayV;
	DWORD i, j;
	double a,e,h,V_periapsis,Altitude;
	DOCKHANDLE hDock, Dock0, Dock1, Dock2;
	bool bAnimationsMade;
	ATTACHMENTHANDLE sat_attach;
	VECTOR3 Rel_Vel;
	double Velocity;
	double FwdZ, AftZ, PlatformZ, DockZ;
	double Drymass, Fuelmass, Thrust, Ispvac;
	bool bDoorsOpen, bWindows, bLegs, bLock, bAero, bHab, bGCR, bStretch, bWeapRing, bRack;
	double FwdHelper, AftHelper;
	double atchmass[15];
	int malf;
	VECTOR3 ofscam[1];
	ATTACHMENTHANDLE  Dockpos,attachpos[15];
	VECTOR3           attach[15];
	VECTOR3			  attachdir[15];
	VECTOR3	          attachrot[15];
	VECTOR3			  gatchdir, gdir, grot, gatchrot;
	VESSEL *AtchV[15];
	MGROUP_TRANSFORM *viewpoint;
	double AttachedMass;
	bool Firstsweep;
	bool RadarFlagForSound;
	bool RadarFlagForClose;
	int numatch;


	SURFHANDLE srf[nsurf];          // handles for panel bitmaps
	SURFHANDLE insignia_tex;        // vessel-specific fuselage markings

};







/* basic vessel class setup */

void GAGARIN::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void GAGARIN::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void GAGARIN::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void GAGARIN::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void GAGARIN::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void GAGARIN::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int GAGARIN::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


void GAGARIN::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC)
{
	// draw the default HUD
	VESSEL2::clbkDrawHUD (mode, hps, hDC);

	// show gear deployment status
	if ((bLegs) && ((gear_status == DOOR_OPEN || (gear_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)))) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			Rectangle (hDC, cx-d/2, cy-d*5, cx+d/2, cy-d*4);
			Rectangle (hDC, cx-d/2, cy+d*2, cx+d/2, cy+d);
			Rectangle (hDC, cx-d*3, cy-d*2, cx-d*2, cy-d);
			Rectangle (hDC, cx+d*2, cy-d*2, cx+d*3, cy-d);
		}
	}


	// show hab deployment status
	if ((bHab) && ((hab_status == DOOR_OPEN || (hab_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)))) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			Rectangle (hDC, cx-d*3, cy+d*5.8, cx+d*3, cy+d*6.2);
			Rectangle (hDC, cx-d*6, cy+d*5, cx-d*3, cy+d*7);
			Rectangle (hDC, cx+d*6, cy+d*5, cx+d*3, cy+d*7);
			TextOut (hDC, cx-d*1, cy+d*6.5, "HAB", 3);
		}
	}

	// show engine doors deployment status
	if ((bAero) && ((engdoors_status == DOOR_OPEN || (engdoors_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)))) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			TextOut (hDC, cx-d*3, cy+d*3, "ENG DOORS", 9);
		}
	}

	// show RCS mode
	if (oapiCockpitMode() == COCKPIT_VIRTUAL) {
		switch (GetAttitudeMode()) {
		case RCS_ROT:
			TextOut (hDC, 0, hps->H-13, "RCS ROT", 7);
			break;
		case RCS_LIN:
			TextOut (hDC, 0, hps->H-13, "RCS_LIN", 7);
			break;
		}
	}
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new GAGARIN (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (GAGARIN*)vessel;
}

