// ==============================================================
// ORBITER vessel module: NASA Manned Manuveuring Unit (MMU)
// ==============================================================

#include "Generic_EVA.hpp"
#include <cstdio>

GENERIC_EVA::GENERIC_EVA(OBJHANDLE hVessel, int iFlightModel)
: VESSEL2(hVessel, iFlightModel)
{
	phRCS = NULL;
	for(int i = 0; i<12+8; i++)
	{
		thRCS[i] = NULL;
	}
	height = minheight;
//	higher = true;
	bHandArmed = false;
	bUseNumPad = false;
	bDisplayAttachments = true;
	LHandMass = 0.0;
	RHandMass = 0.0;
	KEY1 = KEY2 = KEY3 = KEY4 = KEY5 = KEY6 = KEY7 = KEY8 = KEY9 = false;
	hgemini_leva = NULL;
	THRUST_ATT_VAC = 0.0;
	ISP_FIRST_VAC = 0.0;
	PROP_MASS = 0.0;
	DRYMASS = 0.0;
	OnInit = false;

}

GENERIC_EVA::~GENERIC_EVA()
{
	
	phRCS = NULL;
	for(int i = 0; i<12+8; i++)
	{
		thRCS[i] = NULL;
	}
	height = minheight;
//	higher = true;
	bHandArmed = false;
	bUseNumPad = false;
	bDisplayAttachments = false;
	LHandMass = 0.0;
	RHandMass = 0.0;
	KEY1 = KEY2 = KEY3 = KEY4 = KEY5 = KEY6 = KEY7 = KEY8 = KEY9 = false;
	hgemini_leva = NULL;
	THRUST_ATT_VAC = 0.0;
	ISP_FIRST_VAC = 0.0;
	PROP_MASS = 0.0;
	DRYMASS = 0.0;
	
}

void GENERIC_EVA::boardVessel()
{
	// "Abort Emergency"
	//return to /board vessel
	char name[256];
	char vname[256];
	UINT i;
	VECTOR3 gpos, grms;

	strcpy (vname, GetName());
	for (i=0;i<strlen(vname)+1;i++)
		name[i]=0;

	for (i=0;i<strlen(vname)-4;i++)
		name[i]=vname[i];
	
	OBJHANDLE hvessel=oapiGetVesselByName(name);
	VESSEL *v = oapiGetVesselInterface (hvessel);
	OBJHANDLE hndVessel;
			
	Local2Global (_V(0,0,0), grms);  // global position of RMS tip

	v->GetGlobalPos(gpos);
	if ((hvessel != 0) && (dist(gpos,grms) < v->GetSize()))
	{
		hndVessel = GetAttachmentStatus(LHand);
		if (hndVessel != 0) {
			if (oapiGetMass(hndVessel) < 100.0)
				oapiDeleteVessel(hndVessel);
			else DetachChild (LHand);
		}
		hndVessel = GetAttachmentStatus(RHand);
		if (hndVessel != 0) {
			if (oapiGetMass(hndVessel) < 100.0)
				oapiDeleteVessel(hndVessel);
			else DetachChild (RHand);
		}

		ClearMeshes();


	oapiSetFocusObject(hvessel);

//	SetEnableFocus(false);
	if (OnInit)
		MakeADummy();
	oapiDeleteVessel(GetHandle());
				//((Multi_Stage*)oapiGetVesselInterface(hVessel))->Message(100,0);
	}
	else {

//		Local2Global (_V(0,0,0), grms);  // global position of RMS tip
		for (DWORD j = 0; j < oapiGetVesselCount(); j++) 
		{
			hV = oapiGetVesselByIndex (j);
			if (hV == GetHandle()) 
				continue; // we don't want to grapple ourselves ...
			if (hV == hvessel)
				continue; // we already checked this one!
			oapiGetGlobalPos (hV, &gpos);
			if ((dist (gpos, grms) < oapiGetSize (hV)) && (oapiGetSize(hV) > 2.0)) { // in range
					VESSEL *vess = oapiGetVesselInterface (hV);

					if (vess->GetEnableFocus()) {
					
						hndVessel = GetAttachmentStatus(LHand);
						if (hndVessel != 0) {
							oapiDeleteVessel(hndVessel);
						}
						hndVessel = GetAttachmentStatus(RHand);
						if (hndVessel != 0) {
							oapiDeleteVessel(hndVessel);
						}

					ClearMeshes();


					oapiSetFocusObject(hV);

//					SetEnableFocus(false);
					if (OnInit)
						MakeADummy();
					oapiDeleteVessel(GetHandle());

					}
			}
		}
	}


}

void GENERIC_EVA::clbkDrawHUD(int mode, const HUDPAINTSPEC* hps, HDC hdc)
{
	VESSEL2::clbkDrawHUD(mode, hps, hdc);

	int d = hps->Markersize/2;
	int cx = hps->CX, cy = hps->CY;

	if (bHandArmed)
		TextOut(hdc, 10, int(hps->CY * 0.45), "1 <- GRAB -> 2", 14);

	if (GetAttachmentStatus(LHand)) {
			Rectangle (hdc, 10, int(cy*0.4), int(10+d), int(cy*0.4-d));
			Rectangle (hdc, 10, int(cy*0.4-1.5*d), int(10+0.33*d), int(cy*0.4-d));
			Rectangle (hdc, int(10+0.33*d), int(cy*0.4-d), int(10+0.667*d), int(cy*0.4-1.5*d));
			Rectangle (hdc, int(10+0.67*d), int(cy*0.4-d), int(10+d), int(cy*0.4-1.5*d));
			Rectangle (hdc, int(10+d), int(cy*0.4-0.25*d), int(10+1.4*d), int(cy*0.4-1*d));
	} else {
			Rectangle (hdc, 10, int(cy*0.4), int(10+d), int(cy*0.4-d));
			Rectangle (hdc, 10, int(cy*0.4-2*d), int(10+0.33*d), int(cy*0.4-d));
			Rectangle (hdc, int(10+0.33*d), int(cy*0.4-d), int(10+0.667*d), int(cy*0.4-2*d));
			Rectangle (hdc, int(10+0.67*d), int(cy*0.4-d), int(10+d), int(cy*0.4-2*d));
			Rectangle (hdc, int(10+d), int(cy*0.4-0.25*d), int(10+1.4*d), int(cy*0.4-1.5*d));
	}

	if (GetAttachmentStatus(RHand)) {
			Rectangle (hdc, (10+4*d), int(cy*0.4), int((10+4*d)+d), int(cy*0.4-d));
			Rectangle (hdc, (10+4*d), int(cy*0.4-1.5*d), int((10+4*d)+0.33*d), int(cy*0.4-d));
			Rectangle (hdc, int((10+4*d)+0.33*d), int(cy*0.4-d), int((10+4*d)+0.667*d), int(cy*0.4-1.5*d));
			Rectangle (hdc, int((10+4*d)+0.67*d), int(cy*0.4-d), int((10+4*d)+d), int(cy*0.4-1.5*d));
			Rectangle (hdc, int(10+4*d), int(cy*0.4-0.25*d), int((10+4*d)-0.4*d), int(cy*0.4-1*d));
	} else {
			Rectangle (hdc, (10+4*d), int(cy*0.4), int((10+4*d)+d), int(cy*0.4-d));
			Rectangle (hdc, (10+4*d), int(cy*0.4-2*d), int((10+4*d)+0.33*d), int(cy*0.4-d));
			Rectangle (hdc, int((10+4*d)+0.33*d), int(cy*0.4-d), int((10+4*d)+0.667*d), int(cy*0.4-2*d));
			Rectangle (hdc, int((10+4*d)+0.67*d), int(cy*0.4-d), int((10+4*d)+d), int(cy*0.4-2*d));
			Rectangle (hdc, int(10+4*d), int(cy*0.4-0.25*d), int((10+4*d)-0.4*d), int(cy*0.4-1.5*d));
	}

	if(bDisplayAttachments)
	{
		drawAttachments(hps, hdc);
	}
}

void GENERIC_EVA::clbkSetClassCaps(FILEHANDLE cfg)
{
	//read config file parameters here
//	bool bItem;
	char meshname[31], ambiancename[31];
	UINT i = 0;
	char* pLine;
	MyID=ConnectToOrbiterSoundDLL3(GetHandle());

	SoundOptionOnOff3(MyID,PLAYCABINRANDOMAMBIANCE,true);
	
	while(oapiReadScenario_nextline(cfg, pLine))
	{
		if(!strnicmp(pLine, "Meshname", 8))
		{
			char* pLine2 = pLine + 8;
			while(*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if(*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%31s", &meshname);
				hgemini_leva		= oapiLoadMeshGlobal (meshname);
			}

		}
		else if(!strnicmp(pLine, "Ambiance", 8))
		{
			char* pLine2 = pLine + 8;
			while(*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if(*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%31s", &ambiancename);
			}
			ReplaceStockSound3(MyID,ambiancename,REPLACE_AIR_CONDITIONNING);
			SoundOptionOnOff3(MyID,PLAYCABINRANDOMAMBIANCE,false);

		}
		else if(!strnicmp(pLine, "MaxAttitudeThrust", 17))
		{
			char* pLine2 = pLine + 17;
			while(*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if(*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &THRUST_ATT_VAC);
			}

		}
		else if(!strnicmp(pLine, "Isp", 3))
		{
			char* pLine2 = pLine + 3;
			while(*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if(*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &ISP_FIRST_VAC);
			}

		}
		else if(!strnicmp(pLine, "Mass", 4))
		{
			char* pLine2 = pLine + 4;
			while(*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if(*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &DRYMASS);
			}

		}
		else if(!strnicmp(pLine, "MaxFuel", 7))
		{
			char* pLine2 = pLine + 7;
			while(*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if(*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf", &PROP_MASS);
			}

		}
		else if(!strnicmp(pLine, "OFS_SEAT", 8))
		{
			char* pLine2 = pLine + 8;
			while(*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if(*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf%lf%lf", &OFS_SEAT.x, &OFS_SEAT.y, &OFS_SEAT.z);
			}

		}
		else if(!strnicmp(pLine, "OFS_LHAND", 9))
		{
			char* pLine2 = pLine + 9;
			while(*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if(*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf%lf%lf", &OFS_LHAND.x, &OFS_LHAND.y, &OFS_LHAND.z);
			}

		}
		else if(!strnicmp(pLine, "OFS_RHAND", 9))
		{
			char* pLine2 = pLine + 9;
			while(*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if(*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%lf%lf%lf", &OFS_RHAND.x, &OFS_RHAND.y, &OFS_RHAND.z);
			}

		}
		else if(!strnicmp(pLine, "UseNumPadForWalking", 19))
		{
			char* pLine2 = pLine + 19;
			while(*pLine2 != '=' && *pLine2 != '\0')
			{
				pLine2++;
			}
			if(*pLine2 != '\0')
			{
				pLine2++;
				sscanf(pLine2, "%d", &bUseNumPad);
			}

		}
	}

/*	if(oapiReadItem_bool(cfg, "UseNumPadForWalking", bItem))
	{
		bUseNumPad = bItem;
	}
*/	//

	ResetClassCaps();
}

void GENERIC_EVA::ResetClassCaps()
{
	SetSize (1.0);
	SetTouchdownPoints(_V(0.0,-minheight,10.0),_V(-10.0,-minheight,-4.0),_V(10.0,-minheight,-4.0));
	SetSurfaceFrictionCoeff(0.3,0.8);
	SetCameraOffset(_V(0.0,0.75,0.0));
	SetEmptyMass (DRYMASS);
	SetPMI (_V(.35,.13,0.35));
	SetCrossSections (_V(1.64,.77,1.64));
	SetCW (0.3, 0.3, .3, .3);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (0);
	SetBankMomentScale (0);
	SetLiftCoeffFunc (0); 
	AddMesh (hgemini_leva);
	//SetMaxFuelMass (20.0);
	//SetISP (350.0*SEC);
	//SetMaxThrust (ENGINE_MAIN, 31.5);
	//SetMaxThrust (ENGINE_RETRO, 31.5);
	//SetMaxThrust (ENGINE_HOVER, 50.0);
	//SetMaxThrust (ENGINE_ATTITUDE, 31.5);
	createRCS();

	char name[256];
	char vname[256];
	UINT i;
	strcpy (vname,GetName());
	for (i=0;i<strlen(vname)+1;i++)
		name[i]=0;
	for (i=0;i<strlen(vname)-4;i++)
		name[i]=vname[i];
	gvessel=oapiGetVesselByName(name);
	//SetStage(vessel);

	//	  					 child     pos        dir       rot      handle
	Seat  = CreateAttachment (true,  OFS_SEAT,   _V(0,1,0), _V(0,0,1), "BUTT");
	LHand = CreateAttachment (false, OFS_LHAND, _V(0,0,1), _V(0,1,0), "HANL");
	RHand = CreateAttachment (false, OFS_RHAND, _V(0,0,1), _V(0,1,0), "HANR");
	//vessel->SetEngineLevel(ENGINE_HOVER,0.0);

}

int GENERIC_EVA::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{

	VESSELSTATUS eva;
	if (!down) 
		return 0; // only process keydown events

	sprintf(oapiDebugString(), "");

	if (KEYMOD_SHIFT (kstate)) 
	{
	}
	else if (KEYMOD_CONTROL (kstate)) 
	{
		// insert ctrl key combinations here
		if (key == OAPI_KEY_1) 
		{ 
			
			// "Grab"
			bDisplayAttachments = !bDisplayAttachments;
			return 1;
		}
	}
	else 
	{ // unmodified keys
		if (key == OAPI_KEY_G) 
		{ 
			// "Grab"
			bHandArmed = true;
			return 1;
		}
		if ((key == OAPI_KEY_1) && (bHandArmed))  { // "Left hand"
			bHandArmed = false;
			takeObjectLeft();	
			return 1;
		}
		if ((key == OAPI_KEY_2) && (bHandArmed))  { // "Right hand"
			bHandArmed = false;
			takeObjectRight();
			return 1;
		}
		if (key == OAPI_KEY_E) 
		{ 
			boardVessel();
			return 1;
		}

		if (key == OAPI_KEY_S) // "Strap/Unstrap" 
		{

			Local2Global (OFS_SEAT, grms);  // global position of RMS tip
			
			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...
			for (DWORD i = 0; i < oapiGetVesselCount(); i++) 
			{
				hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) 
					continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < oapiGetSize (hV)) { // in range
					
//					sprintf(oapiDebugString(),"Distance is too far to strap in!");

					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (false);
					for (DWORD j = 0; j < nAttach; j++) 
					{
						// now scan all attachment points of the candidate
						hAtt = v->GetAttachmentHandle (false, j);
						const char *id = v->GetAttachmentId (hAtt);
						if (strncmp (id, "SEAT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						if (dist (gpos, grms) < v->GetSize()) { // found one!

							// check whether satellite is currently clamped into payload bay
							if (hV == v->GetAttachmentStatus (Seat))
							{
								v->DetachChild (hAtt);
								sprintf(oapiDebugString(),"Unstrapped");
							}
							else
							{
								GetStatus(eva);
								if (eva.status == 0)
								{
									v->AttachChild (GetHandle(), hAtt, Seat);
									sprintf(oapiDebugString(),"Strapped in!");
									oapiSetFocusObject (hV);
								}
							}
							return 1;
						}
					}
				}
			
			}

		
			return 1;			
		}

	}	
	return 0;
}



int GENERIC_EVA::clbkConsumeDirectKey(char* keymap)
{
	
	if(bUseNumPad)
	{
		if (KEYDOWN(keymap, OAPI_KEY_NUMPAD1) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1 && !bHandArmed) // "Walk Right" 
		{
			KEY1 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_NUMPAD2) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1 && !bHandArmed) // "Walk Right" 
		{
			KEY2 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_NUMPAD3) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY3 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_NUMPAD4) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY4 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_NUMPAD5) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY5 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_NUMPAD6) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY6 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_NUMPAD7) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY7 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_NUMPAD8) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY8 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_NUMPAD9) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY9 = true;
		}
	} else {
		if (KEYDOWN(keymap, OAPI_KEY_1) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1 && !bHandArmed) // "Walk Right" 
		{
			KEY1 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_2) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1 && !bHandArmed) // "Walk Right" 
		{
			KEY2 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_3) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY3 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_4) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY4 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_5) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY5 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_6) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY6 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_7) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY7 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_8) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY8 = true;
		}
		else if (KEYDOWN(keymap, OAPI_KEY_9) && !KEYMOD_CONTROL(keymap) && !KEYMOD_SHIFT(keymap) && GetFlightStatus()==1) // "Walk Right" 
		{
			KEY9 = true;
		}
	}
	return 0;
}

void GENERIC_EVA::clbkLoadStateEx(FILEHANDLE scn, void* vs)
{
	OnInit = true;
	char* line;
	while(oapiReadScenario_nextline(scn, line))
	{
		ParseScenarioLineEx(line, vs);
	}
//	bHandArmed = false;
//	ResetClassCaps();
}

void GENERIC_EVA::clbkPostStep(double fSimT, double fDeltaT, double fMJD)
{


	if (GetAttachmentStatus(RHand) != NULL)
		RHandMass = oapiGetMass(GetAttachmentStatus(RHand));
	if (GetAttachmentStatus(LHand) != NULL)
		LHandMass = oapiGetMass(GetAttachmentStatus(LHand));
	SetEmptyMass (DRYMASS+LHandMass+RHandMass);

}

void GENERIC_EVA::clbkPreStep(double fSimT, double fDeltaT, double fMJD)
{
	/*
	if (GetFlightStatus() == 1 && !KEY8 && !KEY2 && !KEY4 && !KEY6)
	{
		higher = false;

		VECTOR3 gpos, grms;
		Local2Global (OFS_SEAT, grms);  // global position of RMS tip
			
		// Are we close to something (like the LM?)  If so, set higher....
		// Not very scalable ...
		for (DWORD i = 0; i < oapiGetVesselCount(); i++) 
		{
			OBJHANDLE hV = oapiGetVesselByIndex (i);
			if (hV == GetHandle()) 
				continue; // we don't want to check ourselves ...
			oapiGetGlobalPos (hV, &gpos);
			if (dist (gpos, grms) < 1.5) 
				higher = true;	
		}

		// GRADUALLY change the touchdown height according to our need.

		if ((higher) && (height < maxheight))
			height+=0.1;
		if ((!higher) && (height > minheight))
			height-=0.5;

		if (((higher) && (height < maxheight)) || ((!higher) && (height > minheight)))
			SetTouchdownPoints(_V(0.0,-height,10.0),_V(-10.0,-height,-5.0),_V(10.0,-height,-5.0));

	}

*/

	if(!bHandArmed)
		if (KEY1 || KEY2 || KEY3 || KEY4 || KEY5 || KEY6 || KEY7 || KEY8 || KEY9) 
			moveEVA(fSimT, fDeltaT, fMJD);



	
	//vessel->SetEngineLevel(ENGINE_HOVER,0.0);	// There are no hover engines.  What are you thinking?
}

void GENERIC_EVA::clbkSaveState(FILEHANDLE scn)
{
	VESSEL2::clbkSaveState(scn);
}



void GENERIC_EVA::MakeADummy()
{
	VESSELSTATUS vs;
	GetStatus (vs);
	oapiCreateVessel ("Dummy", "Dummy", vs);
}

void GENERIC_EVA::createRCS()
{
	//instead of relying on obsolete functions, create the RCS by adding propellant resources and thrusters.

	phRCS = CreatePropellantResource(PROP_MASS);
	
	//forward RCS
	thRCS[0] = CreateThruster(_V(-0.15, 0.5, -0.1), _V(0.0, 0.0, 1.0), 10.0, phRCS, 1200.0, 0.0);
	thRCS[1] = CreateThruster(_V( 0.15, 0.5, -0.1), _V(0.0, 0.0, 1.0), 10.0, phRCS, 1200.0, 0.0);

	thRCS[2] = CreateThruster(_V(-0.15, -0.5, -0.1), _V(0.0, 0.0, 1.0), 10.0, phRCS, 1200.0, 0.0);
	thRCS[3] = CreateThruster(_V( 0.15, -0.5, -0.1), _V(0.0, 0.0, 1.0), 10.0, phRCS, 1200.0, 0.0);

	//backward RCS
	thRCS[4] = CreateThruster(_V(-0.15, 0.5, 0.1), _V(0.0, 0.0, -1.0), 10.0, phRCS, 1200.0, 0.0);
	thRCS[5] = CreateThruster(_V( 0.15, 0.5, 0.1), _V(0.0, 0.0, -1.0), 10.0, phRCS, 1200.0, 0.0);

	thRCS[6] = CreateThruster(_V(-0.15, -0.5, 0.1), _V(0.0, 0.0, -1.0), 10.0, phRCS, 1200.0, 0.0);
	thRCS[7] = CreateThruster(_V( 0.15, -0.5, 0.1), _V(0.0, 0.0, -1.0), 10.0, phRCS, 1200.0, 0.0);

	//Roll RCS / Side RCS
	thRCS[8] = CreateThruster(_V(-0.15, 0.5, -0.1), _V(1.0, 0.0, 0.0), 10.0, phRCS, 1200.0, 0.0);
	thRCS[9] = CreateThruster(_V( 0.15, 0.5, -0.1), _V(-1.0, 0.0, 0.0), 10.0, phRCS, 1200.0, 0.0);

	thRCS[10] = CreateThruster(_V(-0.15, -0.5, -0.1), _V(1.0, 0.0, 0.0), 10.0, phRCS, 1200.0, 0.0);
	thRCS[11] = CreateThruster(_V( 0.15, -0.5, -0.1), _V(-1.0, 0.0, 0.0), 10.0, phRCS, 1200.0, 0.0);

	//upward RCS
	thRCS[12] = CreateThruster(_V(-0.15, -0.5, 0.1), _V(0.0, 1.0, 0.0), 10.0, phRCS, 1200.0, 0.0);
	thRCS[13] = CreateThruster(_V( 0.15, -0.5, 0.1), _V(0.0, 1.0, 0.0), 10.0, phRCS, 1200.0, 0.0);

	thRCS[14] = CreateThruster(_V(-0.15, -0.5, -0.1), _V(0.0, 1.0, 0.0), 10.0, phRCS, 1200.0, 0.0);
	thRCS[15] = CreateThruster(_V( 0.15, -0.5, -0.1), _V(0.0, 1.0, 0.0), 10.0, phRCS, 1200.0, 0.0);

	//downward RCS
	thRCS[16] = CreateThruster(_V(-0.15, 0.5, 0.1), _V(0.0, -1.0, 0.0), 10.0, phRCS, 1200.0, 0.0);
	thRCS[17] = CreateThruster(_V( 0.15, 0.5, 0.1), _V(0.0, -1.0, 0.0), 10.0, phRCS, 1200.0, 0.0);

	thRCS[18] = CreateThruster(_V(-0.15, 0.5, -0.1), _V(0.0, -1.0, 0.0), 10.0, phRCS, 1200.0, 0.0);
	thRCS[19] = CreateThruster(_V( 0.15, 0.5, -0.1), _V(0.0, -1.0, 0.0), 10.0, phRCS, 1200.0, 0.0);

	CreateThrusterGroup(thRCS, 4,THGROUP_ATT_FORWARD);
	CreateThrusterGroup(thRCS + 4, 4, THGROUP_ATT_BACK);
	CreateThrusterGroup(thRCS+12, 4, THGROUP_ATT_UP);
	CreateThrusterGroup(thRCS + 16, 4, THGROUP_ATT_DOWN);

	THRUSTER_HANDLE thTemp[4];

	thTemp[0] = thRCS[8];
	thTemp[1] = thRCS[10];

	CreateThrusterGroup(thTemp, 2, THGROUP_ATT_RIGHT);

	thTemp[0] = thRCS[9];
	thTemp[1] = thRCS[11];

	CreateThrusterGroup(thTemp, 2, THGROUP_ATT_LEFT);

	thTemp[0] = thRCS[0];
	thTemp[1] = thRCS[1];
	thTemp[2] = thRCS[6];
	thTemp[3] = thRCS[7];

	CreateThrusterGroup(thTemp, 4, THGROUP_ATT_PITCHDOWN);

	thTemp[0] = thRCS[2];
	thTemp[1] = thRCS[3];
	thTemp[2] = thRCS[4];
	thTemp[3] = thRCS[5];

	CreateThrusterGroup(thTemp, 4, THGROUP_ATT_PITCHUP);

	thTemp[0] = thRCS[1];
	thTemp[1] = thRCS[3];
	thTemp[2] = thRCS[4];
	thTemp[3] = thRCS[6];

	CreateThrusterGroup(thTemp, 4, THGROUP_ATT_YAWRIGHT);

	thTemp[0] = thRCS[0];
	thTemp[1] = thRCS[2];
	thTemp[2] = thRCS[5];
	thTemp[3] = thRCS[7];

	CreateThrusterGroup(thTemp, 4, THGROUP_ATT_YAWLEFT);

	thTemp[0] = thRCS[9];
	thTemp[1] = thRCS[10];

	CreateThrusterGroup(thTemp, 2, THGROUP_ATT_BANKLEFT);

	thTemp[0] = thRCS[8];
	thTemp[1] = thRCS[11];

	CreateThrusterGroup(thTemp, 2, THGROUP_ATT_BANKRIGHT);
}


void GENERIC_EVA::drawAttachments(const HUDPAINTSPEC* hps, HDC hdc)
{
		

	VECTOR3 my_gpos;

	GetGlobalPos(my_gpos);

	VECTOR3 gX;
	VECTOR3 gY;
	VECTOR3 gZ;

	GlobalRot(_V(1.0, 0.0, 0.0), gX);
	GlobalRot(_V(0.0, 1.0, 0.0), gY);
	GlobalRot(_V(0.0, 0.0, 1.0), gZ);

	TextOut(hdc, (int)(hps->W * 0.3), hps->CY, "DISP:ATT", 8);

	for (DWORD i = 0; i < oapiGetVesselCount(); i++) 
	{
		OBJHANDLE hV = oapiGetVesselByIndex (i);
		if (hV == GetHandle()) 
			continue; // we don't want to grapple ourselves ...
	
		oapiGetGlobalPos (hV, &gpos);


		if (dist (gpos, my_gpos) < 25.0 + oapiGetSize (hV)) 
		{ 
			// in range
			VESSEL *v = oapiGetVesselInterface (hV);

			//Filter object if object too heavy to be hold on the ground
			if(v->GetMass() > 50.0 && GetFlightStatus() == 1)
				continue;

			DWORD nAttach = v->AttachmentCount (true);
			for (DWORD j = 0; j < nAttach; j++) 
			{ // now scan all child attachment points of the candidate
				ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
				const char *id = v->GetAttachmentId (hAtt);

				//Only display object which can be grappled with our hands.
				v->GetAttachmentParams (hAtt, pos, dir, rot);
				v->Local2Global (pos, gpos);

				VECTOR3 rpos = gpos - my_gpos;
				
				VECTOR3 att_loc = _V(dotp(gX,rpos), dotp(gY,rpos), dotp(gZ, rpos));

				double Az = atan2(att_loc.x, att_loc.z) * DEG;
				double El = -1.0*atan2(att_loc.y, att_loc.z) * DEG;

				int bX1 = (int)(Az * hps->Scale) + hps->CX - hps->Markersize;
				int bY1 = (int)(El * hps->Scale) + hps->CY - hps->Markersize;
				int bX2 = (int)(Az * hps->Scale) + hps->CX + hps->Markersize;
				int bY2 = (int)(El * hps->Scale) + hps->CY + hps->Markersize;

				Rectangle(hdc, bX1, bY1, bX2, bY2);

				if(!strnicmp(id, "HAN", 3))
				{
					//Highlight all attachment points which are specifically for hands
					Rectangle(hdc, bX1-2, bY1-2, bX2+2, bY2+2);
				}
			}
		}
	}	
}

void GENERIC_EVA::moveEVA(double fSimT, double fDeltaT, double fMJD)
{
	double cm ;
	double lat;
	double lon;
//	double cap;
	double timeW=1.0;

	/*sprintf(oapiDebugString(), "moveEVA %f [%c %c %c %c %c %c %c %c %c]", 
		fSimT,
		KEY1?'y':'n', KEY2?'y':'n', KEY3?'y':'n', 
		KEY4?'y':'n', KEY5?'y':'n', KEY6?'y':'n', 
		KEY7?'y':'n', KEY8?'y':'n', KEY9?'y':'n');
	*/

	VESSELSTATUS2 eva;
	eva.version = 2;
	eva.flag = NULL;

	GetStatusEx(&eva);
	if (eva.status != 1) return;

	if (GetGravityRef() != GetSurfaceRef())				// I.E., Phobos
		return;

	timeW=oapiGetTimeAcceleration();

	if (timeW >100.0)
		oapiSetTimeAcceleration(100);

	lon=eva.surf_lng;
	lat=eva.surf_lat;

	cm = 0.86e-6 * fDeltaT;
	//Walking speed divided by planet circumspherence
	cm = WALKING_SPEED  * fDeltaT / (oapiGetSize(GetSurfaceRef()) * 2 * PI);

	if (KEY1)
	{
		eva.surf_hdg = eva.surf_hdg - TURN_SPEED * fDeltaT;

		if(eva.surf_hdg <= -PI)
		{
			eva.surf_hdg = eva.surf_hdg + 2 * PI;
		}
		KEY1=false;
	}
	
	else if (KEY3)
	{
		eva.surf_hdg = eva.surf_hdg + TURN_SPEED * fDeltaT;
		if(eva.surf_hdg >= PI ) 
		{
			eva.surf_hdg = eva.surf_hdg - 2*PI;
		}
		KEY3=false;
	}
	
	else if (KEY2)
	{
		lat = lat - cos(eva.surf_hdg) * cm;
		lon = lon - sin(eva.surf_hdg) * cm;
		KEY2=false;
	}
	
	else if (KEY4 )
	{
		lat = lat + sin(eva.surf_hdg) * 0.25*cm;
		lon = lon - cos(eva.surf_hdg) * 0.25*cm;
		KEY4=false;
	}
	
	else if (KEY5)
	{
		KEY5=false;
	}
	
	else if (KEY6 )
	{
		lat = lat - sin(eva.surf_hdg) * 0.25 * cm;
		lon = lon + cos(eva.surf_hdg) * 0.25 * cm;
		KEY6=false;
	}
	
	else if (KEY7)
	{
		KEY7=false;
	}
//	else if (KEY8||GoHover && !Astro){// we go ahead whatever our headign
	
	else if (KEY8)
	{// we go ahead whatever our headign
		lat = lat + cos(eva.surf_hdg) * cm;
		lon = lon + sin(eva.surf_hdg) * cm;
		KEY8=false;
	}

	else if (KEY9){
		KEY9=false;
	}

	eva.surf_lng=lon;
	eva.surf_lat=lat;
	eva.status = 1;

	DefSetStateEx(&eva);
	//KEY8 = false;
}


void GENERIC_EVA::takeObjectLeft()
{
	VECTOR3 gpos, grms, pos, dir, rot;
	Local2Global (OFS_LHAND, grms);  // global position of RMS tip
			
	// Search the complete vessel list for a grappling candidate.
	// Not very scalable ...
	for (DWORD i = 0; i < oapiGetVesselCount(); i++) 
	{
		OBJHANDLE hV = oapiGetVesselByIndex (i);
		if (hV == GetHandle()) 
			continue; // we don't want to grapple ourselves ...
	
		oapiGetGlobalPos (hV, &gpos);
		if (dist (gpos, grms) < 3.0*oapiGetSize (hV)) 
		{ 
			// in range
			VESSEL *v = oapiGetVesselInterface (hV);
			DWORD nAttach = v->AttachmentCount (true);
			for (DWORD j = 0; j < nAttach; j++) 
			{ // now scan all attachment points of the candidate
				ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
				const char *id = v->GetAttachmentId (hAtt);

//				if (!strncmp (id, "SEAT", 4)) 
//					continue; // attachment point not compatible
				v->GetAttachmentParams (hAtt, pos, dir, rot);
				v->Local2Global (pos, gpos);
				if (dist (gpos, grms) < v->GetSize()) 
				{ 
					// found one!
					// check whether satellite is currently clamped into payload bay
					if (hV == GetAttachmentStatus (LHand)) 
					{
						DetachChild (LHand);
						LHandMass = 0.0;
						SetEmptyMass (DRYMASS+LHandMass+RHandMass);
					}
					else 
					{
						VECTOR3 Fw;
						v->GetWeightVector(Fw);
						if(length(Fw) <= MAX_WEIGHTFORCE)
						{
							AttachChild (hV, LHand, hAtt);
							LHandMass = v->GetMass();
							if ((GetAttachmentStatus (RHand)) == (GetAttachmentStatus (LHand))) {
								DetachChild (RHand);
								RHandMass = 0.0;
							}
//							RHandMass = oapiGetMass(GetAttachmentStatus (RHand));
							SetEmptyMass (DRYMASS+LHandMass+RHandMass);
						}
					}
					return;
				}
			}
		}
	}
}

void GENERIC_EVA::takeObjectRight()
{
	VECTOR3 gpos, grms, pos, dir, rot;
	Local2Global (OFS_RHAND, grms);  // global position of RMS tip
			
	// Search the complete vessel list for a grappling candidate.
	// Not very scalable ...
	for (DWORD i = 0; i < oapiGetVesselCount(); i++) 
	{
		OBJHANDLE hV = oapiGetVesselByIndex (i);
		if (hV == GetHandle()) 
			continue; // we don't want to grapple ourselves ...

		oapiGetGlobalPos (hV, &gpos);
		if (dist (gpos, grms) < 3.0*oapiGetSize (hV)) { // in range
			VESSEL *v = oapiGetVesselInterface (hV);
			DWORD nAttach = v->AttachmentCount (true);
			for (DWORD j = 0; j < nAttach; j++) 
			{ 
				// now scan all attachment points of the candidate
				ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
				const char *id = v->GetAttachmentId (hAtt);

				if (!strncmp (id, "SEAT", 4)) 
					continue; // attachment point not compatible
				v->GetAttachmentParams (hAtt, pos, dir, rot);
				v->Local2Global (pos, gpos);

				if (dist (gpos, grms) < v->GetSize()) 
				{ 
					// found one!
					// check whether satellite is currently clamped into payload bay
					if (hV == GetAttachmentStatus (RHand)) {
						DetachChild (RHand);
						RHandMass = 0.0;
						SetEmptyMass (DRYMASS+LHandMass+RHandMass);
					}
					else {
						VECTOR3 Fw;
						v->GetWeightVector(Fw);
						if(length(Fw) <= MAX_WEIGHTFORCE)
						{
							AttachChild (hV, RHand, hAtt);
							RHandMass = v->GetMass();
							if ((GetAttachmentStatus (RHand)) == (GetAttachmentStatus (LHand))) {
								DetachChild (LHand);
								LHandMass = 0.0;
							}
							SetEmptyMass (DRYMASS+LHandMass+RHandMass);
						}
					}
					return;
				}
			}
		}
	}
}


// ==============================================================
// API interface
// ==============================================================

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new GENERIC_EVA(hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
//	sprintf(oapiDebugString(),"Exit");
//		if (vessel) delete (GENERIC_EVA*)vessel;
	delete static_cast<GENERIC_EVA*>(vessel);
}
