/**
*
* $Id: m53.cpp
* (c) 2005 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* Autopilot by rjcroy
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "m53.h"


// 1. vertical lift component (wings and body)
// Borrowed from Mike Norman's Shuttle code

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5,  0.02, 0.655,     0.46,   0.32, 0.12, 0.05, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,     0,  0.002,  0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	// change for drag increase at high Mach:
	// Subsonic Equivalents

	double Cl0, Cl1, Cd0, Cd1, Cd2;
	if(M>=5.0) {
		Cl0 = -0.187040;
		Cl1 =  1.275600;
		Cd0 =  0.128540;
		Cd1 = -0.402900;
		Cd2 =  1.539960;
		*cl = Cl0 + Cl1*aoa;
		*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
 	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
	if(M<1.25) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.02 + 0.2*oapiGetInducedDrag (*cl, 6.5, 0.92);
	}
	else if(M<5.0) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.025 + 0.25*oapiGetInducedDrag (*cl, 4.1, 0.85);
// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
	}
}


// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 2.5, 0.8) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	
	if (M > 1.0) *cd *= 0.6;
	else *cd *= 0.3;
}


M53::M53 (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	strcat (meshname,"2001/m-53");

	hM53				= oapiLoadMeshGlobal (meshname);

	tex = oapiRegisterExhaustTexture("RealExhaust");

	/* set the propellant handles to NULL so I can check if I need to 
	/* create them in the SetMainStage() function. */

	ph_m53=NULL;
	bBoosterMade=false;
	bControlsMade=false;
	gear_status=DOOR_CLOSED;
	gear_proc=0.0;
	oldsimt = -0.01;
	Vold = _V(0,0,0);
	BrakesOn = false;

	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
	RequestLoadVesselWave3(MyID,PLAYGEAR,"Sound\\2001\\gear.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYBANG,"Sound\\2001\\stg2sep.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYJETT,"Sound\\2001\\fasep.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYJET,"Sound\\2001\\jet.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,MYRADARSOUND,"Sound\\2001\\rusbeep.wav",INTERNAL_ONLY);
	RequestLoadVesselWave3(MyID,MYRADARCLOSE,"Sound\\2001\\rusclose.wav",INTERNAL_ONLY);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();   
 
	MakeAnimations();

	Firstsweep = true;

}


M53::~M53()
{

}

void M53::MakeAnimations()
{

	anim_gear = 1.0;

	anim_gear = CreateAnimation (1);
/*	anim_lelevator = CreateAnimation (0.5);
	anim_relevator = CreateAnimation (0.5);
	anim_laileron = CreateAnimation (0.5);
	anim_raileron = CreateAnimation (0.5);
	anim_rudder = CreateAnimation (0.5);
*/

/*	static UINT LGearDoorGrp[1] = {10};
	static MGROUP_ROTATE LGearDoor (0, LGearDoorGrp, 1,
		_V(-1.038, 0.68,0.3), _V(0,0,1), (float)(-130*RAD));
*/
	static UINT LGrGrp[1] = {9};
	static MGROUP_ROTATE LGr (0, LGrGrp, 1,
		_V(-6.45,4.73,-12.8), _V( 0,0, 1), (float)(-90*RAD));

	static UINT LWhl[3] = {10, 13,17};
	static MGROUP_TRANSLATE LWheels (0, LWhl, 3,
		_V( 0,-1.8,0));
		
	static UINT RGrGrp[1] = {11};
	static MGROUP_ROTATE RGr (0, RGrGrp, 1,
		_V( 6.45,4.73,-12.8), _V( 0,0,-1), (float)(-90*RAD));

	static UINT RWhl[3] = {12, 14,18};
	static MGROUP_TRANSLATE RWheels (0, RWhl, 3,
		_V( 0,-1.8,0));
		



	static UINT NoseGrGrp[3] = {7,8,15};
	static MGROUP_ROTATE NoseGear (0, NoseGrGrp, 3,
		_V( 0.0, 1.975, 15.85), _V( 1,0, 0), (float)( 120*RAD));


	Gear1 = AddAnimationComponent (anim_gear, 0.2, 0.85, &LGr);
	AddAnimationComponent (anim_gear, 0.6, 0.85, &LWheels, Gear1);

	Gear2 = AddAnimationComponent (anim_gear, 0.2, 0.85, &RGr);
	AddAnimationComponent (anim_gear, 0.6, 0.85, &RWheels, Gear2);

//	Gear3 = AddAnimationComponent (anim_gear, 0.4, 0.75, &RGr);
//	AddAnimationComponent (anim_gear, 0.1, 0.35, &RGrStrut, Gear3);
//	AddAnimationComponent (anim_gear, 0.2, 0.35, &RWheels, Gear3);
//	AddAnimationComponent (anim_gear, 0.1, 0.2, &RWheels2, Gear3);
	AddAnimationComponent (anim_gear, 0.05, 0.95, &NoseGear);

/*
Known Groups:

  0 Windows
  1 Wing
  2 Canard
  3 Fuselage
  4 Engines
  5 Fans
  6 Nozzles
  7 Nose Gear Strut
  8 Nose Gear Wheels
  9 L Upr Strut
  10 L Wheel
  11 R Upr Strut
  12 R Wheel
  13 L Lwr Strut
  14 R Lwr Strut
  15 Nose Tires
  16 Nozzle Closures
  17 L Tires
  18 R Tires
  19 VStabs
  20 VStabs

*/
/*
	// ***** Elevon animation *****
	static UINT LElevonGrp[1] = {19};
	static MGROUP_ROTATE LElevon (0, LElevonGrp, 1,
		_V(-13,1.5,-32.57), _V(-1,0.0, 0), (float)(-35*RAD));
	AddAnimationComponent (anim_lelevon, 0, 1, &LElevon);
	static UINT RElevonGrp[1] = {20};
	static MGROUP_ROTATE RElevon (0, RElevonGrp, 1,
		_V( 13,1.5,-32.57), _V( 1,0.0, 0), (float)( 35*RAD));
	AddAnimationComponent (anim_relevon, 0, 1, &RElevon);

	// ***** Elevator animation *****
	static UINT LElevatorGrp[1] = {32};
	static MGROUP_ROTATE LElevator (0, LElevatorGrp, 1,
		_V(-4.35,1.1,-30.57), _V(-1,0.0, 0.3), (float)(-25*RAD));
	AddAnimationComponent (anim_lelevator, 0, 1, &LElevator);
	static UINT RElevatorGrp[1] = {33};
	static MGROUP_ROTATE RElevator (0, RElevatorGrp, 1,
		_V( 4.35,1.1,-30.57), _V( 1,0.0, 0.3), (float)( 25*RAD));
	AddAnimationComponent (anim_relevator, 0, 1, &RElevator);
*/
/*
	// ***** Elevator trim animation *****
	static UINT LElevatorTrimGrp[1] = {3};
	static MGROUP_ROTATE LElevatorTrim (5, LElevatorTrimGrp, 1,
		_V(-2.15,0.83,-3.74), _V(-0.25,0.45,-0.4), (float)(-10*RAD));
	anim_lelevatortrim = CreateAnimation (0.5);
	AddAnimationComponent (anim_lelevatortrim, 0, 1, &LElevatorTrim, wing1);
	static UINT RElevatorTrimGrp[1] = {5};
	static MGROUP_ROTATE RElevatorTrim (5, RElevatorTrimGrp, 1,
		_V( 2.15,0.83,-3.74), _V( 0.25,0.45,-0.4), (float)( 10*RAD));
	anim_relevatortrim = CreateAnimation (0.5);
	AddAnimationComponent (anim_relevatortrim, 0, 1, &RElevatorTrim, wing2);
*/
/*	// ***** Aileron animation *****
	static UINT LAileronGrp[1] = {36};
	static MGROUP_ROTATE LAileron (0, LAileronGrp, 1,
		_V( 27.95,2.17,-20.27), _V(-1,0.1, 0.15), (float)(-20*RAD));
	AddAnimationComponent (anim_laileron, 0, 1, &LAileron, wing1);

	static UINT RAileronGrp[1] = {35};
	static MGROUP_ROTATE RAileron (0, RAileronGrp, 1,
		_V(-27.95,2.17,-20.27), _V( 1,0.1, 0.15), (float)(-20*RAD));
	AddAnimationComponent (anim_raileron, 0, 1, &RAileron, wing2);

	// ***** Rudder animation *****
	static UINT RudderGrp[1] = {30};
	static MGROUP_ROTATE Rudder (0, RudderGrp, 1,
		_V( 0,7.17,-35.625), _V(0,1,-0.15), (float)(-11*RAD));
	AddAnimationComponent (anim_rudder, 0, 1, &Rudder);

*/

}


void M53::MakeAirfoils()
{
	ClearAirfoilDefinitions();
	ClearVariableDragElements();


		CreateVariableDragElement (&gear_proc, 0.8, _V(0, -1, 0));


//					align,         attack pt,         coeff func, c, S,   AR
	CreateAirfoil (LIFT_VERTICAL, _V(0,1,-1.281),    VLiftCoeff, 35.89, 585.07, 1.751);
	// wing and body lift+drag components

	CreateAirfoil (LIFT_HORIZONTAL, _V(0,2,-23), HLiftCoeff, 7,  140.22, 1.04);
	// vertical stabiliser and body lift and drag components


	MakeControlSurfaces();
}


void M53::MakeControlSurfaces()
{

//  Aargh!  Can't delete control surfaces
//  Need to make sure we declare once and only once

	if (bControlsMade) return;
	bControlsMade = true;

	CreateControlSurface (AIRCTRL_ELEVATOR,      18.37,  0.8, _V(  8.5,0,-26), AIRCTRL_AXIS_XPOS); //, anim_lelevator);
	CreateControlSurface (AIRCTRL_ELEVATOR,      18.37,  0.8, _V( -8.5,0,-26), AIRCTRL_AXIS_XPOS); //, anim_relevator);
	CreateControlSurface (AIRCTRL_RUDDER,        21.74,  0.7, _V(   0, 0,-23), AIRCTRL_AXIS_YPOS); //, anim_rudder);
	CreateControlSurface (AIRCTRL_RUDDER,        21.74,  0.7, _V(   0, 0,-23), AIRCTRL_AXIS_YPOS); //, anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,       18.37, -0.8, _V(-8.5,0,-26), AIRCTRL_AXIS_XPOS); //, anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,       18.37, -0.8, _V( 8.5,0,-26), AIRCTRL_AXIS_XNEG); //, anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM,  7.25,  0.8, _V(  8.5,0,-23), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM,  7.25,  0.8, _V( -8.5,0,-23), AIRCTRL_AXIS_XPOS);

}




void M53::UpdateAnimations()
{	

	// animate landing gear
	if (gear_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * GEAR_OPERATING_SPEED;
		if (gear_status == DOOR_CLOSING) {
			if (gear_proc > 0.0)
				gear_proc = max (0.0, gear_proc-da);
			else {
				gear_status = DOOR_CLOSED;
			}
		} else  { // door opening
			if (gear_proc < 1.0)
				gear_proc = min (1.0, gear_proc+da);
			else {
				gear_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_gear, gear_proc);
	}

/*
	// animate wings
	if (wings_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * WINGS_OPERATING_SPEED;
		if (wings_status == DOOR_CLOSING) {
			if (wings_proc > 0.0)
				wings_proc = max (0.0, wings_proc-da);
			else {
				wings_status = DOOR_CLOSED;
			}
		} else  { // door opening
			if (wings_proc < 1.0)
				wings_proc = min (1.0, wings_proc+da);
			else {
				wings_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_wings, wings_proc);
	}
*/
}

void M53::ActivateLandingGear (DoorStatus action)
{
	gear_status = action;
}

void M53::RevertLandingGear ()
{
	ActivateLandingGear (gear_status == DOOR_CLOSED || gear_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}


void M53::AirBreathingEngine()
// Thanks to David Hopkins for publishing his AirBreathingEngine source.  
{//=========================================================
double alt2;
double eas2;
double cf1, cf2;
double ActualThrust;


alt2 = GetAltitude();
eas2 = GetAirspeed ();

alt2 = alt2*3.28;
alt2 = alt2*0.78;		// a kludge that keeps the "airbreathing" engine operating at higher altitude
eas2 = eas2*2.236;

cf1 = 1.67e-6*(eas2*eas2)-8.8789e-4*(eas2)+0.9977;	
cf2 = 3.0e-10*(alt2*alt2)-3.0000e-5*(alt2)+0.9934;

// Nasty kludge to keep the thrust from increasing dramatically above 100kft
if (alt2 > 50000)
	cf2 = 0.25*(1-((alt2-50000)*.0001));
if (cf2 < 0) cf2 = 0;
if (cf1 > 3.0) cf1 = 3.0;

ActualThrust = cf1*cf2*THRUST_JET_VAC;
if (GetMachNumber() > 3.0)
	ActualThrust = ActualThrust * (1-(GetMachNumber()*2-3.0*2));

//sprintf(oapiDebugString(), "cf1: %.3f, cf2:  %.3f, cf1*cf2:  %.3f ActualThrust:  %.3f", cf1, cf2, cf1*cf2, ActualThrust);

SetThrusterMax0 (th_first[0],ActualThrust);
SetThrusterMax0 (th_first[1],ActualThrust);
SetThrusterMax0 (th_first[2],ActualThrust);
SetThrusterMax0 (th_first[3],ActualThrust);

double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN);


		if(ThrusterActive>0.0f)
			{
			// we calculate the volume at wich it must be played
			double volume=110+(145*ThrusterActive);
			double freq=11050+(9000*ThrusterActive);
			PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
			}
		else if(IsPlaying3(MyID,PLAYJET))
			{
			// else shut down the sound if it's playing
			StopVesselWave3(MyID,PLAYJET);
			}


//SetThrusterMax0 (th_first[0],ActualThrust);
};

void M53::SetGearParameters (double state)
{

	double geary = ((1.0-state)*1.0);
	if ((gear_status == DOOR_OPENING) || (gear_status == DOOR_CLOSING)) {
		SetTouchdownPoints (_V(0,0+1.5*geary,16.95), _V(-6.8,0+geary,-9.0), _V( 6.8,0+geary,-9.0));
	}
	if (!BrakesOn) {
		SetSurfaceFrictionCoeff (0.001, 0.4);
	} else {
		SetSurfaceFrictionCoeff (0.1, 0.4);		// Brakes on!
	}

	double steerforce;
	double steerangle;

	steerforce = 40.0;
	if (GetAirspeed() < 40.0)
		steerforce *=8.0;
	steerforce = steerforce*GetMass();
	if (GetAirspeed () < 5.0) steerforce*=(GetAirspeed()/5); // from 1* to 0*

	steerangle = GetControlSurfaceLevel (AIRCTRL_RUDDER);
	steerforce = steerforce * steerangle;
	if ((GetAltitude() > 0.1) || (state < 0.0)) steerforce = 0.0;

	AddForce (_V(steerforce, 0, 0), _V(0, 0, 12.0));
	AddForce (_V(-steerforce, 0, 0), _V(0, 0, -12.0));


	// sprintf (oapiDebugString(), "%0.4f", steerforce );


}


void M53::TimeStep(  double simt )
{
	RussianBeeps();

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */

	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
			SetMainStage();
	}


	if (!bControlsMade)
		MakeControlSurfaces();

	

	AirBreathingEngine();
	double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN);
	if(ThrusterActive>0.0f)
		{
		// we calculate the volume at wich it must be played
		double volume=110+(145*ThrusterActive);
		double freq=11050+(9000*ThrusterActive);
		PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
		}
	else if(IsPlaying3(MyID,PLAYJET))
		{
		// else shut down the sound if it's playing
		StopVesselWave3(MyID,PLAYJET);
		}


	UpdateAnimations();
	SetGearParameters(gear_proc);

}


void M53::RussianBeeps()
{
		double Altitude=0;
		double Distance=1000;
		oapiGetFocusAltitude(&Altitude);
		Local2Global (_V(0,0,0), grms);  // global position of RMS tip
		for (int i = 0; i < oapiGetVesselCount(); i++) {
			hV = oapiGetVesselByIndex (i);
			if (hV == GetHandle()) continue; // we don't want to check ourselves ...
			oapiGetGlobalPos (hV, &gpos);
			if (((dist (gpos, grms) - oapiGetSize(hV) - oapiGetSize(GetHandle())) < Distance) && ((dist (gpos, grms) - oapiGetSize(hV) - oapiGetSize(GetHandle())) > 10.0))
				Distance = dist(gpos,grms) - oapiGetSize(hV) - oapiGetSize(GetHandle());
		}
		if ((Distance < Altitude) || (Altitude == 0))
			Altitude = Distance;			// Play beeps based on object distance instead!



		// ------------------------------------------------------
		// here we play our radar sound that slide with altitude
		// ------------------------------------------------------
		if(Altitude<600&&Altitude>25)
		{
			PlayVesselWave3(MyID,MYRADARSOUND,LOOP,255,(int)(16000+((600-Altitude)*10)));		// play looped and a frequency that increase with altitude
			RadarFlagForSound=TRUE;
			if(RadarFlagForClose=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARCLOSE);
				RadarFlagForClose = FALSE;
			}
		}
		else if (Altitude<25&&Altitude>10) 
		{
			PlayVesselWave3(MyID,MYRADARCLOSE,LOOP,255);		// play looped 
			RadarFlagForClose=TRUE;
			if(RadarFlagForSound=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARSOUND);
				RadarFlagForSound = FALSE;
			}
		}
		else {
			if(RadarFlagForSound=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARSOUND);
				RadarFlagForSound = FALSE;
			}
			if(RadarFlagForClose=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARCLOSE);
				RadarFlagForClose = FALSE;
			}
		}
}


/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */


void M53::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void M53::SetClassCaps(  FILEHANDLE cfg )
{

	CheckCentralMessage();

	SetMainStage();
}

void M53::SaveState( FILEHANDLE scn )
{
	char cbuf[256];

	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */


//	Undock(ALLDOCKS);

	sprintf (cbuf, "%d %0.4f", gear_status, gear_proc);
	oapiWriteScenario_string (scn, "GEAR", cbuf);
	sprintf (cbuf, meshname);
	oapiWriteScenario_string (scn, "MESH", cbuf);

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void M53::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		if (!strnicmp (line, "GEAR", 4)) {
		sscanf (line+4, "%d%lf", &gear_status, &gear_proc);
		} else if(!strnicmp (line, "MESH", 4) ) {
			sscanf( line+4, "%s", &meshname);

		} else ParsePayloadString(line);

		ParseScenarioLineEx (line, vs);
	}



	/* always set the stage whenever something may have changed */
	SetMainStage();

	RevertLandingGear();
	RevertLandingGear();

}


void M53::PlaceComponents()
{
	COMMON_MESH tmp;


	tmp.MeshHandle(hM53);
	tmp.MeshPosition(_V(0,0,0));
	AddMeshEx (tmp);


}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void M53::SetMainStage( )
{

	SoundOptionOnOff3(MyID,PLAYMAINTHRUST,true);
	hM53				= oapiLoadMeshGlobal (meshname);

	SetM53Stage();
}


void M53::SetM53Stage( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (45);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass (JET_DRYMASS + CalculateMass() );

	SetTouchdownPoints (_V(0,0.0,16.95), _V(-6.8,0.0,-9.0), _V( 6.8,0.0,-9.0));
	SetSurfaceFrictionCoeff (0.025, 0.4);
//	SetTouchdownPoints (_V(0,-10,-11), _V(-10,10,-11), _V(10,10,-11));
	SetPMI (_V(188,202,20));
	SetCrossSections (_V(176,489,22));
	SetCW (0.05, 0.2, 0.7, 0.7);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
//	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */
	if (!ph_m53)  
		ph_m53  = CreatePropellantResource(JET_PROP_MASS,JET_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_m53); 

	th_first[0] = CreateThruster ( _V( 2.80, 0.0,-23.75), _V(0,0,1), THRUST_JET_VAC, ph_m53, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[0], _V( 2.80, 3.768,-23.75), &contrail);
	th_first[1] = CreateThruster ( _V( 0.95, 0.0,-23.75), _V(0,0,1), THRUST_JET_VAC, ph_m53, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[1], _V( 0.95, 3.768,-23.75), &contrail);
	th_first[2] = CreateThruster ( _V(-0.95, 0.0,-23.75), _V(0,0,1), THRUST_JET_VAC, ph_m53, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[2], _V(-0.95, 3.768,-23.75), &contrail);
	th_first[3] = CreateThruster ( _V(-2.80, 0.0,-23.75), _V(0,0,1), THRUST_JET_VAC, ph_m53, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[3], _V(-2.80, 3.768,-23.75), &contrail);


	thg_first = CreateThrusterGroup (th_first, 4, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 20.0, 2.0, _V( 2.80, 3.768,-23.75), _V(0,0,-1), tex);
	AddExhaust (th_first[1], 20.0, 2.0, _V( 0.95, 3.768,-23.75), _V(0,0,-1), tex);
	AddExhaust (th_first[2], 20.0, 2.0, _V(-0.95, 3.768,-23.75), _V(0,0,-1), tex);
	AddExhaust (th_first[3], 20.0, 2.0, _V(-2.80, 3.768,-23.75), _V(0,0,-1), tex);
//	numengines = 4;

	SoundOptionOnOff3(MyID,PLAYMAINTHRUST,false);

	EnableTransponder(true);

}



int M53::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	if (!fPressed) return 0; // only process keydown events
	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else { // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
				VESSEL *depV;
				OBJHANDLE hVessel;
				OverridePayloadVelocity(_V(0,+0.25,0));
				depV = DeployPayload();
				hVessel=oapiGetVesselByName(depV->GetName());
				if (hVessel != 0)
					oapiSetFocusObject(hVessel);
					

					/* CVE-lite function ! */
					/* IMPORTANT */
					/* call DeployPayload() to jettison the GPDS payloads in sequence */

					/* IMPORTANT */
					/* Always re-set the stage after any kind of change */
				SetMainStage();
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_B)) 
		{ // Toggle the brakes
			if (oapiAcceptDelayedKey (OAPI_KEY_B, 1.0))
			{
				if (BrakesOn)
					BrakesOn = false;
				else BrakesOn = true;
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_G)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_G, 1.0)) {
				// Toggle the Gear
				RevertLandingGear ();
				StopVesselWave3(MyID,PLAYGEAR);
				PlayVesselWave3(MyID,PLAYGEAR);
			}
			return 1;
		}

	}
	return 0;
}



int M53::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}



