/**
*
* $Id: M53.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/cvelmessage.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1


const double ISP_JET_SL			=   10500.0*G; 
const double ISP_JET_VAC		=   17500.0*G; //***


const double THRUST_JET_VAC		=   768652.0/2.0; //
const double JET_DRYMASS		=   124720.0; //***
const double JET_PROP_MASS		=   92988.0; //***

const double R_EARTH = 6371000;			// radius of the earth (m)
const double U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
const double APO_ALT = 240.0;


const VECTOR3 M53_OFS			= _V(0,0,0);
const VECTOR3 CAMERA_OFS		= _V(-0.42, 6.5,15.0);

const double GEAR_OPERATING_SPEED = 0.25;
const double WINGS_OPERATING_SPEED = 0.25;

const double ROLL = 0.0;            // 180.0 to go to orbit heads-down.  Intermediate values probably won't work.

#define PLAYGEAR					 20		// the gear sound
#define PLAYBANG					 24		// the jettison sound
#define PLAYJETT					 25     // entry shield jettison sound
#define PLAYJET						 26     // the jet noise sound
#define MYRADARSOUND				  2     // Custom radar sound
#define MYRADARCLOSE				  3     // Custom radar sound

/* make your own vessel class (named M53 here, for example)
   subclassed from COMMON_VESSEL_EX */

class M53 : public COMMON_VESSEL_EX {
public:
	M53 (OBJHANDLE hObj, int fmodel);
	~M53();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } gear_status, bgear_status, brakes_status;

	void MakeAnimations();
	void MakeAirfoils();
	void MakeControlSurfaces();
	void UpdateAnimations();
	void ActivateLandingGear (DoorStatus action);
	void RevertLandingGear();
	void SetGearParameters (double state);
	void AirBreathingEngine();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void RussianBeeps();
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void PlaceComponents();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage();  
	void SetM53Stage();
	bool AzInput (void *id, char *str, void *data);
	bool IncInput (char *rstr);
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);
	double pitchinput, yawinput, rollinput;
	VESSELSTATUS vst;
	int stage;

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC);

protected:

	PROPELLANT_HANDLE ph_m53;
	THRUSTER_HANDLE   th_first[12];
	THGROUP_HANDLE    thg_first;
	MESHHANDLE		  hM53;
	ANIMATIONCOMPONENT_HANDLE wing11, wing12, wing13, wing14, Gear1, Gear2, Gear3, Gear4, nose1,
		Gear18, Gear19, Gear1A, Elev11, Rudd11, Elev12, Rudd12, Elev13, Rudd13, Elev14, Rudd14;
	SURFHANDLE		  tex,texLH2,texRCS, retex;
	UINT anim_rudder,anim_laileron,anim_raileron,anim_lelevator,anim_relevator,anim_lelevatortrim,anim_relevatortrim;
	UINT anim_gear;     // handle for landing gear animation
	double gear_proc;
	bool BrakesOn;
	VECTOR3 Vold, Vnew;
	double dt, oldsimt;
	int MyID;
	VECTOR3 gpos, grms, pos, dir, rot;
	OBJHANDLE hV;
	bool Firstsweep;
	bool RadarFlagForSound;
	bool RadarFlagForClose;
	char *key;
	char *message;

	bool bBoosterMade, bControlsMade;
	char meshname[80];

};








/* basic vessel class setup */

void M53::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void M53::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void M53::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void M53::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void M53::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void M53::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int M53::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


void M53::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC)
{
	// draw the default HUD
	VESSEL2::clbkDrawHUD (mode, hps, hDC);

	// show gear deployment status
	if (gear_status == DOOR_OPEN || (gear_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			Rectangle (hDC, cx-d/2, cy-d*5, cx+d/2, cy-d*4);
			Rectangle (hDC, cx-d*3, cy-d*2, cx-d*2, cy-d);
			Rectangle (hDC, cx+d*2, cy-d*2, cx+d*3, cy-d);
		}
	}

	// show brakes deployment status
	if (BrakesOn) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			TextOut (hDC, cx-d*9, cy+d*1.2, "BRAKES", 6);
		}
	}

}

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new M53 (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (M53*)vessel;
}