/**
*
* $Id: Moonbus.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/cvelmessage.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include <cstdio>
#include <cmath>
#include <string>


#define InitFuelLevel 1

#define PLAYTREADS						 27     // the tank noise sound



#define AID_MFD1_BBUTTONS       14
#define AID_MFD1_LBUTTONS       15
#define AID_MFD1_RBUTTONS       16
#define AID_MFD2_BBUTTONS       17
#define AID_MFD2_LBUTTONS       18
#define AID_MFD2_RBUTTONS       19
#define AID_MFD1_PWR          1024
#define AID_MFD1_SEL          1025
#define AID_MFD1_MNU          1026
#define AID_MFD2_PWR          1027
#define AID_MFD2_SEL          1028
#define AID_MFD2_MNU          1029

// All data from astronautix.com
const double ISP_FIRST_SL		=     444.0*G; 
const double ISP_FIRST_VAC		=     444.0*G; //***
const double THRUST_FIRST_VAC	=    7000.0*G/6.0; //***
const double MAX_ATT_FIRST		=     250.0*G;
const double FIRST_PROP_MASS	=   12000.0; //*** 
const double MOONBUS_MAX_BURN	=     625.0; //***
const double MOONBUS_DRYMASS	=    5250.0; //***

const VECTOR3 EVA_OFS			= _V(0,0.0,-6.0);

const double HATCH_OPERATING_SPEED = 0.125;


/* make your own vessel class (named MOONBUS here, for example)
   subclassed from COMMON_VESSEL_EX */

class MOONBUS : public COMMON_VESSEL_EX {
//class MOONBUS : public VESSEL2 {
public:
	MOONBUS (OBJHANDLE hObj, int fmodel);
	~MOONBUS();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } hatch_status;

	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void MoveEVA();
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void SeparateEVA();
	void ActivateHatch (DoorStatus action);
	void RevertHatch();
	void UpdateAnimations();
	void RedrawPanel_MFDButton (SURFHANDLE surf, int mfd, int side);


	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 

	void SetAttControlsMoonbus();    // this is to set up the thrusters. I like to keep them separate.
	int  ConsumeKey(char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeDirectKey (char *keystate );
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	bool clbkLoadVC (int id);
	bool clbkVCMouseEvent (int id, int event, VECTOR3 &p);
	bool clbkVCRedrawEvent (int id, int event, SURFHANDLE surf);

protected:
	PROPELLANT_HANDLE ph_first;
	THRUSTER_HANDLE   th_first[6];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hMoonbus;
	SURFHANDLE		  tex;
	bool KEY1, KEY2, KEY3, KEY4, KEY5, KEY6, KEY7, KEY8, KEY9;
	double simdt, lastt, simt;

	UINT anim_hatch;	// handle for aft hatch animation
	double hatch_proc;
	int MyID;

	int erasecount;
	OBJHANDLE hvessel, hV;
	VCHUDSPEC huds;
	VCMFDSPEC mfds_left;
	VCMFDSPEC mfds_right;
	VCMFDSPEC mfds_2;
	VCMFDSPEC mfds_3;

};

typedef struct {
	HINSTANCE hDLL;
	HFONT font[2];
	DWORD col[4];
	HBRUSH brush[4];
	HPEN pen[2];
} GDIParams;





/* basic vessel class setup */


bool MOONBUS::clbkLoadVC (int id)
{
//	int i;

//	ReleaseSurfaces();
//	InitVC (id);
	SetCameraDefaultDirection (_V(0,0,1)); // forward
//	oapiVCRegisterHUD (&huds);
	oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
	oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
	oapiVCRegisterMFD (2, &mfds_2);
	oapiVCRegisterMFD (3, &mfds_3);

//	oapiOpenMFD (MFD_HAL9000, 2);
//	oapiOpenMFD (MFD_HAL9000, 3);
	oapiSendMFDKey (2, OAPI_KEY_ESCAPE);
	oapiSendMFDKey (2, OAPI_KEY_F1);
	oapiSendMFDKey (2, OAPI_KEY_9);
	oapiSendMFDKey (3, OAPI_KEY_ESCAPE);
	oapiSendMFDKey (3, OAPI_KEY_F1);
	oapiSendMFDKey (3, OAPI_KEY_9);

		SURFHANDLE tex1 = oapiGetTextureHandle (hMoonbus, 6);

		oapiVCRegisterArea (AID_MFD1_LBUTTONS, _R(112, 214, 255, 224), PANEL_REDRAW_USER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED|PANEL_MOUSE_ONREPLAY, PANEL_MAP_BACKGROUND, tex1);
		oapiVCSetAreaClickmode_Quadrilateral (AID_MFD1_LBUTTONS, _V(0.864,0.398,3.775), _V(0.822,0.398,3.775), _V(0.822,0.290,3.654), _V(0.864,0.290,3.654));
		oapiVCRegisterArea (AID_MFD1_RBUTTONS, _R(112, 224, 255, 234), PANEL_REDRAW_USER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED|PANEL_MOUSE_ONREPLAY, PANEL_MAP_BACKGROUND, tex1);
		oapiVCSetAreaClickmode_Quadrilateral (AID_MFD1_RBUTTONS, _V(0.566,0.398,3.775), _V(0.514,0.398,3.775), _V(0.514,0.290,3.654), _V(0.566,0.290,3.654));
		oapiVCRegisterArea (AID_MFD2_LBUTTONS, _R(112, 234, 255, 244), PANEL_REDRAW_USER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED|PANEL_MOUSE_ONREPLAY, PANEL_MAP_BACKGROUND, tex1);
		oapiVCSetAreaClickmode_Quadrilateral (AID_MFD2_LBUTTONS, _V(0.324,0.356,3.719), _V(0.287,0.356,3.719), _V(0.287,0.255,3.617), _V(0.324,0.255,3.617));
		oapiVCRegisterArea (AID_MFD2_RBUTTONS, _R(112, 244, 255, 254), PANEL_REDRAW_USER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_LBPRESSED|PANEL_MOUSE_ONREPLAY, PANEL_MAP_BACKGROUND, tex1);
		oapiVCSetAreaClickmode_Quadrilateral (AID_MFD2_RBUTTONS, _V(0.170,0.356,3.719), _V(0.144,0.356,3.719), _V(0.144,0.255,3.617), _V(0.170,0.255,3.617));

		oapiVCRegisterArea (AID_MFD1_PWR, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
		oapiVCSetAreaClickmode_Spherical (AID_MFD1_PWR, _V(-0.1914,1.009,7.2775), 0.01);
		oapiVCRegisterArea (AID_MFD1_SEL, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
		oapiVCSetAreaClickmode_Spherical (AID_MFD1_SEL, _V(-0.0670,1.009,7.2775), 0.01);
		oapiVCRegisterArea (AID_MFD1_MNU, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
		oapiVCSetAreaClickmode_Spherical (AID_MFD1_MNU, _V(-0.0485,1.009,7.2775), 0.01);

		oapiVCRegisterArea (AID_MFD2_PWR, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
		oapiVCSetAreaClickmode_Spherical (AID_MFD2_PWR, _V(0.0483,1.009,7.2775), 0.01);
		oapiVCRegisterArea (AID_MFD2_SEL, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
		oapiVCSetAreaClickmode_Spherical (AID_MFD2_SEL, _V(0.1726,1.009,7.2775), 0.01);
		oapiVCRegisterArea (AID_MFD2_MNU, PANEL_REDRAW_NEVER, PANEL_MOUSE_LBDOWN|PANEL_MOUSE_ONREPLAY);
		oapiVCSetAreaClickmode_Spherical (AID_MFD2_MNU, _V(0.1913,1.009,7.2775), 0.01);



	return true;
}

bool MOONBUS::clbkVCMouseEvent (int id, int event, VECTOR3 &p)
{
	static int ctrl = 0, mode = 0;
	static double py = 0;

	// Process other controls
	switch (id) {
	case AID_MFD1_LBUTTONS:
	case AID_MFD1_RBUTTONS:
	case AID_MFD2_LBUTTONS:
	case AID_MFD2_RBUTTONS: {
		double dp;
		if (modf (p.y*23.0/4.0, &dp) < 0.75) {
			int mfd = (id <= AID_MFD1_RBUTTONS ? MFD_LEFT : MFD_RIGHT);
			int bt = (int)dp + (id == AID_MFD1_LBUTTONS || id == AID_MFD2_LBUTTONS ? 0 : 6);
			oapiProcessMFDButton (mfd, bt, event);
			return true;
		}
		} break;
	case AID_MFD1_PWR:
		oapiToggleMFD_on (MFD_LEFT);
		return true;
	case AID_MFD1_SEL:
		oapiSendMFDKey (MFD_LEFT, OAPI_KEY_F1);
		return true;
	case AID_MFD1_MNU:
		oapiSendMFDKey (MFD_LEFT, OAPI_KEY_GRAVE);
		return true;
	case AID_MFD2_PWR:
		oapiToggleMFD_on (MFD_RIGHT);
		return true;
	case AID_MFD2_SEL:
		oapiSendMFDKey (MFD_RIGHT, OAPI_KEY_F1);
		return true;
	case AID_MFD2_MNU:
		oapiSendMFDKey (MFD_RIGHT, OAPI_KEY_GRAVE);
		return true;
	}
	return false;
}

bool MOONBUS::clbkVCRedrawEvent (int id, int event, SURFHANDLE surf)
{
	int i;

	switch (id) {
	case AID_MFD1_LBUTTONS:
		RedrawPanel_MFDButton (surf, MFD_LEFT, 0);
		return true;
	case AID_MFD1_RBUTTONS:
		RedrawPanel_MFDButton (surf, MFD_LEFT, 1);
		return true;
	case AID_MFD2_LBUTTONS:
		RedrawPanel_MFDButton (surf, MFD_RIGHT, 0);
		return true;
	case AID_MFD2_RBUTTONS:
		RedrawPanel_MFDButton (surf, MFD_RIGHT, 1);
		return true;
	}
	return false;
}

void MOONBUS::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void MOONBUS::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void MOONBUS::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void MOONBUS::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void MOONBUS::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void MOONBUS::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int MOONBUS::clbkConsumeDirectKey(char *keystate )
{
	ConsumeKey( keystate );
	return 0;
}

int MOONBUS::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new MOONBUS (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (MOONBUS*)vessel;
}

