/**
*
* $Id: Titov V.cpp
* (c) 2005 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* Autopilot by rjcroy
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "OrionIII.h"


// 1. vertical lift component (wings and body)
// Borrowed from Mike Norman's Shuttle code

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5,  0.1, 0.8355,    0.72,   0.42, 0.24, 0.10, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,     0,  0.002,  0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	// change for drag increase at high Mach:
	// Subsonic Equivalents

	double Cl0, Cl1, Cd0, Cd1, Cd2;
	if(M>=5.0) {
		Cl0 = -0.187040;
		Cl1 = 1.675600;
		Cd0 = 0.068540;
		Cd1 = -0.21767;
		Cd2 = 1.3632;
		*cl = Cl0 + Cl1*aoa;
		*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
 	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
	if(M<1.25) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.03 + 0.3*oapiGetInducedDrag (*cl, 2.1, 0.92);
	}
	else if(M<5.0) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.04 + 0.6*oapiGetInducedDrag (*cl, 2.1, 0.82);
// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
	}
}

/*
void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, 0*RAD, 15*RAD,25*RAD,30*RAD,60*RAD,180*RAD};
	static const double CL[nabsc]  = {       0,      0,   -0.4,     0,    0.7,     1,   0.8,     0,      0};
	static const double CM[nabsc]  = {       0,      0,  0.014, 0.004,-0.0074,-0.094,-0.012,     0,      0};
	for (int i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = CM[i] + (CM[i+1]-CM[i]) * f;  // aoa-dependent moment coefficient
	double saoa = sin(aoa);
	double pd = 0.015 + 0.4*saoa*saoa;  // profile drag
	*cd = pd + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	// profile drag + (lift-)induced drag + transonic/supersonic wave (compressibility) drag
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}
*/

// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}


																					
void ORIONIII::InitialiseGuidance()
{
	

//	time=0.0;

	pitchinput = 0.0;
	rollinput = 0.0;
	engineignited = false;
	last_simt=0.0;
	ignition_time=0.0;
	first_guidance_pass = true;

	// rjcroy Guidance variables

	INCLINATION = 51.63 ;		// degrees

	guidance = false;			// guidance switch
	Alt_Flag = false;			// True if max launch altitude reached
	MECO_Cond = false;			// True if MECO conditions met
	First_Guidance_Pass = false;
	T_minus_10 = true;			// True at T minus 10 seconds.
	TZero = -100;				// Time Zero - Launch time
	PC_D = 0.9;					// Pitch control coefficient D for d_theta /d_t
	PC_K = 0.2;					//    "    "         "       K  "  theta
	YC_D = 0.5	;				// Yaw Control
	YC_K = 0.3;
	PC_D_FINAL = 0.9;			// Pitch control coefficients for final section.
	PC_K_FINAL = 0.6; 
	D_FINAL  = 0.8;				// Vertical velocity control coefficients
	K_FINAL = 0.8;
	ENG_THRST_2 = 0 ;		// Thrust rating of Stage 2 engine. (N)  
	i_last = 0;					// value of incliniation in previous iteration.
//	Planet_Name = new char[6];
//	strcpy( Planet_Name, "Earth");
	R_EARTH = 6371000;			// radius of the earth (m)
	U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
	Ob_EARTH = 23.45 * RAD ;	// Obliquity of the Earth (rad)
	APO_ALT = 150 ;				// Altitude of apoapsis
//	APO_ALT = 1705;

	//  inclination correction factor constant. (3580)
	ICF_K = 62.008 * INCLINATION*INCLINATION 
			- 6848.7*INCLINATION + 189807;	

	DisplayIncSP = false;		// toggle for displaying the inclination setpoint.

	NewIncSP = false ;			// True if a user has entered a new inclination SP
	PostMECO = false ;			// switch for cleaning up after MECO

	BadHeading = false;			// true if soyuz is at an azimuth not suitable for the G&CS



}

double ORIONIII::GetCPitch(double t)
{
	int i;
	if (t>met[N-1]) return cpitch[N-1];
	i=1;
	while (met[i]<t) i++;
	return cpitch[i-1]+(cpitch[i]-cpitch[i-1])/(met[i]-met[i-1])*(t-met[i-1]);
}

// rjcroy Lateral Guidance System ( called by Guidance_Routine() ).

void ORIONIII::Lateral_Guidance(VESSEL *vessel, double SimT, double SimT_Last)
{
	// Lateral Guidance System 
/*
		double hdg, diff, rate, amount, targetrate, TgtHdg;

		if (MET > 35.0) {
			targetrate = ((GetBank()*180/PI)-180)/-1.0;
			if (GetBank()<0) targetrate = -1.0*targetrate;

			if (MET > 55.0) {

				oapiGetHeading(GetHandle(),&hdg);
//				hdg = vst.arot.y * 180/PI;
				hdg *= 180/PI;
//				hdg += 180;

				GetRelativeVel (hEarth, Rel_Vel);
				GetEquPos( Longitude, Latitude, Radius );
				Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));

				oapiGetRelativeVel(hVessel, hEarth, &Rel_Vel);
				oapiGetRelativePos(hVessel, hEarth, &Rel_Pos);
				
				// Change ecliptic rel. vectors to equatorial vectors.
 				temp.x = Rel_Vel.x;
				temp.y = Rel_Vel.z;
				temp.z = Rel_Vel.y;
				veccpy( Rel_Vel, temp);
				temp.x = Rel_Pos.x;
				temp.y = Rel_Pos.z;
				temp.z = Rel_Pos.y;
				veccpy( Rel_Pos, temp);
				temp.x = Rel_Pos.x ;
				temp.y = cos(-Ob_EARTH)* Rel_Pos.y + sin(-Ob_EARTH)*Rel_Pos.z;
				temp.z =  -sin(-Ob_EARTH)* Rel_Pos.y + cos(-Ob_EARTH)*Rel_Pos.z;
				veccpy( Rel_Pos, temp);
				temp.x = Rel_Vel.x ;
				temp.y = cos(-Ob_EARTH)* Rel_Vel.y + sin(-Ob_EARTH)*Rel_Vel.z;
				temp.z =  -sin(-Ob_EARTH)* Rel_Vel.y + cos(-Ob_EARTH)*Rel_Vel.z;
				veccpy( Rel_Vel, temp);

				h_vector = crossp(Rel_Vel, Rel_Pos);
				h_abs = sqrt((h_vector.x*h_vector.x) + (h_vector.y*h_vector.y) + (h_vector.z*h_vector.z));
				i = 180 - acos(h_vector.z / h_abs) * DEG;
				i_dot = (i - i_last) / (MET - SimT_Last) ;
				i_last = i;
				ICF = Velocity / ICF_K;

				// Calculated error in inclination as output signal
				i_err = (i - INCLINATION) * (150 * ICF) + 250 * ICF * i_dot;
				if (i_err > ICF )
				{	
					i_err = ICF;
				}
				else if ( i_err < -ICF) 
				{
					i_err = -ICF;
				}
							
				double Phrad;
					//= (PadHeading-180)*PI/180;

				if ((Rel_Vel.z < 110.0) || (bgoingsouth)) {
					Phrad = PI/2 + (PI/2 - asin(cos(INCLINATION*PI/180)/cos(Latitude)));
					if ((PadHeading-180) > 180)
						Phrad = 3*PI/2 + (3*PI/2 - asin(cos(INCLINATION*PI/180)/cos(Latitude)));
					TgtHdg = Phrad*DEG;
					TgtHdg -= i_err;
					bgoingsouth = true;
				}
				else {
					Phrad =  asin(cos(INCLINATION*PI/180)/cos(Latitude));
					TgtHdg = Phrad*DEG;
					TgtHdg += i_err;
				}

				if ((TgtHdg > 90.0) || (PadHeading-180 >= 90.0))
					bgoingsouth = true;

				if ((Latitude*DEG >= i*0.98) && (fabs(i-INCLINATION)<0.02))
					bgoingsouth = true;

//				sprintf(oapiDebugString(),"Latitude %.2f  inc %.2f TGTINC %.2f going %1d",Latitude*DEG, i, INCLINATION, bgoingsouth);
//				sprintf(oapiDebugString(),"TgtHdg %.2f  inc %.2f TGTINC %.2f Phrad %.2f PadHdg %.2f tmpy %.2f",TgtHdg, i, INCLINATION, Phrad*DEG,PadHeading-180,Rel_Vel.z);

				diff = hdg - (TgtHdg);
//				if (ROLL < 90.0) diff = hdg - TgtHdg + 180 - ROLL;
//				diff = hdg - (PadHeading - 180);
				if( diff > 180.0 )
					diff -= 360.0;
				else if( diff < -180.0 )
					diff += 360.0;
				targetrate = diff/+1.0;
			}

			if( targetrate > 8.0 )
				targetrate = 8.0;
			else if( targetrate < -8.0 )
				targetrate = -8.0;


			rate = vst.vrot.y * 180/PI;
			amount = fabs (targetrate/8.0);
			if (MET > 0) amount = amount / (MET/10);

//		sprintf(oapiDebugString(),"Bank %.1f Heading %.1f TgtHdg %.1f diff %.1f Targetrate %.1f",GetBank()*180/PI,hdg,TgtHdg,diff, targetrate);

			if( fabs( rate-targetrate ) > 0.05 ) {
				if( rate < targetrate ) {
//					IncThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.05 );
//					SetThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.0 );
					yawinput = -1*amount;
				}
				else {
//					IncThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.05);
//					SetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.0 );
					yawinput = amount;
				}
			}
			else {
//				SetThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.0 );
//				SetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.0 );
				yawinput = 0.0;
			}
		}
*/
}

// rjcroy Guidance & Control System
void ORIONIII::Guidance_Routine(VESSEL *vessel, double SimT, double SimT_Last)
{

	hVessel = GetHandle();
	hEarth = GetSurfaceRef ();
//	hEarth = oapiGetGbodyByName ( Planet_Name );


	if (T_minus_10)
	{
		// Record launch time
		if (TZero == 0){
			TZero = SimT + 1;
		}

		if (MET > -5 && MET < -3) {
//			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0.3);
//			LaunchClamps();
		}
		else if (MET > -3 && MET < 0) {
//			oapiSetEngineLevel(hVessel, ENGINE_MAIN, (0.2*MET + 1));
//			LaunchClamps();
			INCLINATION = 180 - acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG;
		}
		else if (MET > 0)
		{
			T_minus_10 = false;
			First_Guidance_Pass = true;
			if (guidance)
				oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);


			GetStatus(vst);
	
			initialgamma = vst.arot.z * 180/PI;
			launchrotate =	PadHeading - initialheading - 180 + ROLL;

			if( launchrotate < -180.0 )
				launchrotate += 360.0;
			else if( launchrotate > 180.0 )
				launchrotate -= 360.0;

		}
		sprintf(oapiDebugString(),"T - %.1f", fabs(MET));
	}		

	else if (First_Guidance_Pass) 
	{
		
		// Start the main engines
		if (guidance)
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);

		// Unset the first pass flag
		First_Guidance_Pass = false;
	}

	else if (!Alt_Flag) 
	{
		if ((GetEngineLevel(ENGINE_MAIN) == 0.01) && (GetAltitude() > 10.0))
			if (guidance)
				oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);


		if (oapiGetFocusObject() == GetHandle())
			sprintf(oapiDebugString(),"T + %.1f", fabs(MET));

		if (initialgamma == 0.0)
			initialgamma = vst.arot.z * 180/PI;

		// Get correct pitch from table
		Pitch_SP=GetCPitch(MET);
		MJD_Time = oapiGetSimMJD();
		GetElements( elements, MJD_Time );
		Alt_Apoapsis = (elements.a * ( 1 + elements.e )/1000) - 6371; 
		if (Alt_Apoapsis > 100.0) {
			GetHorizonAirspeedVector(horiz);

			Pitch_SP = Pitch_SP/4.0;			// Flydown -- we've met our altitude goal
			double Pitch_zero = -0.02*horiz.y;

			if (ROLL < 90.0)
				if ((GetAOA()*DEG > -0.0))
					Pitch_SP += Pitch_zero+0.9*GetAOA()*DEG;
			else
				if ((GetAOA()*DEG < 0.0))
					Pitch_SP += Pitch_zero-0.9*GetAOA()*DEG;
		}

		if (Pitch_SP < 0.0)
			Pitch_SP = 0.0;

//		sprintf(oapiDebugString(),"Alt_Apoapsis %.1f Pitch_SP %.1f CPitch %.1f",Alt_Apoapsis, Pitch_SP, GetCPitch(MET));

		// Control to pitch setpoint
		Pitch=GetPitch(  );
		Pitch = Pitch * DEG;
		Pitch_Error = Pitch - Pitch_SP;
		if (ROLL < 90.0)
			Pitch_Error = Pitch_SP - Pitch;

		GetAngularVel( Angular_Velocity ); 
		Pitch_Error_Dot = Angular_Velocity.x * DEG; // get pitch change rate in degrees/s

		Pitch_Cntrl_Sig =  - PC_D * Pitch_Error_Dot + PC_K * Pitch_Error;
//		if ((MET > 10.0) &&(GetBank() < PI/2) && (GetBank() > -PI/2))
//			Pitch_Cntrl_Sig *= -1.0;

		if (Pitch_Cntrl_Sig > 1 )
		{
//			SetAttitudeRotLevel( 0, 1); // 0 is y-axis
			pitchinput =-1.0;
		}
		else if (Pitch_Cntrl_Sig < -1) 
		{
//			SetAttitudeRotLevel( 0, -1);
			pitchinput = +1.0;
		}
		else {
//			SetAttitudeRotLevel( 0 , Pitch_Cntrl_Sig);
			pitchinput = -Pitch_Cntrl_Sig;
		}
		
		Yaw_Error = 0.0;

		if( MET > 50.0) pitchinput *= 0.5;

//		if (Pitch > 45.0) pitchinput = +1.0;
		

		// Heading control
//		if (ROLL > 90.0) Lateral_Guidance(this, SimT, SimT_Last);


		// Roll control
		// Adapted from Hendo's Ariane V

//		if (MET < 2.0) SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);
		if (MET < 35.0) pitchinput = -2.0;
//			SetControlSurfaceLevel(AIRCTRL_ELEVATOR,1.0);

		SetControlSurfaceLevel(AIRCTRL_ELEVATOR,0.0);
		if (stage < 1) SetControlSurfaceLevel(AIRCTRL_ELEVATOR,-1.0*pitchinput);


		if( MET > 0.0) {
				
//		vst.version = 2;
//		vst.flag = 0;
		vessel->GetStatus(vst);


		target = initialgamma - launchrotate;

		if( target > 180.0 )
			target -= 360.0;
		else if( target < -180.0 )
			target += 360.0;
					
		diff = vst.arot.z*180/PI - target;

		if( diff > 180.0 )
			diff -= 360.0;
		else if( diff < -180.0 )
			diff += 360.0;

		if (MET > 30.0) diff = 0.0;
		if (ROLL < 90.0) diff = 0.0;

		diff = GetBank()*180/PI + target;
		if (ROLL < 90.0) diff = -1.0*GetBank()*180/PI;

		if ((ROLL < 90.0) && (MET < 110) && (MET > 5)) {
			oapiGetHeading(GetHandle(),&hdg);
			hdg = hdg*180/PI + 180.0;
			if (hdg < 0.0) hdg += 360.0;
			if (hdg > 450.0) hdg -= 360.0;
			diverge = (hdg-PadHeading)*1.7;
			diverge -= 0.8*GetBank()*180/PI;
			if (GetAltitude() < 150.0) {
				if (diverge > 30.0) diverge = 30.0;
				if (diverge < -30.0) diverge = -30.0;
			} else {
				if (diverge > 50.0) diverge = 50.0;
				if (diverge < -50.0) diverge = -50.0;
			}
			diff += diverge;
		}

		if (MET > 110) {
			diff = -1.0*GetBank()*180/PI;
			yawangle = DEG*GetSlipAngle();
			yawinput = yawangle / 15.0;
			if (ROLL > 90.0) yawinput *= -1.0;
		}
//		sprintf(oapiDebugString(),"hdg %.1f diverge %.1f PadHeading %.1f diff %.1f ",hdg,diverge,PadHeading,diff);

//		sprintf(oapiDebugString(),"initialhdg %.1f initialgamma %.1f launchrotate %.1f target %.1f diff %.1f TGT_ %.1f",initialheading,initialgamma, launchrotate, target,diff, PadHeading);

		Yaw_Error = diff;

		targetrate = diff/-1.0;
		if( targetrate > 8.0 )
			targetrate = 8.0;
		else if( targetrate < -8.0 )
			targetrate = -8.0;

//		if (GetBank() > 0.0)
//			Bank_Error = (GetBank()-PI);
//		else Bank_Error = (GetBank()+PI);
//		if (RollAtt == 1)
			Bank_Error = RAD*targetrate;

		rate = vst.vrot.z * 180/PI;
		amount = fabs (targetrate/8.0);
		if (MET > 0) amount = amount / (MET/10);

//		if (amount > 0.05)
			rollinput = diff/25.0;
//		else
//			rollinput = 0.0;
/*

		if( fabs( rate-targetrate ) > 0.05 ) {
			if( rate < targetrate ) {
//				IncThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.05 );
//				SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
				rollinput = -1*amount;
			}
			else {
//				IncThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.05);
//				SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
				rollinput = amount;
			}
		}
		else {
//			SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
//			SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
			rollinput = 0.0;
		}
*/		
//		if ((MET > 60.0) && (diff < 0.5))

//		if (MET > 60.0)
//			Lateral_Guidance(this, SimT, SimT_Last);
//		else
//			yawinput = 0.0;

//		GetAngularVel(AngularVel);

//		yawinput=(-20*AngularVel.y);		

		// END Hendo's roll control


		}

		Altitude=GetAltitude();
		a = ((APO_ALT * 1000 + R_EARTH) + (Altitude + R_EARTH)) / 2;
		e = ((APO_ALT * 1000 + R_EARTH) / a ) - 1;
		h = sqrt(U_EARTH * a * ( 1 - (e * e)));
		V_periapsis = h / (Altitude + R_EARTH);

		
		// Get current velocity
		GetRelativeVel (hEarth, Rel_Vel);
		Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));

		if ((MET > 180) && (Velocity > V_periapsis)) 
		{
			MECO_Cond = true;
			TimeMECO = SimT;
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0);
			Guidance(false);
			sprintf(oapiDebugString(), "");
			pitchinput = 0.0;
			yawinput = 0.0;
			rollinput = 0.0;
		}

		
	}

	else if (MECO_Cond && PostMECO) 
	{
		if ((SimT - TimeMECO) < 10) {
			sprintf(oapiDebugString(), "Guidance OFF.  Inclination (Equatorial): %.3f", i); 
		}	
		else {
			guidance = false;
			sprintf(oapiDebugString(), "");
		}
	}
}
// END Guidance_Routine()


void ORIONIII::GetInclinationSP()
{
	// Gets a new Inclination setting from the pilot
	bool AzInput (void *id, char *str, void *data);
	
//	oapiOpenInputBox ("Inclination Setpoint, e.g. 5163 = 51.63 deg (47 to 56)", IncInput, 0, 25);
	oapiOpenInputBox ("Enter launch azimuth, 0 to 90 (east) to 360", AzInput, 0, 20, (void*)this);

//	GetEquPos( Longitude, Latitude, Radius );
//	sprintf(oapiDebugString(), "Target inclination (Equatorial): %.3f", acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG); 
}


bool AzInput (void *id, char *str, void *data)
{
//	sscanf(str, "%4lf", &PadHeading);
//	PadHeading = PadHeading / 100.0;
//	PadHeading += 180.0;
/*	sscanf(str, "%4lf", &INCLINATION);
	if (INCLINATION > 4700 && INCLINATION < 5600 ) {
		INCLINATION = INCLINATION / 100;
		NewIncSP = true;
	}
*/
	return ((ORIONIII*)data)->IncInput (str);
}

bool ORIONIII::IncInput (char *rstr)
{
	sscanf(rstr, "%5lf", &PadHeading);
//	PadHeading = PadHeading / 100.0;
	PadHeading += 180.0;
	if (PadHeading > 450.0) PadHeading -= 360.0;
	d_heading = PadHeading;
	displaytime = 0.01;
	return true;
}

// END rjcroy Guidance System

ORIONIII::ORIONIII (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{

	AnimationsMade = false;
	strcat (OrionMesh,"2001/OrionBase");
	strcat (Or2BayMesh,"2001/Orion2USAF");
	strcat (Or3BayMesh,"2001/Orion3PanAm");

	tex = oapiRegisterExhaustTexture("2001/Exhaust_Orion");
	texRCS  = oapiRegisterExhaustTexture ("exhaust_atrcs");

	/* set the propellant handles to NULL so I can check if I need to 
	/* create them in the SetMainStage() function. */
	Orion2 = false;
	Orion3 = false;
	ph_booster=NULL; 
	ph_first=NULL; 
	ph_OrionIII=NULL;
	ph_Jets=NULL;
	bShieldJettisoned=false;
	bDeorbitFired=false;
	bThrustTerminated=false;
	bEjected=false;
	bEVA=false;
	bAttSet=false;
	bBoosterMade=false;
	bAutoProgrammed=false;
	bControlsMade=false;
	gear_status=DOOR_CLOSED;
	gear_proc=0.0;
	bgear_status=DOOR_CLOSED;
	bgear_proc=0.0;
	pldoors_status=DOOR_CLOSED;
	pldoors_proc=0.0;
	brakes_status=DOOR_CLOSED;
	brakes_proc=0.0;
	hatch_status=DOOR_CLOSED;
	hatch_proc=0.0;
	jets_status=DOOR_CLOSED;
	jets_proc=0.0;
	Vold = _V(0,0,0);
	BrakesOn = false;
	bAbort = false;
	AbortChecked = false;
	numengines = 2;
	dt = 0.0;
	simt = 0.0;
	oldsimt = -0.01;
	displaytime = 0.01;
	SimT_Last = 0.0;
	PadHeading = 270.0;
	d_heading = 270.0;
	last_simt=0.0;
	ignition_time=0.0;
	initialgamma = 0.0;
	pitchinput = 0.0;
	yawinput = 0.0;
	rollinput = 0.0;
	Orion3Add = 0.0;
	stage = 1;
	atchmass[0] = 0.0;
	atchmass[1] = 0.0;
	atchmass[2] = 0.0;
	atchmass[3] = 0.0;
	atchmass[4] = 0.0;

	//		     			   child        pos           dir       rot      handle
	Dockpos = CreateAttachment (false, _V(0,0,20.0),   _V(0,0,1), _V(0,1,0), "DOCK");
	//		     			   child       pos           dir       rot      handle
	Baypos = CreateAttachment (false, _V(0,0,0.0),   _V(0,0,1), _V(0,-1,0), "X");
//	sat_attach = CreateAttachment (false, OFS_SAT, _V(0,1,0), _V(0,0,1), "X");
	Nose_atch1 = CreateAttachment (false, _V(0,-3.8,15.7), _V(0,0.0045,1), _V(0,1,0), "NG");
	Nose_atch2 = CreateAttachment (true, _V(0,-3.8,15.7), _V(0,0.0045,1), _V(0,1,0), "NG");
	
	attach[0] = _V(0,-0.4,-1.05);
	attachdir[0] = _V(0,0,1);
	attachrot[0] = _V(0,-1,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[0] = CreateAttachment (false, attach[0],   attachdir[0], attachrot[0], "X");
	
	attach[1] = _V(0,-0.4,1.5);
	attachdir[1] = _V(0,0,1);
	attachrot[1] = _V(0,-1,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[1] = CreateAttachment (false, attach[1],   attachdir[1], attachrot[1], "X");

	attach[2] = _V(0,-0.4,4);
	attachdir[2] = _V(0,0,1);
	attachrot[2] = _V(0,-1,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[2] = CreateAttachment (false, attach[2],   attachdir[2], attachrot[2], "X");
	
	attach[3] = _V(0,-0.4,6.5);
	attachdir[3] = _V(0,0,1);
	attachrot[3] = _V(0,-1,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[3] = CreateAttachment (false, attach[3],   attachdir[3], attachrot[3], "X");

	attach[4] = _V(0,-1.35,4);
	attachdir[4] = _V(0,1,0);
	attachrot[4] = _V(0,0,1);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[4] = CreateAttachment (false, attach[4],   attachdir[4], attachrot[4], "X");

	MDD_atch = CreateAttachment (true, _V(0,0.5,0.0), _V(0,-1,0), _V(-1,0,0), "XS");

	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "ORIONIII" ) ) {
		Orion3 = true;
		Orion3Add = PAXMASS;
	}
	else if( strstr( classname, "ORIONIV" ) )
		stage = -3;
	else if( strstr( classname, "ORIONII" ) )
		Orion2 = true;
	else
		stage = 1;

/*  The stages of the Orion II/III:

  -3	Orion IV Booster Stage Only
  -2	Orion with Booster Stage
  -1	N/A
  0		Orion with Drop Tank
  1		Orion upper stage only
  2		Orion upper stage, big engines disabled (enables OMS engines)
  3		Orion upper stage, with jet engine

*/

	if (stage > -3) {
		Dock0 = CreateDock(_V(0,0,20.0),_V(0,0,1),_V(0,1,0));
		Dock1 = CreateDock(_V(0,-0.3,9.05),_V(0,0,-1),_V(0,1,0));
		Dock2 = CreateDock(_V( 1.42,0.54,10.3),_V( 1, 0,0),_V(0,1,0));
	}
	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
//		havesound=false;
//	else
//	{
		havesound=true;
		RequestLoadVesselWave3(MyID,PLAYGEAR,"Sound\\2001\\gear.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYBANG,"Sound\\2001\\stg2sep.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYJETT,"Sound\\2001\\fasep.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYJET,"Sound\\2001\\jet.wav",BOTHVIEW_FADED_FAR);
		RequestLoadVesselWave3(MyID,PLAYDOORS,"Sound\\2001\\Payload_doors.wav",DEFAULT);
//	}

	anim_gear = 0.0;
	anim_bgear = 0.0;
	anim_pldoors = 0.0;
	anim_brakes = 0.0;
	anim_hatch = 0.0;

// Make airfoils for the first time
		hwing = CreateAirfoil2 (LIFT_VERTICAL, _V(0,0,-0.4), VLiftCoeff, 9, 125, 4.35);
		htail = CreateAirfoil2 (LIFT_HORIZONTAL, _V(0,0,-8.5), HLiftCoeff, 3.5, 18, 2.6);


	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();   

/*
	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);
	retex = oapiRegisterReentryTexture("reentry");
	SetReentryTexture(retex,6e6,1,1);
*/
	MakeAnimations();

	Firstsweep = true;

}


ORIONIII::~ORIONIII()
{

}

void ORIONIII::MakeAnimations()
{

	AnimationsMade = true;


	hORION					= oapiLoadMeshGlobal (OrionMesh);
	hORIONTk				= oapiLoadMeshGlobal ("2001/OrionTk");
	hORIONIV				= oapiLoadMeshGlobal ("2001/Orion4");
	hORIONII				= oapiLoadMeshGlobal (Or2BayMesh);
	hORIONIII				= oapiLoadMeshGlobal (Or3BayMesh);
//	hORIONIIITk				= oapiLoadMeshGlobal ("2001/OrionIIITank");
//	hSeat					= oapiLoadMeshGlobal ("gem-ejseat");
//	hSeat2					= oapiLoadMeshGlobal ("gem-ejseat");
//	hAstro					= oapiLoadMeshGlobal ("gem-eva1");
//	hAstro2					= oapiLoadMeshGlobal ("gem-eva1");
	hDummy					= oapiLoadMeshGlobal ("2001/dummy");
	hDummy2					= oapiLoadMeshGlobal ("2001/dummy2");


	anim_gear = CreateAnimation (0);
	anim_bgear = CreateAnimation (0);
	anim_pldoors = CreateAnimation(0);
	anim_hatch = CreateAnimation(0);
	anim_jets = CreateAnimation(0);
	anim_lelevator = CreateAnimation (0.5);
	anim_relevator = CreateAnimation (0.5);
	anim_lelevatortrim = CreateAnimation (0.5);
	anim_relevatortrim = CreateAnimation (0.5);
	anim_raileron = CreateAnimation (0.5);
	anim_laileron = CreateAnimation (0.5);
//	anim_lrudder = CreateAnimation (0.5);
//	anim_rrudder = CreateAnimation (0.5);
	anim_brakes = CreateAnimation (0);

	if (stage < 0) MakeBoosterAnimations();
	if (stage == -3) return;

	static UINT LGearDrGrp[1] = {20};
	static MGROUP_ROTATE LGearDr (0, LGearDrGrp, 1,
		_V(-1.1,-1.85,-2.6), _V(0,0,1), (float)( 105*RAD));
	static UINT LGearGrp[2] = { 3, 29};
	static MGROUP_ROTATE LGear (0, LGearGrp, 2,
		_V(-3.35,-1.7,-2.83), _V(0,0,1), (float)(-105*RAD));

	static UINT RGearDrGrp[1] = {19};
	static MGROUP_ROTATE RGearDr (0, RGearDrGrp, 1,
		_V( 1.1,-1.85,-2.6), _V(0,0,1), (float)(-105*RAD));
	static UINT RGearGrp[12] = { 2, 28};
	static MGROUP_ROTATE RGear (0, RGearGrp, 2,
		_V( 3.35,-1.7,-2.83), _V(0,0,1), (float)( 105*RAD));

	static UINT NoseGearDoorGrp[1] = {10};
	static MGROUP_ROTATE NoseGearDoor (0, NoseGearDoorGrp, 1,
		_V( 0.262,-1.8,16.17), _V(0,0,1), (float)( 110*RAD));
	static UINT NoseGearGrp[2] = {1, 23};
	static MGROUP_ROTATE NoseGear (0, NoseGearGrp, 2,
		_V(0.0,-1.415,14.72), _V(1,0,0), (float)( 65*RAD));


// Known Groups:

// Orion II Bay:
	// 0 Sides
	// 1 L Bay Dr
	// 2 R Bay Dr
	// 3 LOX Tank Cover
	// 4 Logo sides
	// 5 Ribs
	// 6 Interior


// 0 Aft Cowl
// 1 Nose Tires
// 2 R Tires
// 3 L Tires
// 4 Wing LE's
// 5 Main Gear Well
// 6 Greeblie Ring
// 7 Nozzles and Glare Shield
// 8 Little Exhausts
// 9 Fore Hull Sides
// 10 Nose Gear Door
// 11 Fore Hull Top
// 12 Hatch doorway
// 13 Inner Hatch
// 14 Outer Hatch
// 15 Interior Hatch
// 16 Airlock
// 17 R Skid
// 18 L Skid
// 19 R Gear Door
// 20 L Gear Door
// 21 Nozzle hump side
// 22 Nose Gear Box
// 23 Nose Gear Strut
// 24 LOX Tank
// 25 Aft Bulkhead
// 26 Bay Floor
// 27 Cockpit Interior
// 28 R Gear
// 29 L Gear
// 30 Nozzle Hump Top
// 31 L Ail
// 32 L Wing
// 33 R Ail
// 34 R Wing
// 35 HoleFill
// 36 Cockpit Window


	if (Orion2) {
		static UINT LPLDoorGrp[1] = {1};
		static MGROUP_ROTATE LPLDoor (1, LPLDoorGrp, 1,
			_V(-1.533,0.557,3.548), _V( 0.030,-0.045,1), (float)( 170*RAD));
		static UINT RPLDoorGrp[1] = {2};
		static MGROUP_ROTATE RPLDoor (1, RPLDoorGrp, 1,
			_V( 1.533,0.557,3.548), _V(-0.030,-0.045,1), (float)(-170*RAD));
		AddAnimationComponent (anim_pldoors, 0.1, 0.9, &LPLDoor);
		AddAnimationComponent (anim_pldoors, 0.1, 0.9, &RPLDoor);

	}
		static UINT HatchGrp[1] = {14};
		static MGROUP_ROTATE Hatch (0, HatchGrp, 1,
			_V( 1.4,-0.7,10.75), _V( 0.0,1.0,0.0), (float)(-178*RAD));

	AddAnimationComponent (anim_gear, 0.1, 0.25, &LGearDr);
	AddAnimationComponent (anim_gear, 0.2, 0.9, &LGear);
	AddAnimationComponent (anim_gear, 0.1, 0.25, &RGearDr);
	AddAnimationComponent (anim_gear, 0.2, 0.9, &RGear);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &NoseGear);
	AddAnimationComponent (anim_gear, 0.1, 0.4, &NoseGearDoor);

	AddAnimationComponent (anim_hatch, 0.1, 0.9, &Hatch);

//	AddAnimationComponent (anim_wings, 0.1, 0.9, &LWing);
//	AddAnimationComponent (anim_wings, 0.1, 0.9, &RWing);

	// ***** Elevator animation *****
	static UINT LElevatorGrp[1] = {31};
	static MGROUP_ROTATE LElevator (0, LElevatorGrp, 1,
		_V(-5.0,-1.625,-6.55), _V(-0.866,0.0,0.2), (float)(-25*RAD));
	AddAnimationComponent (anim_lelevator, 0, 1, &LElevator);
	static UINT RElevatorGrp[1] = {33};
	static MGROUP_ROTATE RElevator (0, RElevatorGrp, 1,
		_V( 5.0,-1.625,-6.55), _V( 0.866,0.0,0.2), (float)( 25*RAD));
	AddAnimationComponent (anim_relevator, 0, 1, &RElevator);

	// ***** Elevator trim animation *****
	static UINT LElevatorTrimGrp[1] = {31};
	static MGROUP_ROTATE LElevatorTrim (0, LElevatorTrimGrp, 1,
		_V(-5.0,-1.625,-6.55), _V(-0.866,0.0,0.2), (float)(-10*RAD));
	AddAnimationComponent (anim_lelevatortrim, 0, 1, &LElevatorTrim);
	static UINT RElevatorTrimGrp[1] = {33};
	static MGROUP_ROTATE RElevatorTrim (0, RElevatorTrimGrp, 1,
		_V( 5.0,-1.625,-6.55), _V( 0.866,0.0,0.2), (float)( 10*RAD));
	AddAnimationComponent (anim_relevatortrim, 0, 1, &RElevatorTrim);
/*
	// ***** Rudder animation *****
	static UINT RRudderGrp[1] = {9};
	static MGROUP_ROTATE RRudder (0, RRudderGrp, 1,
		_V( 9.19,2.0,-10.315), _V( 0.25,0.51,-0.36), (float)(-30*RAD));
	static UINT LRudderGrp[1] = {16};
	static MGROUP_ROTATE LRudder (0, LRudderGrp, 1,
		_V(-9.19,2.0,-10.315), _V(-0.25,0.51,-0.36), (float)(-30*RAD));
	AddAnimationComponent (anim_rrudder, 0, 1, &RRudder, wing2);
	AddAnimationComponent (anim_lrudder, 0, 1, &LRudder, wing1);
*/
	// ***** Aileron animation *****

	static UINT RAileronGrp[1] = {31};
	static MGROUP_ROTATE RAileron (0, RAileronGrp, 1,
		_V(-5.0,-1.625,-6.55), _V(-0.866,0.0,0.2), (float)( 30*RAD));
	AddAnimationComponent (anim_raileron, 0, 1, &RAileron);

	static UINT LAileronGrp[1] = {33};
	static MGROUP_ROTATE LAileron (0, LAileronGrp, 1,
		_V( 5.0,-1.625,-6.55), _V( 0.866,0.0,0.2), (float)( 30*RAD));
	AddAnimationComponent (anim_laileron, 0, 1, &LAileron);


		// ***** Speedbrake animation *****
/*
//	static UINT RRudderGrp[1] = {12};
	static MGROUP_ROTATE RRudder2 (0, RRudderGrp, 1,
		_V( 9.19,2.0,-9.125), _V( 0.25,0.45,-0.4), (float)(-20*RAD));
//	static UINT LRudderGrp[1] = {13};
	static MGROUP_ROTATE LRudder2 (0, LRudderGrp, 1,
		_V(-9.19,2.0,-9.125), _V(-0.25,0.45,-0.4), (float)( 20*RAD));
*/

//	AddAnimationComponent (anim_brakes, 0, 1, &RRudder2, wing2);
//	AddAnimationComponent (anim_brakes, 0, 1, &LRudder2, wing1);

}


void ORIONIII::MakeBoosterAnimations()
{

	bBoosterMade = true;

	static UINT LGearDoorGrp[1] = {19};
	static MGROUP_ROTATE LGearDoor (2, LGearDoorGrp, 1,
		_V(-4.35,-6.775, 2.5), _V(0,0,1), (float)(-120*RAD));
	static UINT LGearGrp[2] = {28,2};
	static MGROUP_TRANSLATE LGear (2, LGearGrp, 2,
		_V(0,-1.2,0));


	static UINT RGearDoorGrp[1] = {22};
	static MGROUP_ROTATE RGearDoor (2, RGearDoorGrp, 1,
		_V( 4.35,-6.775, 2.5), _V(0,0,1), (float)( 120*RAD));
	static UINT RGearGrp[2] = {30,3};
	static MGROUP_TRANSLATE RGear (2, RGearGrp, 2,
		_V(0,-1.2,0));

	static UINT NoseGearDoorGrp[1] = {11};
	static MGROUP_ROTATE NoseGearDoor (2, NoseGearDoorGrp, 1,
		_V( 0.425,-7.2,36.0), _V(0,0,1), (float)( 110*RAD));
	static UINT NoseGearGrp[2] = {1, 25};
	static MGROUP_ROTATE NoseGear (2, NoseGearGrp, 2,
		_V(0.0,-6.525,34.25), _V(1,0,0), (float)( 65*RAD));


// Known Groups:

// 0 Aft Cowl
// 1 Nose Tires
// 2 L Tires
// 3 R Tires
// 4 LE's
// 5 Rt Nozzles
// 6 Greeblie Ring
// 7 Glare Shield
// 8 Lt Nozzles
// 9 Mid Hull Sides 
// 10 Fore Hull Sides
// 11 Nose Gr Dr
// 12 Fore Hull Top
// 13 Mid Hull Top
// 14 Hatch
// 15 Logo
// 16 Rib Arch
// 17 Ribbed Atch Pt
// 18 Rt Cowl Side
// 19 Lt Gear Door
// 20 Lt Gear Pod
// 21 Rt Gear Pod
// 22 Rt Gear Door
// 23 Lt Cowl Side
// 24 Nose Gear Box
// 25 Nose Gear Strut
// 26 Cockpit Interior
// 27 Lt Struts
// 28 Lt Truck
// 29 Rt Struts
// 30 Rt Truck
// 31 Rt Nozzle Top
// 32 Lt Nozzle Top
// 33 Rt Ail
// 34 Rt Wing
// 35 Lt Ail
// 36 Lt Wing
// 37 Cockpit Window


	Gear11 = AddAnimationComponent (anim_bgear, 0.05, 0.6, &LGearDoor);
	Gear12 = AddAnimationComponent (anim_bgear, 0.4, 0.9, &LGear);
	Gear14 = AddAnimationComponent (anim_bgear, 0.05, 0.6, &RGearDoor);
	Gear15 = AddAnimationComponent (anim_bgear, 0.4, 0.9, &RGear);
	Gear17 = AddAnimationComponent (anim_bgear, 0.4, 0.9, &NoseGear);
	Gear18 = AddAnimationComponent (anim_bgear, 0.1, 0.4, &NoseGearDoor);

	// ***** Elevator animation *****
	static UINT LElevatorGrp[1] = {35};
	static MGROUP_ROTATE LElevator (2, LElevatorGrp, 1,
		_V(-9.0,-1.9,-5.95), _V(-0.866,0.0,0.2), (float)(-25*RAD));
	Elev11 = AddAnimationComponent (anim_lelevator, 0, 1, &LElevator);
	static UINT RElevatorGrp[1] = {33};
	static MGROUP_ROTATE RElevator (2, RElevatorGrp, 1,
		_V( 9.0,-1.9,-5.95), _V( 0.866,0.0,0.2), (float)( 25*RAD));
	Elev12 = AddAnimationComponent (anim_relevator, 0, 1, &RElevator);

	// ***** Elevator trim animation *****
	static UINT LElevatorTrimGrp[1] = {35};
	static MGROUP_ROTATE LElevatorTrim (2, LElevatorTrimGrp, 1,
		_V(-9.0,-1.9,-5.95), _V(-0.866,0.0,0.2), (float)(-10*RAD));
	Elev13 = AddAnimationComponent (anim_lelevatortrim, 0, 1, &LElevatorTrim);
	static UINT RElevatorTrimGrp[1] = {33};
	static MGROUP_ROTATE RElevatorTrim (2, RElevatorTrimGrp, 1,
		_V( 9.0,-1.9,-5.95), _V( 0.866,0.0,0.2), (float)( 10*RAD));
	Elev14 = AddAnimationComponent (anim_relevatortrim, 0, 1, &RElevatorTrim);
/*
	// ***** Rudder animation *****
	static UINT RRudderGrp[1] = {5};
	static MGROUP_ROTATE RRudder (2, RRudderGrp, 1,
		_V( 15.0,0.477,-19.542), _V( 0.25,0.51,-0.36), (float)(-30*RAD));
	static UINT LRudderGrp[1] = {12};
	static MGROUP_ROTATE LRudder (2, LRudderGrp, 1,
		_V(-15.0,0.477,-19.542), _V(-0.25,0.51,-0.36), (float)(-30*RAD));
	Rudd11 = AddAnimationComponent (anim_rrudder, 0, 1, &RRudder, wing12);
	Rudd12 = AddAnimationComponent (anim_lrudder, 0, 1, &LRudder, wing11);
*/
	// ***** Aileron animation *****

	static UINT RAileronGrp[1] = {35};
	static MGROUP_ROTATE RAileron (2, RAileronGrp, 1,
		_V(-9.0,-1.9,-5.95), _V(-0.866,0.0,0.2), (float)( 30*RAD));
	Rudd13 = AddAnimationComponent (anim_raileron, 0, 1, &RAileron);

	static UINT LAileronGrp[1] = {33};
	static MGROUP_ROTATE LAileron (2, LAileronGrp, 1,
		_V( 9.0,-1.9,-5.95), _V( 0.866,0.0,0.2), (float)( 30*RAD));
	Rudd14 = AddAnimationComponent (anim_laileron, 0, 1, &LAileron);


}

void ORIONIII::DelBoosterAnimations()
{

	bBoosterMade = false;

	DelAnimationComponent (anim_bgear,Gear11);
	DelAnimationComponent (anim_bgear,Gear12);
	DelAnimationComponent (anim_bgear,Gear14);
	DelAnimationComponent (anim_bgear,Gear15);
	DelAnimationComponent (anim_bgear,Gear17);
	DelAnimationComponent (anim_bgear,Gear18);
	DelAnimationComponent (anim_lelevator,Elev11);
	DelAnimationComponent (anim_relevator,Elev12);
	DelAnimationComponent (anim_lelevatortrim,Elev13);
	DelAnimationComponent (anim_relevatortrim,Elev14);
	DelAnimationComponent (anim_raileron,Rudd13);
	DelAnimationComponent (anim_laileron,Rudd14);

}

void ORIONIII::MakeAirfoils()
{
//	ClearAirfoilDefinitions();
	ClearVariableDragElements();

	if (stage > 0) {

		CreateVariableDragElement (&brakes_proc, 0.8, _V(0, 0, 0));
		CreateVariableDragElement (&gear_proc, 0.6, _V(0, -1, 0));

// EditAirfoil (AIRFOILHANDLE hAirfoil, DWORD flag, const VECTOR3& ref, AirfoilCoeffFunc cf, double c, double S, double A) const
	//                  alignment      position      func    chord S   AR   
		EditAirfoil (hwing, 0xFF, _V(0,0,-0.4), VLiftCoeff, 9, 125, 4.35);
//		CreateAirfoil (LIFT_VERTICAL, _V(0,0,-0.4), VLiftCoeff, 9, 125, 4.35);
		// wing and body lift+drag components

		EditAirfoil (htail,0xFF, _V(0,0,-8.5), HLiftCoeff, 3.5, 18, 2.6);
		// vertical stabiliser and body lift and drag components

	}
	else if (stage > -1) {

		CreateVariableDragElement (&brakes_proc, 0.8, _V(0, 0, 0));
		CreateVariableDragElement (&gear_proc, 0.6, _V(0, -1, 0));

	//                  alignment      position      func    chord S   AR   
		EditAirfoil (hwing, 0xFF, _V(0,0,-0.4), VLiftCoeff, 9, 125, 4.35);

		// wing and body lift+drag components

		EditAirfoil (htail, 0xFF, _V(0,0,-8.5), HLiftCoeff, 3.5, 18, 2.6);
		// vertical stabiliser and body lift and drag components

	}
	else {

		CreateVariableDragElement (&brakes_proc, 1.6, _V(0, 0, 0));
		CreateVariableDragElement (&gear_proc, 0.6, _V(0, -1, 0));


	//                     alignment      position       func    chord S   AR   
		EditAirfoil (hwing, 0xFF, _V(0,0,-0.3), VLiftCoeff, 18, 225, 3.35);
		// wing and body lift+drag components

		EditAirfoil (htail, 0xFF, _V(0,0,-14.5), HLiftCoeff, 8, 48, 2.6);
		// vertical stabiliser and body lift and drag components


	}
}


void ORIONIII::MakeControlSurfaces()
{

//  Aargh!  Can't delete control surfaces
//  Need to make sure we declare once and only once,
//  but that a booster stage alone doesn't get animations for the upper stage!

	if (bControlsMade) return;
	bControlsMade = true;


	CreateControlSurface (AIRCTRL_ELEVATOR,     14.5,  0.4, _V(  4.2,0,-11.67), AIRCTRL_AXIS_XPOS, anim_lelevator);
	CreateControlSurface (AIRCTRL_ELEVATOR,     14.5,  0.4, _V( -4.2,0,-11.67), AIRCTRL_AXIS_XPOS, anim_relevator);
	CreateControlSurface (AIRCTRL_AILERON,      6.3, -0.4, _V(-9.5,0,-12.75), AIRCTRL_AXIS_XPOS, anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      6.3, -0.4, _V( 9.5,0,-12.75), AIRCTRL_AXIS_XNEG, anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 6.1,  0.5, _V(  4.2,0,-11.67), AIRCTRL_AXIS_XPOS, anim_lelevatortrim);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 6.1,  0.5, _V( -4.2,0,-11.67), AIRCTRL_AXIS_XPOS, anim_relevatortrim);

}



void ORIONIII::BoosterSwitchToJets ()
{


	PARTICLESTREAMSPEC contrail = 
	{
		0, 0.35, 30, 325.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
//		0, 1.25, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};

	SoundOptionOnOff3(MyID,PLAYMAINTHRUST,false);

//	if (GetPropellantMass(ph_Jets) == 0.0)  
		SetPropellantMass(ph_Jets,BJET_PROP_MASS); 

	SetDefaultPropellantResource (ph_Jets); 

	ClearThrusterDefinitions();
	
	th_first[0] = CreateThruster ( _V( 4.95,0.0,-4.15), _V(0, 0,1), BJET_THRUST, ph_Jets, ISP_BJET_VAC, ISP_BJET_VAC);
	th_first[2] = CreateThruster ( _V( 0.00,0.0,-4.15), _V(0, 0,1), BJET_THRUST, ph_Jets, ISP_BJET_VAC, ISP_BJET_VAC);
	th_first[1] = CreateThruster ( _V(-4.95,0.0,-4.15), _V(0, 0,1), BJET_THRUST, ph_Jets, ISP_BJET_VAC, ISP_BJET_VAC);

	numengines = 3;
	thg_first = CreateThrusterGroup (th_first, 3, THGROUP_MAIN);
//	AddExhaust (th_first[0], 3.0, 0.3, tex);

	AddExhaustStream (th_first[0], _V( 5.6,-2.15,-4.15), &contrail);
	AddExhaustStream (th_first[0], _V(-5.6,-2.15,-4.15), &contrail);
	AddExhaustStream (th_first[0], _V( 7.65,-2.15,-4.15), &contrail);
	AddExhaustStream (th_first[0], _V(-7.65,-2.15,-4.15), &contrail);
	AddExhaustStream (th_first[0], _V( 9.7,-2.15,-4.15), &contrail);
	AddExhaustStream (th_first[0], _V(-9.7,-2.15,-4.15), &contrail);
	AddExhaustStream (th_first[0], _V( 11.7,-2.15,-4.15), &contrail);
	AddExhaustStream (th_first[0], _V(-11.7,-2.15,-4.15), &contrail);
	AddExhaustStream (th_first[0], _V( 13.5,-2.15,-4.15), &contrail);
	AddExhaustStream (th_first[0], _V(-13.5,-2.15,-4.15), &contrail);

}
void ORIONIII::UpdateAnimations()
{	

		// animate landing gear
		if (gear_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * GEAR_OPERATING_SPEED;
			if (gear_status == DOOR_CLOSING) {
				if (gear_proc > 0.0)
					gear_proc = max (0.0, gear_proc-da);
				else {
					gear_status = DOOR_CLOSED;
					StopVesselWave3(MyID,PLAYGEAR);
				}
			} else  { // door opening
				if (gear_proc < 1.0)
					gear_proc = min (1.0, gear_proc+da);
				else {
					gear_status = DOOR_OPEN;
					StopVesselWave3(MyID,PLAYGEAR);
				}
			}
			SetAnimation (anim_gear, gear_proc);
		}

		// animate landing bgear
		if (bgear_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * GEAR_OPERATING_SPEED;
			if (bgear_status == DOOR_CLOSING) {
				if (bgear_proc > 0.0)
					bgear_proc = max (0.0, bgear_proc-da);
				else {
					bgear_status = DOOR_CLOSED;
					StopVesselWave3(MyID,PLAYGEAR);
				}
			} else  { // door opening
				if (bgear_proc < 1.0)
					bgear_proc = min (1.0, bgear_proc+da);
				else {
					bgear_status = DOOR_OPEN;
					StopVesselWave3(MyID,PLAYGEAR);
				}
			}
			SetAnimation (anim_bgear, bgear_proc);
		}

		// animate payload bay doors
		if (pldoors_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * PLDOORS_OPERATING_SPEED;
			if (pldoors_status == DOOR_CLOSING) {
				if (pldoors_proc > 0.0)
					pldoors_proc = max (0.0, pldoors_proc-da);
				else {
					pldoors_status = DOOR_CLOSED;
					StopVesselWave3(MyID,PLAYDOORS);
				}
			} else  { // door opening
				if (pldoors_proc < 1.0)
					pldoors_proc = min (1.0, pldoors_proc+da);
				else {
					pldoors_status = DOOR_OPEN;
					StopVesselWave3(MyID,PLAYDOORS);
				}
			}
			SetAnimation (anim_pldoors, pldoors_proc);
		}

		// animate speedbrakes
		if (brakes_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * BRAKES_OPERATING_SPEED;
			if (brakes_status == DOOR_CLOSING) {
				if (brakes_proc > 0.0)
					brakes_proc = max (0.0, brakes_proc-da);
				else {
					brakes_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (brakes_proc < 1.0)
					brakes_proc = min (1.0, brakes_proc+da);
				else {
					brakes_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_brakes, brakes_proc);
		}

		// animate top hatch
		if (!bEjected)
			if (hatch_status >= DOOR_CLOSING) {
				double da = oapiGetSimStep() * HATCH_OPERATING_SPEED;
				if (hatch_status == DOOR_CLOSING) {
					if (hatch_proc > 0.0)
						hatch_proc = max (0.0, hatch_proc-da);
					else {
						hatch_status = DOOR_CLOSED;
					}
				} else  { // door opening
					if (hatch_proc < 1.0)
						hatch_proc = min (1.0, hatch_proc+da);
					else {
						hatch_status = DOOR_OPEN;
					}
				}
				SetAnimation (anim_hatch, hatch_proc);
			}

		// animate jet doors
		if (jets_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * JETS_OPERATING_SPEED;
			if (jets_status == DOOR_CLOSING) {
				if (jets_proc > 0.0)
					jets_proc = max (0.0, jets_proc-da);
				else {
					jets_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (jets_proc < 1.0)
					jets_proc = min (1.0, jets_proc+da);
				else {
					jets_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_jets, jets_proc);
		}


}

void ORIONIII::ActivateLandingGear (DoorStatus action)
{
	gear_status = action;
}

void ORIONIII::RevertLandingGear ()
{
	ActivateLandingGear (gear_status == DOOR_CLOSED || gear_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void ORIONIII::ActivateBoosterGear (DoorStatus action)
{
	bgear_status = action;
}

void ORIONIII::RevertBoosterGear ()
{
	ActivateBoosterGear (bgear_status == DOOR_CLOSED || bgear_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void ORIONIII::ActivatePayloadBayDoors (DoorStatus action)
{
	pldoors_status = action;
}

void ORIONIII::RevertPayloadBayDoors ()
{
	ActivatePayloadBayDoors (pldoors_status == DOOR_CLOSED || pldoors_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void ORIONIII::ActivateBrakes (DoorStatus action)
{
	brakes_status = action;
}

void ORIONIII::RevertBrakes ()
{
	ActivateBrakes (brakes_status == DOOR_CLOSED || brakes_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void ORIONIII::ActivateHatch (DoorStatus action)
{
	hatch_status = action;
}

void ORIONIII::RevertHatch ()
{
	ActivateHatch (hatch_status == DOOR_CLOSED || hatch_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}


void ORIONIII::ActivateJets (DoorStatus action)
{
	jets_status = action;
}

void ORIONIII::RevertJets ()
{
	ActivateJets (jets_status == DOOR_CLOSED || jets_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}


void ORIONIII::SetGearParameters (double state)
{


	double geary = (-1*(gear_proc-1.0)*1.4);
	if (stage < 0)
		geary = (-1*(bgear_proc-1.0)*1.4);

	if ((Firstsweep) || (gear_status == DOOR_OPENING) || (gear_status == DOOR_CLOSING)) {
		if (stage < 0)
			SetTouchdownPoints (_V(0,-9.6+geary*1.5,35.0), _V(-9.15,-8.85+geary*1.3,-1.5), _V(9.15,-8.85+geary*1.3,-1.5));
		else if (stage==0)
			SetTouchdownPoints (_V(0,-6.5+geary,16.0), _V(-9.15,-7.6+geary*1.3,-4.0), _V(9.15,-7.6+geary*1.3,-4.0));
		else
			SetTouchdownPoints (_V(0,-3.8+geary,15.7), _V(-9.15,-3.71+geary,-4.1), _V(9.15,-3.71+geary,-4.1));
	}
	if (stage < 0)
		if ((!BrakesOn) && (bgear_proc == 1.0)) {
			SetSurfaceFrictionCoeff (0.025, 0.4);
		} else {
			SetSurfaceFrictionCoeff (0.3, 0.4);		// Brakes on!
		}
	else if (stage > -1)
		if ((!BrakesOn) && (gear_proc == 1.0)) {
			SetSurfaceFrictionCoeff (0.025, 0.4);
		} else {
			SetSurfaceFrictionCoeff (0.3, 0.4);		// Brakes on!
		}
	
	double steerforce;
	double steerangle;

	steerforce = 4.0;
	if (GetAirspeed() < 40.0)
		steerforce *=5.0;
	steerforce = steerforce*GetMass();
	if ((GetAirspeed ()-3) < 5.0) steerforce*=((GetAirspeed()-3)/5); // from 1* to 0*
	if (GetAirspeed() < 3.0) steerforce = 0.0;

	steerangle = GetControlSurfaceLevel (AIRCTRL_RUDDER);
	steerforce = steerforce * steerangle;
	if ((stage < 0) && (GetAltitude() > 10.5)) steerforce = 0.0;
	if ((stage > -1) && (GetAltitude() > 4.0)) steerforce = 0.0;

	if ((gear_status == DOOR_OPEN) && (fabs(steerforce) > 0.05)) {
		AddForce (_V(steerforce, 0, 0), _V(0, 0, 12.0));
		AddForce (_V(-steerforce, 0, 0), _V(0, 0, -12.0));
	}
	
}


void ORIONIII::SeparateEVA ()
{
	VESSELSTATUS vs;
	GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	vs.status=0;
	VECTOR3 ofs = EVA_OFS;
	ofs.y += hatch_proc;
	Local2Rel (ofs, vs.rpos);
	char name[256], name2[256];
//	strcpy (name, GetName()); strcat (name, "-PLT");


	UINT i;
	for (i=1;i<8;i++) {
			strcpy (name2, GetName()); 
			sprintf(name, "-EV%d", i);
			strcat (name2, name);
			hvessel=oapiGetVesselByName(name2);
			if (hvessel == 0) break;
	}		
	if ( i < 7) 
	{
		oapiCreateVessel (name2, "2001/Astro2001", vs);
		hvessel=oapiGetVesselByName(name2);
		oapiSetFocusObject(hvessel);
	}



//	VESSEL::Create (name, "gemini_leva", vs);
}

void ORIONIII::TimeStep(  double simt )
{
	int i;
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */
	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
			SetMainStage();
	}

	if (!bAutoProgrammed)
		ProgramAuto();
	if (!bControlsMade)
		MakeControlSurfaces();
	
	if ((guidance) && (MET > 0))
		SetThrusterGroupLevel (THGROUP_MAIN,1.0);

	dt = simt - oldsimt;
	oldsimt = simt;
	if ((displaytime > 0) && (oapiGetFocusObject() == GetHandle())) displaytime += dt;

	if ((!guidance) && (displaytime < 5) && (stage < 1) && (stage > -3) && (oapiGetFocusObject() == GetHandle()))
	{
		GetEquPos( Longitude, Latitude, Radius );
		sprintf(oapiDebugString(), "Target inclination (Equatorial): %.3f", (180 - acos(sin(d_heading*PI/180)*cos(Latitude))*DEG));
	}
//	else if (oapiGetFocusObject() == GetHandle())
//		sprintf(oapiDebugString(), " ");

	if (!guidance && !MECO_Cond ) {
		if(!NewIncSP) {
			// Set the Inclination setpoint for the guidance computer
			GetEquPos( Longitude, Latitude, Radius );
				INCLINATION = acos(Clairaut_Constant)*DEG;
		}
	}	

	//Check if guidance is on and if so, call
	if (guidance)
	{
/*		if (BadHeading) 
		{
			sprintf(oapiDebugString(),"Guidance system cannot launch from this heading.");
			guidance = false;
		} 
		else 
*/		{
			// Calculate MET
			if (TZero == -100) 
			{
				MET = -5 ;
				TZero = 0;
			}
			else 
			{
//				if (dt > 0)
//					MET += dt;
//				else MET += 0.01;
				MET = simt - TZero;
			}
			// Call the guidance routine
			Guidance_Routine(this, simt, SimT_Last);
			SimT_Last = MET;
		}
	}
	else if (!MECO_Cond && !PostMECO && (stage < 1) && (!Firstsweep) && (GetAltitude() > GetSize())) {
			if (TZero == -100) 
			{
				MET = -5 ;
				TZero = 0;
			}
			else 
			{
				MET = simt - TZero;
			}
			Guidance_Routine(this, simt, SimT_Last);
	}

// Fix the amount of hypersonic pitch-up

	if ((stage < 2) && (jets_status == DOOR_CLOSED))					// Throttle back to maintain 3.0 G's
		if (((GetThrusterMax0(th_first[0])*numengines)/GetMass()) > 3.0*G) 
			for (i = 0; i < numengines; i++) 
				SetThrusterMax0(th_first[i],0.98*GetThrusterMax0(th_first[i]));

//			SetThrusterGroupLevel(THGROUP_MAIN,(GetThrusterGroupLevel(THGROUP_MAIN)*0.98));

	if ((stage == -3) && (jets_status == DOOR_OPEN)) {
		AirBreathingEngines();
		double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN);
		if(ThrusterActive>0.0f)
			{
			// we calculate the volume at wich it must be played
			double volume=110+(145*ThrusterActive);
			double freq=11050+(9000*ThrusterActive);
			PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
			}
		else if(IsPlaying3(MyID,PLAYJET))
			{
			// else shut down the sound if it's playing
			StopVesselWave3(MyID,PLAYJET);
			}
	}
	if (stage == 3) {
		AirBreathingEngine();
		double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN);
		if(ThrusterActive>0.0f)
			{
			// we calculate the volume at wich it must be played
			double volume=110+(145*ThrusterActive);
			double freq=11050+(9000*ThrusterActive);
			PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
			}
		else if(IsPlaying3(MyID,PLAYJET))
			{
			// else shut down the sound if it's playing
			StopVesselWave3(MyID,PLAYJET);
			}
	}

	if ((!bAttSet) && (stage == 0))				// if we're at altitude with the drop tank, turn on attitude controls anyway
		if (GetAltitude()>100000.0)
			SetAttControlsORIONIII();

	UpdateAnimations();
	VectorThrust();
	SetGearParameters(gear_proc);



	if (stage == 0)
		if (GetPropellantMass(ph_first) == 0) {
			JettisonPart();
			AttachSweep();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();
			SetThrusterGroupLevel (THGROUP_MAIN,1.0);
		}
	if ((stage == -1) || (stage == -2))
		if (GetPropellantMass(ph_booster) == 0) {
			JettisonPart();
			AttachSweep();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();
			SetThrusterGroupLevel (THGROUP_MAIN,1.0);
		}
	if (stage == 1)
		if ((GetAltitude() < 80000.0) && (GetPropellantMass(ph_OrionIII) < 9000.0)) {
				JettisonPart();					// Switch to Stage 2 for the re-entry
				AttachSweep();

				/* IMPORTANT */
				/* Always re-set the stage after any kind of change */
				SetMainStage();
			}

//		MakeANewWing();
		// Make a new wing!
//		SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,0.0);		// Reset the elevator trim to zero, to avoid a big pitch change
		
	if ((stage == 2) && (GetMachNumber() < 3.5) && (GetAltitude() < 45000.0)) {
		stage++;
		SetMainStage();
	}
	if ((stage == -3) && (GetPropellantMass(ph_booster) < 100000.0) && (GetMachNumber() < 3.5) && (GetAltitude() < 45000.0) && (jets_status==DOOR_CLOSED)) {
		RevertJets();		// Extend the jet engines and set them
		BoosterSwitchToJets();
	}

	if (Firstsweep) {
		AttachSweep();
		Firstsweep = false;
		SetMainStage();
	}
}


void ORIONIII::VectorThrust()
{

	if (Firstsweep)
		return;

	if (oapiGetTimeAcceleration() > 10) return;

	if (GetFocus() == GetHandle()) {
		tempP = GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		tempY = GetManualControlLevel(THGROUP_ATT_YAWLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_YAWRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		tempR = GetManualControlLevel(THGROUP_ATT_BANKLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_BANKRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

	} else {
		tempP = 0.0;
		tempY = 0.0;
		tempR = 0.0;
	}

		GetAngularVel(AngularVel);

		tempP+=(2*AngularVel.x);			// Bias the thrust vector AGAINST the angular velocity a bit
		tempY+=(-2*AngularVel.y);			// This will damp angular velocity, preventing loss of control
		tempR+=(2*AngularVel.z);
	

		if (guidance) {
			tempP+=pitchinput;
			tempY+=yawinput;
			tempR+=rollinput;
		}


		if (stage < 1) {
			SetThrusterDir(th_first[0],_V(0.012*tempY,0.012*(tempP-2*tempR),1));
			SetThrusterDir(th_first[1],_V(0.012*tempY,0.012*(tempP+2*tempR),1));
		}
		else if (stage == 1) {
			SetThrusterDir(th_first[0],_V(0.012*tempY,0.012*(tempP-2*tempR),1));
			SetThrusterDir(th_first[1],_V(0.012*tempY,0.012*(tempP+2*tempR),1));
		}
		else if (stage == 2) {
			SetThrusterDir(th_first[0],_V(0.012*tempY,0.012*(tempP+2*tempR),1));
			SetThrusterDir(th_first[1],_V(0.012*tempY,0.012*(tempP-2*tempR),1));
		}

}

void ORIONIII::AirBreathingEngine()
// Thanks to David Hopkins for publishing his AirBreathingEngine source.  
{//=========================================================
double alt2;
double eas2;
double cf1, cf2;
double ActualThrust;


alt2 = GetAltitude();
eas2 = GetAirspeed ();

alt2 = alt2*3.28/2.0;
eas2 = eas2*2.236/3.0;

cf1 = 1.67e-6*(eas2*eas2)-8.8789e-4*(eas2)+0.9977;
cf2 = 3.0e-10*(alt2*alt2)-3.0000e-5*(alt2)+0.9934;

// Nasty kludge to keep the thrust from increasing dramatically above 50kft
if (alt2 > 100000)
	cf2 = 0.25*(1-((alt2-100000)*.0001));
if (cf2 < 0) cf2 = 0;

ActualThrust = cf1*cf2*JET_THRUST;

SetThrusterMax0 (th_first[0],ActualThrust);
};

void ORIONIII::AirBreathingEngines()
// Thanks to David Hopkins for publishing his AirBreathingEngine source.  
{//=========================================================
double alt2;
double eas2;
double cf1, cf2;
double ActualThrust;


alt2 = GetAltitude();
eas2 = GetAirspeed ();

alt2 = alt2*3.28;
eas2 = eas2*2.236;

cf1 = 1.67e-6*(eas2*eas2)-8.8789e-4*(eas2)+0.9977;
cf2 = 3.0e-10*(alt2*alt2)-3.0000e-5*(alt2)+0.9934;

// Nasty kludge to keep the thrust from increasing dramatically above 50kft
if (alt2 > 50000)
	cf2 = 0.25*(1-((alt2-50000)*.0001));
if (cf2 < 0) cf2 = 0;

ActualThrust = cf1*cf2*BJET_THRUST;

SetThrusterMax0 (th_first[0],ActualThrust);
SetThrusterMax0 (th_first[1],ActualThrust);
SetThrusterMax0 (th_first[2],ActualThrust);
};
/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */

void ORIONIII::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void ORIONIII::SetClassCaps(  FILEHANDLE cfg )
{

	CheckCentralMessage();

	SetMainStage();
}

void ORIONIII::SaveState( FILEHANDLE scn )
{
	char cbuf[256];

	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */


//	Undock(ALLDOCKS);

	oapiWriteScenario_int (scn, "CONFIGURATION", stage);
	oapiWriteScenario_int (scn, "EVA", bEVA);
	sprintf (cbuf, "%d %0.4f", gear_status, gear_proc);
	oapiWriteScenario_string (scn, "GEAR", cbuf);
	sprintf (cbuf, "%d %0.4f", bgear_status, bgear_proc);
	oapiWriteScenario_string (scn, "BGEAR", cbuf);
	sprintf (cbuf, "%d %0.4f", pldoors_status, pldoors_proc);
	oapiWriteScenario_string (scn, "DOORS", cbuf);
	sprintf (cbuf, "%d %0.4f", brakes_status, brakes_proc);
	oapiWriteScenario_string (scn, "BRAKES", cbuf);
	sprintf (cbuf, "%d %0.4f", hatch_status, hatch_proc);
	oapiWriteScenario_string (scn, "HATCH", cbuf);
	sprintf (cbuf, "%d %0.4f", jets_status, jets_proc);
	oapiWriteScenario_string (scn, "JETS", cbuf);
//	for (int i = 0; i < 5; i++) {
//		sprintf (cbuf, "%d %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f", i, attach[i].x,attach[i].y,attach[i].z,attachdir[i].x,attachdir[i].y,attachdir[i].z,attachrot[i].x,attachrot[i].y,attachrot[i].z);
//		oapiWriteScenario_string (scn, "ATTACHLOC", cbuf);
//	}
	sprintf (cbuf, OrionMesh);
	oapiWriteScenario_string (scn, "BASICMESH", cbuf);
	if (Orion2) {
		sprintf (cbuf, Or2BayMesh);
		oapiWriteScenario_string (scn, "OR2MESH", cbuf);
	} else if (Orion3) {
		sprintf (cbuf, Or3BayMesh);
		oapiWriteScenario_string (scn, "OR3MESH", cbuf);
	}
/*	if ((attach[0].x != 0.0) || (attach[0].y != -0.1) || (attach[0].z != -1.05)) {
		sprintf (cbuf, "%0.4f %0.4f %0.4f", attach[0].x,attach[0].y,attach[0].z);
		oapiWriteScenario_string (scn, "ATTACHPOINT1", cbuf);
	}
	if ((attach[1].x != 0.0) || (attach[1].y != -0.1) || (attach[1].z != 4)) {
		sprintf (cbuf, "%0.4f %0.4f %0.4f", attach[1].x,attach[1].y,attach[1].z);
		oapiWriteScenario_string (scn, "ATTACHPOINT2", cbuf);
	}
	if ((attach[2].x != 0.0) || (attach[2].y != -1.35) || (attach[2].z != 4)) {
		sprintf (cbuf, "%0.4f %0.4f %0.4f", attach[2].x,attach[2].y,attach[2].z);
		oapiWriteScenario_string (scn, "ATTACHPOINT3", cbuf);
	}
*/
	oapiWriteScenario_float (scn, "TGT_HEADING", PadHeading - 180.0);

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void ORIONIII::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		if (!strnicmp (line, "CONFIGURATION", 13)) {
		    sscanf (line+13, "%d", &stage);
		} else if (!strnicmp (line, "HEADING", 7)) {
			sscanf(line+7, "%lf", &initialheading);
		} else if (!strnicmp (line, "EVA", 3)) {
			sscanf (line+3, "%d", &bEVA);
		} else if (!strnicmp (line, "GEAR", 4)) {
		sscanf (line+4, "%d%lf", &gear_status, &gear_proc);
		} else if (!strnicmp (line, "BGEAR", 5)) {
		sscanf (line+5, "%d%lf", &bgear_status, &bgear_proc);
		} else if (!strnicmp (line, "DOORS", 5)) {
		sscanf (line+5, "%d%lf", &pldoors_status, &pldoors_proc);
		} else if (!strnicmp (line, "BRAKES", 6)) {
		sscanf (line+6, "%d%lf", &brakes_status, &brakes_proc);
		} else if (!strnicmp (line, "HATCH", 5)) {
		sscanf (line+5, "%d%lf", &hatch_status, &hatch_proc);
		} else if (!strnicmp (line, "JETS", 4)) {
		sscanf (line+4, "%d%lf", &jets_status, &jets_proc);
		} else if(!strnicmp (line, "BASICMESH", 9) ) {
			AnimationsMade = false;
			sscanf( line+9, "%s", &OrionMesh);
		} else if(!strnicmp (line, "OR2MESH", 7) ) {
			AnimationsMade = false;
			sscanf( line+7, "%s", &Or2BayMesh);
		} else if(!strnicmp (line, "OR3MESH", 7) ) {
			AnimationsMade = false;
			sscanf( line+7, "%s", &Or3BayMesh);

/*	attach[0] = _V(0,-0.4,-1.05);
	attachdir[0] = _V(0,0,1);
	attachrot[0] = _V(0,-1,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[0] = CreateAttachment (false, attach[0],   attachdir[0], attachrot[0], "X");
*/
//		} else if (!strnicmp (line, "ATTACHLOC", 9)) {
//			VECTOR3 atch, atchdir, atchrot;
//			sscanf (line+9, "%d%lf%lf%lf%lf%lf%lf%lf%lf%lf", i, &atch.x,&atch.y,&atch.z,&atchdir.x,&atchdir.y,&atchdir.z,&atchrot.x,&atchrot.y,&atchrot.z);
//			SetAttachmentParams(attachpos[i],atch,atchdir,atchrot);
		} else if( !strncmp( line, "TGT_HEADING", 11 ) ) {
            sscanf (line+11, "%lf", &d_heading);
			PadHeading = d_heading + 180.0;  // It's not pad heading any more; just target heading
			if (PadHeading > 450.0) PadHeading -= 360.0;
			INCLINATION = acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG;
			d_heading = PadHeading;
/*		} else if (!strnicmp (line, "ATTACHPOINT1", 12)) {
		    sscanf (line+12, "%lf%lf%lf", &attach[0].x,&attach[0].y,&attach[0].z);
			//	   						   pos           dir       rot       
			SetAttachmentParams (attachpos, attach[0],   attachdir[0], _V(0,-1,0));
		} else if (!strnicmp (line, "ATTACHPOINT2", 12)) {
		    sscanf (line+12, "%lf%lf%lf", &attach[1].x,&attach[1].y,&attach[1].z);
			//	   						   pos           dir       rot       
			SetAttachmentParams (attachpos, attach[1],   attachdir[1], _V(0,-1,0));
		} else if (!strnicmp (line, "ATTACHPOINT3", 12)) {
		    sscanf (line+12, "%lf%lf%lf", &attach[2].x,&attach[2].y,&attach[2].z);
			//	   						   pos           dir       rot       
			SetAttachmentParams (attachpos, attach[2],   attachdir[2], _V(0,0,1));
*/		} else ParsePayloadString(line);

		ParseScenarioLineEx (line, vs);
	}

	/* always set the stage whenever something may have changed */
	SetMainStage();

	// Double-tap all the animations so they display right
	RevertLandingGear();
	RevertLandingGear();

	RevertBoosterGear();
	RevertBoosterGear();

	RevertHatch();
	RevertHatch();

	RevertBrakes();
	RevertBrakes();

	RevertPayloadBayDoors();
	RevertPayloadBayDoors();


	if (brakes_proc == 1.0)
		BrakesOn = true;
//	if (wings_proc > 0.0)
//		MakeANewWing();

	ProgramAuto();

}

void ORIONIII::ProgramAuto()
{

	bAutoProgrammed = true;
	N=14;
	double *fmet = new double[N];
	double *fcpitch = new double[N];
	double biastime = -20.0;
//	if (stage == -2) biastime = -138.0;
//	if (stage == -1) biastime = 100.0;
	if (stage == 0) biastime = 130.0;

	fmet[0]					=    0.0;
	fmet[1]					=   45.0;
	fmet[2]					=   65.0;
	fmet[3]					=   95.0;
	fmet[4]					=  155.0+biastime;
	fmet[5]					=  165.0+biastime;
	fmet[6]					=  190.0+biastime;
	fmet[7]					=  210.0+biastime;
	fmet[8]					=  225.0+biastime;
	fmet[9]					=  245.0+biastime;
	fmet[10]				=  258.0+biastime;
	fmet[11]				=  262.0+biastime;
	fmet[12]				=  267.0+biastime;
	fmet[13]				=  270.0+biastime;

	fcpitch[0]				= 20.0;
	fcpitch[1]				= 45.0;
	fcpitch[2]				= 38.0;
	fcpitch[3]				= 32.0;
	fcpitch[4]				= 28.0;
	fcpitch[5]				= 25.0;
	fcpitch[6]				= 23.0;
	fcpitch[7]				= 21.0;
	fcpitch[8]				= 19.0;
	fcpitch[9]				= 14.0;
	fcpitch[10]				= 10.0;
	fcpitch[11]				=  5.0;
	fcpitch[12]				=  2.5;
	fcpitch[13]				=  0.0;
	ProgramPitchMet( N, fmet, fcpitch );
}

void ORIONIII::MakeANewWing()
{
//	                     alignment      position      func    chord S   AR   
		CreateAirfoil (LIFT_VERTICAL, _V(0,0, 0.25), VLiftCoeff, 4, 72, 4.85);

}

void ORIONIII::PlaceComponents()
{
	COMMON_MESH tmp;
/*
	tmp.MeshHandle(hSeat);
	tmp.MeshPosition(EJSEAT_OFS);
	AddMeshEx (tmp);
	tmp.MeshHandle(hSeat2);
	tmp.MeshPosition(EJSEAT_OFS2);
	AddMeshEx (tmp);

	tmp.MeshHandle(hAstro);
	tmp.MeshPosition(ASTRO_OFS);
	AddMeshEx (tmp);
	tmp.MeshHandle(hAstro2);
	tmp.MeshPosition(ASTRO_OFS2);
	AddMeshEx (tmp);
*/
	if (stage > -3) {
		tmp.MeshHandle(hORION);
		tmp.MeshPosition(ORIONIII_OFS);
		AddMeshEx (tmp);
		if (Orion2) {
			tmp.MeshHandle(hORIONII);
			tmp.MeshPosition(ORIONIII_OFS);
			AddMeshEx (tmp);
		} else {
			tmp.MeshHandle(hORIONIII);
			tmp.MeshPosition(ORIONIII_OFS);
			AddMeshEx (tmp);
		}
	}
	else {
		tmp.MeshHandle(hDummy);
		tmp.MeshPosition(ORIONIII_OFS);
		AddMeshEx (tmp);
		tmp.MeshHandle(hDummy2);
		tmp.MeshPosition(ORIONIII_OFS);
		AddMeshEx (tmp);
	}


	if (stage < 0) {
		tmp.MeshHandle(hORIONIV);
		tmp.MeshPosition(BOOSTER_OFS);
		AddMeshEx (tmp);
	}
	if ((stage == 0) || (stage == -1))  {
		tmp.MeshHandle(hORIONTk);
		tmp.MeshPosition(TANK_OFS);
		AddMeshEx (tmp);
	}
}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void ORIONIII::SetMainStage( )
{
	int i;

	if (!AnimationsMade)
		MakeAnimations();

	for (i = 0; i < 5; i++) {
		hV = GetAttachmentStatus(attachpos[i]);
		if (hV != NULL) {
			AtchV[i] = oapiGetVesselInterface (hV);
			atchmass[i] = AtchV[i]->GetMass();
		}
		else atchmass[i] = 0.0;
	}

	if (stage < 0)
		SetBoosterStage();
	else if (stage == 0)
		SetTankStage();
	else if (stage == 1)
		SetSustainStage();
	else if (stage == 2) 
		SetORIONIIIStage();
	else
		SetJetsStage();
}

void ORIONIII::SetBoosterStage( )
{
	int n;


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (35);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	AttachedMass = 0.0;
	for (n = 0; n < 5; n++)
		AttachedMass += atchmass[n];

	if (stage > -3) SetEmptyMass ( Orion3Add + ORIONIII_DRYMASS + BOOSTER_DRYMASS + JET_PROP_MASS + AttachedMass + CalculateMass() );
	else if (stage == -3) SetEmptyMass ( BOOSTER_DRYMASS - BOOSTER_RCSMASS - BJET_PROP_MASS + CalculateMass() );

//	SetTouchdownPoints (_V(-10,-3,-10), _V( 10,-3,-10), _V(0,-3,10));
	SetTouchdownPoints (_V(0,-9.6,35.0), _V(-9.15,-8.85,-1.5), _V(9.15,-8.85,-1.5));
	SetSurfaceFrictionCoeff (0.025, 0.4);
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-9.6,35.0), _V(0,-0.02,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-9.6,35.0), _V(0,-0.02,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-9.6,35.0), _V(0,-0.02,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-9.6,35.0), _V(0,-0.02,1), _V(0,1,0));

//	SetTouchdownPoints (_V(0,-10,-11), _V(-10,10,-11), _V(10,10,-11));
	SetPMI (_V(18,19.5,3.25));
	SetCrossSections (_V(42.4,76.4,12.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS_BOOSTER);
	MakeAirfoils();
	if (!bBoosterMade) MakeBoosterAnimations();
	MakeControlSurfaces();
//	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
	SetMeshVisibilityMode(2,MESHVIS_ALWAYS);


	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hORIONIII, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */
	if (ph_booster==NULL)  
		ph_booster  = CreatePropellantResource(BOOSTER_PROP_MASS,BOOSTER_PROP_MASS*InitFuelLevel,1.0); 
	if (stage > -3)
		if (!ph_OrionIII)  
			ph_OrionIII  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 

//	if (!ph_OrionIII)  
//		ph_OrionIII  = CreatePropellantResource(RCS_PROP_MASS,RCS_PROP_MASS,1.0); 

//	if (stage == -3) DelPropellantResource(ph_OrionIII);
	if (stage < -1) DelPropellantResource(ph_first);

	SetDefaultPropellantResource (ph_booster); 

	th_first[0] = CreateThruster ( _V(-2.75,0,-19.7), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[0], _V(-2.75,-5.5,-19.7), &contrail);
	th_first[1] = CreateThruster ( _V( 2.75,0,-19.7), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[1], _V( 2.75,-5.5,-19.7), &contrail);
	th_first[2] = CreateThruster ( _V(-2.25,0,-19.7), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[2], _V(-2.25,-6.6,-19.7), &contrail);
	th_first[3] = CreateThruster ( _V( 2.25,0,-19.7), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[3], _V( 2.25,-6.6,-19.7), &contrail);
	if (stage > -3) {
		th_first[4] = CreateThruster ( _V(-0.52,0,-12.25), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_SECOND_VAC, ISP_SECOND_SL);
		AddExhaustStream (th_first[0], _V(-0.52,1.4,-12.25), &contrail);
		th_first[5] = CreateThruster ( _V( 0.52,0,-12.25), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_SECOND_VAC, ISP_SECOND_SL);
		AddExhaustStream (th_first[1], _V( 0.52,1.4,-12.25), &contrail);
	}

//



	

//
	if (stage == -3) thg_first = CreateThrusterGroup (th_first, 4, THGROUP_MAIN);
		else thg_first = CreateThrusterGroup (th_first, 6, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 20.0, 2.0, _V(-2.75,-5.5,-19.7), _V(0,0,-1),tex);
	AddExhaust (th_first[1], 20.0, 2.0, _V( 2.75,-5.5,-19.7), _V(0,0,-1), tex);
	AddExhaust (th_first[2], 20.0, 2.0, _V(-2.25,-6.6,-19.7), _V(0,0,-1), tex);
	AddExhaust (th_first[3], 20.0, 2.0, _V( 2.25,-6.6,-19.7), _V(0,0,-1), tex);
	numengines = 4;
	if (stage > -3) {
		AddExhaust (th_first[4], 20.0, 2.0, _V(-0.52,1.4,-12.25), _V(0,-0.1,-1), tex);
		AddExhaust (th_first[5], 20.0, 2.0, _V( 0.52,1.4,-12.25), _V(0,-0.1,-1), tex);
		numengines = 6;
	}

	EnableTransponder(true);
	if (stage == -3) {
		if (!ph_OrionIII)  
			ph_OrionIII  = CreatePropellantResource(BOOSTER_RCSMASS,BOOSTER_RCSMASS,1.0); 
		if (!ph_Jets)  
			ph_Jets  = CreatePropellantResource(BJET_PROP_MASS,BJET_PROP_MASS,1.0); 
		SetAttControlsORIONIIIBoost();
		if (jets_proc > 0.0) BoosterSwitchToJets();
	}
}

void ORIONIII::SetTankStage( )
{

	int n;

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();
	if (bBoosterMade) DelBoosterAnimations();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (20);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	AttachedMass = 0.0;
	for (n = 0; n < 5; n++)
		AttachedMass += atchmass[n];

	SetEmptyMass ( Orion3Add + ORIONIII_DRYMASS + TANK_DRYMASS + JET_PROP_MASS + AttachedMass + CalculateMass() );

	SetTouchdownPoints (_V(0,-6.5,16.0), _V(-9.15,-7.6,-9.0), _V(9.15,-7.6,-9.0));
	SetSurfaceFrictionCoeff (0.025, 0.4);
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-6.5,15.7), _V(0,0.055,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-6.5,15.7), _V(0,0.055,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-6.5,15.7), _V(0,0.055,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-6.5,15.7), _V(0,0.055,1), _V(0,1,0));
	
//	SetTouchdownPoints (_V(0,-10,-11), _V(-10,10,-11), _V(10,10,-11));
	SetPMI (_V(18,19.5,3.25));
	SetCrossSections (_V(42.4,76.4,12.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
//	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);	
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);


	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hORIONIII, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_booster != NULL)
		DelPropellantResource (ph_booster);
	if (ph_first==NULL)  
		ph_first  = CreatePropellantResource(FIRST_PROP_MASS,FIRST_PROP_MASS*InitFuelLevel,1.0); 
	if (!ph_OrionIII)  
		ph_OrionIII  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 
//	if (!ph_OrionIII)  
//		ph_OrionIII  = CreatePropellantResource(RCS_PROP_MASS,RCS_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_first); 

	th_first[0] = CreateThruster ( _V(-0.52,0,-12.25), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_SECOND_VAC, ISP_SECOND_SL);
	AddExhaustStream (th_first[0], _V(-0.52,1.4,-12.25), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	th_first[1] = CreateThruster ( _V( 0.52,0,-12.25), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_SECOND_VAC, ISP_SECOND_SL);
	AddExhaustStream (th_first[1], _V( 0.52,1.4,-12.25), &contrail);
//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);

	thg_first = CreateThrusterGroup (th_first, 2, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 20.0, 2.0, _V(-0.52,1.4,-12.25), _V(0,-0.1,-1), tex);
	AddExhaust (th_first[1], 20.0, 2.0, _V( 0.52,1.4,-12.25), _V(0,-0.1,-1), tex);
	numengines = 2;

	EnableTransponder(true);
}

void ORIONIII::SetSustainStage( )
{
	int n;

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();
	if (bBoosterMade) DelBoosterAnimations();

	PARTICLESTREAMSPEC contrail = 
	{
//		0, 5.0, 5, 150, 0.3, 4.0, 16, 3.0, 
		0, 1.1, 30, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 0.5, 3, 500, 0, 1, 50, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 0, 0.1
	};

	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (20);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	AttachedMass = 0.0;
	for (n = 0; n < 5; n++)
		AttachedMass += atchmass[n];

	SetEmptyMass ( Orion3Add + ORIONIII_DRYMASS + JET_PROP_MASS + AttachedMass + CalculateMass() );

	SetTouchdownPoints (_V(0,-3.8,15.7), _V(-9.15,-3.71,-4.1), _V(9.15,-3.71,-4.1));
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0));
	SetPMI (_V(16,17.5,3.0));
	SetCrossSections (_V(32.4,56.4,12.5));
	SetCW (0.05, 0.1, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,0.0);
//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);

	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hORIONIII, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_first != NULL)
		DelPropellantResource (ph_first);
	if (ph_booster != NULL)
		DelPropellantResource (ph_booster);
//	if (ph_deorbit==NULL)  
//		ph_deorbit  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 
	if (ph_OrionIII==NULL)  
		ph_OrionIII  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_OrionIII); 


	th_first[0] = CreateThruster ( _V(-0.52,0,-12.25), _V(0,0,1), THRUST_FIRST_VAC, ph_OrionIII, ISP_SECOND_VAC, ISP_SECOND_SL);
	AddExhaustStream (th_first[0], _V(-0.52,1.4,-12.25), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	th_first[1] = CreateThruster ( _V( 0.52,0,-12.25), _V(0,0,1), THRUST_FIRST_VAC, ph_OrionIII, ISP_SECOND_VAC, ISP_SECOND_SL);
	AddExhaustStream (th_first[1], _V( 0.52,1.4,-12.25), &contrail);
//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);



	thg_first = CreateThrusterGroup (th_first, 2, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 20.0, 2.0, _V(-0.52,1.4,-12.25), _V(0,-0.1,-1), tex);
	AddExhaust (th_first[1], 20.0, 2.0, _V( 0.52,1.4,-12.25), _V(0,-0.1,-1), tex);
	numengines = 2;

	SetAttControlsORIONIII();
	EnableTransponder(true);
}

void ORIONIII::SetORIONIIIStage( )
{
	int n;

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();
	if (bBoosterMade) DelBoosterAnimations();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");

	SetSize (20);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	AttachedMass = 0.0;
	for (n = 0; n < 5; n++)
		AttachedMass += atchmass[n];

	SetEmptyMass ( Orion3Add + ORIONIII_DRYMASS + AttachedMass + CalculateMass() );

	SetTouchdownPoints (_V(0,-3.8,15.7), _V(-9.15,-3.71,-4.1), _V(9.15,-3.71,-4.1));
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0));
	SetPMI (_V(16.7,18.9,3.6));
	SetCrossSections (_V(24.2,55.4,9.8));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();	
	Guidance (false);
	SetAttitudeRotLevel(_V(0,0,0));
	pitchinput = 0.0;
	yawinput = 0.0;
	rollinput = 0.0;
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);

	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hORIONIII, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_first != NULL)
		DelPropellantResource (ph_first);
	if (ph_booster != NULL)
		DelPropellantResource (ph_booster);
//	if (ph_deorbit != NULL)				
//		DelPropellantResource (ph_deorbit); 
	if (ph_OrionIII==NULL)  
		ph_OrionIII  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 
	if (ph_Jets==NULL)  
		ph_Jets  = CreatePropellantResource(JET_PROP_MASS,JET_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_OrionIII); 

	th_first[0] = CreateThruster ( _V( 0.0, 0.0,-9.26), _V(0,0,1), MAX_ATT_FIRST*4, ph_OrionIII, ISP_RCS_VAC, ISP_RCS_VAC);

	th_first[1] = CreateThruster ( _V( 0.0, 0.0,-9.26), _V(0,0,1), MAX_ATT_FIRST*4, ph_OrionIII, ISP_RCS_VAC, ISP_RCS_VAC);

	thg_first = CreateThrusterGroup (th_first, 2, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 2.5, 0.9, _V(-1.1, 1.12,-11.0), _V(0,0,-1),texRCS);  
	AddExhaust (th_first[1], 2.5, 0.9, _V( 1.1, 1.12,-11.0), _V(0,0,-1),texRCS);  

	numengines = 2;

	SetAttControlsORIONIII();
	EnableTransponder(true);

}


void ORIONIII::SetJetsStage( )
{
	int n;

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();
	if (bBoosterMade) DelBoosterAnimations();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 0.4, 30, 325.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
//		0, 1.25, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");

	SetSize (20);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	AttachedMass = 0.0;
	for (n = 0; n < 5; n++)
		AttachedMass += atchmass[n];

	SetEmptyMass ( Orion3Add + ORIONIII_DRYMASS + AttachedMass + CalculateMass() );
	SetTouchdownPoints (_V(0,-3.8,15.7), _V(-9.15,-3.71,-4.1), _V(9.15,-3.71,-4.1));
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-3.8,15.7), _V(0,-0.0045,1), _V(0,1,0));
	SetPMI (_V(16.7,18.9,3.6));
	SetCrossSections (_V(24.2,55.4,9.8));
	SetCW (0.05, 0.1, 1.4, 1.4);
//	SetRotDrag (_V(0.7,0.7,1.2));
//	SetPitchMomentScale (1e-6);
//	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	Guidance (false);
	SetAttitudeRotLevel(_V(0,0,0));
	pitchinput = 0.0;
	yawinput = 0.0;
	rollinput = 0.0;
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);

	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hORIONIII, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_first != NULL)
		DelPropellantResource (ph_first);
	if (ph_booster != NULL)
		DelPropellantResource (ph_booster);
	if (ph_OrionIII != NULL)				
		DelPropellantResource (ph_OrionIII); 
//	if (ph_OrionIII==NULL)  
//		ph_OrionIII  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 
	if (ph_Jets==NULL)  
		ph_Jets  = CreatePropellantResource(JET_PROP_MASS,JET_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_Jets); 

	th_first[0] = CreateThruster ( _V( 0.0, 0.0,-4.8), _V(0,0.0,1), JET_THRUST, ph_Jets, ISP_JET_VAC, ISP_JET_VAC);
	AddExhaustStream (th_first[0], _V( 3.5, -1.5,-4.8), &contrail);
	AddExhaustStream (th_first[0], _V(-3.5, -1.5,-4.8), &contrail);
	AddExhaustStream (th_first[0], _V( 4.6, -1.5,-4.8), &contrail);
	AddExhaustStream (th_first[0], _V(-4.6, -1.5,-4.8), &contrail);
	AddExhaustStream (th_first[0], _V( 5.7, -1.5,-4.8), &contrail);
	AddExhaustStream (th_first[0], _V(-5.7, -1.5,-4.8), &contrail);
	AddExhaustStream (th_first[0], _V( 6.8, -1.5,-4.8), &contrail);
	AddExhaustStream (th_first[0], _V(-6.8, -1.5,-4.8), &contrail);
	AddExhaustStream (th_first[0], _V( 7.9, -1.5,-4.8), &contrail);
	AddExhaustStream (th_first[0], _V(-7.9, -1.5,-4.8), &contrail);

	thg_first = CreateThrusterGroup (th_first, 1, THGROUP_MAIN);

	numengines = 1;

//	SetAttControlsORIONIII();
	EnableTransponder(true);

}


/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */

void ORIONIII::JettisonPart()
 {

				VESSELSTATUS vs;
				VECTOR3 ofs = _V(0,0,0);
				VECTOR3 vel = _V(0,-1.5,0);
//				ofs = Centaur_OFS;
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
//				SetEngineLevel(ENGINE_MAIN, 0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				vs.vrot.x = 0.0;
//				if (GetEngineLevel(ENGINE_MAIN) > 0.0) 
				vs.vrot.y = 0.0;
				vs.vrot.z = 0.0;
				char name[256];
				if ((stage == -1) || (stage == -2)) {
					DelBoosterAnimations();
					strcpy (name, GetName()); strcat (name, "-Booster");
					OBJHANDLE BSTR = oapiCreateVessel (name, "2001/OrionIV", vs);
					VESSEL *remotevessel = oapiGetVesselInterface(BSTR);
					remotevessel->SetFuelMass(GetPropellantMass(ph_booster));
					delete remotevessel;
					DelPropellantResource (ph_booster); 
					PlayVesselWave3(MyID,PLAYBANG);
					if (stage == -2) stage = 0;		// Go straight from booster to upper stage
				}
				else if (stage == 0) {
					vs.vrot.x = -0.05;
					strcpy (name, GetName()); strcat (name, "-Tank");
					VESSEL::Create (name, "2001/OrionIIITk", vs);	
					DelPropellantResource (ph_first); 
					PlayVesselWave3(MyID,PLAYBANG);
				}
/*				else if (GetThrusterGroupLevel (thg_first) == 0.0) {
					strcpy (name, GetName()); strcat (name, "-Deorbit Stage");
					VESSEL::Create (name, "ORIONIII/ORIONIII_Deorbit", vs);
					DelPropellantResource (ph_deorbit); 
					PlayVesselWave3(MyID,PLAYBANG);
				}
*/
//				else if (stage == 1) {
//					DelPropellantResource (ph_deorbit);
//				}
				else if (stage > 1) stage--;


				stage++;

 }

void ORIONIII::AttachSweep()
{
	
			/* Now scan for nearby attach points; these need to stay in the bay!  */
				
			Local2Global (_V(0,0,0), grms);  // global position of RMS tip
			
			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (i = 0; i < oapiGetVesselCount(); i++) {
				hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < 3.0*oapiGetSize (hV)) { // in range
					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
						if (strncmp (id, "BOLT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						if (dist (gpos, grms) < 0.25) { // found one!
							// check whether satellite is currently clamped into payload bay
							if ((hV == GetAttachmentStatus (Baypos)) && (!Firstsweep))
							{
								AttachChild (hV, Baypos, hAtt);
								atchmass[5] = v->GetMass();
								BayVessel = hV;
								BayV = oapiGetVesselInterface (BayVessel);
							}
							else
							{	
								AttachChild (hV, Baypos, hAtt);
								atchmass[3] = v->GetMass();
								hDock = v->GetDockHandle(0);
								BayVessel = hV;
								BayV = oapiGetVesselInterface (BayVessel);
/*								if (hDock != NULL) {
									v->GetDockParams(hDock,pos, dir, rot);
									pos.z += 4.8;
									if (Dock1 == NULL)
										Dock1 = CreateDock(pos, dir, rot);
									else 
										SetDockParams(Dock1, pos, dir, rot);
									v->DelDock(hDock);
								}
*/
							}
//							return 1;
						}
					}
				}
			}
}


void ORIONIII::AttachVessel()
{
	int n, i, k;
	double bestdist = 100.0;
	OBJHANDLE hVBest = NULL;
	int nbest;
	ATTACHMENTHANDLE hAtt, hAttbest;
	bool AlreadyAttached = false;

			/* Now scan for nearby attach points; these need to stay in the bay!  */
		for (n = 0; n < 5; n++) {
				
			Local2Global (attach[n], grms);  // global position of RMS tip
			GlobalRot (attachdir[n], gatchdir);
			GlobalRot (attachrot[n], gatchrot);

			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (i = 0; i < oapiGetVesselCount(); i++) {
				hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < 25.0 + 3.0*oapiGetSize (hV)) { // in range
					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
//						if (strncmp (id, "BOLT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						v->GlobalRot (dir, gdir);
						v->GlobalRot (rot, grot);
						if ((!GetAttachmentStatus (hAtt)) && (dist (gpos, grms) > 0.0 ) && (dist (gpos, grms) < MAX_GRAPPLING_DIST)) { // found one!
							double angleoff = 0.5*PI*sqrt(pow(gdir.x-gatchdir.x,2.0) + pow(gdir.y-gatchdir.y,2.0) + pow(gdir.z-gatchdir.z,2.0));
							double angleoff2 = 0.5*PI*sqrt(pow(grot.x-gatchrot.x,2.0) + pow(grot.y-gatchrot.y,2.0) + pow(grot.z-gatchrot.z,2.0));
							if ((angleoff > 160*RAD) && (angleoff2 < 20*RAD)) {
								AlreadyAttached = false;
								for (k = 0; k < nAttach; k++) { // now scan all attachment points of the candidate
									if (GetAttachmentStatus(v->GetAttachmentHandle(true,k)) != NULL)
										AlreadyAttached = true;
								}
								if (!AlreadyAttached) 
									if (dist (gpos, grms) < bestdist) {
										bestdist = dist (gpos, grms);
										hVBest = hV;
										nbest = n;
										hAttbest = hAtt;
									}
							}
						}
					}
				}
			}
		}
		if ((hVBest != NULL) && (bestdist < 100.0)) {
			AttachChild (hVBest, attachpos[nbest], hAttbest);
			AtchV[n] = oapiGetVesselInterface (hVBest);
//			return;
		}

}

int ORIONIII::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	if (!fPressed) return 0; // only process keydown events
	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		if (KEYDOWN (keystate, OAPI_KEY_SPACE)) 
		{ // "Join Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_SPACE, 1.0))
			{

			oapiSetShowGrapplePoints (!oapiGetShowGrapplePoints ());

			return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_J)) 
		{ // "Join Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{
			/* CVE-lite function ! */
			/* IMPORTANT */
			/* call DeployPayload() to jettison the GPDS payloads in sequence */
	
			AttachVessel();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();

			return 1;
			}
		}
	}
	else {

		if (KEYDOWN (keystate, OAPI_KEY_J)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{
				VESSEL *depV;
							if ((stage < 2) && (stage > -3) && (pldoors_proc == 0.0)) {
								JettisonPart();
							}
							else if ((stage == -3) && (GetMachNumber() < 3.5) && (GetAltitude() < 45000.0)) {
								SetPropellantMass(ph_booster,0.0);
								if (jets_status==DOOR_CLOSED)
									RevertJets();		// Extend the jet engines and set them
								BoosterSwitchToJets();
							}
							else {
								OverridePayloadVelocity(_V(0,+0.25,0));
								depV = DeployPayload();
								if (depV == NULL) {
									if (GetAttachmentStatus(attachpos[0]) != NULL) {
										DetachChild(attachpos[0],0.02);
										atchmass[0] = 0.0;
									} else if (GetAttachmentStatus(attachpos[1]) != NULL) {
										DetachChild(attachpos[1],0.02);
										atchmass[1] = 0.0;
									} else if (GetAttachmentStatus(attachpos[2]) != NULL) {
										DetachChild(attachpos[2],0.02);
										atchmass[2] = 0.0;
									} else if (GetAttachmentStatus(attachpos[3]) != NULL) {
										DetachChild(attachpos[3],0.02);
										atchmass[3] = 0.0;
									} else if (GetAttachmentStatus(attachpos[4]) != NULL) {
										DetachChild(attachpos[4],0.1);
										atchmass[4] = 0.0;
									} 
								}
							}

							/* CVE-lite function ! */
							/* IMPORTANT */
							/* call DeployPayload() to jettison the GPDS payloads in sequence */

							AttachSweep();

							/* IMPORTANT */
							/* Always re-set the stage after any kind of change */
							SetMainStage();
							if ((stage==1) && (pldoors_proc == 0.0)) SetThrusterGroupLevel (THGROUP_MAIN,1.0);
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_G)) 
		{ // Toggle the Gear
			if (oapiAcceptDelayedKey (OAPI_KEY_G, 1.0))
			{
				if (stage < 0)
					RevertBoosterGear();
				else
					RevertLandingGear ();
				StopVesselWave3(MyID,PLAYGEAR);
				PlayVesselWave3(MyID,PLAYGEAR);
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_K)) 
		{ // Toggle the bay doors
			if ((oapiAcceptDelayedKey (OAPI_KEY_K, 1.0)) && (stage > -3))
				if (Orion2)
				{
					RevertPayloadBayDoors ();
					if (BayVessel != NULL)
						SendCentralMessage(BayV, "MSG", "REVERTPROTRUSIONS");
					StopVesselWave3(MyID,PLAYDOORS);
					PlayVesselWave3(MyID,PLAYDOORS);
	//				if (pldoors_status == DOOR_CLOSING)
	//					Undock(ALLDOCKS);
				}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_B)) 
		{ // Toggle the brakes
			if (oapiAcceptDelayedKey (OAPI_KEY_B, 1.0))
			{
//				if (BrakesOn)
//					BrakesOn = false;
//				else BrakesOn = true;
				RevertBrakes();
				if (brakes_status == DOOR_OPENING)
					BrakesOn = true;
				else BrakesOn = false;
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_P)) 
		{ // Toggle the hatch / airlock
			if (oapiAcceptDelayedKey (OAPI_KEY_P, 1.0))
			{
				if (stage > -3)
					RevertHatch ();
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_O))
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_O, 1.0))
				if (!guidance) { 
					Guidance( true );
//					guidance=true;
					sprintf(oapiDebugString(),"Guidance ON");
					pitchinput = 0.0;
					yawinput = 0.0;
					rollinput = 0.0;
				}
				else {
					Guidance( false );
//					guidance = false;
					SetAttitudeRotLevel(_V(0,0,0));
					pitchinput = 0.0;
					yawinput = 0.0;
					rollinput = 0.0;

					sprintf(oapiDebugString(),"Guidance OFF");
				}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_U)) 
		{ // "Set launch Azimuth"
			if (oapiAcceptDelayedKey (OAPI_KEY_U, 1.0))
			{
				GetInclinationSP();
				return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) 
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				{
				if (hatch_status == DOOR_OPEN)
					{
		//				char name[256];
		//				OBJHANDLE hvessel;
		//				strcpy (name, GetName()); strcat (name, "-EV1");
		//				hvessel=oapiGetVesselByName(name);
		//				if (hvessel == 0)
		//				{
		//					bEVA = true;
							SeparateEVA();
							SetMainStage();
		//					hvessel=oapiGetVesselByName(name);
		//					if (hvessel != 0)
		//						oapiSetFocusObject(hvessel);
		//				}
		//				else
		//				{
		//					oapiDeleteVessel(hvessel);
		//					bEVA = false;
		//					SetMainStage();
		//				};
					}
			}
		return 1;
		}
	}
	return 0;
}




int ORIONIII::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}



void ORIONIII::SetAttControlsORIONIIIBoost()
{
	THRUSTER_HANDLE th_rot[2];

	th_rot[0] = CreateThruster (_V(0,-0.97, 5.25), _V(0, 1,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.97,-5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);


	th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_FIRST/4, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_OrionIII, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_FIRST/4, ph_OrionIII, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

}


void ORIONIII::SetAttControlsORIONIII()
{
	bAttSet = true;
	THRUSTER_HANDLE th_rot[2];

	th_rot[0] = CreateThruster (_V(0,-0.97, 5.25), _V(0, 1,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.97,-5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.53,-1.15, 9.18), _V(-0.5,-.866,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.51,-1.15, 9.325), _V(-0.5,-.866,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.53,-1.15, 9.18), _V( 0.5,-.866,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.51,-1.15, 9.325), _V( 0.5,-.866,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.4,1.78,-8.72), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.348,1.75,-8.51), _V(0, 1.0,0.0),texRCS); 
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.4,1.78,-8.72), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.348,1.75,-8.51), _V(0, 1.0,0.0),texRCS);   
	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.2, 5.25), _V(0,1,0),MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.53,-1.15, 9.18), _V(-0.5,-.866,0.0),texRCS);   
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.53,-1.15, 9.18), _V( 0.5,-.866,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-5.04,-1.78, -4.14), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-5.00,-1.78, -3.94), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 5.04,-1.78, -4.14), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 5.00,-1.78, -3.94), _V(0,-1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0),MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.2, 5.25), _V(0,-1,0),MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.53,-1.15, 9.18), _V(-0.5, .866,0.0),texRCS);   
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.53,-1.15, 9.18), _V( 0.5, .866,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-5.04,-1.46, -4.14), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-5.00,-1.46, -3.94), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 5.04,-1.46, -4.14), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 5.00,-1.46, -3.94), _V(0, 1.0,0.0),texRCS);   
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.53,-1.15, 9.18), _V(-0.5, .866,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.51,-1.15, 9.325), _V(-0.5, .866,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.53,-1.15, 9.18), _V( 0.5, .866,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.51,-1.15, 9.325), _V( 0.5, .866,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V(-1.38,-1.7, -10.8), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 1.38,-1.7, -10.8), _V(0,-1.0,0.0),texRCS);  
	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.53,-1.15, 9.18), _V( 1,0,0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.51,-1.15, 9.325), _V( 1,0,0),texRCS); 
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.4,1.78,-8.72), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.348,1.75,-8.51), _V(-1.0,0,0.0),texRCS); 
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.53,-1.15, 9.18), _V(-1,0,0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.51,-1.15, 9.325), _V(-1,0,0),texRCS); 
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.4,1.78,-8.72), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.348,1.75,-8.51), _V( 1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.53,-1.15, 9.18), _V( 1,0,0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.51,-1.15, 9.325), _V( 1,0,0),texRCS); 
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.4,1.78,-8.72), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.348,1.75,-8.51), _V( 1.0,0,0.0),texRCS); 
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster (_V(-2,0.0,0.0), _V( 1,0,0), MAX_ATT_FIRST, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.53,-1.15, 9.18), _V(-1,0,0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.51,-1.15, 9.325), _V(-1,0,0),texRCS); 
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.4,1.78,-8.72), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.348,1.75,-8.51), _V(-1.0,0,0.0),texRCS); 
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_FIRST/4, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-5.04,-1.78, -4.14), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-5.00,-1.78, -3.94), _V(0,-1.0,0.0),texRCS);   
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 5.04,-1.46, -4.14), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 5.00,-1.46, -3.94), _V(0, 1.0,0.0),texRCS); 
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_OrionIII, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_FIRST/4, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 5.04,-1.78, -4.14), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 5.00,-1.78, -3.94), _V(0,-1.0,0.0),texRCS);   
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-5.04,-1.46, -4.14), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-5.00,-1.46, -3.94), _V(0, 1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_FIRST*2, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 2.5, 0.5, _V( 1.67,-0.90,-11.0), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 2.5, 0.5, _V(-1.67,-0.90,-11.0), _V(0,0,-1),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_FIRST/2, ph_OrionIII, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.53,-1.15, 9.18), _V(-0.707,0,0.707),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.51,-1.15, 9.325), _V(-0.707,0,0.707),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.53,-1.15, 9.18), _V( 0.707,0,0.707),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.51,-1.15, 9.325), _V( 0.707,0,0.707),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}
