/**
*
* $Id: CVEL-ORIONIII.h $
* (c) 2005 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/cvelmessage.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include "../../include/guidance.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

// All data from astronautix.com
const double ISP_FIRST_SL		=     385.0*G; 
const double ISP_FIRST_VAC		=     468.0*G; //***
const double ISP_SECOND_SL		=     385.0*G; 
const double ISP_SECOND_VAC		=     468.0*G; //***
const double ISP_RCS_VAC		=     455.0*G; //***
const double ISP_JET_VAC		=   10500.0*G;
const double ISP_BJET_VAC		=   30500.0*G;
const double THRUST_FIRST_VAC	=  567000.0*G/3.0; //***
const double MAX_ATT_FIRST		=    1500.0*G;
const double JET_THRUST			=   24000.0*G;
const double BJET_THRUST		=   18000.0*G;
const double BOOSTER_PROP_MASS	=  695500.0; 
const double FIRST_PROP_MASS	=  335500.0; 
const double SECOND_PROP_MASS	=	27200.0;
//const double RCS_PROP_MASS		=	 6000.0;
const double BJET_PROP_MASS		=   13500.0;
const double JET_PROP_MASS		=    1000.0;
const double ORIONIII_DRYMASS	=   23800.0-JET_PROP_MASS; //***
const double TANK_DRYMASS		=   12500.0; //***
const double BOOSTER_DRYMASS	=  124000.0; //***
const double BOOSTER_RCSMASS	=     800.0; //***
const double PAXMASS			=   11000.0;	// Mass of pax provisions on Orion III

const VECTOR3 ORIONIII_OFS		= _V(0,0,0);
const VECTOR3 TANK_OFS			= _V(0,0,0);
const VECTOR3 BOOSTER_OFS		= _V(0,0,0);
const VECTOR3 EJSEAT_OFS		= _V(-0.462,-0.924,11.77);
const VECTOR3 EJSEAT_OFS2		= _V( 0.462,-0.924,11.77);
const VECTOR3 ASTRO_OFS			= _V(-0.462,-0.792,11.99);
const VECTOR3 ASTRO_OFS2		= _V( 0.462,-0.792,11.99);
const VECTOR3 EVA_OFS			= _V( 1.5,-0.62,10.2);
const VECTOR3 CAMERA_OFS		= _V(-0.26,-0.076,12.95);
const VECTOR3 CAMERA_OFS_BOOSTER= _V(-0.39, -4.2, 31.90);
const VECTOR3 OFS_SAT			= _V(0,0,0);

const double GEAR_OPERATING_SPEED = 0.25;
const double PLDOORS_OPERATING_SPEED = 0.07125;
const double BRAKES_OPERATING_SPEED = 0.25;
const double HATCH_OPERATING_SPEED = 0.125;
const double WINGS_OPERATING_SPEED = 0.25;
const double JETS_OPERATING_SPEED = 0.5;
const double NOZZLES_OPERATING_SPEED = 0.25;

const double MAX_GRAPPLING_DIST		= 20.0;

const double ROLL = 0.0;            // 180.0 to go to orbit heads-down.  Intermediate values probably won't work.

#define PLAYGEAR					 20		// the gear sound
#define PLAYBANG					 24		// the jettison sound
#define PLAYJETT					 25     // entry shield jettison sound
#define PLAYJET						 26     // the jet noise sound
#define PLAYDOORS					 27     // the payload door sound

/* make your own vessel class (named ORIONIII here, for example)
   subclassed from COMMON_VESSEL_EX */

class ORIONIII : public COMMON_VESSEL_EX, GUIDANCE {
public:
	ORIONIII (OBJHANDLE hObj, int fmodel);
	~ORIONIII();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } gear_status, bgear_status, pldoors_status, brakes_status, hatch_status, wings_status, jets_status, nozzles_status;

	void MakeAnimations();
	void MakeBoosterAnimations();
	void DelBoosterAnimations();
	void MakeAirfoils();
	void MakeControlSurfaces();
	void BoosterSwitchToJets();
	void UpdateAnimations();
	void ActivateLandingGear (DoorStatus action);
	void RevertLandingGear();
	void ActivateBoosterGear (DoorStatus action);
	void RevertBoosterGear();
	void ActivatePayloadBayDoors (DoorStatus action);
	void RevertPayloadBayDoors();
	void ActivateBrakes (DoorStatus action);
	void RevertBrakes();
	void ActivateHatch (DoorStatus action);
	void RevertHatch();
	void ActivateJets (DoorStatus action);
	void RevertJets();
	void SeparateEVA();
	void SetGearParameters (double state);
	void AttachSweep();
	void AttachVessel();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VectorThrust();
	void AirBreathingEngine();
	void AirBreathingEngines();
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void ProgramAuto();
	void MakeANewWing();
	void PlaceComponents();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetBoosterStage();
	void SetMainStage(); 
	void SetTankStage();
	void SetSustainStage();
	void SetORIONIIIStage();
	void SetJetsStage();
//	void SetAttControlsTranstage();
	void SetAttControlsORIONIIIBoost();   
	void SetAttControlsORIONIII();    // this is to set up the thrusters. I like to keep them separate.
	void JettisonPart();
	bool AzInput (void *id, char *str, void *data);
	bool IncInput (char *rstr);
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);
	double pitchinput, yawinput, rollinput;
	VESSELSTATUS vst;
	int stage;

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);

protected:
	double GetCPitch(double t);
	void Lateral_Guidance(VESSEL *vessel, double SimT, double SimT_Last);
	void Guidance_Routine(VESSEL *vessel, double SimT, double SimT_Last);
	void GetInclinationSP();
	void InitialiseGuidance();


	bool Guidance() { return guidance; };
	void Guidance( bool g ) { guidance = g; };

	void DisplayInclinationSetPoint( bool d ) { DisplayIncSP=d; };
	bool DisplayInclinationSetPoint() { return DisplayIncSP; };

	double initialgamma, initialheading, launchrotate;


	PROPELLANT_HANDLE ph_booster,ph_first,ph_OrionIII,ph_Jets;
	THRUSTER_HANDLE   th_first[6];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hORION, hORIONII, hORIONIII,hORIONTk,hORIONIV,hSeat,hAstro,hSeat2,hAstro2,hDummy, hDummy2;
//	ANIMATIONCOMPONENT_HANDLE wing1, wing2, wing3, wing4, Gear1, Gear2, Gear3, Gear4, Gear5, Gear6, Gear7,
//		PLDr1, PLDr2, Hatch1, Jets1, Nozl1;
	ANIMATIONCOMPONENT_HANDLE wing11, wing12, wing13, wing14, Gear11, Gear12, Gear13, Gear14, Gear15, Gear16, Gear17,
		Gear18, Gear19, Jets11, Jets12, Jets13, Nozl11, Elev11, Rudd11, Elev12, Rudd12, Elev13, Rudd13, Elev14, Rudd14;
	SURFHANDLE		  tex,texRCS, retex;
	AIRFOILHANDLE	  hwing, htail;
	UINT anim_lrudder,anim_rrudder,anim_laileron,anim_raileron,anim_lelevator,anim_relevator,anim_lelevatortrim,anim_relevatortrim;
	UINT anim_gear;     // handle for landing gear animation
	UINT anim_bgear;     // handle for booster landing gear animation
	UINT anim_pldoors;     // handle for payload door animation
	UINT anim_brakes;	// handle for speedbrake animation
	UINT anim_hatch;	// handle for top hatch animation
	UINT anim_wings;	// handle for wings animation
	UINT anim_jets;		// handle for jets animation
	UINT anim_nozzles;	// handle for nozzles animation
	double gear_proc;
	double bgear_proc;
	double pldoors_proc;
	double brakes_proc;
	double hatch_proc;
	double wings_proc;
	double jets_proc;
	double nozzles_proc;
	bool BrakesOn, havesound, bShieldJettisoned, bDeorbitFired, bThrustTerminated, bEjected, bEVA, 
		bAbort, AbortChecked, bAttSet,bBoosterMade, bAutoProgrammed, bControlsMade;
	UINT Exh1,Exh2,Exh3,Exh4;
	PSTREAM_HANDLE stream_a_1,stream_b_1,stream_a_2,stream_b_2,stream_a_3,stream_b_3,stream_a_4,stream_b_4;
	VECTOR3 Vold, Vnew;
	double dt, oldsimt;
	int MyID;
	ATTACHMENTHANDLE Dockpos,Baypos, sat_attach, Nose_atch1, Nose_atch2, MDD_atch;
	VECTOR3 gpos, grms, pos, dir, rot;
	OBJHANDLE hV, BayVessel, hvessel;
	VESSEL *BayV;
	DWORD i, j;
	double atchmass[6];
	DOCKHANDLE hDock, Dock0, Dock1, Dock2;
	bool Firstsweep;
	int numengines;
	double tempP, tempY, tempR;
	VECTOR3 AngularVel;
	bool autopilot;
	double Bank_Error, Yaw_Error;
	bool engineignited, bgoingsouth;
	double last_simt;
	double ignition_time;
	double density;
	double displaytime, simt, d_heading;
	double MachPitch;
	bool first_guidance_pass, AnimationsMade;
	double MET, diff, target, hdg, diverge, yawangle, targetrate, rate, amount;
	AIRFOILHANDLE BstrWing, BstrRudder;
	bool Orion2, Orion3;
	double Orion3Add;
	char OrionMesh[80], Or2BayMesh[80], Or3BayMesh[80];
	ATTACHMENTHANDLE  attachpos[5];
	VECTOR3           attach[5];
	VECTOR3			  attachdir[5];
	VECTOR3	          attachrot[5];
	VECTOR3			  gatchdir, gdir, gatchrot, grot, horiz;
	VESSEL *AtchV[5];
	double AttachedMass;
};







/* basic vessel class setup */

void ORIONIII::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void ORIONIII::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void ORIONIII::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void ORIONIII::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void ORIONIII::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void ORIONIII::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int ORIONIII::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}

bool ORIONIII::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	// show gear deployment status
	if ((gear_status == DOOR_OPEN || (gear_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) ||
		(bgear_status == DOOR_OPEN || (bgear_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5))) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			skp->Rectangle (cx-d/2, cy-d*5, cx+d/2, cy-d*4);
			skp->Rectangle (cx-d*3, cy-d*2, cx-d*2, cy-d);
			skp->Rectangle (cx+d*2, cy-d*2, cx+d*3, cy-d);
		}
	}

	// show pldoors deployment status
	if (pldoors_status == DOOR_OPEN || (pldoors_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
//			skp->Arc (cx, cy-d*8.5, cx+d*4, cy-d*6.5, cx, cy-d*6, cx+d*4, cy-d*7.5);
//			skp->Arc (cx-d*4, cy-d*8.5, cx, cy-d*6.5, cx-d*4, cy-d*7.5, cx, cy-d*6);
			skp->Line(cx-d*4, cy-d*8.5, cx-d*3, cy-d*7.5);
			skp->LineTo(cx-d*2, cy-d*7.0);
			skp->LineTo(cx-d, cy-d*6.65);
			skp->LineTo(cx, cy-d*6.5);
			skp->LineTo(cx+d, cy-d*6.65);
			skp->LineTo(cx+d*2, cy-d*7.0);
			skp->LineTo(cx+d*3, cy-d*7.5);
			skp->LineTo(cx+d*4, cy-d*8.5);
			skp->Text (cx-d, cy-d*8, "BAY", 3);
		}
	}



	// show hatch deployment status
	if (hatch_status == DOOR_OPEN || (hatch_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			skp->Text (cx+d*5, cy+d*1.2, "DOOR", 4);
		}
	}

	// show brakes deployment status
	if (BrakesOn) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			skp->Text (cx-d*9, cy+d*1.2, "BRAKES", 6);
		}
	}

	// show RCS mode
	if (oapiCockpitMode() == COCKPIT_VIRTUAL) {
		switch (GetAttitudeMode()) {
		case RCS_ROT:
			skp->Text (0, hps->H-13, "RCS ROT", 7);
			break;
		case RCS_LIN:
			skp->Text (0, hps->H-13, "RCS_LIN", 7);
			break;
		}
	}


// Draw Flight Director
	if ((!MECO_Cond) && (stage < 2)) 
		if ((stage != -3) || (jets_status != DOOR_OPEN)) {
	double fYawError = Yaw_Error*RAD;
	double fBankError = Bank_Error;
	double fPitchError = Pitch_Error;

/*	VECTOR3 rloc = _V(0,0,0);
	HorizonInvRot(rhorizon,rloc);

	double currHdg;
	oapiGetHeading(GetHandle(),&currHdg);
	double fYawError = rhorizon.x - currHdg;
	double fPitchError = rhorizon.y - GetPitch();
	double fBankError = rhorizon.z - (GetBank()- (2*PI));
*/

	double InnCoord = 0.5;
//	if (guidance) InnCoord = 0.25;

	int iX = hps->CX + (int)(fYawError * hps->Scale);
	int iY = hps->CY - (int)(fPitchError * hps->Scale);
	int iR = hps->Markersize / 2;
	if (!guidance)
		skp->Ellipse(iX - iR, iY - iR, iX + iR, iY + iR);


	//two lines for indicating Roll Error
	int iX1 = (int)(1.0 * hps->Markersize * cos(fBankError));
	int iX2 = (int)(InnCoord * hps->Markersize * cos(fBankError));
	int iY1 = (int)(1.0 * hps->Markersize * sin(fBankError));
	int iY2 = (int)(InnCoord * hps->Markersize * sin(fBankError));
	skp->MoveTo(iX + iX1, iY + iY1);
	skp->LineTo(iX + iX2, iY + iY2);

	skp->MoveTo(iX - iX1, iY - iY1);
	skp->LineTo(iX - iX2, iY - iY2);

	//Top Marker for indicating Roll Error
	int iX3 = (int)(0.8 * hps->Markersize * sin(fBankError));
	int iX4 = (int)(InnCoord * hps->Markersize * sin(fBankError));
	int iY3 = (int)(0.8 * hps->Markersize * cos(fBankError));
	int iY4 = (int)(InnCoord * hps->Markersize * cos(fBankError));
	skp->MoveTo(iX + iX3, iY - iY3);
	skp->LineTo(iX + iX4, iY - iY4);

	// draw a diamond, not a circle
	if (guidance) {
		skp->MoveTo(iX + iX4, iY - iY4);
		skp->LineTo(iX + iX2, iY + iY2);
		skp->LineTo(iX - iX4, iY + iY4);
		skp->LineTo(iX - iX2, iY - iY2);
		skp->LineTo(iX + iX4, iY - iY4);
	}
	}
	return true;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ORIONIII (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ORIONIII*)vessel;
}

