/**
*
* $Id: RocketBus.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2005 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/cvelmessage.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include <cstdio>
#include <cmath>
#include <string>


#define InitFuelLevel 1

#define PLAYTREADS						 27     // the tank noise sound
#define MYRADARSOUND				  2     // Custom radar sound
#define MYRADARCLOSE				  3     // Custom radar sound


// All data from astronautix.com
const double ISP_FIRST_SL		=     444.0*G; 
const double ISP_FIRST_VAC		=     444.0*G; //***
const double THRUST_FIRST_VAC	=   11000.0*G/3.0; //***
const double MAX_ATT_FIRST		=     450.0*G;
const double FIRST_PROP_MASS	=   25500.0; //*** 
const double ROCKETBUS_MAX_BURN	=     625.0; //***
const double ROCKETBUS_DRYMASS	=    5850.0; //***

const VECTOR3 EVA_OFS			= _V(0,0.0,-8.5);

const double HATCH_OPERATING_SPEED = 0.125;


/* make your own vessel class (named ROCKETBUS here, for example)
   subclassed from COMMON_VESSEL_EX */

class ROCKETBUS : public COMMON_VESSEL_EX {
//class ROCKETBUS : public VESSEL2 {
public:
	ROCKETBUS (OBJHANDLE hObj, int fmodel);
	~ROCKETBUS();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } hatch_status;

	void RussianBeeps();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void MoveEVA();
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void SeparateEVA();
	void ActivateHatch (DoorStatus action);
	void RevertHatch();
	void UpdateAnimations();


	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 

	void SetAttControlsRocketBus();    // this is to set up the thrusters. I like to keep them separate.
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);

protected:
	PROPELLANT_HANDLE ph_first;
	THRUSTER_HANDLE   th_first[3], th_fwd[2];
	THGROUP_HANDLE    thg_first,thg_fwd,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hRocketBus;
	SURFHANDLE		  tex;
	bool KEY1, KEY2, KEY3, KEY4, KEY5, KEY6, KEY7, KEY8, KEY9;
	double simdt, lastt, simt;

	UINT anim_hatch;	// handle for aft hatch animation
	double hatch_proc;
	int MyID;

	OBJHANDLE hvessel;
	VECTOR3 gpos, grms;
	OBJHANDLE hV;
	bool RadarFlagForSound;
	bool RadarFlagForClose;



};







/* basic vessel class setup */

void ROCKETBUS::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void ROCKETBUS::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void ROCKETBUS::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void ROCKETBUS::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void ROCKETBUS::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void ROCKETBUS::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int ROCKETBUS::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ROCKETBUS (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ROCKETBUS*)vessel;
}

