/**
*
* $Id: CVEL-StationV.cpp,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "StationV.h"

STATIONV::STATIONV (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{


	odds = true;
	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "STATIONVEARLY" ) ) {
		hStationV = oapiLoadMeshGlobal ("2001/StationVEarly");
	} else if( strstr( classname, "STATIONVCOMP" ) ) {
		hStationV = oapiLoadMeshGlobal ("2001/StationVComp");
	} else if( strstr( classname, "STATIONIV" ) ) {
		hStationV = oapiLoadMeshGlobal ("2001/StationIV");
	} else if( strstr( classname, "LANGE" ) ) {
		hStationV = oapiLoadMeshGlobal ("2001/Lange Station");
		odds = false;
	} else
		hStationV = oapiLoadMeshGlobal ("2001/StationV");

	hDockSet[0] = oapiLoadMeshGlobal ("2001/DockSet1");
	if (odds) hDockSet[1] = oapiLoadMeshGlobal ("2001/DockSet2");
	hDockSet[2] = oapiLoadMeshGlobal ("2001/DockSet3");
	if (odds) hDockSet[3] = oapiLoadMeshGlobal ("2001/DockSet4");
	hDockSet[4] = oapiLoadMeshGlobal ("2001/DockSet5");
	if (odds) hDockSet[5] = oapiLoadMeshGlobal ("2001/DockSet6");

	SkipDockProcess = true;
	for(i = 0 ; i < 14 ; i++ )
		dockedobj[i] = NULL;

	dock_point[0] = CreateAttachment(false, _V(0,0,0), _V(0,0,0), _V(0,0,0), "DOCK");
	dock_point[1] = CreateAttachment(false, _V(0,0,0), _V(0,0,0), _V(0,0,0), "DOCK");

	SetUndockRequest = -1;
	EarlyCycle = 0;

	SetDockMode(0);

//	tex = oapiRegisterExhaustTexture("Exhaust_atsme");

	/* set the propellant handle to NULL so I can check if I need to 
	/* create it in the SetMainStage() function. */
//	ph_first=NULL; 

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();   
}


STATIONV::~STATIONV()
{

}


void STATIONV::TimeStep(  double simt )
{
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */

//	int i;

	dt = simt - oldsimt;
	oldsimt = simt;

	if( CheckCentralMessage() )
		SetMainStage();

	if ((EarlyCycle > -1) && (EarlyCycle < 10)) {		// We want to be sure to skip the first few cycles
		SkipDockProcess = true;							// So that if a scenario file puts something on the centerline,
		EarlyCycle++;									// we don't move it.
	}
	else {
		if (EarlyCycle > 0) {
			SkipDockProcess = false;
			SetMainStage();
		}
		EarlyCycle = -1;
	}

//	if (SkipDockProcess)						
//		if ((simt > 10*dt) && (simt < 15*dt))
//			SkipDockProcess = false;					// We want to skip the first time, but not any subsequent

	if (SetUndockRequest >= 0) {						// Release the center docking port
		Undock(SetUndockRequest);						// Sometimes it 'sticks'; this fixes it
		SetUndockRequest = -1;
//		DetachChild(dock_point[0],-0.5);											// Unhook
//		DetachChild(dock_point[1],-0.5);											// Unhook
	}


}


void STATIONV::DockEvent(  int dock, OBJHANDLE connected)
{
	int j;

	if (EarlyCycle > -1)																	// Still in .scn loading process -- don't mess
		return;

	if (SkipDockProcess) {																	// We skip this process on command,
		SkipDockProcess = false;															// such as when we change docks in this routine 
		return;																			 
	}

	VECTOR3 dock_pos, dock_dir, dock_rot;
	VECTOR3 dockpos_rec, dockdir_rec, dockrot_rec;
	VESSEL *v;
	ATTACHMENTHANDLE  dock_receiver;

	dock_receiver = NULL;


	if ((dock < 2) && (connected != NULL)) {												// Something docked to a centerline port
		v = oapiGetVesselInterface (connected);												// Get the interface of the newly docked vessel
	
		if ((v->GetSize() > 8) && (v->GetSize() != 50)) {									// Normal-sized ships (the pod is 50, don't ask me why)
			if (DockingStatus(dock+2) == 0)
				targetdock = dock+2;														// Set target port to the +X side port
			else if (DockingStatus(dock+4) == 0)
				targetdock = dock+4;														// +X is taken, move to -X instead
			else return;																	// No room at the inn, so leave on centerline port
		}
		else {																				// Special routine for the Moonbus and pod; small docking ports
			if (DockingStatus(dock+6) == 0)
				targetdock = dock+6;														
			else if (DockingStatus(dock+8) == 0)
				targetdock = dock+8;
			else if (DockingStatus(dock+10) == 0)
				targetdock = dock+10;														
			else if (DockingStatus(dock+12) == 0)
				targetdock = dock+12;	
			else return;
		}

		GetDockParams(GetDockHandle(dock), dock_pos, dock_dir, dock_rot);
		SetAttachmentParams(dock_point[dock], dock_pos, dock_dir, dock_rot);				// Reset the station's dock_pont to equal the centerline docking port
		v->GetDockParams(v->GetDockHandle(0), dockpos_rec, dockdir_rec, dockrot_rec);

		int docknum = -1;
		int nAttach = v->AttachmentCount (true);
		for (j = 0; j < nAttach; j++) {														// now scan all attachment points of the candidate
			ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
			const char *id = v->GetAttachmentId (hAtt);
			if (!strncmp (id, "DOCK", 4))													// Found one!  Let's re-use it
				docknum = j;
		}

		if (docknum == -1)
			dock_receiver = v->CreateAttachment(true, dockpos_rec, dockdir_rec, dockrot_rec, "DOCK");
		else dock_receiver = v->GetAttachmentHandle (true, docknum);						// Create/reset the target's dock_receiver to equal its centerline docking port (0)

																							// NOTE:  If not docking on port 0, unpredictable (bad) results will ensue!
		AttachChild(connected,dock_point[dock],dock_receiver);								// Hook on!																		
		v->Undock(0);																		// Undock from the centerline port
		GetDockParams(GetDockHandle(targetdock), dock_pos, dock_dir, dock_rot);
		SetAttachmentParams(dock_point[dock], dock_pos, dock_dir, dock_rot);				// Move the attach points over to the new port
		SetUndockRequest = dock;															// Release the center docking port next timestep
	}

	else if ((dock > 1) && (connected == NULL)) {											// Something just undocked from a non-centerline port!
		div_t divresult;																	// This must not be allowed!
		divresult = div(dock,2);															
		if (divresult.rem == 1)
			targetdock = 1;																	// It came from an aft port; move it to port 1
		else targetdock = 0;																// It came from a forward port; move it to port 0
		bool CenterTaken = false;
		if (DockingStatus(targetdock) != 0)													// Centerline port is taken!  Set target to equal source -- no movement!
			CenterTaken = true;

		v = oapiGetVesselInterface (dockedobj[dock]);										// Get our target vessel's interface from the docked list
																							// Hope the list is good!  Can't get the objhandle directly, since it's undocked

		GetDockParams(GetDockHandle(dock), dock_pos, dock_dir, dock_rot);
		SetAttachmentParams(dock_point[targetdock], dock_pos, dock_dir, dock_rot);
		v->GetDockParams(v->GetDockHandle(0), dockpos_rec, dockdir_rec, dockrot_rec);

		int docknum = -1;
		int nAttach = v->AttachmentCount (true);
		for (j = 0; j < nAttach; j++) {														// now scan all attachment points of the candidate
			ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
			const char *id = v->GetAttachmentId (hAtt);
			if (!strncmp (id, "DOCK", 4))													// Found one!  Let's re-use it
				docknum = j;
		}
	
		if (docknum == -1)
			dock_receiver = v->CreateAttachment(true, dockpos_rec, dockdir_rec, dockrot_rec, "DOCK");
		else dock_receiver = v->GetAttachmentHandle (true, docknum);						// Create/reset the target's dock_receiver to equal its centerline docking port (0)

				
		if (GetAttachmentStatus(dock_point[targetdock]) != dockedobj[dock])
			AttachChild(dockedobj[dock],dock_point[targetdock],dock_receiver);				// Hook on!
		if (CenterTaken)																	// Centerline port is taken, so we've re-attached	
			return;																			// Our work is done
		SkipDockProcess = true;
		Undock (dock);
		GetDockParams(GetDockHandle(targetdock), dock_pos, dock_dir, dock_rot);
		SkipDockProcess = true;
		SetAttachmentParams(dock_point[targetdock], dock_pos, dock_dir, dock_rot);			// Move the attach points over to the new port
		DetachChild(dock_point[targetdock],0.0);											// Unhook
	}

	SetMainStage();																			// Reset + get new docked list
}

/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */

void STATIONV::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void STATIONV::SetClassCaps(  FILEHANDLE cfg )
{
	/* ensure the propellent handle is set to NULL before creation */
	ph_first=NULL;
}

void STATIONV::SaveState( FILEHANDLE scn )
{
	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void STATIONV::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		ParsePayloadString(line);


		ParseScenarioLineEx (line, vs);
	}

	/* always set the stage whenever something may have changed */
	SetMainStage();
}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void STATIONV::SetMainStage( )
{

	const VECTOR3 VECTOR_ZERO = _V(0,0,0);

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
//	ClearThrusterDefinitions();
/*
	PARTICLESTREAMSPEC contrail = 
	{
		0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
*/
//	SetSize (4);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
/*	SetEmptyMass ( STATIONV_DRYMASS + CalculateMass() );


	SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
	SetPMI (_V(17,17,1.5));
	SetCrossSections (_V(4,4,2.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 

*/	

	ClearMeshesEx();

	/* we're not using COMMON_MESH today, so draw our own */
	AddMesh(hStationV, &VECTOR_ZERO );

	VECTOR3 dock_pos, dock_dir, dock_rot,add_dock;
	for(int  j = 0 ; j < 6 ; j++ ) {
		GetDockParams(GetDockHandle(j), dock_pos, dock_dir, dock_rot);
		if ((j == 0) || (j == 2) || (j == 4)) {
			add_dock = dock_pos + _V(0,0,-8.00);
			AddMesh(hDockSet[j], &add_dock);
		} else {
			add_dock = dock_pos + _V(0,0, 8.00);
			if (odds) AddMesh(hDockSet[j], &add_dock);
		}
	}

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	Draw();

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */
/*	if (ph_first==NULL)  
		ph_first  = CreatePropellantResource(FIRST_PROP_MASS,FIRST_PROP_MASS*InitFuelLevel,1.0); 


	SetDefaultPropellantResource (ph_first); 

	th_first[0] = CreateThruster ( _V(0.72,0,-9), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[0], _V(0.72,0,-9), &contrail);
	AddExhaustStream (th_first[0], _V(0.72,0,-9), &exhaust_main);

	th_first[1] = CreateThruster ( _V(-0.72,0,-9), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[1], _V(-0.72,0,-9), &contrail);
	AddExhaustStream (th_first[1], _V(-0.72,0,-9), &exhaust_main);

	thg_first = CreateThrusterGroup (th_first, 2, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 5.0, 0.5,tex);
	AddExhaust (th_first[1], 5.0, 0.5,tex);

	SetAttControlsStationV();
*/	EnableTransponder(true);

	for(int  i = 0 ; i < 14 ; i++ )
		dockedobj[i] = GetDockStatus(GetDockHandle(i));

}





/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */
int STATIONV::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	int result = 0;
	if(!fPressed && !KEYMOD_SHIFT(keystate) && !KEYMOD_CONTROL(keystate) && !KEYMOD_ALT(keystate))
	{
		switch(dwKey)
		{ 
		case OAPI_KEY_J:
			result = 1;

			/* CVE-lite function ! */
			/* IMPORTANT */
			/* call DeployPayload() to jettison the GPDS payloads in sequence */

			DeployPayload();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();

			break;

		}
	}
	return result;
}



int STATIONV::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}




void STATIONV::SetAttControlsStationV()
{
	THRUSTER_HANDLE th_rot[2];
	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);
	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);

	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_FIRST, ph_first, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}
