/**
*
* $Id: StationV.h,v 0.1 2006/04/06 Sputnik Exp $
* (c) 2006 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

// All data from astronautix.com
const double ISP_FIRST_SL		=     444.0*G; 
const double ISP_FIRST_VAC		=     444.0*G; //***
const double THRUST_FIRST_VAC	=    7485.0*G; //***
const double MAX_ATT_FIRST		=     250.0*G;
const double FIRST_PROP_MASS	=   21105.0; //*** the total mass of fueled StationV is 23880kg
const double STATIONV_MAX_BURN	=     625.0; //***
const double STATIONV_DRYMASS	=    2775.0; //***


/* make your own vessel class (named STATIONV here, for example)
   subclassed from COMMON_VESSEL_EX */

class STATIONV : public COMMON_VESSEL_EX {
public:
	STATIONV (OBJHANDLE hObj, int fmodel);
	~STATIONV();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void DockEvent(  int dock, OBJHANDLE connected ); // need a timestep loop to conduct the GPDS polling
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 

	void SetAttControlsStationV();    // this is to set up the thrusters. I like to keep them separate.
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkDockEvent (int dock, OBJHANDLE connected);

protected:
	PROPELLANT_HANDLE ph_first;
	THRUSTER_HANDLE   th_first[2];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hStationV, hDockSet[6];
	SURFHANDLE		  tex;
	double			  dt, oldsimt;
	int			      targetdock;
	int				  SetUndockRequest;
	int				  EarlyCycle;
	bool			  SkipDockProcess, recheckdocks, odds;
	OBJHANDLE		  dockedobj[16];
	ATTACHMENTHANDLE  dock_point[2];
};






/* basic vessel class setup */

void STATIONV::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void STATIONV::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void STATIONV::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void STATIONV::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void STATIONV::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void STATIONV::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void STATIONV::clbkDockEvent (int dock, OBJHANDLE connected)
{
	DockEvent(dock,connected );
}

int STATIONV::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new STATIONV (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (STATIONV*)vessel;
}

