/**
*
* $Id: Titov V.cpp
* (c) 2005 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* Autopilot by rjcroy
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "TitovV.h"


// 1. vertical lift component (wings and body)
// Borrowed from Mike Norman's Shuttle code

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5,  0.02, 0.6955,    0.66,   0.42, 0.24, 0.10, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,     0,  0.002,  0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	// change for drag increase at high Mach:
	// Subsonic Equivalents

	double Cl0, Cl1, Cd0, Cd1, Cd2;
	if(M>=5.0) {
		Cl0 = -0.187040;
		Cl1 = 1.675600;
		Cd0 = 0.068540;
		Cd1 = -0.21767;
		Cd2 = 1.3632;
		*cl = Cl0 + Cl1*aoa;
		*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
 	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
	if(M<1.25) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.03 + 0.3*oapiGetInducedDrag (*cl, 2.1, 0.82);
	}
	else if(M<5.0) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.05 + 0.8*oapiGetInducedDrag (*cl, 2.1, 0.65);
// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
	}
}

/*
void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, 0*RAD, 15*RAD,25*RAD,30*RAD,60*RAD,180*RAD};
	static const double CL[nabsc]  = {       0,      0,   -0.4,     0,    0.7,     1,   0.8,     0,      0};
	static const double CM[nabsc]  = {       0,      0,  0.014, 0.004,-0.0074,-0.094,-0.012,     0,      0};
	for (int i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = CM[i] + (CM[i+1]-CM[i]) * f;  // aoa-dependent moment coefficient
	double saoa = sin(aoa);
	double pd = 0.015 + 0.4*saoa*saoa;  // profile drag
	*cd = pd + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	// profile drag + (lift-)induced drag + transonic/supersonic wave (compressibility) drag
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}
*/

// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}


																					
void TITOVV::InitialiseGuidance()
{
	

//	time=0.0;

	pitchinput = 0.0;
	rollinput = 0.0;
	engineignited = false;
	last_simt=0.0;
	ignition_time=0.0;
	first_guidance_pass = true;

	// rjcroy Guidance variables

	INCLINATION = 51.63 ;		// degrees

	guidance = false;			// guidance switch
	Alt_Flag = false;			// True if max launch altitude reached
	MECO_Cond = false;			// True if MECO conditions met
	First_Guidance_Pass = false;
	T_minus_10 = true;			// True at T minus 10 seconds.
	TZero = -100;				// Time Zero - Launch time
	PC_D = 0.9;					// Pitch control coefficient D for d_theta /d_t
	PC_K = 0.2;					//    "    "         "       K  "  theta
	YC_D = 0.5	;				// Yaw Control
	YC_K = 0.3;
	PC_D_FINAL = 0.9;			// Pitch control coefficients for final section.
	PC_K_FINAL = 0.6; 
	D_FINAL  = 0.8;				// Vertical velocity control coefficients
	K_FINAL = 0.8;
	ENG_THRST_2 = 0 ;		// Thrust rating of Stage 2 engine. (N)  
	i_last = 0;					// value of incliniation in previous iteration.
//	Planet_Name = new char[6];
//	strcpy( Planet_Name, "Earth");
	R_EARTH = 6371000;			// radius of the earth (m)
	U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
	Ob_EARTH = 23.45 * RAD ;	// Obliquity of the Earth (rad)
	APO_ALT = 300 ;				// Altitude of apoapsis
//	APO_ALT = 1705;

	//  inclination correction factor constant. (3580)
	ICF_K = 62.008 * INCLINATION*INCLINATION 
			- 6848.7*INCLINATION + 189807;	

	DisplayIncSP = false;		// toggle for displaying the inclination setpoint.

	NewIncSP = false ;			// True if a user has entered a new inclination SP
	PostMECO = false ;			// switch for cleaning up after MECO

	BadHeading = false;			// true if soyuz is at an azimuth not suitable for the G&CS



}

double TITOVV::GetCPitch(double t)
{
	int i;
	if (t>met[N-1]) return cpitch[N-1];
	i=1;
	while (met[i]<t) i++;
	return cpitch[i-1]+(cpitch[i]-cpitch[i-1])/(met[i]-met[i-1])*(t-met[i-1]);
}

// rjcroy Lateral Guidance System ( called by Guidance_Routine() ).

void TITOVV::Lateral_Guidance(VESSEL *vessel, double SimT, double SimT_Last)
{
	// Lateral Guidance System 
/*
		double hdg, diff, rate, amount, targetrate, TgtHdg;

		if (MET > 35.0) {
			targetrate = ((GetBank()*180/PI)-180)/-1.0;
			if (GetBank()<0) targetrate = -1.0*targetrate;

			if (MET > 55.0) {

				oapiGetHeading(GetHandle(),&hdg);
//				hdg = vst.arot.y * 180/PI;
				hdg *= 180/PI;
//				hdg += 180;

				GetRelativeVel (hEarth, Rel_Vel);
				GetEquPos( Longitude, Latitude, Radius );
				Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));

				oapiGetRelativeVel(hVessel, hEarth, &Rel_Vel);
				oapiGetRelativePos(hVessel, hEarth, &Rel_Pos);
				
				// Change ecliptic rel. vectors to equatorial vectors.
 				temp.x = Rel_Vel.x;
				temp.y = Rel_Vel.z;
				temp.z = Rel_Vel.y;
				veccpy( Rel_Vel, temp);
				temp.x = Rel_Pos.x;
				temp.y = Rel_Pos.z;
				temp.z = Rel_Pos.y;
				veccpy( Rel_Pos, temp);
				temp.x = Rel_Pos.x ;
				temp.y = cos(-Ob_EARTH)* Rel_Pos.y + sin(-Ob_EARTH)*Rel_Pos.z;
				temp.z =  -sin(-Ob_EARTH)* Rel_Pos.y + cos(-Ob_EARTH)*Rel_Pos.z;
				veccpy( Rel_Pos, temp);
				temp.x = Rel_Vel.x ;
				temp.y = cos(-Ob_EARTH)* Rel_Vel.y + sin(-Ob_EARTH)*Rel_Vel.z;
				temp.z =  -sin(-Ob_EARTH)* Rel_Vel.y + cos(-Ob_EARTH)*Rel_Vel.z;
				veccpy( Rel_Vel, temp);

				h_vector = crossp(Rel_Vel, Rel_Pos);
				h_abs = sqrt((h_vector.x*h_vector.x) + (h_vector.y*h_vector.y) + (h_vector.z*h_vector.z));
				i = 180 - acos(h_vector.z / h_abs) * DEG;
				i_dot = (i - i_last) / (MET - SimT_Last) ;
				i_last = i;
				ICF = Velocity / ICF_K;

				// Calculated error in inclination as output signal
				i_err = (i - INCLINATION) * (150 * ICF) + 250 * ICF * i_dot;
				if (i_err > ICF )
				{	
					i_err = ICF;
				}
				else if ( i_err < -ICF) 
				{
					i_err = -ICF;
				}
							
				double Phrad;
					//= (PadHeading-180)*PI/180;

				if ((Rel_Vel.z < 110.0) || (bgoingsouth)) {
					Phrad = PI/2 + (PI/2 - asin(cos(INCLINATION*PI/180)/cos(Latitude)));
					if ((PadHeading-180) > 180)
						Phrad = 3*PI/2 + (3*PI/2 - asin(cos(INCLINATION*PI/180)/cos(Latitude)));
					TgtHdg = Phrad*DEG;
					TgtHdg -= i_err;
					bgoingsouth = true;
				}
				else {
					Phrad =  asin(cos(INCLINATION*PI/180)/cos(Latitude));
					TgtHdg = Phrad*DEG;
					TgtHdg += i_err;
				}

				if ((TgtHdg > 90.0) || (PadHeading-180 >= 90.0))
					bgoingsouth = true;

				if ((Latitude*DEG >= i*0.98) && (fabs(i-INCLINATION)<0.02))
					bgoingsouth = true;

//				sprintf(oapiDebugString(),"Latitude %.2f  inc %.2f TGTINC %.2f going %1d",Latitude*DEG, i, INCLINATION, bgoingsouth);
//				sprintf(oapiDebugString(),"TgtHdg %.2f  inc %.2f TGTINC %.2f Phrad %.2f PadHdg %.2f tmpy %.2f",TgtHdg, i, INCLINATION, Phrad*DEG,PadHeading-180,Rel_Vel.z);

				diff = hdg - (TgtHdg);
//				if (ROLL < 90.0) diff = hdg - TgtHdg + 180 - ROLL;
//				diff = hdg - (PadHeading - 180);
				if( diff > 180.0 )
					diff -= 360.0;
				else if( diff < -180.0 )
					diff += 360.0;
				targetrate = diff/+1.0;
			}

			if( targetrate > 8.0 )
				targetrate = 8.0;
			else if( targetrate < -8.0 )
				targetrate = -8.0;


			rate = vst.vrot.y * 180/PI;
			amount = fabs (targetrate/8.0);
			if (MET > 0) amount = amount / (MET/10);

//		sprintf(oapiDebugString(),"Bank %.1f Heading %.1f TgtHdg %.1f diff %.1f Targetrate %.1f",GetBank()*180/PI,hdg,TgtHdg,diff, targetrate);

			if( fabs( rate-targetrate ) > 0.05 ) {
				if( rate < targetrate ) {
//					IncThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.05 );
//					SetThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.0 );
					yawinput = -1*amount;
				}
				else {
//					IncThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.05);
//					SetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.0 );
					yawinput = amount;
				}
			}
			else {
//				SetThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.0 );
//				SetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.0 );
				yawinput = 0.0;
			}
		}
*/
}

// rjcroy Guidance & Control System
void TITOVV::Guidance_Routine(VESSEL *vessel, double SimT, double SimT_Last)
{

	hVessel = GetHandle();
	hEarth = GetSurfaceRef ();


	if (T_minus_10)
	{
		// Record launch time
		if (TZero == 0){
			TZero = SimT + 1;
		}

		if (MET > -5 && MET < -3) {
//			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0.3);
//			LaunchClamps();
		}
		else if (MET > -3 && MET < 0) {
//			oapiSetEngineLevel(hVessel, ENGINE_MAIN, (0.2*MET + 1));
//			LaunchClamps();
			INCLINATION = 180 - acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG;
		}
		else if (MET > 0)
		{
			T_minus_10 = false;
			First_Guidance_Pass = true;
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);

			GetStatus(vst);
	
			initialgamma = vst.arot.z * 180/PI;
			launchrotate =	PadHeading - initialheading - 180 + ROLL;

			if( launchrotate < -180.0 )
				launchrotate += 360.0;
			else if( launchrotate > 180.0 )
				launchrotate -= 360.0;

		}
		sprintf(oapiDebugString(),"T - %.1f", fabs(MET));
	}		

	else if (First_Guidance_Pass) 
	{
		
		// Start the main engines
		oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0.01);

		// Unset the first pass flag
		First_Guidance_Pass = false;
	}

	else if (!Alt_Flag) 
	{
		if ((GetEngineLevel(ENGINE_MAIN) == 0.01) && (GetAltitude() > 10.0))
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);

		if (oapiGetFocusObject() == GetHandle())
			sprintf(oapiDebugString(),"T + %.1f", fabs(MET));

		if (initialgamma == 0.0)
			initialgamma = vst.arot.z * 180/PI;

		// Get correct pitch from table
		Pitch_SP=GetCPitch(MET);
		MJD_Time = oapiGetSimMJD();
		GetElements( elements, MJD_Time );
		Alt_Apoapsis = (elements.a * ( 1 + elements.e )/1000) - 6371; 
		if (Alt_Apoapsis > 100.0) {
			GetHorizonAirspeedVector(horiz);

			Pitch_SP = Pitch_SP/4.0;			// Flydown -- we've met our altitude goal
			double Pitch_zero = -0.02*horiz.y;

			if (ROLL < 90.0)
				if ((GetAOA()*DEG > -1.0))
					Pitch_SP += Pitch_zero+0.9*GetAOA()*DEG;
			else
				if ((GetAOA()*DEG < 1.0))
					Pitch_SP += Pitch_zero-0.9*GetAOA()*DEG;
		}

		if (Pitch_SP < 0.0)
			Pitch_SP = 0.0;

//		if (oapiGetFocusObject() == GetHandle())
//		sprintf(oapiDebugString(),"MET %.1f Alt_Apoapsis %.1f Pitch_SP %.1f CPitch %.1f",MET,Alt_Apoapsis, Pitch_SP, GetCPitch(MET));

		// Control to pitch setpoint
		Pitch=GetPitch(  );
		Pitch = Pitch * DEG;
		Pitch_Error = Pitch - Pitch_SP;
		if (ROLL < 90.0)
			Pitch_Error = Pitch_SP - Pitch;

		GetAngularVel( Angular_Velocity ); 
		Pitch_Error_Dot = Angular_Velocity.x * DEG; // get pitch change rate in degrees/s

		Pitch_Cntrl_Sig =  - PC_D * Pitch_Error_Dot + PC_K * Pitch_Error;
//		if ((MET > 10.0) &&(GetBank() < PI/2) && (GetBank() > -PI/2))
//			Pitch_Cntrl_Sig *= -1.0;

		if (Pitch_Cntrl_Sig > 1 )
		{
//			SetAttitudeRotLevel( 0, 1); // 0 is y-axis
			pitchinput =-1.0;
		}
		else if (Pitch_Cntrl_Sig < -1) 
		{
//			SetAttitudeRotLevel( 0, -1);
			pitchinput = +1.0;
		}
		else {
//			SetAttitudeRotLevel( 0 , Pitch_Cntrl_Sig);
			pitchinput = -Pitch_Cntrl_Sig;
		}
		
		if( MET > 50.0) pitchinput *= 0.5;

		Yaw_Error = 0.0;

//		if (Pitch > 45.0) pitchinput = +1.0;
		
//		SetControlSurfaceLevel(AIRCTRL_ELEVATOR,-1.0*pitchinput);


		// Heading control
//		if (ROLL > 90.0) Lateral_Guidance(this, SimT, SimT_Last);


		// Roll control
		// Adapted from Hendo's Ariane V

//		if (MET < 2.0) SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);
		if (MET < 35.0) pitchinput = -2.0;
//			SetControlSurfaceLevel(AIRCTRL_ELEVATOR,1.0);
	

		if( MET > 0.0) {
				
//		vst.version = 2;
//		vst.flag = 0;
		vessel->GetStatus(vst);


		target = initialgamma - launchrotate;

		if( target > 180.0 )
			target -= 360.0;
		else if( target < -180.0 )
			target += 360.0;
					
		diff = vst.arot.z*180/PI - target;
		if( diff > 180.0 )
			diff -= 360.0;
		else if( diff < -180.0 )
			diff += 360.0;

		if (MET > 30.0) diff = 0.0;
		if (ROLL < 90.0) diff = 0.0;

		diff = GetBank()*180/PI + target;
		if (ROLL < 90.0) diff = -1.0*GetBank()*180/PI;

		if ((ROLL < 90.0) && (MET < 110) && (MET > 5)) {
			oapiGetHeading(GetHandle(),&hdg);
			hdg = hdg*180/PI + 180.0;
			if (hdg < 0.0) hdg += 360.0;
			if (hdg > 450.0) hdg -= 360.0;
			diverge = (hdg-PadHeading)*1.7;
			diverge -= 0.8*GetBank()*180/PI;
			if (GetAltitude() < 150.0) {
				if (diverge > 30.0) diverge = 30.0;
				if (diverge < -30.0) diverge = -30.0;
			} else {
				if (diverge > 50.0) diverge = 50.0;
				if (diverge < -50.0) diverge = -50.0;
			}
			diff += diverge;
		}

		if (MET > 110) {
			diff = -1.0*GetBank()*180/PI;
			yawangle = DEG*GetSlipAngle();
			yawinput = yawangle / 15.0;
			if (ROLL > 90.0) yawinput *= -1.0;
		}
//		sprintf(oapiDebugString(),"hdg %.1f diverge %.1f PadHeading %.1f diff %.1f ",hdg,diverge,PadHeading,diff);

//		sprintf(oapiDebugString(),"initialhdg %.1f initialgamma %.1f launchrotate %.1f target %.1f diff %.1f TGT_ %.1f",initialheading,initialgamma, launchrotate, target,diff, PadHeading);

		Yaw_Error = diff;


		targetrate = diff/-1.0;
		if( targetrate > 8.0 )
			targetrate = 8.0;
		else if( targetrate < -8.0 )
			targetrate = -8.0;

		Bank_Error = RAD*targetrate;
		rate = vst.vrot.z * 180/PI;
		amount = fabs (targetrate/8.0);
		if (MET > 0) amount = amount / (MET/10);

//		if (amount > 0.05)
			rollinput = diff/25.0;
//		else
//			rollinput = 0.0;
/*

		if( fabs( rate-targetrate ) > 0.05 ) {
			if( rate < targetrate ) {
//				IncThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.05 );
//				SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
				rollinput = -1*amount;
			}
			else {
//				IncThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.05);
//				SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
				rollinput = amount;
			}
		}
		else {
//			SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
//			SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
			rollinput = 0.0;
		}
*/		
//		if ((MET > 60.0) && (diff < 0.5))

//		if (MET > 60.0)
//			Lateral_Guidance(this, SimT, SimT_Last);
//		else
//			yawinput = 0.0;

//		GetAngularVel(AngularVel);

//		yawinput=(-20*AngularVel.y);		

		// END Hendo's roll control


		}

		Altitude=GetAltitude();
		a = ((APO_ALT * 1000 + R_EARTH) + (Altitude + R_EARTH)) / 2;
		e = ((APO_ALT * 1000 + R_EARTH) / a ) - 1;
		h = sqrt(U_EARTH * a * ( 1 - (e * e)));
		V_periapsis = h / (Altitude + R_EARTH);

		
		// Get current velocity
		GetRelativeVel (hEarth, Rel_Vel);
		Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));

		if ((MET > 180) && (Velocity > V_periapsis)) 
		{
			MECO_Cond = true;
			TimeMECO = SimT;
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0);
			Guidance(false);
			sprintf(oapiDebugString(), "");
			pitchinput = 0.0;
			yawinput = 0.0;
			rollinput = 0.0;
		}

		
	}

	else if (MECO_Cond && PostMECO) 
	{
		if ((SimT - TimeMECO) < 10) {
			sprintf(oapiDebugString(), "Guidance OFF.  Inclination (Equatorial): %.3f", i); 
		}	
		else {
			guidance = false;
			sprintf(oapiDebugString(), "");
		}
	}
}
// END Guidance_Routine()


void TITOVV::GetInclinationSP()
{
	// Gets a new Inclination setting from the pilot
	bool AzInput (void *id, char *str, void *data);
	
//	oapiOpenInputBox ("Inclination Setpoint, e.g. 5163 = 51.63 deg (47 to 56)", IncInput, 0, 25);
	oapiOpenInputBox ("Enter launch azimuth, 0 to 90 (east) to 360", AzInput, 0, 20, (void*)this);

//	GetEquPos( Longitude, Latitude, Radius );
//	sprintf(oapiDebugString(), "Target inclination (Equatorial): %.3f", acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG); 
}


bool AzInput (void *id, char *str, void *data)
{
//	sscanf(str, "%4lf", &PadHeading);
//	PadHeading = PadHeading / 100.0;
//	PadHeading += 180.0;
/*	sscanf(str, "%4lf", &INCLINATION);
	if (INCLINATION > 4700 && INCLINATION < 5600 ) {
		INCLINATION = INCLINATION / 100;
		NewIncSP = true;
	}
*/
	return ((TITOVV*)data)->IncInput (str);
}

bool TITOVV::IncInput (char *rstr)
{
	sscanf(rstr, "%5lf", &PadHeading);
//	PadHeading = PadHeading / 100.0;
	PadHeading += 180.0;
	if (PadHeading > 450.0) PadHeading -= 360.0;
	d_heading = PadHeading;
	displaytime = 0.01;
	return true;
}

// END rjcroy Guidance System

TITOVV::TITOVV (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{

	tex = oapiRegisterExhaustTexture("2001/Exhaust_Orion");
	texRCS  = oapiRegisterExhaustTexture ("exhaust_atrcs");

	/* set the propellant handles to NULL so I can check if I need to 
	/* create them in the SetMainStage() function. */
	ph_booster=NULL; 
	ph_first=NULL; 
	ph_TitovV=NULL;
	ph_Jets=NULL;
	AnimationsMade = false;
	bShieldJettisoned=false;
	bDeorbitFired=false;
	bThrustTerminated=false;
	bEjected=false;
	bEVA=false;
	bAttSet=false;
	bBoosterMade=false;
	bAutoProgrammed=false;
	bControlsMade=false;
	gear_status=DOOR_CLOSED;
	gear_proc=0.0;
	pldoors_status=DOOR_CLOSED;
	pldoors_proc=0.0;
	brakes_status=DOOR_CLOSED;
	brakes_proc=0.0;
	hatch_status=DOOR_CLOSED;
	hatch_proc=0.0;
	wings_status=DOOR_OPEN;
	wings_proc=1.0;
	MachPitch=1.0;
	jets_status=DOOR_CLOSED;
	jets_proc=0.0;
	nozzles_status=DOOR_CLOSED;
	nozzles_proc=0.0;
	Vold = _V(0,0,0);
	BrakesOn = false;
	bAbort = false;
	AbortChecked = false;
	numengines = 2;
	dt = 0.0;
	simt = 0.0;
	oldsimt = -0.01;
	displaytime = 0.01;
	SimT_Last = 0.0;
	PadHeading = 270.0;
	d_heading = 270.0;
	last_simt=0.0;
	ignition_time=0.0;
	initialgamma = 0.0;
	pitchinput = 0.0;
	yawinput = 0.0;
	rollinput = 0.0;
	stage = 1;
	atchmass[0] = 0.0;
	atchmass[1] = 0.0;
	atchmass[2] = 0.0;
	atchmass[3] = 0.0;
	strcat (tvmesh,"2001/TitovV");

	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "TITOVVNOTK" ) )
		stage = 1;
	else if( strstr( classname, "TITOVV" ) )
		stage = 0; 
	else if( strstr( classname, "TITOVWB" ) )
		stage = -2;
	else if( strstr( classname, "TITOVB" ) )
		stage = -3;
	else
		stage = 2;

/*  The stages of the Titov V:

  -3	Titov V Booster Stage Only
  -2	Titov V with Booster Stage
  -1	Titov V with Drop Tank AND Booster Stage (Zoiks!)
  0		Titov V with Drop Tank
  1		Titov V upper stage only
  2		Titov V upper stage, big engines disabled (enables OMS engines, wing fold-down)
  3		Titov V upper stage, with jet engine

*/


	//		     			   child       pos           dir       rot      handle
	Dockpos = CreateAttachment (false, _V(0,0,14.0),   _V(0,0,1), _V(0,1,0), "DOCK");

	//		     			   child       pos           dir       rot      handle
	Baypos = CreateAttachment (false, _V(0,0,0.0),   _V(0,0,1), _V(0,-1,0), "X");
//	sat_attach = CreateAttachment (false, OFS_SAT, _V(0,1,0), _V(0,0,1), "X");
	if (stage == -3 ) {
		Nose_atch1 = CreateAttachment (false, _V(0,-10.25,16.0), _V(0,-0.0675,1), _V(0,1,0), "NG");
		Nose_atch2 = CreateAttachment (true, _V(0,-10.25,16.0), _V(0,-0.0675,1), _V(0,1,0), "NG");
	}
	else {
		Nose_atch1 = CreateAttachment (false, _V(0,-2.55,14.0), _V(0,-0.0675,1), _V(0,1,0), "NG");
		Nose_atch2 = CreateAttachment (true, _V(0,-2.55,14.0), _V(0,-0.0675,1), _V(0,1,0), "NG");
	}
	attach[0] = _V(0,0.25,-5.45);
	attachdir[0] = _V(0,0,1);
	attachrot[0] = _V(0,-1,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[0] = CreateAttachment (false, attach[0],   attachdir[0], attachrot[0], "X");

	attach[1] = _V(0,0.17,-2.7);
	attachdir[1] = _V(0,0,1);
	attachrot[1] = _V(0,-1,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[1] = CreateAttachment (false, attach[1],   attachdir[1], attachrot[1], "X");

	attach[2] = _V(0,0.1,0);
	attachdir[2] = _V(0,0,1);
	attachrot[2] = _V(0,-1,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[2] = CreateAttachment (false, attach[2],   attachdir[2], attachrot[2], "X");

	attach[3] = _V(0,0.1,2.7);
	attachdir[3] = _V(0,0,1);
	attachrot[3] = _V(0,-1,0);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[3] = CreateAttachment (false, attach[3],   attachdir[3], attachrot[3], "X");

	attach[4] = _V(0,-1.23,0);
	attachdir[4] = _V(0,1,0);
	attachrot[4] = _V(0,0,1);
	//	  		       			     child   pos           dir           rot      handle
	attachpos[4] = CreateAttachment (false, attach[4],   attachdir[4], attachrot[4], "X");

	MDD_atch = CreateAttachment (true, _V(0,3.0,0.0), _V(0,-1,0), _V(-1,0,0), "XS");



	if (stage > -3) {
		Dock0 = CreateDock(_V(0,0,14.0),_V(0,0,1),_V(0,1,0));
		Dock1 = CreateDock(_V(0,0,5.95),_V(0,0,-1),_V(0,1,0));
		Dock2 = CreateDock(_V( 2.06,0.365,7.1),_V( 0.794, 0.6,0.0978),_V(-0.6,0.794,-0.0978));
	}
	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
//		havesound=false;
//	else
//	{
		havesound=true;
		RequestLoadVesselWave3(MyID,PLAYGEAR,"Sound\\2001\\gear.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYBANG,"Sound\\2001\\stg2sep.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYJETT,"Sound\\2001\\fasep.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,PLAYJET,"Sound\\2001\\jet.wav",BOTHVIEW_FADED_FAR);
		RequestLoadVesselWave3(MyID,PLAYDOORS,"Sound\\2001\\Payload_doors.wav",DEFAULT);
		RequestLoadVesselWave3(MyID,MYRADARSOUND,"Sound\\2001\\rusbeep.wav",INTERNAL_ONLY);
		RequestLoadVesselWave3(MyID,MYRADARCLOSE,"Sound\\2001\\rusclose.wav",INTERNAL_ONLY);
		SoundOptionOnOff3(MyID,PLAYRADARBIP,FALSE);

//	}

	anim_gear = 0.0;
	anim_pldoors = 0.0;
	anim_brakes = 0.0;
	anim_hatch = 0.0;
	anim_wings = 1.0;
	anim_bwings = 1.0;
	anim_jets = 0.0;
	anim_nozzles = 0.0;

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();   

/*
	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);
	retex = oapiRegisterReentryTexture("reentry");
	SetReentryTexture(retex,6e6,1,1);
*/
	MakeAnimations();

	Firstsweep = true;
	//                  alignment      position      func    chord S   AR   
		Wing = CreateAirfoil2 (LIFT_VERTICAL, _V(0,0,-0.4), VLiftCoeff, 11, 145, 3.35);
		// wing and body lift+drag components

		Rudder = CreateAirfoil2 (LIFT_HORIZONTAL, _V(0,0,-8.5), HLiftCoeff, 3.5, 28, 2.6);
		// vertical stabiliser and body lift and drag components

}


TITOVV::~TITOVV()
{

}

void TITOVV::MakeAnimations()
{

	AnimationsMade = true;

	hTITOVV					= oapiLoadMeshGlobal (tvmesh);
	hTITOVVTk				= oapiLoadMeshGlobal ("2001/TitovVTank");
	hTITOVB					= oapiLoadMeshGlobal ("2001/TitovB");
//	hTITOVVTk				= oapiLoadMeshGlobal ("2001/TitovVTank");
//	hSeat					= oapiLoadMeshGlobal ("gem-ejseat");
//	hSeat2					= oapiLoadMeshGlobal ("gem-ejseat");
//	hAstro					= oapiLoadMeshGlobal ("gem-eva1");
//	hAstro2					= oapiLoadMeshGlobal ("gem-eva1");
	hDummy					= oapiLoadMeshGlobal ("2001/dummy");

	
	anim_wings = CreateAnimation (1);
	anim_bwings = CreateAnimation (1);
	anim_gear = CreateAnimation (0);
	anim_pldoors = CreateAnimation(0);
	anim_hatch = CreateAnimation(0);
	anim_jets = CreateAnimation(0);
	anim_nozzles = CreateAnimation(0);
	anim_lelevator = CreateAnimation (0.5);
	anim_relevator = CreateAnimation (0.5);
	anim_lelevatortrim = CreateAnimation (0.5);
	anim_relevatortrim = CreateAnimation (0.5);
	anim_raileron = CreateAnimation (0.5);
	anim_laileron = CreateAnimation (0.5);
	anim_lrudder = CreateAnimation (0.5);
	anim_rrudder = CreateAnimation (0.5);
	anim_brakes = CreateAnimation (0);

	if (stage < 0) MakeBoosterAnimations();
	if (stage == -3) return;

	static UINT LGearDoorGrp[1] = {15};
	static MGROUP_ROTATE LGearDoor (0, LGearDoorGrp, 1,
		_V(-2.7,-2.07,-2.5), _V(0,0,1), (float)( 100*RAD));
	static UINT LGearGrp[2] = {3, 35};
	static MGROUP_ROTATE LGear (0, LGearGrp, 2,
		_V( 3.033,-1.75,-3.65), _V(1, 0.3, 0.25), (float)( 75*RAD));

	static UINT RGearDoorGrp[1] = {14};
	static MGROUP_ROTATE RGearDoor (0, RGearDoorGrp, 1,
		_V( 2.7,-2.07,-2.5), _V(0,0,1), (float)(-100*RAD));
	static UINT RGearGrp[2] = {4, 36};
	static MGROUP_ROTATE RGear (0, RGearGrp, 2,
		_V(-3.033,-1.75,-3.65), _V(1,-0.3,-0.25), (float)( 75*RAD));

	static UINT LNoseGearDoorGrp[1] = {13};
	static MGROUP_ROTATE LNoseGearDoor (0, LNoseGearDoorGrp, 1,
		_V(-0.325,-1.20,12.0), _V(0,0.3,1), (float)(-110*RAD));
	static UINT RNoseGearDoorGrp[1] = {12};
	static MGROUP_ROTATE RNoseGearDoor (0, RNoseGearDoorGrp, 1,
		_V( 0.325,-1.20,12.0), _V(0,0.3,1), (float)( 110*RAD));
	static UINT NoseGearGrp[2] = {2, 34};
	static MGROUP_ROTATE NoseGear (0, NoseGearGrp, 2,
		_V(0.0,-1.16,10.78), _V(1,0,0), (float)( 105*RAD));


// Known Groups:

// 0 R Nozzle Ext
// 1 L Nozzle Ext
// 2 Nose tires
// 3 L Tire
// 4 R Tire
// 5 OMS Nozzles
// 6 L Bay Door
// 7 R Bay Door
// 8 L Fin
// 9 L Ail
// 10 L Flap
// 11 R Flap
// 12 R Nose Gr Dr
// 13 L Nose Gr Dr
// 14 L Gr Dr
// 15 R Gr Dr
// 16 R Ail
// 17 R Fin
// 18 Jet Duct
// 19 Jet Inlet Cover
// 20 Hull
// 21 Hatch
// 22 Airlock Inset
// 23 Airlock Door
// 24 Rt Nozzle
// 25 Lt Nozzle
// 26 Cargo bay
// 27 L Gear Bay
// 28 R Gear Bay
// 29 Nose Gear Bay
// 30 Crew Cabin Interior
// 31 Crew Cabin Aft Bhd
// 32 Crew Cabin Fwd Bhd 
// 33 Nozzle extension arms
// 34 Nose Gr Strut
// 35 L Gr Strut
// 36 R Gr Strut
// 37 L Radiator
// 38 R Radiator
// 39 Jet Inner
// 40 Window


	static UINT LPLDoorGrp[2] = {7, 38};
	static MGROUP_ROTATE LPLDoor (0, LPLDoorGrp, 2,
		_V(-1.675,1.492,0.0), _V( 0.050,-0.048,1), (float)( 130*RAD));
	static UINT RPLDoorGrp[2] = {6, 37};
	static MGROUP_ROTATE RPLDoor (0, RPLDoorGrp, 2,
		_V( 1.675,1.492,0.0), _V(-0.050,-0.048,1), (float)(-130*RAD));

	static UINT HatchGrp[1] = {21};
	static MGROUP_TRANSLATE Hatch (0, HatchGrp, 1,
		_V( 0.12,0.0,-1.5));


	// ***** Wing pivot animation *****

	static UINT RWingGrp[1] = {17};
	MGROUP_TRANSFORM *RWing2 = new MGROUP_ROTATE (0, RWingGrp, 1,
		_V(-6.28,-0.505,-3.515), _V(0.35,-0.1,1), (float)(-50*RAD));
	static UINT RAilGrp[1] = {16};
	MGROUP_TRANSFORM *RAil2 = new MGROUP_ROTATE (0, RAilGrp, 1,
		_V(-6.28,-0.505,-3.515), _V(0.35,-0.1,1), (float)( 50*RAD));
	static UINT LWingGrp[1] = {8};
	MGROUP_TRANSFORM *LWing2 = new MGROUP_ROTATE (0, LWingGrp, 1,
		_V( 6.28,-0.505,-3.515), _V(-0.35,-0.1,1), (float)( 50*RAD));
	static UINT LAilGrp[1] = {9};
	MGROUP_TRANSFORM *LAil2 = new MGROUP_ROTATE (0, LAilGrp, 1,
		_V( 6.28,-0.505,-3.515), _V(-0.35,-0.1,1), (float)(-50*RAD));

	wing1 = AddAnimationComponent (anim_wings, 0, 1, RWing2);
	wing2 = AddAnimationComponent (anim_wings, 0, 1, LWing2);
	wing3 = AddAnimationComponent (anim_wings, 0, 1, RAil2);
	wing4 = AddAnimationComponent (anim_wings, 0, 1, LAil2);
/*
	static UINT LWingGrp[2] = {8,9};
	static MGROUP_ROTATE LWing (0, LWingGrp, 2,
		_V( 2.4,0,7.3), _V( 0,1,0), (float)(-40*RAD));
	static UINT RWingGrp[2] = {16,17};
	static MGROUP_ROTATE RWing (0, RWingGrp, 2,
		_V(-2.4,0,7.3), _V( 0,1,0), (float)( 40*RAD));
*/
	static UINT JetGrp[1] = {19};
	static MGROUP_ROTATE Jet (0, JetGrp, 1,
		_V( 0.0,1.0,-6.625), _V(1,0,0), (float)(-32*RAD));

	static UINT NozlGrp[3] = {0, 1, 33};
	static MGROUP_TRANSLATE Nozl (0, NozlGrp, 3,
		_V( 0.0,0.0,-1.2));


	Gear1 = AddAnimationComponent (anim_gear, 0.05, 0.4, &LGearDoor);
	Gear2 = AddAnimationComponent (anim_gear, 0.4, 0.9, &LGear);
	Gear3 = AddAnimationComponent (anim_gear, 0.05, 0.4, &RGearDoor);
	Gear4 = AddAnimationComponent (anim_gear, 0.4, 0.9, &RGear);
	Gear5 = AddAnimationComponent (anim_gear, 0.4, 0.9, &NoseGear);
	Gear6 = AddAnimationComponent (anim_gear, 0.1, 0.7, &LNoseGearDoor);
	Gear7 = AddAnimationComponent (anim_gear, 0.2, 0.7, &RNoseGearDoor);

	PLDr1 = AddAnimationComponent (anim_pldoors, 0.1, 0.9, &LPLDoor);
	PLDr2 = AddAnimationComponent (anim_pldoors, 0.1, 0.9, &RPLDoor);
	Hatch1 = AddAnimationComponent (anim_hatch, 0.1, 0.9, &Hatch);
//	AddAnimationComponent (anim_wings, 0.1, 0.9, &LWing);
//	AddAnimationComponent (anim_wings, 0.1, 0.9, &RWing);
	Jets1 = AddAnimationComponent (anim_jets, 0.1, 0.9, &Jet);
	Nozl1 = AddAnimationComponent (anim_nozzles, 0.1, 0.9, &Nozl);

	// ***** Elevator animation *****
	static UINT LElevatorGrp[1] = {10};
	static MGROUP_ROTATE LElevator (0, LElevatorGrp, 1,
		_V(-4.0,-1.825,-8.715), _V( 1,0,0), (float)( 25*RAD));
	AddAnimationComponent (anim_lelevator, 0, 1, &LElevator);
	static UINT RElevatorGrp[1] = {11};
	static MGROUP_ROTATE RElevator (0, RElevatorGrp, 1,
		_V( 4.0,-1.825,-8.715), _V(-1,0,0), (float)(-25*RAD));
	AddAnimationComponent (anim_relevator, 0, 1, &RElevator);

	// ***** Elevator trim animation *****
	static UINT LElevatorTrimGrp[1] = {10};
	static MGROUP_ROTATE LElevatorTrim (0, LElevatorTrimGrp, 1,
		_V(-4.0,-1.825,-8.715), _V( 1,0,0), (float)( 10*RAD));
	AddAnimationComponent (anim_lelevatortrim, 0, 1, &LElevatorTrim);
	static UINT RElevatorTrimGrp[1] = {11};
	static MGROUP_ROTATE RElevatorTrim (0, RElevatorTrimGrp, 1,
		_V( 4.0,-1.825,-8.715), _V(-1,0,0), (float)(-10*RAD));
	AddAnimationComponent (anim_relevatortrim, 0, 1, &RElevatorTrim);

	// ***** Rudder animation *****
	static UINT RRudderGrp[1] = {9};
	MGROUP_TRANSFORM *RRudder = new MGROUP_ROTATE (0, RRudderGrp, 1,
		_V( 9.19,2.0,-10.315), _V( 0.25,0.51,-0.36), (float)(-30*RAD));
	static UINT LRudderGrp[1] = {16};
	MGROUP_TRANSFORM *LRudder = new MGROUP_ROTATE (0, LRudderGrp, 1,
		_V(-9.19,2.0,-10.315), _V(-0.25,0.51,-0.36), (float)(-30*RAD));
	AddAnimationComponent (anim_rrudder, 0, 1, RRudder, wing2);
	AddAnimationComponent (anim_lrudder, 0, 1, LRudder, wing1);

	// ***** Aileron animation *****

	static UINT RAileronGrp[1] = {9};
	MGROUP_TRANSFORM *RAileron = new MGROUP_ROTATE (0, RAileronGrp, 1,
		_V( 9.19,2.0,-10.315), _V( 0.25,0.51,-0.36), (float)(-30*RAD));
	AddAnimationComponent (anim_raileron, 0, 1, RAileron, wing2);

	static UINT LAileronGrp[1] = {16};
	MGROUP_TRANSFORM *LAileron = new MGROUP_ROTATE (0, LAileronGrp, 1,
		_V(-9.19,2.0,-10.315), _V(-0.25,0.51,-0.36), (float)(-30*RAD));
	AddAnimationComponent (anim_laileron, 0, 1, LAileron, wing1);


		// ***** Speedbrake animation *****
/*
//	static UINT RRudderGrp[1] = {12};
	static MGROUP_ROTATE RRudder2 (0, RRudderGrp, 1,
		_V( 9.19,2.0,-9.125), _V( 0.25,0.45,-0.4), (float)(-20*RAD));
//	static UINT LRudderGrp[1] = {13};
	static MGROUP_ROTATE LRudder2 (0, LRudderGrp, 1,
		_V(-9.19,2.0,-9.125), _V(-0.25,0.45,-0.4), (float)( 20*RAD));
*/

//	AddAnimationComponent (anim_brakes, 0, 1, &RRudder2, wing2);
//	AddAnimationComponent (anim_brakes, 0, 1, &LRudder2, wing1);

}


void TITOVV::MakeBoosterAnimations()
{

	bBoosterMade = true;

	static UINT LGearDoorGrp[1] = {10};
	static MGROUP_ROTATE LGearDoor (1, LGearDoorGrp, 1,
		_V( 5.35,-7.55,-3.0), _V(0,0,1), (float)( 170*RAD));
	static UINT LGearGrp[1] = {26};
	static MGROUP_ROTATE LGear (1, LGearGrp, 1,
		_V(-4.82,-6.10,-5.10), _V(1,-0.25,-0.20), (float)( 75*RAD));
	static UINT LGearTrkGrp[2] = {2, 25};
	static MGROUP_ROTATE LGearTrk (1, LGearTrkGrp, 2,
		_V(-4.85,-6.51,-1.72), _V(1,-0.33,-0.20), (float)(-55*RAD));

	static UINT RGearDoorGrp[1] = {11};
	static MGROUP_ROTATE RGearDoor (1, RGearDoorGrp, 1,
		_V(-5.35,-7.55,-3.0), _V(0,0,1), (float)(-170*RAD));
	static UINT RGearGrp[1] = {29};
	static MGROUP_ROTATE RGear (1, RGearGrp, 1,
		_V( 4.82,-6.10,-5.10), _V(1, 0.25, 0.20), (float)( 75*RAD));
	static UINT RGearTrkGrp[2] = {3, 30};
	static MGROUP_ROTATE RGearTrk (1, RGearTrkGrp, 2,
		_V( 4.85,-6.51,-1.72), _V(1, 0.33, 0.20), (float)(-55*RAD));

	static UINT LNoseGearDoorGrp[1] = {9};
	static MGROUP_ROTATE LNoseGearDoor (1, LNoseGearDoorGrp, 1,
		_V(-0.600,-5.80,26.55), _V(0,0.3,1), (float)(-110*RAD));
	static UINT RNoseGearDoorGrp[1] = {8};
	static MGROUP_ROTATE RNoseGearDoor (1, RNoseGearDoorGrp, 1,
		_V( 0.600,-5.80,26.55), _V(0,0.3,1), (float)( 110*RAD));
	static UINT NoseGearGrp[2] = {0, 22};
	static MGROUP_ROTATE NoseGear (1, NoseGearGrp, 2,
		_V(0.0,-6.00,24.00), _V(1,0,0), (float)( 105*RAD));


// Known Groups:

// 0 Nose Tires
// 1 Nozzle Exts
// 2 L Tires
// 3 R Tires
// 4 L Fin
// 5 L Ail
// 6 L Flap
// 7 R Flap
// 8 R Nose Gr Dr
// 9 L Nose Gr Dr
// 10 L Gr Dr
// 11 R Gr Dr
// 12 R Ail
// 13 R Fin
// 14 Inlet Cover 3
// 15 Inlet Cover 1
// 16 Inlet Cover 2
// 17 Jet Ducts
// 18 Hull
// 19 Nozzles
// 20 Nose Gr Box
// 21 CptInterior
// 22 Nose Gr Strut
// 23 Nozzle Exts
// 24 Jet Duct Interiors
// 25 L Gear Truck
// 26 L Gear Strut
// 27 L Gear Bay
// 28 R Gear Bay
// 29 R Gear Strut
// 30 R Gear Truck
// 31 Window


	// ***** Wing pivot animation *****

	static UINT RWingGrp[1] = {13};
	MGROUP_TRANSFORM *RWing2 = new MGROUP_ROTATE (1, RWingGrp, 1,
		_V(-9.7,-4.7,-8.132), _V(0.25,-0.12,1), (float)(-57*RAD));
	static UINT RAilGrp[1] = {12};
	MGROUP_TRANSFORM *RAil2 = new MGROUP_ROTATE (1, RAilGrp, 1,
		_V(-9.7,-4.7,-8.132), _V(0.25,-0.12,1), (float)( 57*RAD));
	static UINT LWingGrp[1] = {4};
	MGROUP_TRANSFORM *LWing2 = new MGROUP_ROTATE (1, LWingGrp, 1,
		_V( 9.7,-4.7,-8.132), _V(-0.25,-0.12,1), (float)( 57*RAD));
	static UINT LAilGrp[1] = {5};
	MGROUP_TRANSFORM *LAil2 = new MGROUP_ROTATE (1, LAilGrp, 1,
		_V( 9.7,-4.7,-8.132), _V(-0.25,-0.12,1), (float)(-57*RAD));

	wing11 = AddAnimationComponent (anim_bwings, 0, 1, RWing2);
	wing12 = AddAnimationComponent (anim_bwings, 0, 1, LWing2);
	wing13 = AddAnimationComponent (anim_bwings, 0, 1, RAil2);
	wing14 = AddAnimationComponent (anim_bwings, 0, 1, LAil2);
/*
	static UINT LWingGrp[2] = {8,9};
	static MGROUP_ROTATE LWing (1, LWingGrp, 2,
		_V( 2.4,0,7.3), _V( 0,1,0), (float)(-40*RAD));
	static UINT RWingGrp[2] = {16,17};
	static MGROUP_ROTATE RWing (1, RWingGrp, 2,
		_V(-2.4,0,7.3), _V( 0,1,0), (float)( 40*RAD));
*/
	static UINT JetGrp1[1] = {15};
	static MGROUP_ROTATE Jet1 (1, JetGrp1, 1,
		_V( 0.0,-1.9,-11.85), _V(1,0,0), (float)(-32*RAD));
	static UINT JetGrp2[1] = {14};
	static MGROUP_ROTATE Jet2 (1, JetGrp2, 1,
		_V(-5.0,-2.0,-11.85), _V(1, 0.1,0), (float)(-32*RAD));
	static UINT JetGrp3[1] = {16};
	static MGROUP_ROTATE Jet3 (1, JetGrp3, 1,
		_V( 5.0,-2.0,-11.85), _V(1,-0.1,0), (float)(-32*RAD));

	static UINT NozlGrp[2] = {1, 23};
	static MGROUP_TRANSLATE Nozl (1, NozlGrp, 2,
		_V( 0.0,0.0,-1.2));


	Gear11 = AddAnimationComponent (anim_gear, 0.05, 0.4, &LGearDoor);
	Gear12 = AddAnimationComponent (anim_gear, 0.4, 0.9, &LGear);
	Gear13 = AddAnimationComponent (anim_gear, 0.4, 0.9, &LGearTrk,Gear12);
	Gear14 = AddAnimationComponent (anim_gear, 0.05, 0.4, &RGearDoor);
	Gear15 = AddAnimationComponent (anim_gear, 0.4, 0.9, &RGear);
	Gear16 = AddAnimationComponent (anim_gear, 0.4, 0.9, &RGearTrk,Gear15);
	Gear17 = AddAnimationComponent (anim_gear, 0.4, 0.9, &NoseGear);
	Gear18 = AddAnimationComponent (anim_gear, 0.1, 0.7, &LNoseGearDoor);
	Gear19 = AddAnimationComponent (anim_gear, 0.2, 0.7, &RNoseGearDoor);

	Jets11 = AddAnimationComponent (anim_jets, 0.1, 0.9, &Jet1);
	Jets12 = AddAnimationComponent (anim_jets, 0.1, 0.9, &Jet2);
	Jets13 = AddAnimationComponent (anim_jets, 0.1, 0.9, &Jet3);
	Nozl11 = AddAnimationComponent (anim_nozzles, 0.1, 0.9, &Nozl);

	// ***** Elevator animation *****
	static UINT LElevatorGrp[1] = {6};
	static MGROUP_ROTATE LElevator (1, LElevatorGrp, 1,
		_V(-4.0,-7.523,-15.587), _V( 1,0,0), (float)( 25*RAD));
	Elev11 = AddAnimationComponent (anim_lelevator, 0, 1, &LElevator);
	static UINT RElevatorGrp[1] = {7};
	static MGROUP_ROTATE RElevator (1, RElevatorGrp, 1,
		_V( 4.0,-7.523,-15.587), _V(-1,0,0), (float)(-25*RAD));
	Elev12 = AddAnimationComponent (anim_relevator, 0, 1, &RElevator);

	// ***** Elevator trim animation *****
	static UINT LElevatorTrimGrp[1] = {6};
	static MGROUP_ROTATE LElevatorTrim (1, LElevatorTrimGrp, 1,
		_V(-4.0,-7.523,-15.587), _V( 1,0,0), (float)( 10*RAD));
	Elev13 = AddAnimationComponent (anim_lelevatortrim, 0, 1, &LElevatorTrim);
	static UINT RElevatorTrimGrp[1] = {7};
	static MGROUP_ROTATE RElevatorTrim (1, RElevatorTrimGrp, 1,
		_V( 4.0,-7.523,-15.587), _V(-1,0,0), (float)(-10*RAD));
	Elev14 = AddAnimationComponent (anim_relevatortrim, 0, 1, &RElevatorTrim);

	// ***** Rudder animation *****
	static UINT RRudderGrp[1] = {5};
	MGROUP_TRANSFORM *RRudder = new MGROUP_ROTATE (1, RRudderGrp, 1,
		_V( 15.0,0.477,-19.542), _V( 0.25,0.51,-0.36), (float)(-30*RAD));
	static UINT LRudderGrp[1] = {12};
	MGROUP_TRANSFORM *LRudder = new MGROUP_ROTATE (1, LRudderGrp, 1,
		_V(-15.0,0.477,-19.542), _V(-0.25,0.51,-0.36), (float)(-30*RAD));
	Rudd11 = AddAnimationComponent (anim_rrudder, 0, 1, RRudder, wing12);
	Rudd12 = AddAnimationComponent (anim_lrudder, 0, 1, LRudder, wing11);

	// ***** Aileron animation *****

	static UINT RAileronGrp[1] = {5};
	MGROUP_TRANSFORM *RAileron = new MGROUP_ROTATE (1, RAileronGrp, 1,
		_V( 15.0,0.477,-19.542), _V( 0.25,0.51,-0.36), (float)(-30*RAD));
	Rudd13 = AddAnimationComponent (anim_raileron, 0, 1, RAileron, wing12);

	static UINT LAileronGrp[1] = {12};
	MGROUP_TRANSFORM *LAileron = new MGROUP_ROTATE (1, LAileronGrp, 1,
		_V(-15.0,0.477,-19.542), _V(-0.25,0.51,-0.36), (float)(-30*RAD));
	Rudd14 = AddAnimationComponent (anim_laileron, 0, 1, LAileron, wing11);


}

void TITOVV::DelBoosterAnimations()
{

	bBoosterMade = false;

	DelAnimationComponent (anim_bwings,wing11);
	DelAnimationComponent (anim_bwings,wing12);
	DelAnimationComponent (anim_bwings,wing13);
	DelAnimationComponent (anim_bwings,wing14);
	DelAnimationComponent (anim_gear,Gear11);
	DelAnimationComponent (anim_gear,Gear12);
	DelAnimationComponent (anim_gear,Gear13);
	DelAnimationComponent (anim_gear,Gear14);
	DelAnimationComponent (anim_gear,Gear15);
	DelAnimationComponent (anim_gear,Gear16);
	DelAnimationComponent (anim_gear,Gear17);
	DelAnimationComponent (anim_gear,Gear18);
	DelAnimationComponent (anim_gear,Gear19);
	DelAnimationComponent (anim_jets,Jets11);
	DelAnimationComponent (anim_jets,Jets12);
	DelAnimationComponent (anim_jets,Jets13);
	DelAnimationComponent (anim_nozzles,Nozl11);
	DelAnimationComponent (anim_lelevator,Elev11);
	DelAnimationComponent (anim_relevator,Elev12);
	DelAnimationComponent (anim_lelevatortrim,Elev13);
	DelAnimationComponent (anim_relevatortrim,Elev14);
	DelAnimationComponent (anim_rrudder,Rudd11);
	DelAnimationComponent (anim_lrudder,Rudd12);
	DelAnimationComponent (anim_raileron,Rudd13);
	DelAnimationComponent (anim_laileron,Rudd14);

}

void TITOVV::MakeAirfoils()
{
//	ClearAirfoilDefinitions();
	ClearVariableDragElements();

	if (stage > -1) {

		CreateVariableDragElement (&brakes_proc, 0.8, _V(0, 0, 0));
		CreateVariableDragElement (&gear_proc, 0.6, _V(0, -1, 0));
		CreateVariableDragElement (&MachPitch, 1.8, _V(0, 36.0,-8.0));

	//                  alignment      position      func    chord S   AR   
		EditAirfoil (Wing, 0xFF, _V(0,0,-0.4), VLiftCoeff, 9, 145, 4.35);
		// wing and body lift+drag components

		EditAirfoil (Rudder, 0xFF, _V(0,0,-8.5), HLiftCoeff, 3.5, 28, 2.6);
		// vertical stabiliser and body lift and drag components

	}
	else {

		CreateVariableDragElement (&brakes_proc, 1.6, _V(0, 0, 0));
		CreateVariableDragElement (&gear_proc, 0.6, _V(0, -1, 0));
		CreateVariableDragElement (&MachPitch, 2.8, _V(0, 72.0,-16.0));


	//                                alignment      position       func    chord S   AR   
		EditAirfoil (Wing, 0xFF, _V(0,0,-0.2), VLiftCoeff, 18, 265, 3.35);
		// wing and body lift+drag components

		EditAirfoil (Rudder, 0xFF, _V(0,0,-14.5), HLiftCoeff, 8, 58, 2.6);
		// vertical stabiliser and body lift and drag components


	}
}


void TITOVV::MakeControlSurfaces()
{

//  Aargh!  Can't delete control surfaces
//  Need to make sure we declare once and only once,
//  but that a booster stage alone doesn't get animations for the upper stage!

	if (bControlsMade) return;
	bControlsMade = true;

	if (stage > -3) {

		CreateControlSurface (AIRCTRL_ELEVATOR,     14.5,  0.4, _V(  4.2,0,-11.67), AIRCTRL_AXIS_XPOS, anim_lelevator);
		CreateControlSurface (AIRCTRL_ELEVATOR,     14.5,  0.4, _V( -4.2,0,-11.67), AIRCTRL_AXIS_XPOS, anim_relevator);
//		CreateControlSurface (AIRCTRL_RUDDER,       5.3,  0.4, _V(  9.5,0,-12.75), AIRCTRL_AXIS_YPOS, anim_lrudder);
//		CreateControlSurface (AIRCTRL_RUDDER,       5.3,  0.4, _V( -9.5,0,-12.75), AIRCTRL_AXIS_YPOS, anim_rrudder);
		CreateControlSurface (AIRCTRL_AILERON,      6.3, -0.4, _V(-9.5,0,-12.75), AIRCTRL_AXIS_XPOS, anim_raileron);
		CreateControlSurface (AIRCTRL_AILERON,      6.3, -0.4, _V( 9.5,0,-12.75), AIRCTRL_AXIS_XNEG, anim_laileron);
		CreateControlSurface (AIRCTRL_ELEVATORTRIM, 2.7,  0.5, _V(  4.2,0,-11.67), AIRCTRL_AXIS_XPOS, anim_lelevatortrim);
		CreateControlSurface (AIRCTRL_ELEVATORTRIM, 2.7,  0.5, _V( -4.2,0,-11.67), AIRCTRL_AXIS_XPOS, anim_relevatortrim);
	}
	else {
		CreateControlSurface (AIRCTRL_ELEVATOR,     14.5,  0.4, _V(  4.2,0,-16.67), AIRCTRL_AXIS_XPOS, anim_lelevator);
		CreateControlSurface (AIRCTRL_ELEVATOR,     14.5,  0.4, _V( -4.2,0,-16.67), AIRCTRL_AXIS_XPOS, anim_relevator);
//		CreateControlSurface (AIRCTRL_RUDDER,       5.3,  0.4, _V(  9.5,0,-16.75), AIRCTRL_AXIS_YPOS, anim_lrudder);
//		CreateControlSurface (AIRCTRL_RUDDER,       5.3,  0.4, _V( -9.5,0,-16.75), AIRCTRL_AXIS_YPOS, anim_rrudder);
		CreateControlSurface (AIRCTRL_AILERON,      6.3, -0.4, _V(-9.5,0,-16.75), AIRCTRL_AXIS_XPOS, anim_raileron);
		CreateControlSurface (AIRCTRL_AILERON,      6.3, -0.4, _V( 9.5,0,-16.75), AIRCTRL_AXIS_XNEG, anim_laileron);
		CreateControlSurface (AIRCTRL_ELEVATORTRIM, 2.7,  0.5, _V(  4.2,0,-16.67), AIRCTRL_AXIS_XPOS, anim_lelevatortrim);
		CreateControlSurface (AIRCTRL_ELEVATORTRIM, 2.7,  0.5, _V( -4.2,0,-16.67), AIRCTRL_AXIS_XPOS, anim_relevatortrim);
	}
}



void TITOVV::BoosterSwitchToJets ()
{


	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 325.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
//		0, 1.25, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};

	SoundOptionOnOff3(MyID,PLAYMAINTHRUST,false);

	if (ph_booster != NULL)
		DelPropellantResource (ph_booster);
	if (ph_first != NULL)  
		DelPropellantResource (ph_first);
	if (ph_TitovV != NULL)  
		DelPropellantResource (ph_TitovV);
	if (!ph_Jets)  
		ph_Jets  = CreatePropellantResource(BJET_PROP_MASS,BJET_PROP_MASS,1.0); 

	if (GetPropellantMass(ph_Jets) == 0.0)  
		SetPropellantMass(ph_Jets,BJET_PROP_MASS); 

	SetDefaultPropellantResource (ph_Jets); 

	ClearThrusterDefinitions();

	th_first[0] = CreateThruster ( _V( 4.95,0.0,-17.2), _V(0, 0,1), BJET_THRUST, ph_Jets, ISP_BJET_VAC, ISP_BJET_VAC);
	th_first[2] = CreateThruster ( _V( 0.00,0.0,-17.2), _V(0, 0,1), BJET_THRUST, ph_Jets, ISP_BJET_VAC, ISP_BJET_VAC);
	th_first[1] = CreateThruster ( _V(-4.95,0.0,-17.2), _V(0, 0,1), BJET_THRUST, ph_Jets, ISP_BJET_VAC, ISP_BJET_VAC);

	numengines = 3;
	thg_first = CreateThrusterGroup (th_first, 3, THGROUP_MAIN);
//	AddExhaust (th_first[0], 3.0, 0.3, tex);

	AddExhaustStream (th_first[0], _V( 4.95,-2.1,-17.2), &contrail);
	AddExhaustStream (th_first[0], _V( 0.00,-1.8,-17.2), &contrail);
	AddExhaustStream (th_first[0], _V(-4.95,-2.1,-17.2), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

}
void TITOVV::UpdateAnimations()
{	

		// animate landing gear
		if (gear_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * GEAR_OPERATING_SPEED;
			if (gear_status == DOOR_CLOSING) {
				if (gear_proc > 0.0)
					gear_proc = max (0.0, gear_proc-da);
				else {
					gear_status = DOOR_CLOSED;
					StopVesselWave3(MyID,PLAYGEAR);
				}
			} else  { // door opening
				if (gear_proc < 1.0)
					gear_proc = min (1.0, gear_proc+da);
				else {
					gear_status = DOOR_OPEN;
					StopVesselWave3(MyID,PLAYGEAR);
				}
			}
			SetAnimation (anim_gear, gear_proc);
		}

		// animate payload bay doors
		if (pldoors_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * PLDOORS_OPERATING_SPEED;
			if (pldoors_status == DOOR_CLOSING) {
				if (pldoors_proc > 0.0)
					pldoors_proc = max (0.0, pldoors_proc-da);
				else {
					pldoors_status = DOOR_CLOSED;
					StopVesselWave3(MyID,PLAYDOORS);
				}
			} else  { // door opening
				if (pldoors_proc < 1.0)
					pldoors_proc = min (1.0, pldoors_proc+da);
				else {
					pldoors_status = DOOR_OPEN;
					StopVesselWave3(MyID,PLAYDOORS);
				}
			}
			SetAnimation (anim_pldoors, pldoors_proc);
		}

		// animate speedbrakes
		if (brakes_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * BRAKES_OPERATING_SPEED;
			if (brakes_status == DOOR_CLOSING) {
				if (brakes_proc > 0.0)
					brakes_proc = max (0.0, brakes_proc-da);
				else {
					brakes_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (brakes_proc < 1.0)
					brakes_proc = min (1.0, brakes_proc+da);
				else {
					brakes_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_brakes, brakes_proc);
		}

		// animate top hatch
		if (!bEjected)
			if (hatch_status >= DOOR_CLOSING) {
				double da = oapiGetSimStep() * HATCH_OPERATING_SPEED;
				if (hatch_status == DOOR_CLOSING) {
					if (hatch_proc > 0.0)
						hatch_proc = max (0.0, hatch_proc-da);
					else {
						hatch_status = DOOR_CLOSED;
					}
				} else  { // door opening
					if (hatch_proc < 1.0)
						hatch_proc = min (1.0, hatch_proc+da);
					else {
						hatch_status = DOOR_OPEN;
					}
				}
				SetAnimation (anim_hatch, hatch_proc);
			}


		// animate folding wings
		if (wings_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * WINGS_OPERATING_SPEED;
			if (wings_status == DOOR_CLOSING) {
				if (wings_proc > 0.0)
					wings_proc = max (0.0, wings_proc-da);
				else {
					wings_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (wings_proc < 1.0)
					wings_proc = min (1.0, wings_proc+da);
				else {
					wings_status = DOOR_OPEN;
				}
			}
			if (bBoosterMade)
				SetAnimation (anim_bwings, wings_proc);
			else
				SetAnimation (anim_wings, wings_proc);
		}

		// animate jet doors
		if (jets_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * JETS_OPERATING_SPEED;
			if (jets_status == DOOR_CLOSING) {
				if (jets_proc > 0.0)
					jets_proc = max (0.0, jets_proc-da);
				else {
					jets_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (jets_proc < 1.0)
					jets_proc = min (1.0, jets_proc+da);
				else {
					jets_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_jets, jets_proc);
		}

		// animate nozzle extension
		if (nozzles_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * NOZZLES_OPERATING_SPEED;
			if (nozzles_status == DOOR_CLOSING) {
				if (nozzles_proc > 0.0)
					nozzles_proc = max (0.0, nozzles_proc-da);
				else {
					nozzles_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (nozzles_proc < 1.0)
					nozzles_proc = min (1.0, nozzles_proc+da);
				else {
					nozzles_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_nozzles, nozzles_proc);
		}

}

void TITOVV::ActivateLandingGear (DoorStatus action)
{
	gear_status = action;
}

void TITOVV::RevertLandingGear ()
{
	ActivateLandingGear (gear_status == DOOR_CLOSED || gear_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void TITOVV::ActivatePayloadBayDoors (DoorStatus action)
{
	pldoors_status = action;
}

void TITOVV::RevertPayloadBayDoors ()
{
	ActivatePayloadBayDoors (pldoors_status == DOOR_CLOSED || pldoors_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void TITOVV::ActivateBrakes (DoorStatus action)
{
	brakes_status = action;
}

void TITOVV::RevertBrakes ()
{
	ActivateBrakes (brakes_status == DOOR_CLOSED || brakes_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void TITOVV::ActivateHatch (DoorStatus action)
{
	hatch_status = action;
}

void TITOVV::RevertHatch ()
{
	ActivateHatch (hatch_status == DOOR_CLOSED || hatch_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void TITOVV::ActivateWings (DoorStatus action)
{
	wings_status = action;
}

void TITOVV::RevertWings ()
{
	ActivateWings (wings_status == DOOR_CLOSED || wings_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void TITOVV::ActivateJets (DoorStatus action)
{
	jets_status = action;
}

void TITOVV::RevertJets ()
{
	ActivateJets (jets_status == DOOR_CLOSED || jets_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void TITOVV::ActivateNozzles (DoorStatus action)
{
	nozzles_status = action;
}

void TITOVV::RevertNozzles ()
{
	ActivateNozzles (nozzles_status == DOOR_CLOSED || nozzles_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void TITOVV::SetGearParameters (double state)
{

	double geary = (-1*(gear_proc-1.0)*1.4);

	if ((Firstsweep) || (gear_status == DOOR_OPENING) || (gear_status == DOOR_CLOSING)) {
		if (stage < 0)
			SetTouchdownPoints (_V(0,-8.85+geary,16.0), _V(-9.15,-9.95+geary*1.3,-9.0), _V(9.15,-9.95+geary*1.3,-9.0));
		else if (stage==0)
			SetTouchdownPoints (_V(0,-4.05+geary*.8,14.0), _V(-9.15,-3.9+geary,-6.0), _V(9.15,-3.9+geary,-6.0));
		else
			SetTouchdownPoints (_V(0,-2.55+geary*.8,14.0), _V(-9.15,-3.9+geary,-6.0), _V(9.15,-3.9+geary,-6.0));
	}
	if ((!BrakesOn) && (gear_proc == 1.0)) {
		SetSurfaceFrictionCoeff (0.025, 0.4);
	} else {
		SetSurfaceFrictionCoeff (0.3, 0.4);		// Brakes on!
	}


	double steerforce;
	double steerangle;

	steerforce = 4.0;
	if (GetAirspeed() < 40.0)
		steerforce *=5.0;
	steerforce = steerforce*GetMass();
	if ((GetAirspeed ()-3) < 5.0) steerforce*=((GetAirspeed()-3)/5); // from 1* to 0*
	if (GetAirspeed() < 3.0) steerforce = 0.0;

	steerangle = GetControlSurfaceLevel (AIRCTRL_RUDDER);
	steerforce = steerforce * steerangle;
	if ((stage < 0) && (GetAltitude() > 10.5)) steerforce = 0.0;
	if ((stage > -1) && (GetAltitude() > 4.5)) steerforce = 0.0;

	if ((gear_status == DOOR_OPEN) && (fabs(steerforce) > 0.05)) {
		AddForce (_V(steerforce, 0, 0), _V(0, 0, 12.0));
		AddForce (_V(-steerforce, 0, 0), _V(0, 0, -12.0));
	}
}


void TITOVV::SeparateEVA ()
{
	VESSELSTATUS vs;
	GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	vs.status=0;
	VECTOR3 ofs = EVA_OFS;
	ofs.y += hatch_proc;
	Local2Rel (ofs, vs.rpos);
	char name[256], name2[256];
//	strcpy (name, GetName()); strcat (name, "-PLT");


	UINT i;

	for (i=1;i<8;i++) {
			strcpy (name2, GetName()); 
			sprintf(name, "-EV%d", i);
			strcat (name2, name);
			hvessel=oapiGetVesselByName(name2);
			if (hvessel == 0) break;
/*			else  {
				VESSEL *rv = oapiGetVesselInterface(hvessel);
				if (rv->GetEnableFocus() == false) {
					oapiDeleteVessel(hvessel);
					break;
				}
			}
*/	}		
	if ( i < 7) 
	{
		oapiCreateVessel (name2, "2001/Astro2001", vs);
		hvessel=oapiGetVesselByName(name2);
		oapiSetFocusObject(hvessel);
	}



//	VESSEL::Create (name, "gemini_leva", vs);
}


void TITOVV::RussianBeeps()
{
		DWORD i;
		double Altitude=0;
		double Distance=1000;
		oapiGetFocusAltitude(&Altitude);
		Local2Global (_V(0,0,0), grms);  // global position of RMS tip
		for (i = 0; i < oapiGetVesselCount(); i++) {
			hV = oapiGetVesselByIndex (i);
			if (hV == GetHandle()) continue; // we don't want to check ourselves ...
			oapiGetGlobalPos (hV, &gpos);
			if (((dist (gpos, grms) - oapiGetSize(hV) - oapiGetSize(GetHandle())) < Distance) && ((dist (gpos, grms) - oapiGetSize(hV) - oapiGetSize(GetHandle())) > 10.0))
				Distance = dist(gpos,grms) - oapiGetSize(GetHandle());
		}
		if ((Distance < Altitude) || (Altitude == 0))
			Altitude = Distance;			// Play beeps based on object distance instead!



		// ------------------------------------------------------
		// here we play our radar sound that slide with altitude
		// ------------------------------------------------------
		if(Altitude<600&&Altitude>25)
		{
			PlayVesselWave3(MyID,MYRADARSOUND,LOOP,255,(int)(16000+((600-Altitude)*10)));		// play looped and a frequency that increase with altitude
			RadarFlagForSound=TRUE;
			if(RadarFlagForClose=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARCLOSE);
				RadarFlagForClose = FALSE;
			}
		}
		else if (Altitude<25&&Altitude>10) 
		{
			PlayVesselWave3(MyID,MYRADARCLOSE,LOOP,255);		// play looped 
			RadarFlagForClose=TRUE;
			if(RadarFlagForSound=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARSOUND);
				RadarFlagForSound = FALSE;
			}
		}
		else {
			if(RadarFlagForSound=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARSOUND);
				RadarFlagForSound = FALSE;
			}
			if(RadarFlagForClose=TRUE)
			{
				// we stop the sound only once (it would not harm else but it will save a bit of processor time)
				StopVesselWave3(MyID,MYRADARCLOSE);
				RadarFlagForClose = FALSE;
			}
		}
}

void TITOVV::TimeStep(  double simt )
{
	int i;
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */
	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
			SetMainStage();
	}

	RussianBeeps();
	
	if (!bAutoProgrammed)
		ProgramAuto();
	if (!bControlsMade)
		MakeControlSurfaces();
	
	if ((guidance) && (MET > 0))
		SetThrusterGroupLevel (THGROUP_MAIN,1.0);

	dt = simt - oldsimt;
	oldsimt = simt;
	if ((displaytime > 0) && (oapiGetFocusObject() == GetHandle())) displaytime += dt;

	if ((!guidance) && (displaytime < 5) && (stage < 1) && (stage > -3) && (oapiGetFocusObject() == GetHandle()))
	{
		GetEquPos( Longitude, Latitude, Radius );
		sprintf(oapiDebugString(), "Target inclination (Equatorial): %.3f", (180 - acos(sin(d_heading*PI/180)*cos(Latitude))*DEG));
	}
//	else if (oapiGetFocusObject() == GetHandle())
//		sprintf(oapiDebugString(), " ");

	if (!guidance && !MECO_Cond ) {
		if(!NewIncSP) {
			// Set the Inclination setpoint for the guidance computer
			GetEquPos( Longitude, Latitude, Radius );
				INCLINATION = acos(Clairaut_Constant)*DEG;
		}
	}	

	//Check if guidance is on and if so, call
	if (guidance)
	{
/*		if (BadHeading) 
		{
			sprintf(oapiDebugString(),"Guidance system cannot launch from this heading.");
			guidance = false;
		} 
		else 
*/		{
			// Calculate MET
			if (TZero == -100) 
			{
				MET = -5 ;
				TZero = 0;
			}
			else 
			{
//				if (dt > 0)
//					MET += dt;
//				else MET += 0.01;
				MET = simt - TZero;
			}
			// Call the guidance routine
			Guidance_Routine(this, simt, SimT_Last);
			SimT_Last = MET;
		}
	}
	else if (!MECO_Cond && !PostMECO && (stage < 1) && (!Firstsweep) && (GetAltitude() > GetSize())) {
			if (TZero == -100) 
			{
				MET = -5 ;
				TZero = 0;
			}
			else 
			{
				MET = simt - TZero;
			}
			Guidance_Routine(this, simt, SimT_Last);
	}

// Fix the amount of hypersonic pitch-up
	MachPitch = GetMachNumber() - 5.15;
	if (MachPitch > 1.0) MachPitch = 1.0;
	if (MachPitch < 0.0) MachPitch = 0.0;
	if ((stage < 1) && (stage > -3)) MachPitch = 0.0;	// Don't want any while flying UP

	if ((stage < 2) && (jets_status == DOOR_CLOSED))					// Throttle back to maintain 3.5 G's
		if (((GetThrusterMax0(th_first[0])*numengines)/GetMass()) > 3.5*G) 
			for (i = 0; i < numengines; i++) 
				SetThrusterMax0(th_first[i],0.98*GetThrusterMax0(th_first[i]));

	if ((stage == -3) && (jets_status == DOOR_OPEN)) {
		AirBreathingEngines();
		double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN);
		if(ThrusterActive>0.0f)
			{
			// we calculate the volume at wich it must be played
			double volume=110+(145*ThrusterActive);
			double freq=11050+(9000*ThrusterActive);
			PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
			}
		else if(IsPlaying3(MyID,PLAYJET))
			{
			// else shut down the sound if it's playing
			StopVesselWave3(MyID,PLAYJET);
			}
	}
	if (stage == 3) {
		AirBreathingEngine();
		double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN);
		if(ThrusterActive>0.0f)
			{
			// we calculate the volume at wich it must be played
			double volume=110+(145*ThrusterActive);
			double freq=11050+(9000*ThrusterActive);
			PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
			}
		else if(IsPlaying3(MyID,PLAYJET))
			{
			// else shut down the sound if it's playing
			StopVesselWave3(MyID,PLAYJET);
			}
	}

	if ((!bAttSet) && (stage == 0))				// if we're at altitude with the drop tank, turn on attitude controls anyway
		if (GetAltitude()>100000.0)
			SetAttControlsTITOVV();

	UpdateAnimations();
	VectorThrust();
	SetGearParameters(gear_proc);

	if ((GetAltitude() > 17000.0) && (stage < 2) && (nozzles_proc == 0.0)) {
		if (stage > -3) RevertNozzles();		// Extend the nozzles above 17 km.  Setting stage 2 will retract them for re-entry.
		SetThrusterIsp(th_first[0],ISP_SECOND_VAC,ISP_SECOND_SL,101.4e3);
		SetThrusterIsp(th_first[1],ISP_SECOND_VAC,ISP_SECOND_SL,101.4e3);
		if (stage < 0) {
			SetThrusterIsp(th_first[2],ISP_SECOND_VAC,ISP_SECOND_SL,101.4e3);
			SetThrusterIsp(th_first[3],ISP_SECOND_VAC,ISP_SECOND_SL,101.4e3);
			if (stage > -3) {
				SetThrusterIsp(th_first[4],ISP_SECOND_VAC,ISP_SECOND_SL,101.4e3);
				SetThrusterIsp(th_first[5],ISP_SECOND_VAC,ISP_SECOND_SL,101.4e3);
			}
		}
	}

	if (stage == 0)
		if (GetPropellantMass(ph_first) == 0) {
			JettisonPart();
			AttachSweep();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();
			SetThrusterGroupLevel (THGROUP_MAIN,1.0);
		}
	if ((stage == -1) || (stage == -2))
		if (GetPropellantMass(ph_booster) == 0) {
			JettisonPart();
			AttachSweep();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();
			SetThrusterGroupLevel (THGROUP_MAIN,1.0);
		}
	if (stage == 1)
		if ((GetAltitude() < 80000.0) && (GetPropellantMass(ph_TitovV) < 9000.0)) {
				JettisonPart();					// Switch to Stage 2 for the re-entry
				AttachSweep();

				/* IMPORTANT */
				/* Always re-set the stage after any kind of change */
				SetMainStage();
			}
	if ((stage > 1) && (GetMachNumber() < 5.15) && (GetAtmPressure() > 100.0) && (wings_status==DOOR_OPEN))
		RevertWings();

//		MakeANewWing();
		// Make a new wing!
//		SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,0.0);		// Reset the elevator trim to zero, to avoid a big pitch change
		
	if ((stage == -3) && (GetMachNumber() < 5.15) && (GetAtmPressure() > 100.0) && (wings_status==DOOR_OPEN))
		RevertWings();
	if ((stage == 2) && (GetMachNumber() < 0.9) && (GetAltitude() < 30000.0)) {
		stage++;
		RevertJets();		// Extend the jet engines and set Stage 3
		SetMainStage();
	}
	if ((stage == -3) && (jets_status==DOOR_CLOSED))
		if ((GetPropellantMass(ph_booster) < 100000.0) && (GetMachNumber() < 0.9) && (GetAltitude() < 30000.0) && (jets_status==DOOR_CLOSED)) {
		RevertJets();		// Extend the jet engines and set them
		BoosterSwitchToJets();
	}

	if (Firstsweep) {
		AttachSweep();
		Firstsweep = false;
		SetMainStage();
	}
}


void TITOVV::VectorThrust()
{

	if (Firstsweep)
		return;

	if (oapiGetTimeAcceleration() > 10) return;

	if (GetFocus() == GetHandle()) {
		tempP = GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		tempY = GetManualControlLevel(THGROUP_ATT_YAWLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_YAWRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		tempR = GetManualControlLevel(THGROUP_ATT_BANKLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_BANKRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

	} else {
		tempP = 0.0;
		tempY = 0.0;
		tempR = 0.0;
	}
		GetAngularVel(AngularVel);

		tempP+=(2*AngularVel.x);			// Bias the thrust vector AGAINST the angular velocity a bit
		tempY+=(-2*AngularVel.y);			// This will damp angular velocity, preventing loss of control
		tempR+=(2*AngularVel.z);
	

		tempP+=pitchinput;
		tempY+=yawinput;
		tempR+=rollinput;
	


		if (stage < 1) {
			SetThrusterDir(th_first[0],_V(0.012*tempY,0.012*(tempP-2*tempR),1));
			SetThrusterDir(th_first[1],_V(0.012*tempY,0.012*(tempP+2*tempR),1));
		}
		else if (stage == 1) {
			SetThrusterDir(th_first[0],_V(0.012*tempY,0.012*(tempP-2*tempR),1));
			SetThrusterDir(th_first[1],_V(0.012*tempY,0.012*(tempP+2*tempR),1));
		}
		else if (stage == 2) {
			SetThrusterDir(th_first[0],_V(0.012*tempY,0.012*(tempP+2*tempR),1));
			SetThrusterDir(th_first[1],_V(0.012*tempY,0.012*(tempP-2*tempR),1));
		}

}

void TITOVV::AirBreathingEngine()
// Thanks to David Hopkins for publishing his AirBreathingEngine source.  
{//=========================================================
double alt2;
double eas2;
double cf1, cf2;
double ActualThrust;


alt2 = GetAltitude();
eas2 = GetAirspeed ();

alt2 = alt2*3.28;
eas2 = eas2*2.236;

cf1 = 1.67e-6*(eas2*eas2)-8.8789e-4*(eas2)+0.9977;
cf2 = 3.0e-10*(alt2*alt2)-3.0000e-5*(alt2)+0.9934;

// Nasty kludge to keep the thrust from increasing dramatically above 50kft
if (alt2 > 50000)
	cf2 = 0.25*(1-((alt2-50000)*.0001));
if (cf2 < 0) cf2 = 0;

ActualThrust = cf1*cf2*JET_THRUST;

SetThrusterMax0 (th_first[0],ActualThrust);
};

void TITOVV::AirBreathingEngines()
// Thanks to David Hopkins for publishing his AirBreathingEngine source.  
{//=========================================================
double alt2;
double eas2;
double cf1, cf2;
double ActualThrust;


alt2 = GetAltitude();
eas2 = GetAirspeed ();

alt2 = alt2*3.28;
eas2 = eas2*2.236;

cf1 = 1.67e-6*(eas2*eas2)-8.8789e-4*(eas2)+0.9977;
cf2 = 3.0e-10*(alt2*alt2)-3.0000e-5*(alt2)+0.9934;

// Nasty kludge to keep the thrust from increasing dramatically above 50kft
if (alt2 > 50000)
	cf2 = 0.25*(1-((alt2-50000)*.0001));
if (cf2 < 0) cf2 = 0;

ActualThrust = cf1*cf2*BJET_THRUST;

SetThrusterMax0 (th_first[0],ActualThrust);
SetThrusterMax0 (th_first[1],ActualThrust);
SetThrusterMax0 (th_first[2],ActualThrust);
};
/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */

void TITOVV::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void TITOVV::SetClassCaps(  FILEHANDLE cfg )
{

	CheckCentralMessage();

	SetMainStage();
}

void TITOVV::SaveState( FILEHANDLE scn )
{
	char cbuf[256];

	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */


//	Undock(ALLDOCKS);

	oapiWriteScenario_int (scn, "CONFIGURATION", stage);
	oapiWriteScenario_int (scn, "EVA", bEVA);
	sprintf (cbuf, "%d %0.4f", gear_status, gear_proc);
	oapiWriteScenario_string (scn, "GEAR", cbuf);
	sprintf (cbuf, "%d %0.4f", pldoors_status, pldoors_proc);
	oapiWriteScenario_string (scn, "DOORS", cbuf);
	sprintf (cbuf, "%d %0.4f", brakes_status, brakes_proc);
	oapiWriteScenario_string (scn, "BRAKES", cbuf);
	sprintf (cbuf, "%d %0.4f", hatch_status, hatch_proc);
	oapiWriteScenario_string (scn, "HATCH", cbuf);
	sprintf (cbuf, "%d %0.4f", wings_status, wings_proc);
	oapiWriteScenario_string (scn, "WINGS", cbuf);
	sprintf (cbuf, "%d %0.4f", jets_status, jets_proc);
	oapiWriteScenario_string (scn, "JETS", cbuf);
	sprintf (cbuf, "%d %0.4f", nozzles_status, nozzles_proc);
	oapiWriteScenario_string (scn, "NOZZLES", cbuf);
	sprintf (cbuf, tvmesh);
	oapiWriteScenario_string (scn, "TVMESH", cbuf);
	oapiWriteScenario_float (scn, "TGT_HEADING", PadHeading - 180.0);
//	for (int n = 0; n < 5; n++) {
//		sprintf (cbuf, "%d %0.4f %0.4f %0.4f", n,attach[n].x,attach[n].y,attach[n].z);
//		oapiWriteScenario_string (scn, "ATTACHPOINT", cbuf);
//	}
	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void TITOVV::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;
//	int n;
//	VECTOR3 atchtemp = _V(0,0,0);

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		if (!strnicmp (line, "CONFIGURATION", 13)) {
		    sscanf (line+13, "%d", &stage);
		} else if (!strnicmp (line, "HEADING", 7)) {
			sscanf(line+7, "%lf", &initialheading);
		} else if (!strnicmp (line, "EVA", 3)) {
			sscanf (line+3, "%d", &bEVA);
		} else if (!strnicmp (line, "GEAR", 4)) {
		sscanf (line+4, "%d%lf", &gear_status, &gear_proc);
		} else if (!strnicmp (line, "DOORS", 5)) {
		sscanf (line+5, "%d%lf", &pldoors_status, &pldoors_proc);
		} else if (!strnicmp (line, "BRAKES", 6)) {
		sscanf (line+6, "%d%lf", &brakes_status, &brakes_proc);
		} else if (!strnicmp (line, "HATCH", 5)) {
		sscanf (line+5, "%d%lf", &hatch_status, &hatch_proc);
		} else if (!strnicmp (line, "WINGS", 5)) {
		sscanf (line+5, "%d%lf", &wings_status, &wings_proc);
		} else if (!strnicmp (line, "JETS", 4)) {
		sscanf (line+4, "%d%lf", &jets_status, &jets_proc);
		} else if (!strnicmp (line, "NOZZLES", 7)) {
		sscanf (line+7, "%d%lf", &nozzles_status, &nozzles_proc);
		} else if(!strnicmp (line, "TVMESH", 6) ) {
			AnimationsMade = false;
			sscanf( line+6, "%s", &tvmesh);
		} else if( !strncmp( line, "TGT_HEADING", 11 ) ) {
            sscanf (line+11, "%lf", &d_heading);
			PadHeading = d_heading + 180.0;  // It's not pad heading any more; just target heading
			INCLINATION = acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG;
			d_heading = PadHeading;
/*		} else if (!strnicmp (line, "ATTACHPOINT", 11)) {
		    sscanf (line+11, "d%lf%lf%lf", &n,&atchtemp.x,&atchtemp.y,&atchtemp.z);
			attach[n] = atchtemp;
			//	   								     pos           dir       rot       
			if (n<4) SetAttachmentParams (attachpos[n], attach[n],   attachdir[n], _V(0,-1,0));
			else SetAttachmentParams (attachpos[n], attach[n],   attachdir[n], _V(0,0,1));
*/		} else ParsePayloadString(line);

		ParseScenarioLineEx (line, vs);
	}

	/* always set the stage whenever something may have changed */
	SetMainStage();

	// Double-tap all the animations so they display right
	RevertLandingGear();
	RevertLandingGear();

	RevertHatch();
	RevertHatch();

	RevertBrakes();
	RevertBrakes();

	RevertPayloadBayDoors();
	RevertPayloadBayDoors();

	RevertWings();
	RevertWings();

	RevertNozzles();
	RevertNozzles();

	RevertJets();
	RevertJets();

	if (brakes_proc == 1.0)
		BrakesOn = true;
//	if (wings_proc > 0.0)
//		MakeANewWing();

	ProgramAuto();

}

void TITOVV::ProgramAuto()
{

	bAutoProgrammed = true;
	N=14;
	double *fmet = new double[N];
	double *fcpitch = new double[N];
	double biastime = 0.0;
	if (stage == -2) biastime = -85.0;
	if (stage == -1) biastime = 100.0;
	if (stage == 0) biastime = 40.0;

	fmet[0]					=    0.0;
	fmet[1]					=   25.0;
	fmet[2]					=   35.0;
	fmet[3]					=   45.0;
	fmet[4]					=  155.0+biastime;
	fmet[5]					=  165.0+biastime;
	fmet[6]					=  180.0+biastime;
	fmet[7]					=  200.0+biastime;
	fmet[8]					=  235.0+biastime;
	fmet[9]					=  245.0+biastime;
	fmet[10]				=  258.0+biastime;
	fmet[11]				=  262.0+biastime;
	fmet[12]				=  267.0+biastime;
	fmet[13]				=  270.0+biastime;

	fcpitch[0]				= 45.0;
	fcpitch[1]				= 43.0;
	fcpitch[2]				= 40.0;
	fcpitch[3]				= 38.0;
	fcpitch[4]				= 36.0;
	fcpitch[5]				= 30.0;
	fcpitch[6]				= 24.0;
	fcpitch[7]				= 20.0;
	fcpitch[8]				= 16.0;
	fcpitch[9]				= 12.0;
	fcpitch[10]				=  8.0;
	fcpitch[11]				=  4.0;
	fcpitch[12]				=  2.5;
	fcpitch[13]				=  0.0;
	ProgramPitchMet( N, fmet, fcpitch );
}

void TITOVV::MakeANewWing()
{
//	                     alignment      position      func    chord S   AR   
		CreateAirfoil (LIFT_VERTICAL, _V(0,0, 0.25), VLiftCoeff, 4, 72, 4.85);

}

void TITOVV::PlaceComponents()
{
	COMMON_MESH tmp;
/*
	tmp.MeshHandle(hSeat);
	tmp.MeshPosition(EJSEAT_OFS);
	AddMeshEx (tmp);
	tmp.MeshHandle(hSeat2);
	tmp.MeshPosition(EJSEAT_OFS2);
	AddMeshEx (tmp);

	tmp.MeshHandle(hAstro);
	tmp.MeshPosition(ASTRO_OFS);
	AddMeshEx (tmp);
	tmp.MeshHandle(hAstro2);
	tmp.MeshPosition(ASTRO_OFS2);
	AddMeshEx (tmp);
*/
	if (stage > -3) {
		tmp.MeshHandle(hTITOVV);
		tmp.MeshPosition(TITOVV_OFS);
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hDummy);
		tmp.MeshPosition(TITOVV_OFS);
		AddMeshEx (tmp);
	}


	if (stage < 0) {
		tmp.MeshHandle(hTITOVB);
		tmp.MeshPosition(BOOSTER_OFS);
		AddMeshEx (tmp);
	}
	if ((stage == 0) || (stage == -1))  {
		tmp.MeshHandle(hTITOVVTk);
		tmp.MeshPosition(TANK_OFS);
		AddMeshEx (tmp);
	}
}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void TITOVV::SetMainStage( )
{
	int i;

	if (!AnimationsMade)
		MakeAnimations();

	for (i = 0; i < 5; i++) {
		hV = GetAttachmentStatus(attachpos[i]);
		if (hV != NULL) {
			AtchV[i] = oapiGetVesselInterface (hV);
			atchmass[i] = AtchV[i]->GetMass();
		}
		else atchmass[i] = 0.0;
	}

	if (stage < 0)
		SetBoosterStage();
	else if (stage == 0)
		SetTankStage();
	else if (stage == 1)
		SetSustainStage();
	else if (stage == 2) 
		SetTITOVVStage();
	else
		SetJetsStage();
}

void TITOVV::SetBoosterStage( )
{

	int n;

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (30);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	AttachedMass = 0.0;
	for (n = 0; n < 5; n++)
		AttachedMass += atchmass[n];

	if (stage == -1) SetEmptyMass ( TITOVV_DRYMASS + BOOSTER_DRYMASS + TANK_DRYMASS + JET_PROP_MASS + AttachedMass + CalculateMass() );
	else if (stage == -2) SetEmptyMass ( TITOVV_DRYMASS + BOOSTER_DRYMASS + JET_PROP_MASS + AttachedMass + CalculateMass() );
	else if (stage == -3) SetEmptyMass ( BOOSTER_DRYMASS - BOOSTER_RCSMASS - BJET_PROP_MASS + CalculateMass() );

//	SetTouchdownPoints (_V(-10,-3,-10), _V( 10,-3,-10), _V(0,-3,10));
	SetTouchdownPoints (_V(0,-8.85,16.0), _V(-9.15,-9.95,-9.0), _V(9.15,-9.95,-9.0));
	SetSurfaceFrictionCoeff (0.025, 0.4);
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-8.85,16.0), _V(0,0.055,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-8.85,16.0), _V(0,0.055,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-8.85,16.0), _V(0,0.055,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-8.85,16.0), _V(0,0.055,1), _V(0,1,0));

//	SetTouchdownPoints (_V(0,-10,-11), _V(-10,10,-11), _V(10,10,-11));
	SetPMI (_V(18,19.5,3.25));
	SetCrossSections (_V(42.4,76.4,12.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
//	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS_BOOSTER);
	MakeAirfoils();
	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
	SetMeshVisibilityMode(1,MESHVIS_ALWAYS);
	if (stage == -1) SetMeshVisibilityMode(2,MESHVIS_ALWAYS);


	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hTITOVV, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */
	if (ph_booster==NULL)  
		ph_booster  = CreatePropellantResource(BOOSTER_PROP_MASS,BOOSTER_PROP_MASS*InitFuelLevel,1.0); 
	if (stage == -1)
		if (ph_first==NULL)  
			ph_first  = CreatePropellantResource(FIRST_PROP_MASS,FIRST_PROP_MASS,1.0); 
	if (stage > -3)
		if (!ph_TitovV)  
			ph_TitovV  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 

//	if (!ph_TitovV)  
//		ph_TitovV  = CreatePropellantResource(RCS_PROP_MASS,RCS_PROP_MASS,1.0); 

//	if (stage == -3) DelPropellantResource(ph_TitovV);
	if ((stage < -1) && (ph_first)) DelPropellantResource(ph_first);

	SetDefaultPropellantResource (ph_booster); 

	th_first[0] = CreateThruster ( _V(-7.5,0,-15.5), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[0], _V(-7.5,-4.5,-15.5), &contrail);
	th_first[1] = CreateThruster ( _V( 7.5,0,-15.5), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[1], _V( 7.5,-4.5,-15.5), &contrail);
	th_first[2] = CreateThruster ( _V(-2.5,0,-15.5), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[2], _V(-2.5,-4.5,-15.5), &contrail);
	th_first[3] = CreateThruster ( _V( 2.5,0,-15.5), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[3], _V( 2.5,-4.5,-15.5), &contrail);
	if (stage > -3) {
		th_first[4] = CreateThruster ( _V(-3.335,0,-9.7), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_FIRST_VAC, ISP_FIRST_SL);
		AddExhaustStream (th_first[4], _V(-3.335,0,-9.8), &contrail);
		th_first[5] = CreateThruster ( _V( 3.335,0,-9.7), _V(0,0,1), THRUST_FIRST_VAC, ph_booster, ISP_FIRST_VAC, ISP_FIRST_SL);
		AddExhaustStream (th_first[5], _V( 3.335,0,-9.8), &contrail);
	}

	if (stage == -3) thg_first = CreateThrusterGroup (th_first, 4, THGROUP_MAIN);
		else thg_first = CreateThrusterGroup (th_first, 6, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 20.0, 2.0, _V(-7.5,-4.5, -15.5), _V(0,0,-1),tex);
	AddExhaust (th_first[1], 20.0, 2.0, _V( 7.5,-4.5, -15.5), _V(0,0,-1), tex);
	AddExhaust (th_first[2], 20.0, 2.0, _V(-2.5,-4.5, -15.5), _V(0,0,-1), tex);
	AddExhaust (th_first[3], 20.0, 2.0, _V( 2.5,-4.5, -15.5), _V(0,0,-1), tex);
	numengines = 4;
	if (stage > -3) {
		AddExhaust (th_first[4], 20.0, 2.0, tex);
		AddExhaust (th_first[5], 20.0, 2.0, tex);
		numengines = 6;
	}

	EnableTransponder(true);
	if (stage == -3) {
//		if (!ph_TitovV)  
//			ph_TitovV  = CreatePropellantResource(BOOSTER_RCSMASS,BOOSTER_RCSMASS,1.0); 
		if (!ph_Jets)  
			ph_Jets  = CreatePropellantResource(BJET_PROP_MASS,BJET_PROP_MASS,1.0); 
		SetAttControlsTITOVVBoost();
		if (jets_proc > 0.0) BoosterSwitchToJets();
//	}
//	if ((GetAltitude() > 17000.0) && (stage == -3)) {
		nozzles_proc = 1.0;
		nozzles_status = DOOR_OPEN;
		RevertNozzles();		// Freshly invoked booster stage needs to start with nozzles out.  Then we retract them.
		ActivateNavmode(NAVMODE_KILLROT);
	}
}

void TITOVV::SetTankStage( )
{

	int n;

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();
	if (bBoosterMade) DelBoosterAnimations();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (20);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	AttachedMass = 0.0;
	for (n = 0; n < 5; n++)
		AttachedMass += atchmass[n];


	SetEmptyMass ( TITOVV_DRYMASS + TANK_DRYMASS + JET_PROP_MASS + AttachedMass + CalculateMass() );

//	SetTouchdownPoints (_V(-10,-3,-10), _V( 10,-3,-10), _V(0,-3,10));
	SetTouchdownPoints (_V(0,-4.05,14.0), _V(-9.15,-3.9,-3.0), _V(9.15,-3.9,-3.0));
	SetSurfaceFrictionCoeff (0.025, 0.4);
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-4.05,14.0), _V(0,-0.0075,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-4.05,14.0), _V(0,-0.0075,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-4.05,14.0), _V(0,-0.0075,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-4.05,14.0), _V(0,-0.0075,1), _V(0,1,0));
//	SetTouchdownPoints (_V(0,-10,-11), _V(-10,10,-11), _V(10,10,-11));
	SetPMI (_V(18,19.5,3.25));
	SetCrossSections (_V(42.4,76.4,12.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
//	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);	
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
	SetMeshVisibilityMode(1,MESHVIS_ALWAYS);


	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hTITOVV, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_booster != NULL)
		DelPropellantResource (ph_booster);
	if (ph_first==NULL)  
		ph_first  = CreatePropellantResource(FIRST_PROP_MASS,FIRST_PROP_MASS*InitFuelLevel,1.0); 
	if (!ph_TitovV)  
		ph_TitovV  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 
//	if (!ph_TitovV)  
//		ph_TitovV  = CreatePropellantResource(RCS_PROP_MASS,RCS_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_first); 

	th_first[0] = CreateThruster ( _V(-3.335,0,-9.7), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[0], _V(-3.335,0,-9.8), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	th_first[1] = CreateThruster ( _V( 3.335,0,-9.7), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[1], _V( 3.335,0,-9.8), &contrail);
//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);

	thg_first = CreateThrusterGroup (th_first, 2, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 20.0, 2.0, tex);
	AddExhaust (th_first[1], 20.0, 2.0, tex);
	numengines = 2;

	EnableTransponder(true);
}

void TITOVV::SetSustainStage( )
{
	int n;

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();
	if (bBoosterMade) DelBoosterAnimations();

	PARTICLESTREAMSPEC contrail = 
	{
//		0, 5.0, 5, 150, 0.3, 4.0, 16, 3.0, 
		0, 1.1, 30, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 0.5, 3, 500, 0, 1, 50, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 0, 0.1
	};

	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (17);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */

	AttachedMass = 0.0;
	for (n = 0; n < 5; n++)
		AttachedMass += atchmass[n];

	SetEmptyMass ( TITOVV_DRYMASS + JET_PROP_MASS + AttachedMass + CalculateMass() );

	SetTouchdownPoints (_V(0,-2.55,14.0), _V(-9.15,-3.9,-3.0), _V(9.15,-3.9,-3.0));
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0));
	SetPMI (_V(16,17.5,3.0));
	SetCrossSections (_V(32.4,56.4,12.5));
	SetCW (0.05, 0.1, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
//	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,0.0);
//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);

	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hTITOVV, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_first != NULL)
		DelPropellantResource (ph_first);
	if (ph_booster != NULL)
		DelPropellantResource (ph_booster);
//	if (ph_deorbit==NULL)  
//		ph_deorbit  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 
	if (ph_TitovV==NULL)  
		ph_TitovV  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_TitovV); 


	th_first[0] = CreateThruster ( _V(-3.335,0,-9.7), _V(0,0,1), THRUST_FIRST_VAC, ph_TitovV, ISP_SECOND_VAC, ISP_SECOND_SL);
	AddExhaustStream (th_first[0], _V(-3.335,0,-9.8), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	th_first[1] = CreateThruster ( _V( 3.335,0,-9.7), _V(0,0,1), THRUST_FIRST_VAC, ph_TitovV, ISP_SECOND_VAC, ISP_SECOND_SL);
	AddExhaustStream (th_first[1], _V( 3.335,0,-9.7), &contrail);
//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);


	thg_first = CreateThrusterGroup (th_first, 2, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 20.0, 2.0, tex);
	AddExhaust (th_first[1], 20.0, 2.0, tex);
	numengines = 2;

	SetAttControlsTITOVV();
	EnableTransponder(true);
}

void TITOVV::SetTITOVVStage( )
{
	int n;

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();
	if (bBoosterMade) DelBoosterAnimations();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");

	SetSize (17);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	AttachedMass = 0.0;
	for (n = 0; n < 5; n++)
		AttachedMass += atchmass[n];

	SetEmptyMass ( TITOVV_DRYMASS + atchmass[0] + AttachedMass + CalculateMass() );

	SetTouchdownPoints (_V(0,-2.55,14.0), _V(-9.15,-3.9,-3.0), _V(9.15,-3.9,-3.0));
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0));
	SetPMI (_V(16.7,18.9,3.6));
	SetCrossSections (_V(24.2,55.4,9.8));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
//	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	Guidance (false);
	SetAttitudeRotLevel(_V(0,0,0));
	pitchinput = 0.0;
	yawinput = 0.0;
	rollinput = 0.0;

//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);

	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);

	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hTITOVV, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_first != NULL)
		DelPropellantResource (ph_first);
	if (ph_booster != NULL)
		DelPropellantResource (ph_booster);
//	if (ph_deorbit != NULL)				
//		DelPropellantResource (ph_deorbit); 
	if (ph_TitovV==NULL)  
		ph_TitovV  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 
	if (ph_Jets==NULL)  
		ph_Jets  = CreatePropellantResource(JET_PROP_MASS,JET_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_TitovV); 

	th_first[0] = CreateThruster ( _V( 0.0, 0.525,-9.26), _V(0,0,1), MAX_ATT_FIRST*4, ph_TitovV, ISP_RCS_VAC, ISP_RCS_VAC);

	th_first[1] = CreateThruster ( _V( 0.0,-0.525,-9.26), _V(0,0,1), MAX_ATT_FIRST*4, ph_TitovV, ISP_RCS_VAC, ISP_RCS_VAC);

	thg_first = CreateThrusterGroup (th_first, 2, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 2.0, 0.9, texRCS);
	AddExhaust (th_first[1], 2.0, 0.9, texRCS);

	numengines = 2;

	SetAttControlsTITOVV();
	EnableTransponder(true);
	if (nozzles_proc == 1.0)
		RevertNozzles();
	if (jets_status==DOOR_OPEN) {
		stage++;
		RevertJets();		// Extend the jet engines and set Stage 3
		SetMainStage();
	}
}


void TITOVV::SetJetsStage( )
{
	int n;

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();
	if (bBoosterMade) DelBoosterAnimations();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 325.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
//		0, 1.25, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");

	SetSize (17);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	AttachedMass = 0.0;
	for (n = 0; n < 5; n++)
		AttachedMass += atchmass[n];

	SetEmptyMass ( TITOVV_DRYMASS + AttachedMass + CalculateMass() );
	SetTouchdownPoints (_V(0,-2.55,14.0), _V(-9.15,-3.9,-3.0), _V(9.15,-3.9,-3.0));
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0));
	SetPMI (_V(16.7,18.9,3.6));
	SetCrossSections (_V(24.2,55.4,9.8));
	SetCW (0.05, 0.1, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
//	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	Guidance (false);
	SetAttitudeRotLevel(_V(0,0,0));
	pitchinput = 0.0;
	yawinput = 0.0;
	rollinput = 0.0;
//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);

	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);

	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hTITOVV, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_first != NULL)
		DelPropellantResource (ph_first);
	if (ph_booster != NULL)
		DelPropellantResource (ph_booster);
//	if (ph_deorbit != NULL)				
//		DelPropellantResource (ph_deorbit); 
//	if (ph_TitovV==NULL)  
//		ph_TitovV  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 
	if (ph_TitovV != NULL)
		DelPropellantResource (ph_TitovV);
	if (ph_Jets==NULL)  
		ph_Jets  = CreatePropellantResource(JET_PROP_MASS,JET_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_Jets); 

	th_first[0] = CreateThruster ( _V( 0.0, 0.0,-9.5), _V(0,0.0,1), JET_THRUST, ph_Jets, ISP_JET_VAC, ISP_JET_VAC);
	AddExhaustStream (th_first[0], _V( 0.0, 1.83,-9.7), &contrail);


	thg_first = CreateThrusterGroup (th_first, 1, THGROUP_MAIN);

	numengines = 1;

//	SetAttControlsTITOVV();
	EnableTransponder(true);
	if (nozzles_proc == 1.0)
		RevertNozzles();

}


/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */

void TITOVV::JettisonPart()
 {

				VESSELSTATUS vs;
				VECTOR3 ofs = _V(0,0,0);
				VECTOR3 vel = _V(0,-1.5,0);
				if (stage > -1)	
					vel = _V(0,-3.5,6.0);
//				ofs = Centaur_OFS;
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
//				SetEngineLevel(ENGINE_MAIN, 0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				vs.vrot.x = -0.2;
//				if (GetEngineLevel(ENGINE_MAIN) > 0.0) 
				if (stage > -1)
					vs.vrot.x = -0.5;
				vs.vrot.y = 0.0;
				vs.vrot.z = 0.0;
				char name[256];
				if ((stage == -1) || (stage == -2)) {
					DelBoosterAnimations();
					strcpy (name, GetName()); strcat (name, "-Booster");
					OBJHANDLE BSTR = oapiCreateVessel (name, "2001/TitovB", vs);
					VESSEL *remotevessel = oapiGetVesselInterface(BSTR);
					remotevessel->SetFuelMass(GetPropellantMass(ph_booster));
					delete remotevessel;
					DelPropellantResource (ph_booster); 
					PlayVesselWave3(MyID,PLAYBANG);
					if (stage == -2) stage = 0;		// Go straight from booster to upper stage
				}
				else if (stage == 0) {
					strcpy (name, GetName()); strcat (name, "-Tank");
					VESSEL::Create (name, "2001/TitovVTk", vs);	
					DelPropellantResource (ph_first); 
					PlayVesselWave3(MyID,PLAYBANG);
				}
/*				else if (GetThrusterGroupLevel (thg_first) == 0.0) {
					strcpy (name, GetName()); strcat (name, "-Deorbit Stage");
					VESSEL::Create (name, "TITOVV/TITOVV_Deorbit", vs);
					DelPropellantResource (ph_deorbit); 
					PlayVesselWave3(MyID,PLAYBANG);
				}
*/
//				else if (stage == 1) {
//					DelPropellantResource (ph_deorbit);
//				}
				else if (stage > 1) stage--;


				stage++;

 }

void TITOVV::AttachSweep()
{
	
			/* Now scan for nearby attach points; these need to stay in the bay!  */
				
			Local2Global (_V(0,0,0), grms);  // global position of RMS tip
			
			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (i = 0; i < oapiGetVesselCount(); i++) {
				hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < 3.0*oapiGetSize (hV)) { // in range
					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
						if (strncmp (id, "BOLT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						if (dist (gpos, grms) < 0.25) { // found one!
							// check whether satellite is currently clamped into payload bay
							if ((hV == GetAttachmentStatus (Baypos)) && (!Firstsweep))
							{
								AttachChild (hV, Baypos, hAtt);
								atchmass[5] = v->GetMass();
								BayVessel = hV;
								BayV = oapiGetVesselInterface (BayVessel);
							}
							else
							{	
								AttachChild (hV, Baypos, hAtt);
								atchmass[3] = v->GetMass();
								hDock = v->GetDockHandle(0);
								BayVessel = hV;
								BayV = oapiGetVesselInterface (BayVessel);
/*								if (hDock != NULL) {
									v->GetDockParams(hDock,pos, dir, rot);
									pos.z += 4.8;
									if (Dock1 == NULL)
										Dock1 = CreateDock(pos, dir, rot);
									else 
										SetDockParams(Dock1, pos, dir, rot);
									v->DelDock(hDock);
								}
*/
							}
//							return 1;
						}
					}
				}
			}
}


void TITOVV::AttachVessel()
{
	int n;
	DWORD i, k;
	double bestdist = 100.0;
	OBJHANDLE hVBest = NULL;
	int nbest;
	ATTACHMENTHANDLE hAtt, hAttbest;
	bool AlreadyAttached = false;

			/* Now scan for nearby attach points; these need to stay in the bay!  */
		for (n = 0; n < 5; n++) {
				
			Local2Global (attach[n], grms);  // global position of RMS tip
			GlobalRot (attachdir[n], gatchdir);
			GlobalRot (attachrot[n], gatchrot);

			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (i = 0; i < oapiGetVesselCount(); i++) {
				hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < 25.0 + 3.0*oapiGetSize (hV)) { // in range
					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
//						if (strncmp (id, "BOLT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						v->GlobalRot (dir, gdir);
						v->GlobalRot (rot, grot);
						if ((!GetAttachmentStatus (hAtt)) && (dist (gpos, grms) > 0.0 ) && (dist (gpos, grms) < MAX_GRAPPLING_DIST)) { // found one!
							double angleoff = 0.5*PI*sqrt(pow(gdir.x-gatchdir.x,2.0) + pow(gdir.y-gatchdir.y,2.0) + pow(gdir.z-gatchdir.z,2.0));
							double angleoff2 = 0.5*PI*sqrt(pow(grot.x-gatchrot.x,2.0) + pow(grot.y-gatchrot.y,2.0) + pow(grot.z-gatchrot.z,2.0));
							if ((angleoff > 160*RAD) && (angleoff2 < 20*RAD)) {
								AlreadyAttached = false;
								for (k = 0; k < nAttach; k++) { // now scan all attachment points of the candidate
									if (GetAttachmentStatus(v->GetAttachmentHandle(true,k)) != NULL)
										AlreadyAttached = true;
								}
								if (!AlreadyAttached) 
									if (dist (gpos, grms) < bestdist) {
										bestdist = dist (gpos, grms);
										hVBest = hV;
										nbest = n;
										hAttbest = hAtt;
									}
							}
						}
					}
				}
			}
		}
		if ((hVBest != NULL) && (bestdist < 100.0)) {
			AttachChild (hVBest, attachpos[nbest], hAttbest);
			AtchV[n] = oapiGetVesselInterface (hVBest);
//			return;
		}

}

int TITOVV::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	if (!fPressed) return 0; // only process keydown events
	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		if (KEYDOWN (keystate, OAPI_KEY_SPACE)) 
		{ // "Join Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_SPACE, 1.0))
			{

			oapiSetShowGrapplePoints (!oapiGetShowGrapplePoints ());

			return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_J)) 
		{ // "Join Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{
			/* CVE-lite function ! */
			/* IMPORTANT */
			/* call DeployPayload() to jettison the GPDS payloads in sequence */
	
			AttachVessel();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();

			return 1;
			}
		}
	}
	else {

		if (KEYDOWN (keystate, OAPI_KEY_J)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{
				VESSEL *depV;
							if ((stage < 2) && (stage > -3) && (pldoors_proc == 0.0)) {
								JettisonPart();
							}
							else if ((stage == -3) &&  (GetMachNumber() < 0.9) && (GetAltitude() < 30000.0)) {
								SetPropellantMass(ph_booster,0.0);
								if (jets_status==DOOR_CLOSED)
									RevertJets();		// Extend the jet engines and set them
								BoosterSwitchToJets();
								return 1;
							}
							else {
								OverridePayloadVelocity(_V(0,+0.25,0));
								depV = DeployPayload();
								if (depV == NULL) {
									if (GetAttachmentStatus(attachpos[0]) != NULL) {
										DetachChild(attachpos[0],0.02);
										atchmass[0] = 0.0;
									} else if (GetAttachmentStatus(attachpos[1]) != NULL) {
										DetachChild(attachpos[1],0.02);
										atchmass[1] = 0.0;
									} else if (GetAttachmentStatus(attachpos[2]) != NULL) {
										DetachChild(attachpos[2],0.02);
										atchmass[2] = 0.0;
									} else if (GetAttachmentStatus(attachpos[3]) != NULL) {
										DetachChild(attachpos[3],0.02);
										atchmass[3] = 0.0;
									} else if (GetAttachmentStatus(attachpos[4]) != NULL) {
										DetachChild(attachpos[4],0.1);
										atchmass[4] = 0.0;
									}
								}
							}

							/* CVE-lite function ! */
							/* IMPORTANT */
							/* call DeployPayload() to jettison the GPDS payloads in sequence */

							AttachSweep();

							/* IMPORTANT */
							/* Always re-set the stage after any kind of change */
							SetMainStage();
							if ((stage==1) && (pldoors_proc == 0.0)) SetThrusterGroupLevel (THGROUP_MAIN,1.0);
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_G)) 
		{ // Toggle the Gear
			if (oapiAcceptDelayedKey (OAPI_KEY_G, 1.0))
			{
				RevertLandingGear ();
				StopVesselWave3(MyID,PLAYGEAR);
				PlayVesselWave3(MyID,PLAYGEAR);
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_K)) 
		{ // Toggle the bay doors
			if ((oapiAcceptDelayedKey (OAPI_KEY_K, 1.0)) && (stage > -3))
			{
				RevertPayloadBayDoors ();
				StopVesselWave3(MyID,PLAYDOORS);
				PlayVesselWave3(MyID,PLAYDOORS);
				if (BayVessel != NULL)
					SendCentralMessage(BayV, "MSG", "REVERTPROTRUSIONS");
//				if (pldoors_status == DOOR_CLOSING)
//					Undock(ALLDOCKS);
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_B)) 
		{ // Toggle the brakes
			if (oapiAcceptDelayedKey (OAPI_KEY_B, 1.0))
			{
//				if (BrakesOn)
//					BrakesOn = false;
//				else BrakesOn = true;
				RevertBrakes();
				if (brakes_status == DOOR_OPENING)
					BrakesOn = true;
				else BrakesOn = false;
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_P)) 
		{ // Toggle the hatch / airlock
			if (oapiAcceptDelayedKey (OAPI_KEY_P, 1.0))
			{
				if (stage > -3)
					RevertHatch ();
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_O))
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_O, 1.0))
				if (!guidance) { 
					Guidance( true );
//					guidance=true;
					sprintf(oapiDebugString(),"Guidance ON");
					pitchinput = 0.0;
					yawinput = 0.0;
					rollinput = 0.0;
				}
				else {
					Guidance( false );
//					guidance = false;
					SetAttitudeRotLevel(_V(0,0,0));
					pitchinput = 0.0;
					yawinput = 0.0;
					rollinput = 0.0;

					sprintf(oapiDebugString(),"Guidance OFF");
				}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_U)) 
		{ // "Set launch Azimuth"
			if (oapiAcceptDelayedKey (OAPI_KEY_U, 1.0))
			{
				GetInclinationSP();
				return 1;
			}
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) 
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				{
				if (hatch_status == DOOR_OPEN)
					{
		//				char name[256];
		//				OBJHANDLE hvessel;
		//				strcpy (name, GetName()); strcat (name, "-EV1");
		//				hvessel=oapiGetVesselByName(name);
		//				if (hvessel == 0)
		//				{
		//					bEVA = true;
							SeparateEVA();
							SetMainStage();
		//					hvessel=oapiGetVesselByName(name);
		//					if (hvessel != 0)
		//						oapiSetFocusObject(hvessel);
		//				}
		//				else
		//				{
		//					oapiDeleteVessel(hvessel);
		//					bEVA = false;
		//					SetMainStage();
		//				};
					}
			}
		return 1;
		}
	}
	return 0;
}




int TITOVV::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}



void TITOVV::SetAttControlsTITOVVBoost()
{
	THRUSTER_HANDLE th_rot[2];

	th_rot[0] = CreateThruster (_V(0,-0.97, 5.25), _V(0, 1,0), MAX_ATT_FIRST, ph_booster, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.97,-5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_booster, ISP_FIRST_SL);
	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_booster, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_booster, ISP_FIRST_SL);
	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_booster, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_booster, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_booster, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_booster, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);


	th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_FIRST/4, ph_booster, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_booster, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_booster, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_FIRST/4, ph_booster, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

}


void TITOVV::SetAttControlsTITOVV()
{
	bAttSet = true;
	THRUSTER_HANDLE th_rot[2];

	th_rot[0] = CreateThruster (_V(0,-0.97, 5.25), _V(0, 1,0), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.97,-5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.5,-0.35, 13.75), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.5,-0.35, 13.75), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-7.02,0.119,-7.67), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 7.02,0.119,-7.67), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-6.95,0.079,-7.38), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 6.95,0.079,-7.38), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-7.07,0.148,-7.95), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 7.07,0.148,-7.95), _V(0, 1.0,0.0),texRCS);  
	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.5,-0.35, 13.75), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.5,-0.35, 13.75), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V(-7.31,-1.7, -8.55), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 7.31,-1.7, -8.55), _V(0,-1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0),MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0),MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 0.00, 0.07, 13.75), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-7.02,0.119,-7.67), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 7.02,0.119,-7.67), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-6.95,0.079,-7.38), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 6.95,0.079,-7.38), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-7.07,0.148,-7.95), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 7.07,0.148,-7.95), _V(0, 1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 0.0, 0.07, 13.75), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V(-7.31,-1.7, -8.55), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 7.31,-1.7, -8.55), _V(0,-1.0,0.0),texRCS);  
	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 0.2,0.05, 13.75), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V(-7.31,-1.7,-8.55), _V(-1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.5, 0.35, _V(-0.2, 0.05, 13.75), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 7.31,-1.7, -8.55), _V( 1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.5, 0.35, _V(-0.2, 0.05, 13.75), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V(-7.31,-1.7, -8.55), _V( 1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster (_V(-2,0.0,0.0), _V( 1,0,0), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 0.2, 0.05, 13.75), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 7.31,-1.7, -8.55), _V(-1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_FIRST/4, ph_TitovV, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.5, 0.35, _V(-7.31,-1.7, -8.55), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 7.02,0.119,-7.67), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 6.95,0.079,-7.38), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 7.07,0.148,-7.95), _V(0, 1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_TitovV, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_FIRST/4, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 7.31,-1.7, -8.55), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-7.02,0.119,-7.67), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-6.95,0.079,-7.38), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-7.07,0.148,-7.95), _V(0, 1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 0.0, 0.525,-9.26), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 0.0,-0.525,-9.26), _V(0,0,-1),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_FIRST, ph_TitovV, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.5, 0.35, _V( 0.2, 0.05, 13.75), _V(0,0,1),texRCS);  
	AddExhaust (th_rot[0], 1.5, 0.35, _V(-0.2, 0.05, 13.75), _V(0,0,1),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}
