/**
*
* $Id: CVEL-TITOVV.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/cvelmessage.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include "../../include/guidance.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

// All data from astronautix.com
const double ISP_FIRST_SL		=     365.0*G; 
const double ISP_FIRST_VAC		=     465.0*G; //***
const double ISP_SECOND_SL		=     445.0*G; 
const double ISP_SECOND_VAC		=     470.0*G; //***
const double ISP_RCS_VAC		=     455.0*G; //***
const double ISP_JET_VAC		=   15500.0*G;
const double ISP_BJET_VAC		=   30500.0*G;
const double THRUST_FIRST_VAC	=  707560.0*G/3.0; //***
const double MAX_ATT_FIRST		=    1660.0*G;
const double JET_THRUST			=   21000.0*G;
const double BJET_THRUST		=   18000.0*G;
const double BOOSTER_PROP_MASS	=  625566.0; 
const double FIRST_PROP_MASS	=  325566.0; 
const double SECOND_PROP_MASS	=	31528.0;
//const double RCS_PROP_MASS		=	 6000.0;
const double BJET_PROP_MASS		=   12000.0;
const double JET_PROP_MASS		=    2000.0;
const double TITOVV_DRYMASS		=   24500.0-JET_PROP_MASS; //***
const double TANK_DRYMASS		=   17250.0; //***
const double BOOSTER_DRYMASS	=   97250.0; //***
const double BOOSTER_RCSMASS	=     800.0; //***

const VECTOR3 TITOVV_OFS		= _V(0,0,0);
const VECTOR3 TANK_OFS			= _V(0,0,0);
const VECTOR3 BOOSTER_OFS		= _V(0,0,0);
const VECTOR3 EJSEAT_OFS		= _V(-0.462,-0.924,11.77);
const VECTOR3 EJSEAT_OFS2		= _V( 0.462,-0.924,11.77);
const VECTOR3 ASTRO_OFS			= _V(-0.462,-0.792,11.99);
const VECTOR3 ASTRO_OFS2		= _V( 0.462,-0.792,11.99);
const VECTOR3 EVA_OFS			= _V( 2.86,0.6,7.2);
const VECTOR3 CAMERA_OFS		= _V(-0.516, 0.378,11.928);
const VECTOR3 CAMERA_OFS_BOOSTER= _V(-0.77, -3.51,25.65);
const VECTOR3 OFS_SAT			= _V(0,0,0);

const double GEAR_OPERATING_SPEED = 0.25;
const double PLDOORS_OPERATING_SPEED = 0.07125;
const double BRAKES_OPERATING_SPEED = 0.25;
const double HATCH_OPERATING_SPEED = 0.125;
const double WINGS_OPERATING_SPEED = 0.25;
const double JETS_OPERATING_SPEED = 0.5;
const double NOZZLES_OPERATING_SPEED = 0.25;

const double MAX_GRAPPLING_DIST		= 20.0;


const double ROLL = 0.0;            // 180.0 to go to orbit heads-down.  Intermediate values probably won't work.

#define PLAYGEAR					 20		// the gear sound
#define PLAYBANG					 24		// the jettison sound
#define PLAYJETT					 25     // entry shield jettison sound
#define PLAYJET						 26     // the jet noise sound
#define PLAYDOORS					 27     // the payload door sound
#define MYRADARSOUND				  2     // Custom radar sound
#define MYRADARCLOSE				  3     // Custom radar sound

/* make your own vessel class (named TITOVV here, for example)
   subclassed from COMMON_VESSEL_EX */

class TITOVV : public COMMON_VESSEL_EX, GUIDANCE {
public:
	TITOVV (OBJHANDLE hObj, int fmodel);
	~TITOVV();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } gear_status, pldoors_status, brakes_status, hatch_status, wings_status, jets_status, nozzles_status;

	void MakeAnimations();
	void MakeBoosterAnimations();
	void DelBoosterAnimations();
	void MakeAirfoils();
	void MakeControlSurfaces();
	void BoosterSwitchToJets();
	void UpdateAnimations();
	void ActivateLandingGear (DoorStatus action);
	void RevertLandingGear();
	void ActivatePayloadBayDoors (DoorStatus action);
	void RevertPayloadBayDoors();
	void ActivateBrakes (DoorStatus action);
	void RevertBrakes();
	void ActivateHatch (DoorStatus action);
	void RevertHatch();
	void ActivateWings (DoorStatus action);
	void RevertWings();
	void ActivateJets (DoorStatus action);
	void RevertJets();
	void ActivateNozzles (DoorStatus action);
	void RevertNozzles();
	void SeparateEVA();
	void SetGearParameters (double state);
	void AttachSweep();
	void AttachVessel();
	void RussianBeeps();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VectorThrust();
	void AirBreathingEngine();
	void AirBreathingEngines();
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void ProgramAuto();
	void MakeANewWing();
	void PlaceComponents();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetBoosterStage();
	void SetMainStage(); 
	void SetTankStage();
	void SetSustainStage();
	void SetTITOVVStage();
	void SetJetsStage();
//	void SetAttControlsTranstage();
	void SetAttControlsTITOVVBoost();   
	void SetAttControlsTITOVV();    // this is to set up the thrusters. I like to keep them separate.
	void JettisonPart();
	bool AzInput (void *id, char *str, void *data);
	bool IncInput (char *rstr);
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
//	void clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
//	void clbkRenderHUD (int mode, const HUDPAINTSPEC *hps, SURFHANDLE hTex);

protected:
	double GetCPitch(double t);
	void Lateral_Guidance(VESSEL *vessel, double SimT, double SimT_Last);
	void Guidance_Routine(VESSEL *vessel, double SimT, double SimT_Last);
	void GetInclinationSP();
	void InitialiseGuidance();
	double pitchinput, yawinput, rollinput;
	VESSELSTATUS vst;
	int stage;


	bool Guidance() { return guidance; };
	void Guidance( bool g ) { guidance = g; };

	void DisplayInclinationSetPoint( bool d ) { DisplayIncSP=d; };
	bool DisplayInclinationSetPoint() { return DisplayIncSP; };

	double initialgamma, initialheading, launchrotate;


	PROPELLANT_HANDLE ph_booster,ph_first,ph_TitovV,ph_Jets;
	THRUSTER_HANDLE   th_first[6];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hTITOVV,hTITOVVTk,hTITOVB,hSeat,hAstro,hSeat2,hAstro2,hDummy;
	ANIMATIONCOMPONENT_HANDLE wing1, wing2, wing3, wing4, Gear1, Gear2, Gear3, Gear4, Gear5, Gear6, Gear7,
		PLDr1, PLDr2, Hatch1, Jets1, Nozl1;
	ANIMATIONCOMPONENT_HANDLE wing11, wing12, wing13, wing14, Gear11, Gear12, Gear13, Gear14, Gear15, Gear16, Gear17,
		Gear18, Gear19, Jets11, Jets12, Jets13, Nozl11, Elev11, Rudd11, Elev12, Rudd12, Elev13, Rudd13, Elev14, Rudd14;
	SURFHANDLE		  tex,texRCS, retex;
	UINT anim_lrudder,anim_rrudder,anim_laileron,anim_raileron,anim_lelevator,anim_relevator,anim_lelevatortrim,anim_relevatortrim;
	UINT anim_gear;     // handle for landing gear animation
	UINT anim_pldoors;     // handle for payload door animation
	UINT anim_brakes;	// handle for speedbrake animation
	UINT anim_hatch;	// handle for top hatch animation
	UINT anim_wings;	// handle for wings animation
	UINT anim_bwings;	// handle for booster wings animation
	UINT anim_jets;		// handle for jets animation
	UINT anim_nozzles;	// handle for nozzles animation
	double gear_proc;
	double pldoors_proc;
	double brakes_proc;
	double hatch_proc;
	double wings_proc;
	double jets_proc;
	double nozzles_proc;
	bool BrakesOn, havesound, bShieldJettisoned, bDeorbitFired, bThrustTerminated, bEjected, bEVA, 
		bAbort, AbortChecked, bAttSet,bBoosterMade, bAutoProgrammed, bControlsMade;
	UINT Exh1,Exh2,Exh3,Exh4;
	PSTREAM_HANDLE stream_a_1,stream_b_1,stream_a_2,stream_b_2,stream_a_3,stream_b_3,stream_a_4,stream_b_4;
	VECTOR3 Vold, Vnew;
	double dt, oldsimt;
	int MyID;
	ATTACHMENTHANDLE Dockpos, Baypos, sat_attach, Nose_atch1, Nose_atch2, MDD_atch;
	VECTOR3 gpos, grms, pos, dir, rot;
	OBJHANDLE hV, BayVessel, hvessel;
	VESSEL *BayV;
	DWORD i, j;
	double atchmass[6];
	DOCKHANDLE hDock, Dock0, Dock1, Dock2;
	bool Firstsweep;
	int numengines;
	double tempP, tempY, tempR;
	VECTOR3 AngularVel;
	bool autopilot;
	double Bank_Error, Yaw_Error;
	bool engineignited, bgoingsouth;
	double last_simt;
	double ignition_time;
	double density;
	double displaytime, simt, d_heading;
	double MachPitch;
	bool first_guidance_pass, AnimationsMade;
	double MET, diff, target, hdg, diverge, yawangle, targetrate, rate, amount;
	AIRFOILHANDLE Wing, Rudder, BstrWing, BstrRudder;
	char tvmesh[80];
	ATTACHMENTHANDLE  attachpos[5];
	VECTOR3           attach[5];
	VECTOR3			  attachdir[5];
	VECTOR3           attachrot[5];
	VECTOR3			  gatchdir, gdir, gatchrot, grot, horiz;
	VESSEL *AtchV[5];
	double AttachedMass;
	bool RadarFlagForSound;
	bool RadarFlagForClose;

};







/* basic vessel class setup */

void TITOVV::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void TITOVV::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void TITOVV::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void TITOVV::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void TITOVV::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void TITOVV::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int TITOVV::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


bool TITOVV::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{

//	if (oapiCockpitMode() != COCKPIT_VIRTUAL) return false;

	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	// show gear deployment status
	if (gear_status == DOOR_OPEN || (gear_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			skp->Rectangle (cx-d/2, cy-d*5, cx+d/2, cy-d*4);
			skp->Rectangle (cx-d*3, cy-d*2, cx-d*2, cy-d);
			skp->Rectangle (cx+d*2, cy-d*2, cx+d*3, cy-d);
		}
	}

	// show pldoors deployment status
	if (pldoors_status == DOOR_OPEN || (pldoors_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
//			skp->Arc (cx, cy-d*8.5, cx+d*4, cy-d*6.5, cx, cy-d*6, cx+d*4, cy-d*7.5);
//			skp->Arc (cx-d*4, cy-d*8.5, cx, cy-d*6.5, cx-d*4, cy-d*7.5, cx, cy-d*6);
			skp->Line(cx-d*4, cy-d*8.5, cx-d*3, cy-d*7.5);
			skp->LineTo(cx-d*2, cy-d*7.0);
			skp->LineTo(cx-d, cy-d*6.65);
			skp->LineTo(cx, cy-d*6.5);
			skp->LineTo(cx+d, cy-d*6.65);
			skp->LineTo(cx+d*2, cy-d*7.0);
			skp->LineTo(cx+d*3, cy-d*7.5);
			skp->LineTo(cx+d*4, cy-d*8.5);
			skp->Text (cx-d, cy-d*8, "BAY", 3);
		}
	}


	// show hatch deployment status
	if (hatch_status == DOOR_OPEN || (hatch_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			skp->Text (cx+d*5, cy+d*1.2, "DOOR", 4);
		}
	}

	// show brakes deployment status
	if (BrakesOn) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			skp->Text (cx-d*9, cy+d*1.2, "BRAKES", 6);
		}
	}

	// show RCS mode
	if (oapiCockpitMode() == COCKPIT_VIRTUAL) {
		switch (GetAttitudeMode()) {
		case RCS_ROT:
			skp->Text (0, hps->H-13, "RCS ROT", 7);
			break;
		case RCS_LIN:
			skp->Text (0, hps->H-13, "RCS_LIN", 7);
			break;
		}
	}


// Draw Flight Director
	if ((!MECO_Cond) && (stage < 2)) 
		if ((stage != -3) || (jets_status != DOOR_OPEN)) {
	double fYawError = Yaw_Error*RAD;
	double fBankError = Bank_Error;
	double fPitchError = Pitch_Error;

/*	VECTOR3 rloc = _V(0,0,0);
	HorizonInvRot(rhorizon,rloc);

	double currHdg;
	oapiGetHeading(GetHandle(),&currHdg);
	double fYawError = rhorizon.x - currHdg;
	double fPitchError = rhorizon.y - GetPitch();
	double fBankError = rhorizon.z - (GetBank()- (2*PI));
*/

	double InnCoord = 0.5;
//	if (guidance) InnCoord = 0.25;

	int iX = hps->CX + (int)(fYawError * hps->Scale);
	int iY = hps->CY - (int)(fPitchError * hps->Scale);
	int iR = hps->Markersize / 2;
	if (!guidance)
		skp->Ellipse(iX - iR, iY - iR, iX + iR, iY + iR);


	//two lines for indicating Roll Error
	int iX1 = (int)(1.0 * hps->Markersize * cos(fBankError));
	int iX2 = (int)(InnCoord * hps->Markersize * cos(fBankError));
	int iY1 = (int)(1.0 * hps->Markersize * sin(fBankError));
	int iY2 = (int)(InnCoord * hps->Markersize * sin(fBankError));
	skp->MoveTo(iX + iX1, iY + iY1);
	skp->LineTo(iX + iX2, iY + iY2);

	skp->MoveTo(iX - iX1, iY - iY1);
	skp->LineTo(iX - iX2, iY - iY2);

	//Top Marker for indicating Roll Error
	int iX3 = (int)(0.8 * hps->Markersize * sin(fBankError));
	int iX4 = (int)(InnCoord * hps->Markersize * sin(fBankError));
	int iY3 = (int)(0.8 * hps->Markersize * cos(fBankError));
	int iY4 = (int)(InnCoord * hps->Markersize * cos(fBankError));
	skp->MoveTo(iX + iX3, iY - iY3);
	skp->LineTo(iX + iX4, iY - iY4);

	// draw a diamond, not a circle
	if (guidance) {
		skp->MoveTo(iX + iX4, iY - iY4);
		skp->LineTo(iX + iX2, iY + iY2);
		skp->LineTo(iX - iX4, iY + iY4);
		skp->LineTo(iX - iX2, iY - iY2);
		skp->LineTo(iX + iX4, iY - iY4);
	}
	}

	if (GetMachNumber() < 5.0) {
	//Draw Equivalent airspeed & pressure altitude
	if(GetAtmPressure() > 0.0)
	{
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * cx;
		const double EAS_EXP = 2.0/7.0;
		const double P0 = 1.01325E6;

		double fEAS = 661.48 * sqrt(5.0 * GetAtmPressure()/ATMP * (pow(GetDynPressure() / GetAtmPressure() + 1, EAS_EXP) - 1.0));

		sprintf(pszBuffer, "%6.1f KEAS", fEAS);

		skp->Text(bx, cy - d, pszBuffer, 11);
	}
	if (GetAtmPressure() > 7000.0)				  // About 60,000'; use pressure altitude
	{
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * cx;
		const double EAS_EXP = 2.0/7.0;
		const double P0 = 1.01325E6;

		bx = 0.5 * (cx + hps->W) - d * 8;

		double fHp = (1.0 - pow(GetAtmPressure()/ATMP, 0.190263)) * (288.15/0.00198122);

		long H_press = (long)(fHp);
		H_press = H_press - H_press % 50;

		sprintf(pszBuffer, "%6d'  ", H_press);

		skp->Text(bx, cy - d, pszBuffer, 9);
	} else {									// Do inertial altitude instead
		int d = hps->Markersize/2;		
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * cx;
		const double EAS_EXP = 2.0/7.0;
		const double P0 = 1.01325E6;

		bx = 0.5 * (cx + hps->W) - d * 8;

		double fIalt = GetAltitude()/0.3048;

		long I_Alt = (long)(fIalt);
		I_Alt = I_Alt - I_Alt % 50;

		sprintf(pszBuffer, "%6d' I", I_Alt);

		skp->Text(bx, cy - d, pszBuffer, 9);
	}

	//Draw Radar altitude
	if(GetAltitude() < 2500.0 * 0.3048)
	{
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * (cx + hps->W) - d * 8;
		const double GEAR_Y = -1.35;

		sprintf(pszBuffer, "%6dR", static_cast<long>((GetAltitude() + GEAR_Y)/0.3048));

		skp->Text(bx, cy + d, pszBuffer, 7);
	}


	//Draw G-Meter

		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;

		char pszBuffer[40];
		int bx = 0.5 * cx;

		VECTOR3 ForceVec, WeightVec;
		GetForceVector(ForceVec);
		GetWeightVector(WeightVec);
		ForceVec -= WeightVec;
		double G_level = sqrt(pow(ForceVec.x,2)+pow(ForceVec.y,2)+pow(ForceVec.z,2)) / (9.81*GetMass());

		sprintf(pszBuffer, "  %.2f G", G_level);

		skp->Text(bx, cy + d, pszBuffer, strlen(pszBuffer));

	
	}
	return true;
}


/*
// --------------------------------------------------------------
// Render custom HUD elements
// --------------------------------------------------------------
void TITOVV::clbkRenderHUD (int mode, const HUDPAINTSPEC *hps, SURFHANDLE hTex)
{
	VESSEL3::clbkRenderHUD (mode, hps, hTex);

	static float texw = 512.0f, texh = 256.0f;
	float cx = (float)hps->CX, cy = (float)hps->CY;
	int i, nvtx = 0, nidx = 0;
	static NTVERTEX vtx[12+16+4];
	static WORD idx[18+36+6];
	static float scl = 0;
	static NTVERTEX vgear[12];
	static NTVERTEX vnose[16];
	static NTVERTEX vbrk[4];
	static WORD igear[18] = {
		0,3,1, 3,0,2,
		4,7,5, 7,4,6,
		8,11,9, 11,8,10
	};
	static WORD inose[36] = {
		0,1,2, 2,3,0,
		0,6,1, 6,7,1,
		1,8,2, 8,9,2,
		2,10,3, 10,11,3,
		3,4,0, 0,4,5,
		12,15,13, 12,14,15
	};
	static WORD ibrk[6] = {
		0,3,1, 0,2,3
	};

	if (scl != hps->Markersize*0.25f) { // resize
		scl = hps->Markersize*0.25f;
		memset (vgear, 0, 12*sizeof(NTVERTEX));
		float x[12] = {-4,-2,-4,-2,2,4,2,4,-1,1,-1,1};
		float y[12] = {-2,-2,-4,-4,-2,-2,-4,-4,-6,-6,-8,-8};
		for (i = 0; i < 12; i++) {
			vgear[i].x = cx + x[i]*scl;
			vgear[i].y = cy + y[i]*scl;
			vgear[i].tu = (405.0f + (18.0f * (i%2)))/texw;
			vgear[i].tv = (104.0f - (18.0f * ((i%4)/2)))/texh;
		}
		memset (vnose, 0, 16*sizeof(NTVERTEX));
		float xn[16] = {0,1,0,-1,-31,-30,30,31,31,30,-30,-31,  -13,13,-13,13};
		float yn[16] = {-1,0,1,0,-30,-31,-31,-30,30,31,31,30,  -25,-25,-28.9,-28.9};
		float un[16] = {392.5f, 397.0f, 392.5f, 388.0f, 388.0f, 392.5f, 392.5f, 397.0f, 397.0f, 392.5f, 392.5f, 388.0f,    124.0f, 204.0f, 124.0f, 204.0f};
		float vn[16] = {92.0f, 96.5f, 101.0f, 96.5f, 96.5f, 92.0f, 92.0f, 96.5f, 96.5f, 101.0f, 101.0f, 96.5f,             118.0f, 118.0f, 106.0f, 106.0f};
		for (i = 0; i < 16; i++) {
			vnose[i].x = cx + xn[i]*scl*0.4f;
			vnose[i].y = cy + yn[i]*scl*0.4f;
			vnose[i].tu = un[i]/texw;
			vnose[i].tv = vn[i]/texh;
		}
		memset (vbrk, 0, 4*sizeof(NTVERTEX));
		float xb[4] = {-9.1f, 9.1f, -9.1f, 9.1f};
		float yb[4] = {-30.0f, -30.0f, -33.9f, -33.9f};
		float ub[4] = {205.0f, 261.0f, 205.0f, 261.0f};
		float vb[4] = {118.0f, 118.0f, 106.0f, 106.0f};
		for (i = 0; i < 4; i++) {
			vbrk[i].x = cx + xb[i]*scl*0.4f;
			vbrk[i].y = cy + yb[i]*scl*0.4f;
			vbrk[i].tu = ub[i]/texw;
			vbrk[i].tv = vb[i]/texh;
		}
	}

	// show gear deployment status
	if (gear_status == DOOR_OPEN || (gear_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		memcpy (vtx+nvtx, vgear, 12*sizeof(NTVERTEX));
		for (i = 0; i < 18; i++) idx[nidx+i] = igear[i]+nvtx;
		nvtx += 12;
		nidx += 18;
	}

	// show nosecone status
	if (pldoors_status == DOOR_OPEN || (pldoors_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		double tmp;
		if (pldoors_status == DOOR_CLOSED || modf (oapiGetSimTime(), &tmp) < 0.5) {
			memcpy (vtx+nvtx, vnose, 16*sizeof(NTVERTEX));
			for (i = 0; i < 36; i++) idx[nidx+i] = inose[i]+nvtx;
			nvtx += 16;
			nidx += 36;
		}
	}

	// show airbrake status
	if (brakes_status != DOOR_CLOSED) {
		double tmp;
		if (brakes_status == DOOR_OPEN || modf (oapiGetSimTime(), &tmp) < 0.5) {
			memcpy (vtx+nvtx, vbrk, 4*sizeof(NTVERTEX));
			for (i = 0; i < 6; i++) idx[nidx+i] = ibrk[i]+nvtx;
			nvtx += 4;
			nidx += 6;
		}
	}

	if (nvtx) {
		MESHGROUP grp = {vtx, idx, nvtx, nidx, 0, 0, 0, 0, 0};
		MESHHANDLE hmesh = oapiCreateMesh (1, &grp);
		oapiRenderHUD (hmesh, &hTex);
		oapiDeleteMesh (hmesh);
	}
/*
	if (nvtx) {
		MESHGROUP grp = {vtx, idx, nvtx, nidx, 0, 0, 0, 0, 0};
		MESHHANDLE hmesh = oapiCreateMesh (1, &grp);
		oapiRenderHUD (hmesh, &hTex);
		oapiDeleteMesh (hmesh);
	}
*/
//}

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new TITOVV (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (TITOVV*)vessel;
}


