/**
*
* $Id: Titov-G.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2006 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/orbitersdk.h"
#include "../../include/orbitersoundsdk3.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

// All data from astronautix.com
const double ISP_FIRST_SL		=     447.0*G; 
const double ISP_FIRST_VAC		=     447.0*G; //***
const double THRUST_FIRST_VAC	=   11485.0*G; //***
const double MAX_ATT_FIRST		=     250.0*G;
const double FIRST_PROP_MASS	=   11900.0; //*** the total mass of fueled TitovG is 13000kg
const double TITOVG_MAX_BURN	=     625.0; //***
const double TITOVG_DRYMASS		=    1100.0; //***

#define PLAYGEAR					 20		// the gear sound
#define PLAYBANG					 24		// the jettison sound
#define PLAYJETT					 25     // entry shield jettison sound
#define PLAYJET						 26     // the jet noise sound
#define PLAYDOORS					 27     // the payload door sound
#define MYRADARSOUND				  2     // Custom radar sound
#define MYRADARCLOSE				  3     // Custom radar sound


/* make your own vessel class (named TITOVG here, for example)
   subclassed from COMMON_VESSEL_EX */

class TITOVG : public COMMON_VESSEL_EX {
public:
	TITOVG (OBJHANDLE hObj, int fmodel);
	~TITOVG();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void AttachVessel();
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void RussianBeeps();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 

	void SetAttControlsTitovG();    // this is to set up the thrusters. I like to keep them separate.
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);

protected:
	PROPELLANT_HANDLE ph_first;
	THRUSTER_HANDLE   th_first[2];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hTitovG;
	SURFHANDLE		  tex;
	ATTACHMENTHANDLE  attachpos;
	VECTOR3           attach;
	VECTOR3 gpos, grms, pos, dir, rot, attachdir, gatchdir, gdir;
	OBJHANDLE hV, AtchVessel, hvessel;
	VESSEL *AtchV;
	double atchmass;
	int MyID;
	bool RadarFlagForSound;
	bool RadarFlagForClose;

};







/* basic vessel class setup */


// Initialisation

/* basic vessel class setup */

void TITOVG::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void TITOVG::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void TITOVG::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void TITOVG::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void TITOVG::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void TITOVG::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int TITOVG::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new TITOVG (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (TITOVG*)vessel;
}

/*
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	TITOVG *TitovG= new TITOVG( hvessel, flightmodel );
	return TitovG;
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel) 
{
	if (vessel) delete (TITOVG*)vessel;
}

// Keyboard interface handler
DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	TITOVG *TitovG = (TITOVG*)vessel;
	return TitovG->ConsumeKey(  keystate );
}

// Keyboard interface handler
DLLCLBK int ovcConsumeBufferedKey (VESSEL *vessel, DWORD dwKey, bool fPressed, const char *keystate)
{
	TITOVG *TitovG = (TITOVG*)vessel;
	return TitovG->ConsumeBufferedKey( dwKey, fPressed, keystate );
}

DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	TITOVG *TitovG = (TITOVG*)vessel;
	TitovG->SetClassCaps(  cfg );
}

DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{
	TITOVG *TitovG = (TITOVG*)vessel;
	TitovG->TimeStep(  simt );
}

// Read status from scenario file
DLLCLBK void ovcLoadStateEx (VESSEL *vessel, FILEHANDLE scn, void *vs)
{
	TITOVG *TitovG = (TITOVG*)vessel;
	TitovG->LoadState(  scn, vs );
}


// Save status to scenario file
DLLCLBK void ovcSaveState (VESSEL *vessel, FILEHANDLE scn)
{
	TITOVG *TitovG = (TITOVG*)vessel;
	TitovG->SaveState(  scn );

}

*/