/**
*
* $Id: b2707.cpp
* (c) 2005 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* Autopilot by rjcroy
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "b2707.h"


// 1. vertical lift component (wings and body)
// Borrowed from Mike Norman's Shuttle code

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5,  0.02, 0.655,     0.46,   0.32, 0.12, 0.05, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,     0,  0.002,  0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	// change for drag increase at high Mach:
	// Subsonic Equivalents

	double Cl0, Cl1, Cd0, Cd1, Cd2;
	if(M>=5.0) {
		Cl0 = -0.187040;
		Cl1 =  1.275600;
		Cd0 =  0.178540;
		Cd1 = -0.402900;
		Cd2 =  1.539960;
		*cl = Cl0 + Cl1*aoa;
		*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
 	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
	if(M<1.25) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.03 + 0.2*oapiGetInducedDrag (*cl, 6.5, 0.92);
	}
	else if(M<5.0) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.035 + 0.25*oapiGetInducedDrag (*cl, 4.1, 0.85);
// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
	}
}


// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 2.5, 0.8) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	
	if (M > 1.0) *cd *= 0.6;
	else *cd *= 0.3;
}


B2707::B2707 (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	strcat (meshname,"2001/b2707");

	hB2707				= oapiLoadMeshGlobal (meshname);

	tex = oapiRegisterExhaustTexture("RealExhaust");

	/* set the propellant handles to NULL so I can check if I need to 
	/* create them in the SetMainStage() function. */

	ph_b2707=NULL;
	bBoosterMade=false;
	bControlsMade=false;
	gear_status=DOOR_CLOSED;
	gear_proc=0.0;
	wings_status=DOOR_OPEN;
	wings_proc=1.0;
	oldsimt = -0.01;
	Vold = _V(0,0,0);
	BrakesOn = false;

	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
	RequestLoadVesselWave3(MyID,PLAYGEAR,"Sound\\2001\\gear.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYBANG,"Sound\\2001\\stg2sep.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYJETT,"Sound\\2001\\fasep.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYJET,"Sound\\2001\\jet.wav",BOTHVIEW_FADED_FAR);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();   
 
	MakeAnimations();

	Firstsweep = true;

}


B2707::~B2707()
{

}

void B2707::MakeAnimations()
{

	anim_gear = 0.0;
	anim_wings = 0.0;
	anim_nose = 1.0;

	anim_gear = CreateAnimation (0);
	anim_wings = CreateAnimation (1);
	anim_nose = CreateAnimation (0);
	anim_lelevator = CreateAnimation (0.5);
	anim_relevator = CreateAnimation (0.5);
	anim_laileron = CreateAnimation (0.5);
	anim_raileron = CreateAnimation (0.5);
	anim_rudder = CreateAnimation (0.5);
	anim_lelevon = CreateAnimation (0.5);
	anim_relevon = CreateAnimation (0.5);


/*	static UINT LGearDoorGrp[1] = {10};
	static MGROUP_ROTATE LGearDoor (0, LGearDoorGrp, 1,
		_V(-1.038, 0.68,0.3), _V(0,0,1), (float)(-130*RAD));
*/
	static UINT LFSuptGrp[1] = {7};
	static MGROUP_ROTATE LFSupt (0, LFSuptGrp, 1,
		_V(-1.3,-0.7,-5.17), _V( 1,0,-0.45), (float)( 110*RAD));
	static UINT LFGrGrp[2] = {1,6};
	static MGROUP_ROTATE LFGear (0, LFGrGrp, 2,
		_V(-1.3,-0.11,-1.85), _V( 1,0,-0.45), (float)(-90*RAD));

	static UINT LASuptGrp[1] = {8};
	static MGROUP_ROTATE LASupt (0, LASuptGrp, 1,
		_V(-1.3,-0.9,-14.48), _V( 1,0,-0.45), (float)( 110*RAD));
	static UINT LAGrGrp[2] = {2,9};
	static MGROUP_ROTATE LAGear (0, LAGrGrp, 2,
		_V(-1.3,-0.285,-11.19), _V( 1,0,-0.45), (float)(-90*RAD));

	static UINT RFSuptGrp[1] = {11};
	static MGROUP_ROTATE RFSupt (0, RFSuptGrp, 1,
		_V( 1.3,-0.7,-5.17), _V( 1,0,0.45), (float)( 110*RAD));
	static UINT RFGrGrp[2] = {3,10};
	static MGROUP_ROTATE RFGear (0, RFGrGrp, 2,
		_V( 1.3,-0.11,-1.85), _V( 1,0, 0.45), (float)(-90*RAD));

	static UINT RASuptGrp[1] = {12};
	static MGROUP_ROTATE RASupt (0, RASuptGrp, 1,
		_V( 1.3,-0.9,-14.48), _V( 1,0,0.45), (float)( 110*RAD));
	static UINT RAGrGrp[2] = {4,13};
	static MGROUP_ROTATE RAGear (0, RAGrGrp, 2,
		_V( 1.3,-0.285,-11.19), _V( 1,0, 0.45), (float)(-90*RAD));
	
	static UINT NoseGrGrp[2] = {0,5};
	static MGROUP_ROTATE NoseGear (0, NoseGrGrp, 2,
		_V( 0.0, 0.145, 38.37), _V( 1,0, 0), (float)(-90*RAD));

/*
	static UINT LTailGearGrp[1] = {23};
	static MGROUP_ROTATE LTailGear (0, LTailGearGrp, 1,
		_V( 0.74,-0.35,-4.0), _V(0,0.0,1.0), (float)(-45*RAD));
	static UINT LTailTransGrp[1] = {23};
	static MGROUP_TRANSLATE LTailTrans (0, LTailTransGrp, 1,
		_V( 0.0,0.0,-0.30));
	static UINT RTailGearGrp[1] = {22};
	static MGROUP_ROTATE RTailGear (0, RTailGearGrp, 1,
		_V(-0.74,-0.35,-4.0), _V(0,0.0,1.0), (float)( 45*RAD));
	static UINT RTailTransGrp[1] = {22};
	static MGROUP_TRANSLATE RTailTrans (0, RTailTransGrp, 1,
		_V( 0.0,0.0,-0.30));
*/
// Known Groups:

// 0 Nose Tire
// 1 LF Tires
// 2 LA Tires
// 3 RF Tires
// 4 RA Tires
// 5 Nose strut
// 6 LF Truck
// 7 LF Strut
// 8 LA Strut
// 9 LA Truck
// 10 RF Truck
// 11 RF Strut
// 12 RA Strut
// 13 RA Truck
// 14 L Wing
// 15 L Flat
// 16 R Flat
// 17 L LE
// 18 R LE
// 19 L Multivon
// 20 R Multivon
// 21 L Tail
// 22 R Tail
// 23 VStab
// 24 Ventral
// 25 LAftAngle
// 26 Nac4
// 27 Nac1
// 28 Nac2
// 29 Nac3
// 30 Rudder
// 31 R Wing
// 32 L Elevon
// 33 R Elevon
// 34 RAftAngle
// 35 L Aileron
// 36 R Aileron
// 37 FuseBlob
// 38 Pylons
// 39 Fuse
// 40 NoseFront
// 41 NoseAft
// 42 NoseMid

/*
	static UINT LPLDoorGrp[1] = {0};
	static MGROUP_ROTATE LPLDoor (0, LPLDoorGrp, 1,
		_V(-0.488,0.774,0.0), _V(0,0,1), (float)( 150*RAD));
	static UINT RPLDoorGrp[1] = {0};
	static MGROUP_ROTATE RPLDoor (0, RPLDoorGrp, 1,
		_V( 0.488,0.774,0.0), _V(0,0,1), (float)(-150*RAD));
	static UINT HatchGrp[1] = {0};
	static MGROUP_ROTATE Hatch (0, HatchGrp, 1,
		_V( 0.0,1.049,1.978), _V(1,0,0), (float)( 155*RAD));
//		_V(-0.422,0.8,6.25), _V(0,0,1), (float)( 130*RAD));
*/

//	AddAnimationComponent (anim_gear, 0.05, 0.4, &LGearDoor);
	Gear1 = AddAnimationComponent (anim_gear, 0.4, 0.75, &LFSupt);
	AddAnimationComponent (anim_gear, 0.75, 0.95, &LFGear, Gear1);
	Gear2 = AddAnimationComponent (anim_gear, 0.4, 0.75, &LASupt);
	AddAnimationComponent (anim_gear, 0.75, 0.95, &LAGear, Gear2);
	Gear3 = AddAnimationComponent (anim_gear, 0.4, 0.75, &RFSupt);
	AddAnimationComponent (anim_gear, 0.75, 0.95, &RFGear, Gear3);
	Gear4 = AddAnimationComponent (anim_gear, 0.4, 0.75, &RASupt);
	AddAnimationComponent (anim_gear, 0.75, 0.95, &RAGear, Gear4);
	AddAnimationComponent (anim_gear, 0.05, 0.95, &NoseGear);


	// ***** Wing pivot animation *****

	static UINT RWingGrp[1] = {31};
	static MGROUP_ROTATE RWing2 (0, RWingGrp, 1,
		_V( 5.75,0.23,-9.52), _V(0,1,0), (float)(-40*RAD));
	static UINT RAilGrp[1] = {35};
	static MGROUP_ROTATE RAil2 (0, RAilGrp, 1,
		_V( 5.75,0.23,-9.52), _V(0,1,0), (float)(-80*RAD));
	static UINT LWingGrp[1] = {14};
	static MGROUP_ROTATE LWing2 (0, LWingGrp, 1,
		_V(-5.75,0.23,-9.52), _V(0,1,0), (float)( 40*RAD));
	static UINT LAilGrp[1] = {36};
	static MGROUP_ROTATE LAil2 (0, LAilGrp, 1,
		_V(-5.75,0.23,-9.52), _V(0,1,0), (float)( 80*RAD));
	wing1 = AddAnimationComponent (anim_wings, 0, 1, &RWing2);
	wing2 = AddAnimationComponent (anim_wings, 0, 1, &LWing2);
//	AddAnimationComponent (anim_wings, 0, 1, &RAil2);
//	AddAnimationComponent (anim_wings, 0, 1, &LAil2);
//	wing3 = AddAnimationComponent (anim_wings, 0, 1, &RWing2);
//	wing4 = AddAnimationComponent (anim_wings, 0, 1, &LWing2);

	static UINT MidNoseGrp[1] = {42};
	static MGROUP_ROTATE MidNose (0, MidNoseGrp, 1,
		_V( 0, 1.342,38.46), _V(1,0,0), (float)( 10*RAD));
	static UINT FrontNoseGrp[1] = {40};
	static MGROUP_ROTATE FrontNose (0, FrontNoseGrp, 1,
		_V( 0, 0.585,43.766), _V(1,0,0), (float)(-10*RAD));
	nose1 = AddAnimationComponent (anim_nose, 0,1, &MidNose);
	AddAnimationComponent (anim_nose, 0,1, &FrontNose, nose1);

	// ***** Elevon animation *****
	static UINT LElevonGrp[1] = {19};
	static MGROUP_ROTATE LElevon (0, LElevonGrp, 1,
		_V(-13,1.5,-32.57), _V(-1,0.0, 0), (float)(-35*RAD));
	AddAnimationComponent (anim_lelevon, 0, 1, &LElevon);
	static UINT RElevonGrp[1] = {20};
	static MGROUP_ROTATE RElevon (0, RElevonGrp, 1,
		_V( 13,1.5,-32.57), _V( 1,0.0, 0), (float)( 35*RAD));
	AddAnimationComponent (anim_relevon, 0, 1, &RElevon);

	// ***** Elevator animation *****
	static UINT LElevatorGrp[1] = {32};
	static MGROUP_ROTATE LElevator (0, LElevatorGrp, 1,
		_V(-4.35,1.1,-30.57), _V(-1,0.0, 0.3), (float)(-25*RAD));
	AddAnimationComponent (anim_lelevator, 0, 1, &LElevator);
	static UINT RElevatorGrp[1] = {33};
	static MGROUP_ROTATE RElevator (0, RElevatorGrp, 1,
		_V( 4.35,1.1,-30.57), _V( 1,0.0, 0.3), (float)( 25*RAD));
	AddAnimationComponent (anim_relevator, 0, 1, &RElevator);
/*
	// ***** Elevator trim animation *****
	static UINT LElevatorTrimGrp[1] = {3};
	static MGROUP_ROTATE LElevatorTrim (5, LElevatorTrimGrp, 1,
		_V(-2.15,0.83,-3.74), _V(-0.25,0.45,-0.4), (float)(-10*RAD));
	anim_lelevatortrim = CreateAnimation (0.5);
	AddAnimationComponent (anim_lelevatortrim, 0, 1, &LElevatorTrim, wing1);
	static UINT RElevatorTrimGrp[1] = {5};
	static MGROUP_ROTATE RElevatorTrim (5, RElevatorTrimGrp, 1,
		_V( 2.15,0.83,-3.74), _V( 0.25,0.45,-0.4), (float)( 10*RAD));
	anim_relevatortrim = CreateAnimation (0.5);
	AddAnimationComponent (anim_relevatortrim, 0, 1, &RElevatorTrim, wing2);
*/
	// ***** Aileron animation *****
	static UINT LAileronGrp[1] = {36};
	static MGROUP_ROTATE LAileron (0, LAileronGrp, 1,
		_V( 27.95,2.17,-20.27), _V(-1,0.1, 0.15), (float)(-20*RAD));
	AddAnimationComponent (anim_laileron, 0, 1, &LAileron, wing1);

	static UINT RAileronGrp[1] = {35};
	static MGROUP_ROTATE RAileron (0, RAileronGrp, 1,
		_V(-27.95,2.17,-20.27), _V( 1,0.1, 0.15), (float)(-20*RAD));
	AddAnimationComponent (anim_raileron, 0, 1, &RAileron, wing2);

	// ***** Rudder animation *****
	static UINT RudderGrp[1] = {30};
	static MGROUP_ROTATE Rudder (0, RudderGrp, 1,
		_V( 0,7.17,-35.625), _V(0,1,-0.15), (float)(-11*RAD));
	AddAnimationComponent (anim_rudder, 0, 1, &Rudder);



}


void B2707::MakeAirfoils()
{
	ClearAirfoilDefinitions();
	ClearVariableDragElements();


		CreateVariableDragElement (&gear_proc, 0.8, _V(0, -1, 0));


//					align,         attack pt,       coeff func, c, S,   AR
	CreateAirfoil (LIFT_VERTICAL, _V(0,0,-3.2),    VLiftCoeff, 27, 1066, 6.55);
	// wing and body lift+drag components

	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-50.5), HLiftCoeff, 14,  95, 3.625);
	// vertical stabiliser and body lift and drag components


	MakeControlSurfaces();
}


void B2707::MakeControlSurfaces()
{

//  Aargh!  Can't delete control surfaces
//  Need to make sure we declare once and only once

	if (bControlsMade) return;
	bControlsMade = true;

	CreateControlSurface (AIRCTRL_ELEVATOR,      46.9,  0.8, _V(  3.2,0,-34.67), AIRCTRL_AXIS_XPOS, anim_lelevator);
	CreateControlSurface (AIRCTRL_ELEVATOR,      46.9,  0.8, _V( -3.2,0,-34.67), AIRCTRL_AXIS_XPOS, anim_relevator);
	CreateControlSurface (AIRCTRL_RUDDER,        12.0,  0.7, _V(   0, 0,-33.75), AIRCTRL_AXIS_YPOS, anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,       46.1, -0.8, _V(-3.2,0,-34.58), AIRCTRL_AXIS_XPOS, anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,       46.1, -0.8, _V( 3.2,0,-34.58), AIRCTRL_AXIS_XNEG, anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM,  41.9,  0.8, _V(  3.2,0,-34.67), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM,  41.9,  0.8, _V( -3.2,0,-34.67), AIRCTRL_AXIS_XPOS);

}




void B2707::UpdateAnimations()
{	

	// animate landing gear
	if (gear_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * GEAR_OPERATING_SPEED;
		if (gear_status == DOOR_CLOSING) {
			if (gear_proc > 0.0)
				gear_proc = max (0.0, gear_proc-da);
			else {
				gear_status = DOOR_CLOSED;
			}
		} else  { // door opening
			if (gear_proc < 1.0)
				gear_proc = min (1.0, gear_proc+da);
			else {
				gear_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_gear, gear_proc);
	}

/*
	// animate wings
	if (wings_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * WINGS_OPERATING_SPEED;
		if (wings_status == DOOR_CLOSING) {
			if (wings_proc > 0.0)
				wings_proc = max (0.0, wings_proc-da);
			else {
				wings_status = DOOR_CLOSED;
			}
		} else  { // door opening
			if (wings_proc < 1.0)
				wings_proc = min (1.0, wings_proc+da);
			else {
				wings_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_wings, wings_proc);
	}
*/
}

void B2707::ActivateLandingGear (DoorStatus action)
{
	gear_status = action;
}

void B2707::RevertLandingGear ()
{
	ActivateLandingGear (gear_status == DOOR_CLOSED || gear_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void B2707::ActivateWings (DoorStatus action)
{
	wings_status = action;
}

void B2707::RevertWings ()
{
	ActivateWings (wings_status == DOOR_CLOSED || wings_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}


void B2707::AirBreathingEngine()
// Thanks to David Hopkins for publishing his AirBreathingEngine source.  
{//=========================================================
double alt2;
double eas2;
double cf1, cf2;
double ActualThrust;


alt2 = GetAltitude();
eas2 = GetAirspeed ();

alt2 = alt2*3.28;
alt2 = alt2*0.78;		// a kludge that keeps the "airbreathing" engine operating at higher altitude
eas2 = eas2*2.236;

cf1 = 1.67e-6*(eas2*eas2)-8.8789e-4*(eas2)+0.9977;	
cf2 = 3.0e-10*(alt2*alt2)-3.0000e-5*(alt2)+0.9934;

// Nasty kludge to keep the thrust from increasing dramatically above 100kft
if (alt2 > 50000)
	cf2 = 0.25*(1-((alt2-50000)*.0001));
if (cf2 < 0) cf2 = 0;
if (cf1 > 3.0) cf1 = 3.0;

ActualThrust = cf1*cf2*THRUST_JET_VAC;
if (GetMachNumber() > 2.6)
	ActualThrust = ActualThrust * (1-(GetMachNumber()*2-2.6*2));

//sprintf(oapiDebugString(), "cf1: %.3f, cf2:  %.3f, cf1*cf2:  %.3f ActualThrust:  %.3f", cf1, cf2, cf1*cf2, ActualThrust);

SetThrusterMax0 (th_first[0],ActualThrust);
SetThrusterMax0 (th_first[1],ActualThrust);
SetThrusterMax0 (th_first[2],ActualThrust);
SetThrusterMax0 (th_first[3],ActualThrust);

double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN);


		if(ThrusterActive>0.0f)
			{
			// we calculate the volume at wich it must be played
			double volume=110+(145*ThrusterActive);
			double freq=11050+(9000*ThrusterActive);
			PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
			}
		else if(IsPlaying3(MyID,PLAYJET))
			{
			// else shut down the sound if it's playing
			StopVesselWave3(MyID,PLAYJET);
			}


//SetThrusterMax0 (th_first[0],ActualThrust);
};

void B2707::SetGearParameters (double state)
{

	double geary = ((1.0-state)*1.0);
	if ((gear_status == DOOR_OPENING) || (gear_status == DOOR_CLOSING)) {
		SetTouchdownPoints (_V(0,-3.8+1.5*geary,38.0), _V(-2.4,-4.7+geary,-10.0), _V( 2.4,-4.7+geary,-10.0));
	}
	if (gear_proc == 1.0) {
		SetSurfaceFrictionCoeff (0.025, 0.4);
	} else 
	{
		SetSurfaceFrictionCoeff (5, 0.5);		
		if(GroundContact()) 
		{
			//crashed on the ground

			gear_proc = 0.0;
			gear_status = DOOR_OPEN;
		}
		
	}


	double steerforce;
	double steerangle;

	steerforce = 40.0;
	if (GetAirspeed() < 40.0)
		steerforce *=8.0;
	steerforce = steerforce*GetMass();
	if (GetAirspeed () < 5.0) steerforce*=(GetAirspeed()/5); // from 1* to 0*

	steerangle = GetControlSurfaceLevel (AIRCTRL_RUDDER);
	steerforce = steerforce * steerangle;
	if ((GetAltitude() > 5.5) || (state < 1.0)) steerforce = 0.0;

	AddForce (_V(steerforce, 0, 0), _V(0, 0, 12.0));
	AddForce (_V(-steerforce, 0, 0), _V(0, 0, -12.0));


	// sprintf (oapiDebugString(), "%0.4f", steerforce );


}


void B2707::TimeStep(  double simt )
{
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */

	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
			SetMainStage();
	}


	if (!bControlsMade)
		MakeControlSurfaces();

//
//	Make the wings and nose sweep with Mach Number
//
	wings_proc = ((GetMachNumber()*2)-1.0);
	wings_proc = 1-wings_proc;
	if (wings_proc < 0.0) wings_proc = 0.0;
	if (wings_proc > 1.0) wings_proc = 1.0;
	nose_proc = wings_proc;
	SetAnimation (anim_wings, wings_proc);
	SetAnimation (anim_nose, nose_proc);
//
//  End Mach-Sweep function
//


//
// I wanted the little elevons at the tail to act as control surfaces, but to fade out as we speed up towards Mach
// So they're not treated as control surfaces at all, but a separate animation
// This necessitated the following kludgy code:
//

	lelevon_des = 0.5+(GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE)+GetManualControlLevel(THGROUP_ATT_BANKLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_BANKRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE))*(1-GetMachNumber());
	relevon_des = 0.5+(GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE)+GetManualControlLevel(THGROUP_ATT_BANKRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_BANKLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE))*(1-GetMachNumber());
	if (GetMachNumber() > 1.0) {
		relevon_des = 0.5;
		lelevon_des = 0.5;
	}
	if (relevon_des > relevon_proc)
		relevon_proc += 0.01;
	if (relevon_des < relevon_proc)
		relevon_proc -= 0.01;
	if (lelevon_des > lelevon_proc)
		lelevon_proc += 0.01;
	if (lelevon_des < lelevon_proc)
		lelevon_proc -= 0.01;
	SetAnimation (anim_lelevon, lelevon_proc);
	SetAnimation (anim_relevon, relevon_proc);

//
// End little elevon lockout code
//	

	AirBreathingEngine();
	double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN);
	if(ThrusterActive>0.0f)
		{
		// we calculate the volume at wich it must be played
		double volume=110+(145*ThrusterActive);
		double freq=11050+(9000*ThrusterActive);
		PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
		}
	else if(IsPlaying3(MyID,PLAYJET))
		{
		// else shut down the sound if it's playing
		StopVesselWave3(MyID,PLAYJET);
		}


	UpdateAnimations();
	SetGearParameters(gear_proc);

}


/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */


void B2707::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void B2707::SetClassCaps(  FILEHANDLE cfg )
{

	CheckCentralMessage();

	SetMainStage();
}

void B2707::SaveState( FILEHANDLE scn )
{
	char cbuf[256];

	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */


//	Undock(ALLDOCKS);

	sprintf (cbuf, "%d %0.4f", gear_status, gear_proc);
	oapiWriteScenario_string (scn, "GEAR", cbuf);
	sprintf (cbuf, "%d %0.4f", wings_status, wings_proc);
	oapiWriteScenario_string (scn, "WINGS", cbuf);
	sprintf (cbuf, meshname);
	oapiWriteScenario_string (scn, "MESH", cbuf);

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void B2707::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		if (!strnicmp (line, "GEAR", 4)) {
		sscanf (line+4, "%d%lf", &gear_status, &gear_proc);
		} else if (!strnicmp (line, "WINGS", 5)) {
		sscanf (line+6, "%d%lf", &wings_status, &wings_proc);
		} else if(!strnicmp (line, "MESH", 4) ) {
			sscanf( line+4, "%s", &meshname);

		} else ParsePayloadString(line);

		ParseScenarioLineEx (line, vs);
	}



	/* always set the stage whenever something may have changed */
	SetMainStage();

	RevertLandingGear();
	RevertLandingGear();
	RevertWings();
	RevertWings();

}


void B2707::PlaceComponents()
{
	COMMON_MESH tmp;


	tmp.MeshHandle(hB2707);
	tmp.MeshPosition(_V(0,0,0));
	AddMeshEx (tmp);


}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void B2707::SetMainStage( )
{

	SoundOptionOnOff3(MyID,PLAYMAINTHRUST,true);
	hB2707				= oapiLoadMeshGlobal (meshname);

	SetB2707Stage();
}


void B2707::SetB2707Stage( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (30);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass (JET_DRYMASS + CalculateMass() );

	SetTouchdownPoints (_V(0,-3.8,38.0), _V(-2.4,-4.7,-10.0), _V( 2.4,-4.7,-10.0));
	SetSurfaceFrictionCoeff (0.025, 0.4);
//	SetTouchdownPoints (_V(0,-10,-11), _V(-10,10,-11), _V(10,10,-11));
	SetPMI (_V(504,517,20.1));
	SetCrossSections (_V(462.4,1138,12.5));
	SetCW (0.05, 0.2, 0.7, 0.7);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
//	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */
	if (!ph_b2707)  
		ph_b2707  = CreatePropellantResource(JET_PROP_MASS,JET_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_b2707); 

	SetMaxWheelbrakeForce (7.5*GetMass());

	th_first[0] = CreateThruster ( _V( 8.33, 0.6,-37.4), _V(0,0,1), THRUST_JET_VAC, ph_b2707, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[0], _V( 8.33, 0.6,-37.4), &contrail);
	th_first[1] = CreateThruster ( _V( 2.50,-0.6,-37.0), _V(0,0,1), THRUST_JET_VAC, ph_b2707, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[1], _V( 2.50,-0.6,-37.0), &contrail);
	th_first[2] = CreateThruster ( _V(-2.50,-0.6,-37.0), _V(0,0,1), THRUST_JET_VAC, ph_b2707, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[2], _V(-2.50,-0.6,-37.0), &contrail);
	th_first[3] = CreateThruster ( _V(-8.33, 0.6,-37.4), _V(0,0,1), THRUST_JET_VAC, ph_b2707, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[3], _V(-8.33, 0.6,-37.4), &contrail);


	thg_first = CreateThrusterGroup (th_first, 4, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 20.0, 2.0, _V( 8.33, 0.6,-37.4), _V(0,0,-1), tex);
	AddExhaust (th_first[1], 20.0, 2.0, _V( 2.50,-0.6,-37.4), _V(0,0,-1), tex);
	AddExhaust (th_first[2], 20.0, 2.0, _V(-2.50,-0.6,-37.4), _V(0,0,-1), tex);
	AddExhaust (th_first[3], 20.0, 2.0, _V(-8.33, 0.6,-37.4), _V(0,0,-1), tex);
//	numengines = 4;

	SoundOptionOnOff3(MyID,PLAYMAINTHRUST,false);

	EnableTransponder(true);

}



int B2707::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	if (!fPressed) return 0; // only process keydown events
	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else { // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
				VESSEL *depV;
				OBJHANDLE hVessel;
				OverridePayloadVelocity(_V(0,+0.25,0));
				depV = DeployPayload();
				hVessel=oapiGetVesselByName(depV->GetName());
				if (hVessel != 0)
					oapiSetFocusObject(hVessel);
					

					/* CVE-lite function ! */
					/* IMPORTANT */
					/* call DeployPayload() to jettison the GPDS payloads in sequence */

					/* IMPORTANT */
					/* Always re-set the stage after any kind of change */
				SetMainStage();
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_B)) 
		{ // Toggle the brakes
			if (oapiAcceptDelayedKey (OAPI_KEY_B, 1.0))
			{
				if (BrakesOn)
					BrakesOn = false;
				else BrakesOn = true;
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_G)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_G, 1.0)) {
				// Toggle the Gear
				RevertLandingGear ();
				StopVesselWave3(MyID,PLAYGEAR);
				PlayVesselWave3(MyID,PLAYGEAR);
			}
			return 1;
		}

	}
	return 0;
}



int B2707::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}



