/**
*
* $Id: B2707.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/cvelmessage.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1


const double ISP_JET_SL			=    8500.0*G; 
const double ISP_JET_VAC		=   10500.0*G; //***


const double THRUST_JET_VAC		=   28677.0*G*2.5; //
const double JET_DRYMASS		=   130308.0; //***
const double JET_PROP_MASS		=   306175.0-34020.0-JET_DRYMASS; //***

const double R_EARTH = 6371000;			// radius of the earth (m)
const double U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
const double APO_ALT = 240.0;


const VECTOR3 B2707_OFS			= _V(0,0,0);
const VECTOR3 CAMERA_OFS		= _V(-0.33, 1.1,42.6);

const double GEAR_OPERATING_SPEED = 0.25;
const double WINGS_OPERATING_SPEED = 0.25;

const double ROLL = 0.0;            // 180.0 to go to orbit heads-down.  Intermediate values probably won't work.

#define PLAYGEAR					 20		// the gear sound
#define PLAYBANG					 24		// the jettison sound
#define PLAYJETT					 25     // entry shield jettison sound
#define PLAYJET						 26     // the jet noise sound

/* make your own vessel class (named B2707 here, for example)
   subclassed from COMMON_VESSEL_EX */

class B2707 : public COMMON_VESSEL_EX {
public:
	B2707 (OBJHANDLE hObj, int fmodel);
	~B2707();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } gear_status, bgear_status, pldoors_status, brakes_status, hatch_status, wings_status, jets_status, nozzles_status;

	void MakeAnimations();
	void MakeAirfoils();
	void MakeControlSurfaces();
	void UpdateAnimations();
	void ActivateLandingGear (DoorStatus action);
	void RevertLandingGear();
	void ActivateWings (DoorStatus action);
	void RevertWings();
	void SetGearParameters (double state);
	void AirBreathingEngine();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void PlaceComponents();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage();  
	void SetB2707Stage();
	bool AzInput (void *id, char *str, void *data);
	bool IncInput (char *rstr);
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);
	double pitchinput, yawinput, rollinput;
	VESSELSTATUS vst;
	int stage;

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
//	void clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);

protected:

	PROPELLANT_HANDLE ph_booster,ph_first,ph_b2707,ph_escape1,ph_escape2,ph_escape3,ph_Jets;
	THRUSTER_HANDLE   th_first[12], th_fake[1];
	THGROUP_HANDLE    thg_first,thg_first2,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hB2707;
	ANIMATIONCOMPONENT_HANDLE wing11, wing12, wing13, wing14, Gear1, Gear2, Gear3, Gear4, nose1,
		Gear18, Gear19, Gear1A, Elev11, Rudd11, Elev12, Rudd12, Elev13, Rudd13, Elev14, Rudd14;
	SURFHANDLE		  tex,texLH2,texRCS, retex;
	UINT anim_rudder,anim_laileron,anim_raileron,anim_lelevator,anim_relevator,anim_lelevatortrim,anim_relevatortrim;
	UINT anim_lelevon, anim_relevon;
	UINT anim_gear;     // handle for landing gear animation
	UINT anim_nose;     // handle for nose animation	
	UINT anim_wings;	// handle for wings animation
	double gear_proc;
	double lelevon_proc, relevon_proc, lelevon_des, relevon_des;
	double nose_proc;
	double wings_proc;
	bool BrakesOn, bBayOpens, bSphereJettisoned, bDeorbitFired, bThrustTerminated, bEjected, bEVA, bAbort, AbortChecked, bConeJettisoned;
	UINT Exh1,Exh2,Exh3,Exh4;
	PSTREAM_HANDLE stream_a_1,stream_b_1,stream_a_2,stream_b_2,stream_a_3,stream_b_3,stream_a_4,stream_b_4;
	VECTOR3 Vold, Vnew;
	double dt, oldsimt;
	int MyID;
	ATTACHMENTHANDLE Baypos;
	VECTOR3 gpos, grms, pos, dir, rot;
	OBJHANDLE hV, BayVessel;
	VESSEL *BayV;
	DWORD i, j;
	double atchmass;
	double a,e,h,V_periapsis,Altitude;
	DOCKHANDLE hDock, Dock0;
	bool Firstsweep, AutoPilot;
	char *key;
	char *message;
	VECTOR3 Rel_Vel;
	double Velocity, MachPitch;
	ANIMATIONCOMPONENT_HANDLE wing1, wing2, wing3, wing4;
	double tempP, tempY, tempR;
	VECTOR3 AngularVel;
	bool autopilot;
	bool engineignited, bgoingsouth;
	double last_simt;
	double ignition_time;
	double density;
	double displaytime, simt, d_heading;
	bool first_guidance_pass, bBoosterMade, bControlsMade, bAutoProgrammed;
	bool Burn1, Burn2, Burn3;
	double MET, diff, target, hdg, diverge, yawangle, targetrate, rate, amount;
	AIRFOILHANDLE BstrWing, BstrRudder;
	char meshname[80];

};










/* basic vessel class setup */

void B2707::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void B2707::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void B2707::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void B2707::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void B2707::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void B2707::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int B2707::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}

bool B2707::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	// show gear deployment status
	if (gear_status == DOOR_OPEN || (gear_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			skp->Rectangle (cx-d/2, cy-d*5, cx+d/2, cy-d*4);
			skp->Rectangle (cx-d*3, cy-d*2, cx-d*2, cy-d);
			skp->Rectangle (cx+d*2, cy-d*2, cx+d*3, cy-d);
			skp->Rectangle (cx-d*3, cy-d/2, cx-d*2, cy+d/2);
			skp->Rectangle (cx+d*2, cy-d/2, cx+d*3, cy+d/2);
		}
	}

	//Draw Equivalent airspeed & pressure altitude
	if(GetAtmPressure() > 0.0)
	{
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * cx;
		const double EAS_EXP = 2.0/7.0;
		const double P0 = 1.01325E6;

		double fEAS = 661.48 * sqrt(5.0 * GetAtmPressure()/ATMP * (pow(GetDynPressure() / GetAtmPressure() + 1, EAS_EXP) - 1.0));

		sprintf(pszBuffer, "%6.1f KEAS", fEAS);

		skp->Text(bx, cy - 5*d, pszBuffer, 11);

		bx = 0.5 * (cx + hps->W) - d * 8;

		double fHp = (1.0 - pow(GetAtmPressure()/ATMP, 0.190263)) * (288.15/0.00198122);

		long H_press = (long)(fHp);
		H_press = H_press - H_press % 50;

		sprintf(pszBuffer, "%6d'", H_press);

		skp->Text(bx, cy - 5*d, pszBuffer, 7);
	}

	//Draw Radar altitude
	if(GetAltitude() < 2500.0 * 0.3048)
	{
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * (cx + hps->W) - d * 8;
		const double GEAR_Y = -1.35;

		sprintf(pszBuffer, "%6dR", static_cast<long>((GetAltitude() + GEAR_Y)/0.3048));

		skp->Text(bx, cy - 3*d, pszBuffer, 7);
	}


	//Draw G-Meter

		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;

		char pszBuffer[40];
		int bx = 0.5 * cx;

		VECTOR3 ForceVec, WeightVec;
		GetForceVector(ForceVec);
		GetWeightVector(WeightVec);
		ForceVec -= WeightVec;
		double G_level = sqrt(pow(ForceVec.x,2)+pow(ForceVec.y,2)+pow(ForceVec.z,2)) / (9.81*GetMass());

		sprintf(pszBuffer, "  %.2f G", G_level);

		skp->Text(bx, cy - 3*d, pszBuffer, strlen(pszBuffer));



	return true;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new B2707 (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (B2707*)vessel;
}


